/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

/*
 * If a TSA Form need customization at client side, this is done here.
 * Particularly useful for improving the layout or implementing special event handlers.
 */
define(
  
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ]
  
  , function( $
            , TV ) {
    
    //-------------------------------------------------------------------------
    
    logger = tsa.getLogger( 'FormCustomizer' );
    
    //-------------------------------------------------------------------------
    
    /**
     * @constructor
     * @param {Object} properties Property map.
     */
    tsa.FormCustomizer = function(properties) {
      
      tsa.FormCustomizer.instance = this;
      
      logger.info('Form Customizer created.');
    };
    
    /**
     * Run the HTML Form customization process.
     */
    tsa.FormCustomizer.prototype.run = function() {
      
      logger.info('Form Customizer processing started: ' + $('form').length + ' form(s) will be processed.');
      
      $('form').each($.proxy(function(formIndex,form) {
        
        /** @type {string} */ var action = $(form).attr('action');
        
        if ((typeof action) === 'string') {
          
          if ((action.indexOf('/discovery/update/com.ibm.servicestation.DiscoverySchedule'     )>=0) ||
              (action.indexOf('/transmission/update/com.ibm.servicestation.FullCollectionJob' )>=0) ||
              (action.indexOf('/inventorysettings/update/com.ibm.servicestation.inventory'    )>=0) ||
              (action.indexOf('/schedrestart/update/com.ibm.servicestation.schedrestart'      )>=0)) {
            
            this.customizeScheduleForm(form);
          }
        }
      }
      , this));
      
      this.showRequiredLegendIfNeeded();
    };
    
    /**
     * This function should be executed when new forms are loaded.
     * Note: this contains a function previously included into the
     *       legacy HTML template: adding the "* Required fields"
     *       legend when required fields are present.
     */
    tsa.FormCustomizer.prototype.showRequiredLegendIfNeeded = function() {
      /* ----------------------------------------------------
       * Required legend must appear if there are required
       * fields in the form(s).
       * ----------------------------------------------------
       */
      /** @type {boolean} */ var requiredFieldsDetected = ( $( 'span.ibm-required' ).length > 0 );
      /** @type {Element} */ var containerDiv           = $('div.ibm-container').get( 0 );
      /** @type {boolean} */ var requiredLegendDetected = ( $( '.tsa-required-legend' ).length > 0 );
      
      if( requiredFieldsDetected && ( containerDiv !== null ) &&
          !requiredLegendDetected &&
          TV.typeIsObject( typeof tsa.i18n ) && ( tsa.i18n !== null ) &&
          TV.typeIsString( typeof tsa.i18n.labelRequiredLegend ) )
      {
        logger.info( 'Required fields detected: adding the "fields required" legend.' );
        $( '<p>' + tsa.i18n.labelRequiredLegend + '</p>' ).insertBefore( containerDiv );
      }
    };
    
    /**
     * Customize the Schedule Form.
     * @private
     * @param {HTMLForm} form The form.
     */
    tsa.FormCustomizer.prototype.customizeScheduleForm = function(form) {
      
      /** @type {Element}         */ var weekModeRadio      = $(form).find('#daymode1').get(0);
      /** @type {Element}         */ var monthModeRadio     = $(form).find('#daymode2').get(0);
      /** @type {Element}         */ var weekDaysRow        = $(form).find('#day1').closest('tr').get(0);
      /** @type {Element}         */ var monthDaysRow       = $(form).find('#monthday1').closest('tr').get(0);
      /** @type {Element}         */ var monthDaysParent    = $(form).find('#monthday1').parent().get(0);
      /** @type {Element}         */ var newMonthDaysParent = document.createElement('div');
      /** @type {Array.<Element>} */ var inputs             = [];
      /** @type {Array.<Element>} */ var labels             = [];
      /** @type {number}          */ var weekDaysChecked    = 0;
      /** @type {number}          */ var x                  = 46;
      /** @type {number}          */ var y                  = 20;
      /** @type {number}          */ var cols               = 7;
      /** @type {number}          */ var rows               = 0;
      
      // Day mode changed event handler:
      
      var dayModeChanged = function(event) {
        
        $(weekDaysRow ).css( { 'display' : ( weekModeRadio  .checked ? '' : 'none' ) } );
        $(monthDaysRow).css( { 'display' : ( monthModeRadio .checked ? '' : 'none' ) } );
        
        if (weekModeRadio  .checked) $('input[name=monthday]').prop('checked', true);
        if (monthModeRadio .checked) $('input[name=day]'     ).prop('checked', true);
        
        // If the user triggered the event, clear the check-boxes:
        if (event !== null) {
          
          if (weekModeRadio  .checked) $('input[name=day]'     ).prop('checked', false);
          if (monthModeRadio .checked) $('input[name=monthday]').prop('checked', false);
        }
      };
      
      $('input[type=radio][name=daymode]').on('change', dayModeChanged);
      
      // How many week days are checked?
      
      for (var weekday=1; weekday<=7; weekday++) {
        
        weekDaysChecked += ($('#day' + weekday).get(0).checked ? 1 : 0);
      }
      
      // Check a day mode radio if no one is initially checked:
      
      if (!weekModeRadio.checked && !monthModeRadio.checked) {
        
        weekModeRadio  .checked = (weekDaysChecked !== 7);
        monthModeRadio .checked = (weekDaysChecked === 7);
      }
      
      // Calling the day mode change event handler:
      
      dayModeChanged(null);
      
      // Retrieving month days' HTML Elements:
      
      $(monthDaysParent).children().each(function(childIndex, child) {
        
        /** @type {string} */ var tag = child.tagName.toLowerCase();
        
        if      (tag === 'input') inputs.push(child);
        else if (tag === 'label') labels.push(child);
      });
      
      // Calculating the number of rows:
      
      rows = Math.floor(inputs.length / cols) + 1;
      
      // Inserting the new month days' parent into DOM:
      
      $(monthDaysParent).after(newMonthDaysParent);
      
      $(newMonthDaysParent).css( { 'position' : 'relative'
                                 , 'width'    : (cols * x) + 'px'
                                 , 'height'   : (rows * y) + 'px' } );
      
      // Inserting and reorganising the month days' HTML Elements:
      
      $(inputs).each(function(inputIndex, input) {
        
        /** @type {Element} */ var div      = document.createElement('div');
        /** @type {Element} */ var label    = labels[inputIndex];
        /** @type {number}  */ var colIndex = (inputIndex % cols);
        /** @type {number}  */ var rowIndex = Math.floor(inputIndex / cols);
        
        $(input).css( { 'display'     : 'inline' } );
        $(label).css( { 'display'     : 'inline'
                      , 'font-weight' : 'normal' } );
        
        $(newMonthDaysParent).append(div);
        $(div).append(input);
        $(div).append(label);
        
        $(div).css( { 'position' : 'absolute'
                    , 'left'     : (colIndex * x) + 'px'
                    , 'top'      : (rowIndex * y) + 'px'
                    , 'width'    : x
                    , 'height'   : y } );
      });
      
      // Removing the old month days' parent from DOM:
      
      $(monthDaysParent).remove();
    };
    
    /**
     * The instance.
     * @type {tsa.FormCustomizer}
     */
    tsa.FormCustomizer.instance = null;
    
    /**
     * Instance getter (and lazy constructor).
     * @return {tsa.FormCustomizer}
     */
    tsa.FormCustomizer.getInstance = function() {
      
      if (tsa.FormCustomizer.instance === null) {
        
        new tsa.FormCustomizer();
      }
      
      return tsa.FormCustomizer.instance;
    };
    
    return tsa.FormCustomizer;
  }
);