/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'PopStateHandler' );
    
    //-------------------------------------------------------------------------
    
    /**
     * Window Pop State Event Handler.
     * This class only has static methods; it should not be instantiated.
     */
    tsa.PopStateHandler = function()
    {
      logger.error( 'This class is not supposed to be instantiated.' );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * The enabled flag.
     * @private
     * @type {boolean}
     */
    tsa.PopStateHandler.enabledFlag = false;
    
    //-------------------------------------------------------------------------
    
    tsa.PopStateHandler.onPopState = function( event )
    {
      if( TV.typeIsObject( typeof event                  ) && ( event       !== null ) &&
          TV.typeIsObject( typeof event.state            ) && ( event.state !== null ) &&
          TV.typeIsString( typeof event.state.action     ) &&
          TV.typeIsObject( typeof event.state.parameters ) )
      {
        logger.info( 'The state popped from the window history will be processed.' );
        
        require(
          [ 'tsa/cmd/ShowCommand' ],
          function( ShowCommand )
          {
            ( new ShowCommand( { action     : event.state.action
                               , parameters : event.state.parameters } )
            ).execute();
          }
        );
      }
      else
      {
        logger.info( 'The state popped from the window history has been ignored: unrecognized.' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    tsa.PopStateHandler.enable = function()
    {
      if( !tsa.PopStateHandler.enabledFlag )
      {
        try
        {
          window[ 'onpopstate' ] = tsa.PopStateHandler.onPopState;
          logger.info( 'Pop State Handler enabled.' );
        }
        catch( e )
        {
          logger.warn( 'Pop State Handler could not be enabled (unsupported).' );
        }
        
        tsa.PopStateHandler.enabledFlag = true;
      }
    };
    
    //-------------------------------------------------------------------------
    
    tsa.PopStateHandler.disable = function()
    {
      if( tsa.PopStateHandler.enabledFlag )
      {
        try
        {
          window[ 'onpopstate' ] = tsa.PopStateHandler.disabledOnPopState;
          logger.info( 'Pop State Handler disabled.' );
        }
        catch( e )
        {
          logger.warn( 'Pop State Handler could not be disabled (unsupported).' );
        }
        
        tsa.PopStateHandler.enabledFlag = false;
      }
    };
    
    //-------------------------------------------------------------------------
    
    tsa.PopStateHandler.disabledOnPopState = function( event )
    {
      logger.info( 'Pop State ignored (the Pop State Handler is currently disabled).' );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.PopStateHandler;
  }
);