/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

/**
 * This class concentrates many common verification methods (all of them are
 * static, don't instantiate this class).
 */

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'VerificationToolkit' );
    
    //-------------------------------------------------------------------------
    
    /**
     * This class is not supposed to be instantiated.
     */
    tsa.VerificationToolkit = function( parameters )
    {
      logger.error( 'This class is not supposed to be instantiated.' );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Regular expression for IPv4 validation.
     * @private
     * @const
     * @type {RegExp}
     */
    tsa.VerificationToolkit.IP_REGEXP = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
    
    //-------------------------------------------------------------------------
    /**
     * Is valid IPv4?
     * @param {string} ipv4 The IPv4 string.
     * @return {boolean}
     */
    tsa.VerificationToolkit.isValidIPv4 = function( ipv4 )
    {
      return ( TV.typeIsString( typeof ipv4 ) && tsa.VerificationToolkit.IP_REGEXP.test( ipv4 ) );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.VerificationToolkit;
  }
);

//-----------------------------------------------------------------------------