/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'AjaxResponse' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  Parameters:
     *    level    Either: 'info' (or 'information'), 'warn' (or 'warning') or 'error'.
     *    message  The message.
     *  
     *  @param {{level   : string,
     *           message : string}} parameters */
    
    tsa.cmd.SystemMessage = function( parameters )
    {
      /** @type {string} */ var level   = 'info';
      /** @type {string} */ var message = '';
      
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        /** @type {string} */ var level   = ( TV.typeIsString( typeof parameters.level   ) ? parameters.level.toLowerCase() : 'info' );
        /** @type {string} */ var message = ( TV.typeIsString( typeof parameters.message ) ? parameters.message             : ''     );
        
        // Sanitizing level...
        if (level === 'information') {
          level = 'info';
        } else if (level === 'warning') {
          level = 'warn';
        } else if((level !== 'info') && (level !== 'warn') && (level !== 'error')) {
          level = 'info';
        }
      }
      
      this.level   = level;
      this.message = message;
    };
    
    //-----------------------------------------------------------------------
    
    /** @type {string} */ tsa.cmd.SystemMessage.prototype.level   = '';
    /** @type {string} */ tsa.cmd.SystemMessage.prototype.message = '';
    
    /** @param {string} arg */ tsa.cmd.SystemMessage.prototype.setLevel   = function( arg ) { this.level   = ( TV.typeIsString( typeof arg ) ? arg : '' ); };
    /** @param {string} arg */ tsa.cmd.SystemMessage.prototype.setMessage = function( arg ) { this.message = ( TV.typeIsString( typeof arg ) ? arg : '' ); };
    
    /** @return {string} */ tsa.cmd.SystemMessage.prototype.getLevel   = function() { return this.level;   };
    /** @return {string} */ tsa.cmd.SystemMessage.prototype.getMessage = function() { return this.message; };
    
    //-----------------------------------------------------------------------
    
    /**
     * Create a SystemMessage instance from a plain object parsed from JSON.
     * @param {Object} json Plain object resulting from parsing JSON.
     * @return {tsa.cmd.AjaxResponse} A SystemMessage instance.
     */
    tsa.cmd.SystemMessage.buildFromJSON = function( o )
    {
      /** @type {tsa.cmd.SystemMessage} */ var instance = null;
      
      if( TV.typeIsObject( typeof o ) && ( o !== null ) )
      {
        instance = new tsa.cmd.SystemMessage( o );
      }
      else
      {
        logger.error( 'Could not build SystemMessage (invalid argument).' );
      }
      
      return instance;
    };
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @param {{className : string,
     *           message   : string,
     *           cause     : tsa.cmd.Throwable}} parameters */
    
    tsa.cmd.Throwable = function( parameters )
    {
      // Private properties. Initializing them using default values:
      /** @type {string}            */ this.className = '';
      /** @type {string}            */ this.message   = '';
      /** @type {tsa.cmd.Throwable} */ this.cause     = null;
      
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        if( TV.typeIsString( typeof parameters.className ) ) this.className = parameters.className;
        if( TV.typeIsString( typeof parameters.message   ) ) this.message   = parameters.message;
        if( TV.typeIsObject( typeof parameters.cause     ) ) this.cause     = parameters.cause;
      }
    };
    
    //-----------------------------------------------------------------------
    
    /** @param {string}            arg */ tsa.cmd.Throwable.prototype.setClassName = function( arg ) { this.className = ( TV.typeIsString( typeof arg ) ? arg : ''   ); };
    /** @param {string}            arg */ tsa.cmd.Throwable.prototype.setMessage   = function( arg ) { this.message   = ( TV.typeIsString( typeof arg ) ? arg : ''   ); };
    /** @param {tsa.cmd.Throwable} arg */ tsa.cmd.Throwable.prototype.setCause     = function( arg ) { this.cause     = ( TV.typeIsObject( typeof arg ) ? arg : null ); };
    
    /** @return {string}            */ tsa.cmd.Throwable.prototype.getClassName = function() { return this.className; };
    /** @return {string}            */ tsa.cmd.Throwable.prototype.getMessage   = function() { return this.message;   };
    /** @return {tsa.cmd.Throwable} */ tsa.cmd.Throwable.prototype.getCause     = function() { return this.cause;     };
    
    //-----------------------------------------------------------------------
    
    /**
     * Create a Throwable instance from a plain object parsed from JSON.
     * @param {Object} json Plain object resulting from parsing JSON.
     * @return {tsa.cmd.AjaxResponse} A Throwable instance.
     */
    tsa.cmd.Throwable.buildFromJSON = function( o )
    {
      /** @Ttype {tsa.cmd.Throwable} */ var instance = null;
      
      if( TV.typeIsObject( typeof o ) )
      {
        if( o !== null )
        {
          instance = new tsa.cmd.Throwable();
          
          instance.setClassName ( TV.typeIsString( typeof o.name    ) ? o.name    : '' );
          instance.setMessage   ( TV.typeIsString( typeof o.message ) ? o.message : '' );
          instance.setCause     ( TV.typeIsObject( typeof o.cause   ) ? tsa.cmd.Throwable.buildFromJSON( o.cause ) : null );
        }
      }
      else
      {
        logger.error( 'Could not build Throwable (invalid argument type: ' + ( typeof o ) + ').' );
      }
      
      return instance;
    };
    
    //-----------------------------------------------------------------------
    
    /** Constructor. Parameters:
     *    success     Was the request successfully processed?
     *    payload     If the request was successfully processed, the result will be stored here.
     *    sysMessages System messages.
     *    exception   If the request was not successfully processed, was an exception catched?
     *  
     *   @param {{success     : boolean,
     *            payload     : Object.<string,?>,
     *            sysMessages : Array.<tsa.cmd.SystemMessage>,
     *            exception   : tsa.cmd.Throwable}} parameters */
    
    tsa.cmd.AjaxResponse = function( parameters )
    {
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        if( TV.typeIsBoolean ( typeof parameters.success     ) ) this.success     = parameters.success;
        if( TV.typeIsObject  ( typeof parameters.payload     ) ) this.payload     = parameters.payload;
        if( TV.typeIsObject  ( typeof parameters.sysMessages ) ) this.sysMessages = parameters.sysMessages;
        if( TV.typeIsObject  ( typeof parameters.exception   ) ) this.exception   = parameters.exception;
      }
    };
    
    //-------------------------------------------------------------------------
    
    // Public properties (to be set at construction time):
    
    /** @type {boolean}           */ tsa.cmd.AjaxResponse.prototype.success     = false;
    /** @type {Object.<string,?>} */ tsa.cmd.AjaxResponse.prototype.payload     = null;
    /** @type {string}            */ tsa.cmd.AjaxResponse.prototype.sysMessages = null;
    /** @type {tsa.cmd.Throwable} */ tsa.cmd.AjaxResponse.prototype.exception   = null;
        /** @type {string}        */ tsa.cmd.AjaxResponse.prototype.csrfToken   = null;
    
    //-------------------------------------------------------------------------
    
    /** @param {boolean}                       arg */ tsa.cmd.AjaxResponse.prototype.setSuccess     = function( arg ) { this.success     = ( TV.typeIsBoolean( typeof arg ) ? arg : false ); };
    /** @param {Object.<string,?>}             arg */ tsa.cmd.AjaxResponse.prototype.setPayload     = function( arg ) { this.payload     = ( TV.typeIsObject(  typeof arg ) ? arg : null  ); };
    /** @param {Array.<tsa.cmd.SystemMessage>} arg */ tsa.cmd.AjaxResponse.prototype.setSysMessages = function( arg ) { this.sysMessages = ( TV.typeIsObject(  typeof arg ) ? arg : null  ); };
    /** @param {string}                        arg */ tsa.cmd.AjaxResponse.prototype.setCSRFToken   = function( arg ) { this.csrfToken   = ( TV.typeIsString(  typeof arg ) ? arg : null  ); };
    /** @param {tsa.cmd.Throwable}             arg */ tsa.cmd.AjaxResponse.prototype.setException   = function( arg ) { this.exception   = ( TV.typeIsObject(  typeof arg ) ? arg : null  ); };
    
    /** @return {boolean}                       */ tsa.cmd.AjaxResponse.prototype.getSuccess     = function() { return this.success;     };
    /** @return {Object.<string,?>}             */ tsa.cmd.AjaxResponse.prototype.getPayload     = function() { return this.payload;     };
    /** @return {Array.<tsa.cmd.SystemMessage>} */ tsa.cmd.AjaxResponse.prototype.getSysMessages = function() { return this.sysMessages; };
    /** @return {string}                        */ tsa.cmd.AjaxResponse.prototype.getCSRFToken   = function() { return this.csrfToken;   };
    /** @return {tsa.cmd.Throwable}             */ tsa.cmd.AjaxResponse.prototype.getException   = function() { return this.exception;   };
    
    //-------------------------------------------------------------------------
    
    /**
     * Create a AjaxResponse instance from a plain object parsed from JSON.
     * @param {Object} json Plain object resulting from parsing JSON.
     * @return {tsa.cmd.AjaxResponse} An AjaxResponse instance (or null if the
     *                                given argument is invalid).
     */
    tsa.cmd.AjaxResponse.buildFromJSON = function( o )
    {
      /** @type {tsa.cmd.AjaxResponse} */ var instance = null;
      
      if( TV.typeIsObject( typeof o ) )
      {
        if( o !== null )
        {
          if( TV.typeIsBoolean ( typeof o.success ) )
          {
            /** @type {Array.<tsa.cmd.SystemMessage>} */ var sysMsgs  = [];
            
            instance = new tsa.cmd.AjaxResponse();
            
            instance.setSuccess     ( TV.typeIsBoolean( typeof o.success ) && o.success );
            instance.setPayload     ( ( TV.typeIsObject( typeof o.payload ) && ( o.payload !== null ) ) ? o.payload : {} );
            instance.setException   ( TV.typeIsObject( typeof o.systemMessages ) ? tsa.cmd.Throwable.buildFromJSON( o.exception ) : null );
            instance.setSysMessages ( sysMsgs );
            instance.setCSRFToken   ( o.csrfToken );
            
            if( TV.typeIsObject( typeof o.systemMessages ) && TV.isInstanceOf( o.systemMessages, Array ) )
            {
              $.each( o.systemMessages, function( i, sysMsg )
              {
                sysMsgs.push( tsa.cmd.SystemMessage.buildFromJSON( sysMsg ) );
              } );
            }
          }
          else
          {
            logger.error( 'Could not build Ajax Response (invalid argument, it doesn\'t contain a success flag).' );
          }
        }
      }
      else
      {
        logger.error( 'Could not build Ajax Response (invalid argument type: ' + ( typeof o ) + ').' );
      }
      
      return instance;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.AjaxResponse;
  }
);