/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

/* ----------------------------------------------------------------------------
 * The execution of this command creates the UI Elements to be shown.
 * It does not necessarily mean the server has to be contacted.
 * ----------------------------------------------------------------------------
 */
define(
  [ 'jquery'
  , 'tsa/FormCustomizer'
  , 'tsa/PopStateHandler'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/AjaxResponse'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/LeftNav'
  ],
  
  function( $
          , FormCustomizer
          , PopStateHandler
          , TV
          , AjaxResponse
          , Command
          , LeftNav
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ShowCommand' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.Command
     *  Parameters (in addition to supper-class ones):
     *    replaceCurrentHistoryEntry : Replace the current history entry.
     *    
     *  @param {{replaceCurrentHistoryEntry : (undefined|boolean|string)}} properties The properties. */
    
    tsa.cmd.ShowCommand =  function( properties )
    {
      /** @type {boolean} */ var replaceCurrentHistoryEntry = false;
      
      if( !TV.typeIsObject( typeof properties ) || ( properties === null ) ) properties = /** @type {?} */ ( {} );
      
      // Forcing ajax off.
      properties.ajax = false;
      properties.secureToken = false;
      // Calling the constructor of the super-class...
      Command.prototype.constructor.call( this, properties );
      
      if( TV.typeIsString( typeof properties.replaceCurrentHistoryEntry  ) ) replaceCurrentHistoryEntry = ( properties.replaceCurrentHistoryEntry === ( '' + true ) );
      if( TV.typeIsBoolean( typeof properties.replaceCurrentHistoryEntry ) ) replaceCurrentHistoryEntry = /** @type {boolean} */ ( properties.replaceCurrentHistoryEntry );
      
      this.setReplaceCurrentHistoryEntry ( replaceCurrentHistoryEntry );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.ShowCommand.prototype = new Command( {} );
    /** @type {?} */ ( tsa.cmd.ShowCommand.prototype ).constructor = tsa.cmd.ShowCommand;
    
    //-------------------------------------------------------------------------
    
    /** Replace Current Browser History Entry Flag.
     *  (Replacing the current history entry after successful command execution,
     *  instead of pushing a new entry into the history stack).
     *  @private
     *  @type {boolean}
     */
    tsa.cmd.ShowCommand.prototype.replaceCurrentHistoryEntry = false;
    
    /** Flag that indicates the private "show" method has been called.
     *  @private
     *  @type {boolean}
     */
    tsa.cmd.ShowCommand.prototype.showMethodCalled = false;
    
    /** Flag that indicates this instance was created for an auto-refresh operation.
     *  @private
     *  @type {boolean}
     */
    tsa.cmd.ShowCommand.prototype.createdForAutoRefresh = false;
    
    //-------------------------------------------------------------------------
    
    /**
     * The last ShowCommand successfully executed.
     * @private
     * @type {tsa.cmd.ShowCommand}
     */
    tsa.cmd.ShowCommand.lastInstanceSuccessfullyExecuted = null;
    
    //-------------------------------------------------------------------------
    
    /**
     * Build a copy of this ShowCommand.
     * The event handlers will not be copied/attached.
     * @param {boolean=} forAutoRefresh Set this flag if this command is being built
     *                                  for the auto-refresh feature. Optional. Default: false.
     * @return {tsa.cmd.ShowCommand}
     */
    tsa.cmd.ShowCommand.prototype.clone = function( forAutoRefresh )
    {
      /** @type {tsa.cmd.ShowCommand} */ var copy = null;
      
      forAutoRefresh = TV.typeIsBoolean( typeof forAutoRefresh ) && forAutoRefresh;
      
      copy = new tsa.cmd.ShowCommand( { action                     : this.getAction()
                                      , baseURL                    : this.getBaseURL()
                                      , ajax                       : this.isAjax()
                                      , blockUI                    : this.shouldUIBeBlocked()
                                      , clearSystemMessages        : ( forAutoRefresh ? true : this.shouldSystemMessagesBeCleared() )
                                      , printSystemMessages        : this.shouldSystemMessagesBePrinted()
                                      , parameters                 : this.getParameters()
                                      , cancelable                 : this.isCancelable()
                                      , eventHandlers              : {}
                                      , replaceCurrentHistoryEntry : false } );
      
      copy.createdForAutoRefresh = forAutoRefresh;
      return copy;
    };
    
    //-------------------------------------------------------------------------
    
    /** Remove remaining elements created by widgets or third party
     *  libraries, and that will not be needed anymore.
     */
    tsa.cmd.ShowCommand.removeGarbageFromDocument = function()
    {
      // Only if needed.
    };
      
    //-------------------------------------------------------------------------
    
    /**
     * @return {tsa.cmd.ShowCommand} The last instance successfully executed.
     */
    tsa.cmd.ShowCommand.getLastInstanceSuccessfullyExecuted = function()
    {
      return tsa.cmd.ShowCommand.lastInstanceSuccessfullyExecuted;
    };
    
    //-------------------------------------------------------------------------
    
    /** Set the Replace Current Browser History Entry Flag.
     *  (Replacing the current history entry after successful command execution,
     *  instead of pushing a new entry into the history stack).
     *  @param {boolean} b
     */
    
    tsa.cmd.ShowCommand.prototype.setReplaceCurrentHistoryEntry = function( b )
    {
      this.replaceCurrentHistoryEntry = ( TV.typeIsBoolean( typeof b ) ? b : false );
    };
    
    //-------------------------------------------------------------------------
    
    /** Get the Replace Current Browser History Entry Flag.
     *  (Replacing the current history entry after successful command execution,
     *  instead of pushing a new entry into the history stack).
     *  @return {boolean} */
    
    tsa.cmd.ShowCommand.prototype.shouldCurrentHistoryEntryBeReplaced = function()
    {
      return this.replaceCurrentHistoryEntry;
    };
    
    //-------------------------------------------------------------------------
    
    /** Replace content.
     *  @private
     *  @param {Element} */
    
    tsa.cmd.ShowCommand.replaceContent = function( element )
    {
      $( '#dynamic-content' ).empty();
      $( '#dynamic-content' ).append( element );
    };
    
    //-------------------------------------------------------------------------
    
    /** Show the resulting HTML Element (the result of a successful command execution)
     *  or an error message (if the command execution failed).
     *  @param {(Element|string)} result The HTML Element (if successful command execution) or
     *                                   a string containing an error message.
     */
    
    tsa.cmd.ShowCommand.prototype.show = function( result )
    {
      /** @type {Element} */ var errorMessageDiv = null;
      
      this.showMethodCalled = true;
      
      if( TV.typeIsObject    ( typeof result         ) && ( result !== null ) &&
         !TV.typeIsUndefined ( typeof result.tagName ) )
      {
        tsa.cmd.ShowCommand.replaceContent( /** @type {Element} */ ( result ) );
        window.scrollTo( 0, 0 );
      }
      else
      {
        require( [ 'tsa/widgets/ContentArea' ], $.proxy( function( ContentArea )
        {
          /** @type {tsa.widgets.ContentArea} */ var ca = new ContentArea( {} );
          
          if( !TV.typeIsString( typeof result ) ) result = '' + result;
          
          ca.appendMessage( { level  : 'INFO'
                            , text   : result
                            , border : true } );
          
          this.show( ca.getDomNode() );
        }
        , this ) );
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show nothing. Convenience wrapper method.
     * Internally it calls: <code>this.show(document.createElement("br"))</code>.
     */
    
    tsa.cmd.ShowCommand.prototype.showNothing = function()
    {
      this.show( document.createElement( 'br' ) );
    };
    
    //-------------------------------------------------------------------------
    
    /** @override */
    
    tsa.cmd.ShowCommand.prototype.executeCommand = function()
    {
      // Attaching required event handlers...
      
      $( this ).on( 'executionFinished', $.proxy( function( event )
      {
        /** @type {tsa.cmd.AjaxResponse} */ var response = ( TV.typeIsObject( typeof event.response ) ? event.response : null );
        
        if( ( response                  !== null ) &&
            ( response.getSysMessages() !== null ) )
        {
          $.each( response.getSysMessages(), function( sysMsgIndex, sysMsg )
          {
            tsa.printSysMessage( sysMsg );
          } );
        }
        
        // The "show" method was not called? log it, something might be wrong...
        if( !this.showMethodCalled )
        {
          /** @type {string} */ var action = this.getAction();
          
          logger.warn( 'The "show" method was not called' + ( ( action.length > 0 ) ? ' (' + action + ')' : '' ) + '.' );
        }
        
        // Some customizations need to be applied over legacy forms...
        FormCustomizer.getInstance().run();
      }
      , this ) );
      
      // Clean the content area...
      tsa.cmd.ShowCommand.replaceContent( document.createElement( 'br' ) );
      
      // Reset some other settings...
      tsa.updateAutoRefreshSettings( null );
      tsa.setLayout( 'normal' );
      tsa.changesCanBeLost( false );
      
      // Hide the Java Logo. If it needs to be displayed, the command needs to
      // explicitly show it.
      tsa.showJavaLogo( false );
      
      // Remove auxiliary, remaining elements added by third party libraries to the document...
      tsa.cmd.ShowCommand.removeGarbageFromDocument();
      
      // Calling the handler method...
      this.findAndInvokeHandler();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * The purpose of this method is to build the HTML Element that will be
     * placed in the content area. The implementation must call this class'
     * "show" method for this purpose.
     */
    
    tsa.cmd.ShowCommand.prototype.findAndInvokeHandler = function()
    {
      if( TV.typeIsString( typeof this.getAction() ) )
      {
        /** @type {tsa.cmd.Command} */ var heartbeatCommand   = null;
        /** @type {number}          */ var sinceLast          = 0;
        /** @type {number}          */ var heartbeatThreshold = 60; // seconds
        
        /* If more than one minute has passed since the last contact with the server has been made,
         * a heartbeat command will be issued, so the session status could be known in advance and
         * just in case this command displays static content without contacting the server; the
         * session status is always updated in the case the server is contacted afterwards.
         */
        if( ( tsa.cmd.Command.getLastExecutionTime() === null ) ||
            ( ( sinceLast = ( ( new Date() ).getTime() - tsa.cmd.Command.getLastExecutionTime().getTime() ) ) > ( heartbeatThreshold * 1000 ) ) )
        {
          logger.info( 'Ajax heartbeat will be executed'
                     + ( ( sinceLast > 0 ) ? ' (last command was executed ' + ( sinceLast / 1000 ) + ' seconds ago).' : '.' ) );
          
          heartbeatCommand = new Command( { action              : 'main/ajaxHeartbeat'
                                          , clearSystemMessages : false
                                          , printSystemMessages : false 
                                          // do not enable secure token for heartBeat
                                          , secureToken         : false } );
          
          if( this.createdForAutoRefresh )
          {
            heartbeatCommand.setParameter( 'fromAutoRefresh', true );
          }
        }
        
        /**
         * Heartbeat command common handler.
         * For handling a successful response, provide the AjaxResponse instance.
         * For handling a failed response, provide either the AjaxResponse instance
         * (it success flag set to false) or <code>null</code>.
         * If the heartbeat was not issued because it was not necessary, don't provide
         * arguments (undefined).
         * @param {tsa.cmd.AjaxResponse=} heartbeatResponse
         */
        var commonHandler = $.proxy( function( heartbeatResponse )
        {
          /** @type {string} */ var action = this.getAction();
          // disable network panel
          $("#ibm-secondary-navigation").hide();
          /* ------------------------------------------------------------------
           * If necessary, override the action depending on the session status.
           * ------------------------------------------------------------------ */
          if( TV.typeIsObject( typeof heartbeatResponse ) )
          {
            // Invalid session?
            if( !TV.isInstanceOf( heartbeatResponse, AjaxResponse ) || !heartbeatResponse.getSuccess() )
            {
              // The session have been invalidated. Show the login screen:
              if( ( action !== 'admin/login'  ) &&
                  ( action !== 'admin/logout' ) )
              {
                action = 'admin/login';
              }
            }
          }
          
          /* ------------------------------------------------------------------
           * Time to handle the action:
           * ------------------------------------------------------------------ */
          
          /** @type {Array.<string>} */ var actionChunks       = action.split( '/' );
          /** @type {sting}          */ var defaultCommandName = 'index';
          /** @type {string}         */ var handlerName        = actionChunks[0] + 'Handler';
          /** @type {string}         */ var commandName        = ( ( actionChunks.length > 1 ) ? actionChunks[1] : defaultCommandName );
          /** @type {string}         */ var entityId           = ( ( actionChunks.length > 2 ) ? actionChunks[2] : '' );
          /** @type {string}         */ var methodName         = '_' + commandName;
          /** @type {string}         */ var handlerModule      = 'tsa/cmd/handlers/'
                                                               + ( handlerName = ( handlerName.substring( 0, 1 ).toUpperCase()
                                                                                 + handlerName.substring( 1 ) ) );
          if( tsa.widgets.LeftNav.getInstance()!= null && actionChunks[0] === 'main' )
          {
            /** @type {object} */ var leftNav = tsa.widgets.LeftNav.getInstance();
            leftNav.focusMainMenu();
          }
          if( entityId.length > 0 ) {
            this.setParameter('entityId', entityId);
          }
          
          require(
            [ 'jquery'
            , 'tsa/TypeVerificator'
            , handlerModule
            ],
            $.proxy( function( $
                             , TV
                             , Handler )
            {
              try
              {
                /** @type {Object}  */ var handlerInstance = new Handler( { showCommand : this } );
                /** @type {?number} */ var refreshInterval = handlerInstance.shouldAutoRefresh( commandName );
                
                var triggerEvents = $.proxy( function( success )
                {
                  $( this ).trigger( { type     : ( success ? 'executionSucceeded' : 'executionFailed' )
                                     , response : null
                                     , command  : this } );
                  $( this ).trigger( { type     : 'executionFinished'
                                     , response : null
                                     , command  : this } );
                  $( this ).off();
                  
                  tsa.cmd.ShowCommand.lastInstanceSuccessfullyExecuted = this;
                  tsa.updateAutoRefreshSettings( refreshInterval );
                }
                , this );
                
                if( TV.typeIsFunction( typeof handlerInstance[ methodName ] ) )
                {
                  handlerInstance[ methodName ]();
                  triggerEvents( true );
                }
                else
                {
                  /** @type {Element} */ var elementToShow = document.createElement( 'div' );
                  
                  this.show( elementToShow );
                  
                  logger.warn( 'The handler "' + handlerName + '" doesn\'t have the requested method: "' + methodName + '";'
                             + ' retrieving the HTML from server will be attempted (legacy UI mode).' );
                  
                  $.get( tsa.cmd.Command.URL_SERVLET_CONTEXT + action, $.proxy( function( html )
                  {
                    /** @type {string}  */ var flashMarkerStart   = '<!-- FLASH_BEGIN -->';
                    /** @type {string}  */ var flashMarkerEnd     = '<!-- FLASH_END -->';
                    /** @type {number}  */ var flashIndexStart    = html.indexOf( flashMarkerStart );
                    /** @type {number}  */ var flashIndexEnd      = html.indexOf( flashMarkerEnd   );
                    /** @type {string}  */ var contentMarkerStart = '<!-- CONTENT_BEGIN -->';
                    /** @type {string}  */ var contentMarkerEnd   = '<!-- CONTENT_END -->';
                    /** @type {number}  */ var contentIndexStart  = html.indexOf( contentMarkerStart );
                    /** @type {number}  */ var contentIndexEnd    = html.indexOf( contentMarkerEnd   );
                    
                    if( ( flashIndexStart >= 0 ) && ( flashIndexEnd > flashIndexStart ) )
                    {
                      /** @type {Element} */ var flashDiv = null;
                      
                      flashIndexStart += flashMarkerStart.length;
                      flashDiv = $( html.substring( flashIndexStart, flashIndexEnd ) ).get(0);
                      
                      $( flashDiv ).find( 'li' ).each( function( liIndex, li )
                      {
                        try {
                          /** @type {Element} */ var icon       = $( li ).find( 'img' ).get(0);
                          /** @type {Object}  */ var sysMsgData = { level: 'info', message: $.trim( $( li ).text() ) };
                          
                          if      ( icon.src.indexOf( 'Error' ) >= 0 ) sysMsgData.level = 'error';
                          else if ( icon.src.indexOf( 'Warn'  ) >= 0 ) sysMsgData.level = 'warn';
                          
                          tsa.printSysMessage( sysMsgData );
                        } catch (error) {
                          logger.warn( 'Unexpected legacy content at flash message #' + ( liIndex + 1 ) + ': format unknown.' );
                        }
                      } );
                    }
                    else
                    {
                      logger.warn( 'Unexpected legacy content: flash messages could\'t not be retrieved.' );
                    }
                    
                    if( ( contentIndexStart >= 0 ) && ( contentIndexEnd > contentIndexStart ) )
                    {
                      contentIndexStart += contentMarkerStart.length;
                      html = html.substring( contentIndexStart, contentIndexEnd )
                                 .replace( /id=\"ibm-pcon\"/g    , 'id="legacy-ibm-pcon"'    )
                                 .replace( /id=\"ibm-content\"/g , 'id="legacy-ibm-content"' );
                    }
                    else
                    {
                      logger.error( 'Unexpected legacy content: it could\'t not be processed.' );
                      html = '<!-- CONTENT NOT FOUND -->';
                    }
                    
                    $( elementToShow ).html( html );
                    
                    triggerEvents( true );
                    tsa.unblockUI();
                  }
                  , this ) ).fail( $.proxy( function( jqXHR, textStatus, errorThrown )
                  {
                    logger.error( 'Legacy content couldn\'t be loaded from server (' + errorThrown + ').' );
                    $( elementToShow ).html( '<!-- CONTENT COULD NOT BE LOADED -->' );
                    triggerEvents( false );
                    tsa.unblockUI();
                  }
                  , this ) );
                  
                  // Block UI until content is loaded...
                  tsa.blockUI();
                }
                
                // No exceptions at this point? Update URL...
                this.updateWindowURL();
              }
              catch ( error )
              {
                logger.error( error );
                logger.error( 'Error caught when trying to execute method "' + methodName + '" at command handler "' + handlerName + '".' );
                this.showNothing();
                triggerEvents( false );
              }
            }
            , this ) ); // End of require.
        }
        , this ); // End of common handler.
        
        if( heartbeatCommand === null )
        {
          // Heartbeat not necessary; just execute the common handler:
          commonHandler();
        }
        else
        {
          // Attaching the event handlers to the heartbeat:
          $( heartbeatCommand ).on( 'executionSucceeded' , $.proxy( function( event ) { commonHandler( event.response ); }, this ) );
          $( heartbeatCommand ).on( 'executionFailed'    , $.proxy( function( event ) { commonHandler( null           ); }, this ) );
          
          // Executing the heartbeat:
          heartbeatCommand.execute();
        }
      }
      else
      {
        logger.error( 'Invalid action type: ' + ( typeof this.getAction() ) + '.' );
        
        $( this ).trigger( { type : 'executionFailed'   , response : null , command : this } );
        $( this ).trigger( { type : 'executionFinished' , response : null , command : this } );
        $( this ).off();
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Update the window URL, using this command action.
     */
    tsa.cmd.ShowCommand.prototype.updateWindowURL = function()
    {
      try
      {
        if( !TV.typeIsUndefined( typeof window[ 'history' ][ 'pushState'    ] ) &&
            !TV.typeIsUndefined( typeof window[ 'history' ][ 'replaceState' ] ) )
        {
          /** @type {Object} */ var parameters            = this.getParameters();
          /** @type {string} */ var aParameterName        = '';
          /** @type {string} */ var theEncodedQueryString = '';
          /** @type {Object} */ var theStateData          = null;
          /** @type {string} */ var theHistoryEntryTitle  = document.title;
          /** @type {string} */ var theStateURL           = '';
          
          for( aParameterName in parameters )
          {
            if( aParameterName !== 'entityId' && aParameterName !== 'CSRFToken' )
            {
              if( theEncodedQueryString.length > 0 ) theEncodedQueryString += '&';
              
              theEncodedQueryString += encodeURIComponent( aParameterName                    ) + '='
                                    +  encodeURIComponent( parameters[ aParameterName ] + '' );
            }
          }
          
          theStateData = { action     : this.getAction(),
                           parameters : parameters };
          
          theStateURL =       tsa.attributes.servletPath
                      + '/' + tsa.attributes.controllerName
                      + '/' + this.getAction()
                      +       ( ( theEncodedQueryString.length > 0 ) ? '?' + theEncodedQueryString : '' );
          
          PopStateHandler.disable();
          logger.info( 'Updating window URL: "' + theStateURL + '"...' );
          window[ 'history' ][ this.shouldCurrentHistoryEntryBeReplaced() ? 'replaceState' : 'pushState' ]( theStateData, theHistoryEntryTitle, theStateURL );
          PopStateHandler.enable();
          logger.info( '...Window URL updated.' );
        }
      }
      catch( error )
      {
        // This one is not that important
        logger.warn( 'Window URL couldn\'t be set.' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.ShowCommand;
  }
);