/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/ShowCommand'
  ],
  
  function( $
          , TV
          , ShowCommand
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'AbstractHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  Parameters:
     *    showCommand : Required. The show command that is calling this handler.
     *    
     *  @param {{showCommand : (tsa.cmd.ShowCommand)}} parameters The parameters. */
    
    tsa.cmd.handlers.AbstractHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = {};
      
      if( !TV.typeIsUndefined( typeof parameters.showCommand ) )
      {
        if( TV.typeIsObject( typeof parameters.showCommand ) &&
            TV.isInstanceOf( parameters.showCommand, ShowCommand ) )
        {
          this.showCommand = parameters.showCommand;
        }
        else
        {
          logger.error( 'Invalid ShowCommand instance provided.' );
        }
      }
    };
    
    //-------------------------------------------------------------------------
    
    /** The show command that is calling this handler.
     *  @private
     *  @type {tsa.cmd.ShowCommand} */
    
    tsa.cmd.handlers.AbstractHandler.prototype.showCommand = null;
    
    //-------------------------------------------------------------------------
    
    /**
     * @return {tsa.cmd.ShowCommand} The show command that created this handler.
     */
    tsa.cmd.handlers.AbstractHandler.prototype.getShowCommand = function()
    {
      return this.showCommand;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Convenience method, shortcut for: <code>this.showCommand.show</code>
     * @param {(Element|string)} elementOrMsg Element or message to show.
     */
    tsa.cmd.handlers.AbstractHandler.prototype.show = function( elementOrMsg )
    {
      if( TV.typeIsObject( typeof this.showCommand ) && ( this.showCommand !== null ) )
      {
        this.showCommand.show( elementOrMsg );
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Convenience method, shortcut for: <code>this.showCommand.showNothing</code>
     */
    tsa.cmd.handlers.AbstractHandler.prototype.showNothing = function()
    {
      if( TV.typeIsObject( typeof this.showCommand ) && ( this.showCommand !== null ) )
      {
        this.showCommand.showNothing();
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Convenience method, shortcut for: <code>this.showCommand.getParameter</code>
     * @param {string} name The name of the parameter.
     * @return {?string}     The value of the parameter, or null if not found.
     */
    tsa.cmd.handlers.AbstractHandler.prototype.getParameter = function( name )
    {
      /** @type {?string} */ var value = null;
      
      if( TV.typeIsString( typeof name             ) &&
          TV.typeIsObject( typeof this.showCommand ) && ( this.showCommand !== null ) )
      {
        value = this.showCommand.getParameter( name );
      }
      
      return value;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Action should auto-refresh?
     * Override as needed.
     * @param  {string}  commandName The command name (example: 'index').
     * @return {?number}             Number of seconds for auto-refresh. Or <code>null</code>
     *                               for disabling auto-refresh.
     */
    tsa.cmd.handlers.AbstractHandler.prototype.shouldAutoRefresh = function( commandName )
    {
      return null;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.AbstractHandler;
  }
);