/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/LeftNav'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ContentArea
          , FormPanel
          , LeftNav
          , TablePanel )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'AdminHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.AdminHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.AdminHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.AdminHandler.prototype ).constructor = tsa.cmd.handlers.AdminHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show administration description screen.
     */
    tsa.cmd.handlers.AdminHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelAdministrationHeading } );
      
      contentArea.appendMessage( { text : i18n.labelAdministrationCaption } );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show login screen.
     */
    tsa.cmd.handlers.AdminHandler.prototype._login = function()
    {
      /** @type {string}  */ var loginLockDuration = this.getParameter( 'loginLockDuration' );
      /** @type {boolean} */ var loginLocked       = TV.typeIsString( typeof loginLockDuration );
      
      if( LeftNav.getInstance() !== null )
      {
        LeftNav.getInstance().setFlags( { whole : false } );
      }
      
      if( loginLocked )
      {
        this.showLoginLockedScreen( loginLockDuration );
      }
      else
      {
        /** @type {Object}                  */ var i18n             = tsa.i18n;
        /** @type {Element}                 */ var inputForUsername = null;
        /** @type {Element}                 */ var inputForPassword = null;
        /** @type {Element}                 */ var messageContainer = document.createElement( 'p' );
        /** @type {tsa.widgets.ContentArea} */ var contentArea      = new ContentArea ( { title        : i18n.labelLoginHeading } );
        /** @type {tsa.widgets.FormPanel}   */ var formPanel        = new FormPanel   ( { border       : false
                                                                                        , cancelable   : false
                                                                                        , action       : 'admin/login'
                                                                                        , showReqMarks : false
                                                                                        // Manual secure token only for login page
                                                                                        , csrfRequired : false } );
        
        inputForUsername = $( '<input type="text" id="username" name="username" value="" required />' ).get(0);
        inputForPassword = $( '<input type="password" id="password" name="password" value="" required />' ).get(0);
        
        contentArea.append( formPanel.getDomNode() );
        
        $( messageContainer ).css({ 'font-size' : '70%'});
        $( messageContainer ).append( document.createTextNode(i18n.labelLoginResetUserPassword ));
        $( messageContainer ).append( document.createElement( 'br' ));
        $( messageContainer ).append( document.createElement( 'br' ));
        $( messageContainer ).append( document.createTextNode(i18n.labelLoginResetAdminPassword ));
        
        contentArea.append( messageContainer );
        
        formPanel.putHiddenInput( 'ajax', 'true' );
        formPanel.appendHeaderRow( '', i18n.labelLoginCaption );
        formPanel.appendSpacer();
        formPanel.appendFieldRow( { label    : i18n.labelLoginUser
                                  , required : true
                                  , element  : inputForUsername } );
        formPanel.appendFieldRow( { label    : i18n.labelLoginPassword
                                  , required : true
                                  , element  : inputForPassword } );
        formPanel.appendButton( { type    : 'arrow'
                                , label   : i18n.buttonLogin
                                , onclick : function( event ) {
                                          formPanel.putHiddenInput("CSRFToken",$( "#CSRFToken" ).val());
                                          formPanel.submit(); } } );
        formPanel.appendSpacer();
        
        // Ensure the session message is empty:
        tsa.cmd.handlers.AdminHandler.clearSessionMessage();
        
        // Show the content area we have just built:
        this.show( contentArea.getDomNode() );
        
        // Force focus into the username field:
        $( inputForUsername ).focus();
        
        // Submit the form when the user press ENTER when in the password field:
        $( inputForPassword ).on( 'keyup', function( event )
        {
          if( TV.typeIsNumber( typeof event.keyCode ) && ( event.keyCode === 13 ) )
          {
            formPanel.putHiddenInput("CSRFToken",$( "#CSRFToken" ).val());
            formPanel.submit();
          }
        } );
        
        // Java Logo must be shown in this screen:
        tsa.showJavaLogo( true );
        
        // Handler in case the login request is executed successfully:
        $( formPanel ).on( 'executionSucceeded', $.proxy( function( event )
        {
          /** @type {tsa.cmd.Command}     */ var i18nCmd = null;
          /** @type {tsa.cmd.ShowCommand} */ var showCmd = new ShowCommand( { action : 'main/index' } );
          
          // Build the left navigation menu in case it does not exist:
          if( LeftNav.getInstance() === null )
          {
            // If the left navigation menu does not exist, it means the i18n
            // labels have not been loaded; load them now and build the menu
            // when such labels are available.
            var onI18nSucceeded = function( event )
            {
              tsa.i18n = event.response.payload.i18nLabels;
              logger.info( 'I18n labels retrieved.' );
              // Build the menu:
              tsa.buildLeftNav();
              // Go to default action:
              showCmd.execute();
            };
            
            var onI18nFailed = function( event ) {
              logger.error( 'I18n labels couldn\'t be retrieved.' );
            };
            
            i18nCmd = new Command( { action        : 'i18n/getI18nLabels'
                                   , eventHandlers : { executionSucceeded : onI18nSucceeded
                                                     , executionFailed    : onI18nFailed }
                                   , secureToken   : false } );
            i18nCmd.execute();
          }
          else
          {
            // Show the menu (it was hidden):
            LeftNav.getInstance().setFlags( { whole : true } );
            // Go to default action:
            showCmd.execute();
          }
        }
        , this ) );
        
        // Handler in case the login request failed:
        $( formPanel ).on( 'executionFailed', $.proxy( function( event )
        {
          /** @type {tsa.cmd.AjaxResponse} */ var response   = null;
          /** @type {Object}               */ var payload    = null;
          /** @type {Object}               */ var attributes = null;
          
          $( inputForPassword ).val( '' );
          
          if( TV.typeIsObject    ( typeof event               ) && ( event                               !== null ) &&
              TV.typeIsObject    ( typeof event.response      ) && ( ( response   = event.response     ) !== null ) &&
              TV.typeIsObject    ( typeof response.payload    ) && ( ( payload    = response.payload   ) !== null ) &&
              TV.typeIsObject    ( typeof payload.attributes  ) && ( ( attributes = payload.attributes ) !== null ) &&
             !TV.typeIsUndefined ( typeof attributes.duration ) )
          {
            this.showLoginLockedScreen( attributes.duration );
          }
        }
        , this ) );
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show the login locked out screen.
     * @param {string} loginLockDuration The formatted duration (example: "1 min, 13 secs").
     */
    tsa.cmd.handlers.AdminHandler.prototype.showLoginLockedScreen = function( loginLockDuration )
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelLoginErrorHeading } );
      
      contentArea.appendMessage( { text : i18n.labelLoginErrorCaption } );
      contentArea.appendMessage( { text : i18n.labelLoginDurationPre + ' ' + loginLockDuration + ' '
                                        + i18n.labelLoginDurationPost } );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * @return {Element}
     */
    tsa.cmd.handlers.AdminHandler.prototype._logout = function()
    {
      // Show an empty screen until we receive a successful response from the server:
      this.showNothing();
      
      var onLogoutFinished = $.proxy( function( event )
      {
        /** @type {Object}                  */ var i18n          = tsa.i18n;
        /** @type {tsa.widgets.ContentArea} */ var contentArea   = new ContentArea ( {} );
        /** @type {Element}                 */ var message1      = document.createElement( 'h3' );
        /** @type {Element}                 */ var message2      = document.createElement( 'h5' );
        /** @type {Element}                 */ var linkContainer = document.createElement( 'p' );
        /** @type {Element}                 */ var homeLink      = document.createElement( 'a' );
        
        $( message1 ).append( document.createTextNode( i18n.labelLogoutHeading ) );
        $( message1 ).append( document.createElement( 'br' ) );
        $( message1 ).append( document.createTextNode( i18n.labelLogoutCaption ) );
        $( message2 ).text( i18n.labelLogoutNote );
        $( homeLink ).attr( 'href', tsa.cmd.Command.URL_CONTROLLER_BASE + 'admin/login' );
        $( homeLink ).text( i18n.labelLogoutHome );
        $( linkContainer ).append( homeLink );
        
        $( homeLink      ).addClass( 'ibm-anchor-up-link' );
        $( linkContainer ).addClass( 'ibm-back-link'      );
        
        contentArea.append( message1 );
        contentArea.append( message2 );
        contentArea.append( linkContainer );
        
        if( LeftNav.getInstance() !== null )
        {
          LeftNav.getInstance().setFlags( { whole : false } );
        }
        
        tsa.cmd.handlers.AdminHandler.clearSessionMessage();
        this.show( contentArea.getDomNode() );
      }
      , this );
      
      var onLogoutSucceeded = function( event )
      {
        logger.info( 'The server has successfully logged the user out.' );
      };
      
      var onLogoutFailed = function( event )
      {
        logger.info( 'The logout command failed. This usually means the server had already '
                   + 'logged the user out, due to a session timeout. Therefore, the error system '
                   + 'message has not been displayed, although it could be checked in the '
                   + 'console log.' );
      };
      
      /* Issue a logout command in case this is a manual logout. This command could fail in
       * case the server has already logged the user out (automatic logout after the session
       * expired). Therefore, the logout screen must be shown in any case, even if the command
       * fails; the error system message will be also cleared, as the command is expected to
       * fail if the server had already logged the user out.
       */
      ( new Command( { action              : 'admin/logout'
                     , cancelable          : false
                     , parameters          : { ajax : 'true' }
                     , printSystemMessages : false
                     , eventHandlers       : { executionSucceeded : onLogoutSucceeded
                                             , executionFailed    : onLogoutFailed
                                             , executionFinished  : onLogoutFinished } } )
      ).execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Switch debug flag.
     */
    tsa.cmd.handlers.AdminHandler.prototype._debug = function()
    {
      /** @type {?string} */ var entityId = this.getParameter( 'entityId' );
      
      ( new Command( { action     : 'admin/ajaxDebug'
                     , parameters : { flag : TV.typeIsString( typeof entityId ) &&
                                             ( ( entityId.toLowerCase() === 'on'   ) ||
                                               ( entityId.toLowerCase() === 'true' ) ) } } )
      ).execute();
    };
    
    //-------------------------------------------------------------------------
    /**
     * Clear the session message.
     */
    tsa.cmd.handlers.AdminHandler.clearSessionMessage = function()
    {
      try
      {
        /** @type {string} */ var userPrincipalBackup = tsa.attributes.userPrincipal;
        
        tsa.attributes.userPrincipal = '';
        tsa.updateSessionMessage();
        logger.info( 'Session message cleared.' );
        tsa.attributes.userPrincipal = userPrincipalBackup;
      }
      catch (error)
      {
        logger.warn( 'Session message couldn\'t be cleared.' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.AdminHandler;
  }
);