/*
 * IBM Technical Support Appliance Version 2 Release 3 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'AuthenticationstatusHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.AuthenticationstatusHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.AuthenticationstatusHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.AuthenticationstatusHandler.prototype ).constructor = tsa.cmd.handlers.AuthenticationstatusHandler;
    
    //sidebar to show info of elements
    tsa.cmd.handlers.AuthenticationstatusHandler.prototype.infoParagraph   = null;
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.AuthenticationstatusHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelAuthenticationStatusHeading 
                                                                               , helpDoc : 'sshelp_authentication_status_panel.html'} );
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel = null;
      
      contentArea.appendMessage( { text : i18n.labelAuthenticationStatusCaption } );
      
      /** @type {element} */ var disclaimerText = tsa.NBSP + "<strong>" + i18n.labelNote + ":</strong><p>" + i18n.labelAuthenticationStatusDisclaimer+"</p>";
      contentArea.appendMessage( { level : 'INFO'  , text : disclaimerText , textIsHtml : true } );
      
      this.infoParagraph = document.createElement("div");
      
      /** @type {element} */ var paragraph       = document.createElement("p");
      $(paragraph).text(i18n.labelAuthenticationStatusNoObjectSelected);
      $(this.infoParagraph).append(paragraph);
      
      contentArea.appendSidebarPanel( { title: i18n.labelAuthenticationStatusObjectInfo, body: this.infoParagraph } );
      
      /** @type {function()}*/
      var entryHandler = function( entry )
      {
          
        /** @type {Array.<(string|Element)>} */ var result     = null;
        /** @type {Element}         */ var aAddress            = document.createElement("a");
        /** @type {Element}         */ var sAddress            = document.createElement("strong");

        $(aAddress).text(entry.ipAddress);
        $(aAddress).append(sAddress);
        $(aAddress).attr("href","#")
        
        result = [aAddress,entry.lastAttempted,entry.lastSuccessful];
        
        /** @type {function()}*/
        var displayInfoTrigger = $.proxy(function(){
            
            /** @type {tsa.cmd.Command}*/ var getInfoCmd = new Command( { action:'authenticationstatus/getInfo/' + entry.ipAddress
                                                                                     , clearSystemMessages : false } );
            getInfoCmd.execute();
            
            $( getInfoCmd ).on( 'executionSucceeded', $.proxy(function(e) {
                object = e.response.payload.object;
                $(this.infoParagraph).empty();
                /** @type {function()}*/
                var templateInfo = function(name, value){
                    /** @type {Element}*/ var pTemplate  = document.createElement("p");
                    /** @type {Element}*/ var sTemplate  = document.createElement("strong");
                    /** @type {Element}*/ var brTemplate = $("<br class='break'>").get(0);
                    $(sTemplate).text(name+":");
                    $(pTemplate).append(sTemplate, brTemplate, value);
                    return pTemplate;
                }
                if(object.ipAddress != null){
                    if(tsa.isDebugModeOn()){
                        $(this.infoParagraph).append(templateInfo(i18n.labelDiscoRunsPID, object.ipAddress));
                    }
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatusAddress, object.ipAddress));
                }
                if(object.lastAttempted!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatusLastAttempted, object.lastAttempted));
                }
                if(object.lastSuccessful!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatusLastSuccessful, object.lastSuccessful));
                }
                if(object.portsOpen!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatusPortsOpen, object.portsOpen));
                }
                if(object.successfulCredential!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatusuccessfulCredential, object.successfulCredential));
                }
                if(object.credentialChangeDate!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelProblemDeviceCredentialChangeDate, object.credentialChangeDate));
                }
                if(object.applicableCredentials!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelProblemDeviceApplicableCredentials, object.applicableCredentials));
                }
                if(object.deviceScopes!=null){
                    $(this.infoParagraph).append(templateInfo(i18n.labelAuthenticationStatuscopes, object.deviceScopes));
                }
            }, this) );
       
       },this);
       
       $(aAddress).on('click',displayInfoTrigger);
       
       return result;
       
      };
      
      tablePanel   = new TablePanel( { title        : i18n.labelAuthenticationStatusObjectsHeading
          , summary      : i18n.labelSummaryStatusDesc
          , emptyMsg     : i18n.labelSummaryNoStatus
          , ajaxAction   : 'authenticationstatus/getList'
          , entryHandler : $.proxy(entryHandler,this)
          , columns      : [ { id : 'address'    , label : i18n.labelAuthenticationStatusAddress         , width : '25%' , sortable : true }
                           , { id : 'attempt'    , label : i18n.labelAuthenticationStatusLastAttempted   , width : '25%' , sortable : true }
                           , { id : 'successful' , label : i18n.labelAuthenticationStatusLastSuccessful  , width : '25%' , sortable : true }] } );
      
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
//    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.AuthenticationstatusHandler;
  }
);