/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/Command'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , ButtonFactory
          , Command
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'BackupHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.BackupHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.BackupHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.BackupHandler.prototype ).constructor = tsa.cmd.handlers.BackupHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.BackupHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        	= tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea 	= null;
      /** @type {tsa.widgets.FormPanel}   */ var dateForm		= null;
      /** @type {tsa.widgets.FormPanel}   */ var configForm     = null;
      /** @type {tsa.widgets.FormPanel}   */ var backupForm     = null;
      /** @type {tsa.widgets.FormPanel}   */ var restoreForm    = null;
      /** @type {tsa.cmd.Command}         */ var getBackupCmd	= new Command( { action  : 'backup/getBackupInfo' } );
      
      contentArea = new ContentArea ( { title   : i18n.labelBackupHeading
                                      , helpDoc : 'sshelp_backup_panel.html' } );
     
      contentArea.appendMessage( { text : i18n.labelBackupCaption } );
      
      //need to manually create the required legend because the forms can't be link
      var legendP = document.createElement( 'p' );
      $( legendP ).addClass( 'tsa-required-legend-global' );
      
      var legendChunks = tsa.i18n.labelRequiredLegend.split( '*' );
      var legendSpan   = document.createElement( 'span' );
      $( legendSpan ).addClass( 'tsa-required-legend' );
      
      if( legendChunks.length === 2 ){
        $( legendSpan ).append( document.createTextNode( legendChunks[0] )
                              , $( '<span class="ibm-required"> * </span>' ).get(0)
                              , document.createTextNode( legendChunks[1] ) );
      }else{
        $( legendSpan ).text( tsa.i18n.labelRequiredLegend );
      }
      $( legendP ).append( legendSpan );
      
      contentArea.append( legendP );
      
      
      //BACKUP DATE
	  dateForm = new FormPanel( { title : i18n.labelBackupAgeHeading } );
	  var div =  document.createElement( 'div' );
	  $( div ).css( { 'font-size' : '1em', 'padding' : '0px' } );
	  $( div ).attr('id', 'lastBackupDiv' );
	  dateForm.appendJointRow( div );
      
 
      //CONFIG SUMMARY
	  configForm = new FormPanel( { title  : i18n.labelBackupSummaryHeading, action : 'backup/ajaxView' } );
	  configForm.setDescription( i18n.labelBackupSummaryDesc);
	  var viewCmd = new Command({ action : 'backup/ajaxView', clearSystemMessages : false,  blockUI : false });
	  var viewButton = ButtonFactory.buildButton ({
		  type : 'arrow'
			   , grayout : false
			   , label   : i18n.buttonBackupSummaryView
			   , onclick : function( event ){
				  viewCmd.execute();
	    	}
	   });
	      		  
	  $( viewCmd ).on( 'executionSucceeded', function( event ){
		  var allInfo  = event.response.payload.config;
		  var viewConfig = document.createElement( 'div' );
		    			  
		  //Contact
		  var contactInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.contact );
		  $( viewConfig ).append ( contactInfo );
		  
		  //System location
		  var locationInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.location );
		  $( viewConfig ).append ( locationInfo );
		
		  //Clock
		  var clockInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.clock );
		  $( viewConfig ).append ( clockInfo );
		  
		  //Network
		  var networkInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.network );
		  $( viewConfig ).append ( networkInfo );
		  
		  //Connectivity
		  var connectInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.connect );
		  $( viewConfig ).append ( connectInfo );
		  
		  //Log
		  var logInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.log );
		  $( viewConfig ).append ( logInfo );
		  
		  //Discovery
		  var discoveryInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.discovery );
		  $( viewConfig ).append ( discoveryInfo );
		  
		  //Inventory
		  var inventoryInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.inventory );
		  $( viewConfig ).append ( inventoryInfo );
		  
		  //Transmission
		  var transmissionInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.transmission );
		  $( viewConfig ).append ( transmissionInfo );
		  
		  if( tsa.isDebugModeOn() ){
			  //Discovery manager
			  var managerInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.manager );
			  $( viewConfig ).append ( managerInfo );
	  	  }
		  
		  //Scopes
		  var scopeInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.scopes );
		  $( viewConfig ).append ( scopeInfo );
		  
		  //Creds
		  var credInfo = tsa.cmd.handlers.BackupHandler.buildConfigInfo( allInfo.creds );
		  $( viewConfig ).append ( credInfo );
		  
		  
		  //Back to top 
		  var backPar = document.createElement( 'p' );
    	  var backLink = $( '<a class="ibm-anchor-up-link" href="#ibm-content">'+i18n.buttonBackToTop+'</a>' ).get(0);
    	  $( backPar ).append( backLink );
    	  $( viewConfig ).append ( backPar );
    	  
    	  
		  //display config info
		  var configWindow = window.open("", "configwindow", "status=1, width=500, height=700");
		  configWindow.document.title = i18n.labelBannerTitle;
		  $( configWindow.document.body).append( viewConfig );
		   
	  });


	  var configDiv = document.createElement( 'div' );
	  $( configDiv ).append( viewButton );
	  configForm.appendJointRow( viewButton );

	  //BACKUP
	  backupForm = new FormPanel( { title  : i18n.labelBackupBackupHeading, action : 'backup/ajaxBackup', showReqHeader : false } );
	  backupForm.setDescription( i18n.labelBackupBackupDesc);
	  backupForm.appendFieldRow( { label : i18n.labelBackUpZipPassword
		  , hint :  i18n.labelBackUpZipPasswordDesc
		  , required : true
		  , element  : $( '<input type="password" size="50" name="backupPassword" id="backupPassword" value="" required/>' ).get(0)
	  } );
	  
	  backupForm.appendFieldRow( { label : i18n.labelBackUpZipConfirmPassword
		  , required : true
		  , element  : $( '<input type="password" size="50" name="confirmPassword" id="confirmPassword" value="" required/>' ).get(0)
	  } );

	  var hrDiv = document.createElement( 'div' );
	  $( hrDiv ).attr('class', 'ibm-rule');
	  $( hrDiv ).append ( document.createElement( 'hr' ) );
	  backupForm.appendJointRow( hrDiv );
	  
	  
	  var backupButton = ButtonFactory.buildButton ({
		  type : 'arrow'
			  , grayout : false
			  , label   : i18n.buttonBackupBackup
			  , onclick : function( event ){
				  backupForm.submit();
			  }
	  });

	  var backupDiv = document.createElement( 'div' );
	  $( backupDiv ).append( backupButton );
	  backupForm.appendJointRow( backupButton );
	  
	 $( backupForm ).on( 'executionSucceeded', function( event )
	 {
		 //wait 2 seconds to make sure info icon gets displayed
		 setTimeout(function() {
			 window.location = tsa.cmd.Command.URL_SERVLET_CONTEXT + 'backup/download/backup.zip';
		 }, 2000);
	 });
	  

	  //RESTORE
	  restoreForm = new FormPanel( { title  : i18n.labelBackupRestoreHeading, action : 'backup/ajaxRestore', showReqHeader : false } );
	  var descDiv = document.createElement( 'div' );
	  var descP1  = document.createElement( 'p' );
	  var descP2  = document.createElement( 'p' );
	  $( descP1  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px'} );
	  $( descP1  ).append( i18n.labelBackupRestoreDesc );
	  $( descP2  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px', 'padding-bottom' : '0px'} );
	  $( descP2  ).append( i18n.labelBackupRestoreDesc2 );
	  $( descDiv ).append( descP1 );
	  $( descDiv ).append( descP2 );
	  restoreForm.appendJointRow( descDiv );
	 
	  var rsHrDiv = document.createElement( 'div' );
	  $( rsHrDiv ).attr('class', 'ibm-rule');
	  $( rsHrDiv ).append ( document.createElement( 'hr' ) );
	  restoreForm.appendJointRow( rsHrDiv );
	  
	  restoreForm.appendFieldRow( { label : i18n.labelBackupRestore
		  , required  : true
		  , aRequire  : false
		  , element   : $( '<input type="file" id="file" name="file" />' ).get(0)
	  } );

	  restoreForm.appendFieldRow( { label : i18n.labelBackUpZipPassword
		  , hint     :  i18n.labelRestoreZipPasswordDesc
		  , element  : $( '<input type="password" size="50" name="restorePassword" id="restorePassword" value="" />' ).get(0)
	  } );

	  var hrDiv2 = document.createElement( 'div' );
	  $( hrDiv2 ).attr('class', 'ibm-rule');
	  $( hrDiv2 ).append ( document.createElement( 'hr' ) );
	  restoreForm.appendJointRow( hrDiv2 );
	  
	  var restoreButton = ButtonFactory.buildButton ({
		  type : 'arrow'
			  , grayout : false
			  , label   : i18n.buttonBackupRestore
			  , onclick : function( event ){
                  if( confirm(tsa.i18n.labelBackupRestoreConfirm) ){
                      restoreForm.submit();
                  }
			  }
	  });
	  
	  

	  var restoreDiv = document.createElement( 'div' );
	  $( restoreDiv ).append( restoreButton );
	  restoreForm.appendJointRow( restoreButton );

	  contentArea.append( dateForm.getDomNode() );
	  contentArea.append( configForm.getDomNode() );
	  contentArea.append( backupForm.getDomNode() );
	  contentArea.append( restoreForm.getDomNode() );

      $( getBackupCmd ).on( 'executionSucceeded', function( event ){
    	  $("#lastBackupDiv").text(event.response.payload.lastBackup);
      }); 
    
      //get backup info
      getBackupCmd.execute();
      
 	  this.show( contentArea.getDomNode() );
    	  
    };
    
    //-------------------------------------------------------------------------
    
     /**
     * @param  {Object}  entry Configuration detail entry.
     * @return {Element}       HTML Element containing details for a configuration entry
     */
    tsa.cmd.handlers.BackupHandler.buildConfigInfo = function( list )
    {
      var configDiv  = document.createElement( 'div' );
      var configSpan = document.createElement( 'span' );
	  
      $( configSpan ).css( { 'font-weight' : 'bold', 'font-size' : 'small', 'font-family' : 'arial' } );
	  $( configSpan ).append ( list[0].group );
	  $( configDiv ).append ( configSpan );
	  
	  $.each( list, function( entryIndex, entry )
	  {
		  if(entryIndex == 0)	//this is the category heading
	        	return;
	  
		  var groupDiv = document.createElement( 'div' );
		  var keySpan = document.createElement( 'span' );
		  var valSpan = document.createElement( 'span' );
		 
		  $.each(entry, function(key, val){
			  $( groupDiv ).css( {'display' : 'table-row', 'width' : 'auto', 'clear' : 'both'});
			  
			  if(key == 'subgroup'){
				  var spaceDiv = document.createElement( 'div' );
			      $( spaceDiv ).css( {'display' : 'table-row', 'width' : 'auto', 'clear' : 'both', 'margin-bottom' : '10px'});
			      $( spaceDiv ).append( tsa.NBSP );
			      $( configDiv ).append ( spaceDiv );
			  
				  $( keySpan ).css( {'float' : 'left', 'display' : 'table-column', 'width' : '200px', 'margin-left' : '25px', 'margin-bottom' : '3px', 'font-weight' : 'bold', 'font-size': 'small', 'font-family' : 'arial'});
			      $( keySpan ).append( val );
			  }else{
	        	  $( keySpan ).css( {'float' : 'left', 'display' : 'table-column', 'width' : '200px', 'margin-left' : '25px', 'margin-bottom' : '3px', 'font-size': 'small', 'font-family' : 'arial'});
	   	      	  $( keySpan ).append( key + ":" );
	   	      	  
	   	      	  $( valSpan ).css( {'float' : 'left', 'display' : 'table-column', 'width' : '200px', 'margin-bottom' : '3px' , 'font-size': 'small', 'font-family' : 'arial'});
	   	      	  $( valSpan ).append( val );
	          }
	    
	       $( groupDiv ).append( keySpan );
	       $( groupDiv ).append( valSpan );
	       $( configDiv ).append ( groupDiv );
		  });
	  });
	 

	  var spaceDiv2 = document.createElement( 'div' );
      $( spaceDiv2 ).css( {'display' : 'table-row', 'width' : 'auto', 'clear' : 'both', 'margin-bottom' : '10px'});
      $( spaceDiv2 ).append( tsa.NBSP );
      $( configDiv ).append ( spaceDiv2 );
	  
	  return configDiv;
    }

    
    return tsa.cmd.handlers.BackupHandler;
  }
);