/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
*/

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
    , 'tsa/TypeVerificator'
    , 'tsa/cmd/handlers/AbstractHandler'
    , 'tsa/widgets/ContentArea'
    , 'tsa/widgets/FormPanel'
    , 'tsa/widgets/TablePanel'
    , 'tsa/widgets/ButtonFactory'
    , 'tsa/cmd/Command'
    , 'tsa/data/Countries'
  ],
  
  function( $
    , TV
    , AbstractHandler
    , ContentArea
    , FormPanel
    , TablePanel
    , ButtonFactory
    , Command
    , Countries
    )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'CertificatesHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
    *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.CertificatesHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.CertificatesHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CertificatesHandler.prototype ).constructor = tsa.cmd.handlers.CertificatesHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
    */
    tsa.cmd.handlers.CertificatesHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        	= tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea 	= null;
      /** @type {tsa.widgets.FormPanel}   */ var customcertMgtForm    = null;
      /** @type {tsa.widgets.FormPanel}   */ var importCertForm    = null;
      /** @type {tsa.widgets.FormPanel}   */ var certImportMgtForm = null;
      /** @type {tsa.widgets.FormPanel}   */ var certMgtForm    = null;
      /** @type {tsa.widgets.FormPanel}   */ var importServCertForm    = null;
      /** @type {tsa.widgets.TablePanel}  */ var certStatusTbl  = null;
      /** @type {tsa.cmd.Command}         */ var getInfoCmd		= new Command( { action : 'certificates/getInfo' } );
      /** @type {Element}                 */ var selectCountries        = document.createElement( 'select' );
      
      
      contentArea = new ContentArea ( { title : i18n.labelCertificateHeading
                                      , helpDoc : 'sshelp_certificates_panel.html' } );
      
      contentArea.appendMessage( { text : i18n.labelCertificateCaption } );
      
      //need to manually create the required legend because the forms can't be link
      var legendP = document.createElement( 'p' );
      $( legendP ).addClass( 'tsa-required-legend-global' );
      
      var legendChunks = tsa.i18n.labelRequiredLegend.split( '*' );
      var legendSpan   = document.createElement( 'span' );
      $( legendSpan ).addClass( 'tsa-required-legend' );
      
      if( legendChunks.length === 2 ){
        $( legendSpan ).append( document.createTextNode( legendChunks[0] )
          , $( '<span class="ibm-required"> * </span>' ).get(0)
          , document.createTextNode( legendChunks[1] ) );
      }else{
        $( legendSpan ).text( tsa.i18n.labelRequiredLegend );
      }
      $( legendP ).append( legendSpan );
      
      contentArea.append( legendP );
      

      //CERTIFICATE STATUS
      var certDiv  = document.createElement( 'div' );
      var certDiv2 = document.createElement( 'div' );
      var certH2   = document.createElement( 'h2'  );
      
      $( certDiv  ).addClass( 'ibm-container ibm-inner-data-table-last');
      $( certH2   ).append( i18n.labelCertificateSSLServerCertStatusHeading );
      $( certDiv  ).append( certH2 );
      $( certDiv2 ).addClass( 'ibm-container-body');
      
      var certP = document.createElement( 'p' );
      var src  = tsa.attributes.imagesPath + '/' + tsa.i18n[ 'imageStatusInfo16' ];
      var certIcon = document.createElement( 'img' );
      var certSpan = document.createElement( 'span' );
      $( certSpan ).attr('id', 'certStatus');
      $( certSpan ).css({'font-style' : 'italic'});
      
      $( certIcon ).attr( 'src', src  );
      $( certIcon ).attr( 'alt', i18n.labelInfo );
      $( certIcon ).css( { 'vertical-align' : 'top', 'border' : '0' } );
      $( certP ).append( certIcon );
      $( certP ).append( certSpan );
      $( certDiv2 ).append( certP );
      $( certDiv  ).append( certDiv2 );
      
      var table = document.createElement( 'table' );
      var tbody = document.createElement( 'tbody' );
      $( table ).attr('border', '0');
      $( table ).attr('cellpadding', '0');
      $( table ).attr('cellspacing', '0');
      $( table ).attr('role', 'presentation');
      $( table ).addClass('ibm-data-table');
      
      var tr1 = document.createElement( 'tr' );
      var issuedByTd1 = document.createElement( 'td' );
      var issuedByTd2 = document.createElement( 'td' );
      var issuedBySpan = document.createElement( 'span' );
      $( tr1 ).css({'vertical-align' : 'top'})
      $( issuedByTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerCertIssuer + ":"));
      $( tr1 ).append(issuedByTd1);
      $( issuedBySpan ).attr('id', 'issuedBy');
      $( issuedBySpan ).css('font-weight', 'bold');
      $( issuedByTd2 ).append(issuedBySpan);
      $( tr1 ).append(issuedByTd2);
      $( tbody ).append(tr1);
      
      var tr2 = document.createElement( 'tr' );
      var issuedToTd1 = document.createElement( 'td' );
      var issuedToTd2 = document.createElement( 'td' );
      var issuedToSpan = document.createElement( 'span' );
      $( tr2 ).css({'vertical-align' : 'top'})
      $( issuedToTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerCertSubject + ":"));
      $( tr2 ).append(issuedToTd1);
      $( issuedToSpan ).attr('id', 'issuedTo');
      $( issuedToTd2 ).append(issuedToSpan);
      $( tr2 ).append(issuedToTd2);
      $( tbody ).append(tr2);
      
      var tr3 = document.createElement( 'tr' );
      var serialTd1 = document.createElement( 'td' );
      var serialTd2 = document.createElement( 'td' );
      var serialSpan = document.createElement( 'span' );
      $( tr3 ).css({'vertical-align' : 'top'})
      $( serialTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerCertSerialNumber + ":"));
      $( tr3 ).append(serialTd1);
      $( serialSpan ).attr('id', 'serial');
      $( serialTd2 ).append(serialSpan);
      $( tr3 ).append(serialTd2);
      $( tbody ).append(tr3);
      
      var tr4 = document.createElement( 'tr' );
      var algorithmTd1 = document.createElement( 'td' );
      var algorithmTd2 = document.createElement( 'td' );
      var algorithmSpan = document.createElement( 'span' );
      $( tr4 ).css({'vertical-align' : 'top'})
      $( algorithmTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerCertSigAlgoName + ":" ));
      $( tr4 ).append(algorithmTd1);
      $( algorithmSpan ).attr('id', 'algorithm');
      $( algorithmTd2 ).append(algorithmSpan);
      $( tr4 ).append(algorithmTd2);
      $( tbody ).append(tr4);
      
      var tr5 = document.createElement( 'tr' );
      var issuedOnTd1 = document.createElement( 'td' );
      var issuedOnTd2 = document.createElement( 'td' );
      var issuedOnSpan = document.createElement( 'span' );
      $( tr5 ).css({'vertical-align' : 'top'})
      $( issuedOnTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerCertNotBefore + ":"));
      $( tr5 ).append(issuedOnTd1);
      $( issuedOnSpan ).attr('id', 'issuedOn');
      $( issuedOnTd2 ).append(issuedOnSpan);
      $( tr5 ).append(issuedOnTd2);
      $( tbody ).append(tr5);
      
      var tr6 = document.createElement( 'tr' );
      var expiresOnTd1 = document.createElement( 'td' );
      var expiresOnTd2 = document.createElement( 'td' );
      var expiresOnSpan = document.createElement( 'span' );
      $( tr6 ).css({'vertical-align' : 'top'})
      $( expiresOnTd1 ).append( document.createTextNode(i18n.labelCertificateSSLServerNotAfter + ":"));
      $( tr6 ).append(expiresOnTd1);
      $( expiresOnSpan ).attr('id', 'expiresOn');
      $( expiresOnTd2 ).append(expiresOnSpan);
      $( tr6 ).append(expiresOnTd2);
      $( tbody ).append(tr6);
            
      /** @type {ButtonFactory}                   */ var generateSelfSignedCertButton = ButtonFactory.buildButton ({
        type : 'arrow'
        , label   : i18n.buttonCertificateGenerateServerCert
        , onclick : function( event ){
          /** @type {tsa.cmd.Command}           */ var genSelfCertCmd = new Command({ action : 'certificates/ajaxGenSelfSignCert', clearSystemMessages : true,  blockUI : false });
          if (confirm(i18n.labelCertificateServerCertGenerateConfirm)) {
            genSelfCertCmd.execute();
          }
          
        }
      });
      
      
      $( table ).append(tbody);
      $( certDiv2 ).append(table);
      
      $( certDiv ).append(certDiv2);
      $( certDiv ).append( generateSelfSignedCertButton );
      
      $( certDiv ).hide();
      
      contentArea.append( certDiv );
      $( generateSelfSignedCertButton ).hide();
      

      //Generate CSR
      customcertMgtForm = new FormPanel( { title : i18n.labelCertificateCustCertKeyMgtHeading
        , action : 'certificates/ajaxGenerateCSR'
        , showReqHeader : false} );
      
      var descDiv = document.createElement( 'div' );
      var descP1  = document.createElement( 'p' );
      
      $( descP1  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px'} );
      $( descP1  ).append( i18n.labelCertificateCustCertKeyMgtDesc );
      $( descDiv ).append( descP1 );
      customcertMgtForm.appendJointRow( descDiv );
      
      var cmHrDiv = document.createElement( 'div' );
      $( cmHrDiv ).attr('class', 'ibm-rule');
      $( cmHrDiv ).append ( document.createElement( 'hr' ) );
      customcertMgtForm.appendJointRow( cmHrDiv );
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertCommonName
        , required : true
        , element  : $( '<input type="text" size="50" name="cnInfo" id="cnInfo" value="" required/>' ).get(0)
      } );
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertOrgUnit
        , required : true
        , element  : $( '<input type="text" size="50" name="organizationUnit" id="organizationUnit" value="" required/>' ).get(0)
      } );
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertOrg
        , required : true
        , element  : $( '<input type="text" size="50" name="organization" id="organization" value="" required/>' ).get(0)
      } );
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertCity
        , required : true
        , element  : $( '<input type="text" size="50" name="city" id="city" value="" required/>' ).get(0)
      } );
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertState
        , required : true
        , element  : $( '<input type="text" size="50" name="state" id="state" value="" required/>' ).get(0)
      } );
      
      selectCountries.name = 'locationCountry';
      selectCountries.id   = 'locationCountry';
      
      $.each( Countries.getCountries(), function( countryIndex, country )
        {
          /** @type {Element} */ var opt = document.createElement( 'option' );
          opt.value = country.id;
          $( opt ).text( country.id + "-" + i18n[country.name.label] );
          $( selectCountries ).append( opt );
        } );
      
      
      customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertCountry
        , required : true
        , hint     : i18n.labelRegistrationSyslocCountryDesc
        , element  : selectCountries
      } );

     customcertMgtForm.appendFieldRow( { label : i18n.labelCertificateCustCertTime
        , required : true
        , element  : $( '<input type="text" size="50" name="duration" id="duration" required/>' ).get(0)
      } );
       
      var hrDiv2 = document.createElement( 'div' );
      $( hrDiv2 ).attr('class', 'ibm-rule');
      $( hrDiv2 ).append ( document.createElement( 'hr' ) );
      customcertMgtForm.appendJointRow( hrDiv2 );
      
      /** @type {tsa.widgets.ButtonFactory} */ var generateCertButton = ButtonFactory.buildButton ({
          type 	  : 'arrow'
          , label   : i18n.buttonCertificateGenerateCustCertKey
          , onclick : function( event ){
          if ( confirm( i18n.labelCertificateCSRGenerateConfirm ) ) {
            customcertMgtForm.submit();
          }
          
        }
      });
      
      $( customcertMgtForm ).on( 'executionFailed', function( event )
        {
        if( TV.typeIsObject( typeof event.response.payload.errors ) )
        {
          for( /** @type {string} */ var fieldName in event.response.payload.errors )
          {
            customcertMgtForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
          }
        }
      } );
      
      $( customcertMgtForm ).on( 'executionSucceeded', function( event ) {
          //wait 2 seconds to make sure info icon gets displayed
          setTimeout(function() {
            window.location = tsa.cmd.Command.URL_SERVLET_CONTEXT + 'certificates/download/tsacert.csr';
          }, 2000);
      });
      
      /** @type {tsa.widgets.ButtonFactory} */ var cmContainer = ButtonFactory.buildButtonContainer();
      $( cmContainer ).append( generateCertButton );
      $( cmContainer ).append( document.createElement( 'br' ) );
      customcertMgtForm.appendJointRow( cmContainer );
      contentArea.append( customcertMgtForm.getDomNode() );
      

      //Import Multiple Certificate
      importCertForm = new FormPanel( { title  : i18n.labelImportSignersHeading, action : 'certificates/ajaxImportMultiSignCert',showReqHeader : false } );
      var descDiv = document.createElement( 'div' );
      var descP1  = document.createElement( 'p' );
      var descP2  = document.createElement( 'p' );
      var descP3  = document.createElement( 'p' );
      $( descP1  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px'} );
      $( descP1  ).append( i18n.labelImportCertDesc );
      $( descP2  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px', 'padding-bottom' : '0px'} );
      $( descP2  ).append( i18n.labelImportCertDesc2 );
      $( descDiv ).append( descP1 );
      $( descDiv ).append( descP2 );
      importCertForm.appendJointRow( descDiv );
      
      var rsHrDiv = document.createElement( 'div' );
      $( rsHrDiv ).attr('class', 'ibm-rule');
      $( rsHrDiv ).append ( document.createElement( 'hr' ) );
      importCertForm.appendJointRow( rsHrDiv );
      
      
      importCertForm.appendFieldRow( { label : i18n.labelRootImportCert
        , required  : true
        , aRequire  : false
        , element   : $( '<input type="file" id="rootCert" name="rootCert" />' ).get(0)
      } );
      
      var rsHrDiv = document.createElement( 'div' );
      $( rsHrDiv ).attr('class', 'ibm-rule');
      $( rsHrDiv ).append ( document.createElement( 'hr' ) );
      importCertForm.appendJointRow( rsHrDiv );
      
      
      importCertForm.appendFieldRow( { label : i18n.labelIntermediatryImportCert
        , aRequire  : false
        , element   : $( '<input type="file" id="interCert0" name="interCert0" />' ).get(0)
      } );
      
      importCertForm.appendFieldRow( { label : i18n.labelIntermediatryImportCert
        , aRequire  : false
        , element   : $( '<input type="file" id="interCert1" name="interCert1" />' ).get(0)
      } );
      
      importCertForm.appendFieldRow( { label : i18n.labelIntermediatryImportCert
        , aRequire  : false
        , element   : $( '<input type="file" id="interCert2" name="interCert2" />' ).get(0)
      } );
      
      var rsHrDiv = document.createElement( 'div' );
      $( rsHrDiv ).attr('class', 'ibm-rule');
      $( rsHrDiv ).append ( document.createElement( 'hr' ) );
      importCertForm.appendJointRow( rsHrDiv );
      
      importCertForm.appendFieldRow( { label : i18n.labelTSACertImportCert
        , required  : true
        , aRequire  : false
        , element   : $( '<input type="file" id="cert" name="cert" />' ).get(0)
      } );
      
      var hrDiv2 = document.createElement( 'div' );
      $( hrDiv2 ).attr('class', 'ibm-rule');
      $( hrDiv2 ).append ( document.createElement( 'hr' ) );
      importCertForm.appendJointRow( hrDiv2 );
      
      /** @type {tsa.widgets.ButtonFactory} */ var importCertButton = ButtonFactory.buildButton ({
        type : 'arrow'
        , grayout : false
        , label   : i18n.buttonImportSignersCertificate
        , onclick : function( event ){
          if( confirm(i18n.labelCertificateMultiSignerSSLServerCertMgtUploadConfirm) ){
            importCertForm.submit();
          }
        }
      });
      
      var importCertDiv = document.createElement( 'div' );
      $( importCertDiv ).append( importCertButton );
      importCertForm.appendJointRow( importCertButton );
      contentArea.append( importCertForm.getDomNode() );
      
      
      $( getInfoCmd ).on( 'executionSucceeded', function( event ){
        //Certificate status
        $(certDiv).show();
        if(event.response.payload.hasCert){
          $("#certStatus").text(tsa.NBSP + event.response.payload.status);
          $("#issuedBy").text(event.response.payload.issuedBy);
          $("#issuedTo").text(event.response.payload.issuedTo);
          $("#serial").text(event.response.payload.serial);
          $("#algorithm").text(event.response.payload.algorithm);
          $("#issuedOn").text(event.response.payload.issuedOn);
          $("#expiresOn").text(event.response.payload.expiresOn);
          if(event.response.payload.defaultCert){
            $(generateSelfSignedCertButton).show();
          }
          $(certStatusTbl).show();
          
          
        }else if(event.response.payload.isNewCertPending){
          $("#certStatus").text(tsa.NBSP + i18n.labelCertificateSSLServerCertInstallPending);
          
        }else{
          $("#certStatus").text(tsa.NBSP + i18n.labelCertificateSSLServerCertNotInstalled);
        }

      });
      

      //Import CA Certificate
      certImportMgtForm = new FormPanel( { title : i18n.labelImportCACertHeading
        , action 		: 'certificates/ajaxUploadCACert'
        , showReqHeader : false} );
      
      var descDiv = document.createElement( 'div' );
      var descP1  = document.createElement( 'p' );
      var descP2  = document.createElement( 'p' );
      
      $( descP1  ).css({ 'font-size' : '1.0em', 'padding-left' : '0px'} );
      $( descP1  ).append( i18n.labelImportCACertDesc );
      $( descDiv ).append( descP1 );
      certImportMgtForm.appendJointRow( descDiv );
      
      var cmHrDiv = document.createElement( 'div' );
      $( cmHrDiv ).attr('class', 'ibm-rule');
      $( cmHrDiv ).append ( document.createElement( 'hr' ) );
      certImportMgtForm.appendJointRow( cmHrDiv );
      
      certImportMgtForm.appendFieldRow( { label : i18n.labelCertificateSSLServerCertPassword
        , required : true
        , element  : $( '<input type="password" size="50" name="password" id="password" value="" required/>' ).get(0)
      } );
      
      certImportMgtForm.appendFieldRow( { label : i18n.labelCertificateSSLServerCertPassword2
        , required : true
        , element  : $( '<input type="password" size="50" name="confirmPassword" id="confirmPassword" value="" required/>' ).get(0)
      } );
      
      certImportMgtForm.appendFieldRow( { label : i18n.labelImportCACert
        , required  : true
        , aRequire  : false
        , element   : $( '<input type="file" id="file" name="file" />' ).get(0)
      } );
      
      
      var hrDiv2 = document.createElement( 'div' );
      $( hrDiv2 ).attr('class', 'ibm-rule');
      $( hrDiv2 ).append ( document.createElement( 'hr' ) );
      certImportMgtForm.appendJointRow( hrDiv2 );
      
      /** @type {ButtonFactory}                   */ var customCACertButton = ButtonFactory.buildButton ({
        type : 'arrow'
        , label   : i18n.buttonImportCertificate
        , onclick : function( event ){
          if (confirm(i18n.labelCertificateSSLServerCertMgtUploadConfirm)) {
            certImportMgtForm.submit();
          }
        }
      });
      
      var importCACertDiv = document.createElement( 'div' );
      $( importCACertDiv ).append( customCACertButton );
      certImportMgtForm.appendJointRow( customCACertButton );
      contentArea.append( certImportMgtForm.getDomNode() );
      
      // Get Certificate info
      getInfoCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.CertificatesHandler;
  }
);
