/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ConnsettingsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ConnsettingsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ConnsettingsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ConnsettingsHandler.prototype ).constructor = tsa.cmd.handlers.ConnsettingsHandler;
    
    //-------------------------------------------------------------------------
    
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.divSLP                   = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.radioSLPEnabled          = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.radioSLPDisabled         = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.rowEMCSMISHTTPSPorts     = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.rowEMCSMISHTTPPorts      = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.inputEMCSMISHTTPSPorts   = null;
    /** @private ** @type {Element} */ tsa.cmd.handlers.ConnsettingsHandler.prototype.inputEMCSMISHTTPPorts    = null;
    /** @private ** @type {string}  */ tsa.cmd.handlers.ConnsettingsHandler.prototype.prevValEMCSMISHTTPSPorts = '';
    /** @private ** @type {string}  */ tsa.cmd.handlers.ConnsettingsHandler.prototype.prevValEMCSMISHTTPPorts  = '';
    
    //-------------------------------------------------------------------------
    
    /**
     * Build HTML Elements for the EMC Settings form.
     * @private
     */
    tsa.cmd.handlers.ConnsettingsHandler.prototype.buildEMCFields = function()
    {
      /** @type {Object}  */ var i18n             = tsa.i18n;
      /** @type {Element} */ var slpEnabledLabel  = document.createElement( 'label' );
      /** @type {Element} */ var slpDisabledLabel = document.createElement( 'label' );
      
      this.divSLP                 = document.createElement( 'div'   );
      this.radioSLPEnabled        = document.createElement( 'input' );
      this.radioSLPDisabled       = document.createElement( 'input' );
      this.inputEMCSMISHTTPSPorts = document.createElement( 'input' );
      this.inputEMCSMISHTTPPorts  = document.createElement( 'input' );
      
      $( this.inputEMCSMISHTTPSPorts ).attr( 'type', 'text'                 );
      $( this.inputEMCSMISHTTPSPorts ).attr( 'name', 'EMC_SMIS_HTTPS_PORTS' );
      $( this.inputEMCSMISHTTPSPorts ).attr( 'id'  , 'EMC_SMIS_HTTPS_PORTS' );
      $( this.inputEMCSMISHTTPPorts  ).attr( 'type', 'text'                 );
      $( this.inputEMCSMISHTTPPorts  ).attr( 'name', 'EMC_SMIS_HTTP_PORTS'  );
      $( this.inputEMCSMISHTTPPorts  ).attr( 'id'  , 'EMC_SMIS_HTTP_PORTS'  );
      $( this.radioSLPEnabled        ).attr( 'type', 'radio'                );

      $( this.radioSLPDisabled       ).attr( 'type', 'radio'                );
      $(this.radioSLPEnabled).on("click", $.proxy(function(){
        $(this.radioSLPDisabled).prop('checked', false);
      }, this));
      $(this.radioSLPDisabled).on("click", $.proxy(function(){
        $(this.radioSLPEnabled).prop('checked', false);
       }, this));
      
      $( this.radioSLPEnabled  ).val( 'enabled'  );
      $( this.radioSLPDisabled ).val( 'disabled' );
      
      $( slpEnabledLabel  ).css( { 'display' : 'block' } );
      $( slpDisabledLabel ).css( { 'display' : 'block' } );
      
      $( slpEnabledLabel  ).append( this.radioSLPEnabled  );
      $( slpEnabledLabel  ).append( document.createTextNode( i18n.labelConnSettingsEnabled + ' (' + i18n.labelConnSettingsDefault + ')' ) );
      $( slpDisabledLabel ).append( this.radioSLPDisabled );
      $( slpDisabledLabel ).append( document.createTextNode( i18n.labelConnSettingsDisabled ) );
      
      $( this.divSLP ).append( slpEnabledLabel );
      $( this.divSLP ).append( slpDisabledLabel );
      
      $( this.inputEMCSMISHTTPSPorts ).on( 'change', function( event ) { tsa.changesCanBeLost( true ); tsa.clearSysMessages(); } );
      $( this.inputEMCSMISHTTPPorts  ).on( 'change', function( event ) { tsa.changesCanBeLost( true ); tsa.clearSysMessages(); } );
      
      $( this.radioSLPEnabled  ).on( 'click', $.proxy( this.onSLPChange, this ) );
      $( this.radioSLPDisabled ).on( 'click', $.proxy( this.onSLPChange, this ) );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Event handler: SLP Discovery field changed.
     * @param {Event} event The event. Optional. A programmatic call to this function
     *                      (null event) will behave differently from a user-generated
     *                      event triggered call.
     */
    tsa.cmd.handlers.ConnsettingsHandler.prototype.onSLPChange = function( event )
    {
      /** @type {boolean} */ var slpEnabled   = ( this.radioSLPEnabled.checked );
      /** @type {boolean} */ var programmatic = !( TV.typeIsObject( typeof event ) && ( event !== null ) ); 
      
      $( this.rowEMCSMISHTTPSPorts ).css( { 'opacity' : ( slpEnabled ? '0.6' : '1.0' ) } );
      $( this.rowEMCSMISHTTPPorts  ).css( { 'opacity' : ( slpEnabled ? '0.6' : '1.0' ) } );
      
      this.inputEMCSMISHTTPSPorts .disabled = slpEnabled;
      this.inputEMCSMISHTTPPorts  .disabled = slpEnabled;
      
      if( !programmatic )
      {
        if( slpEnabled )
        {
          this.prevValEMCSMISHTTPSPorts = $( this.inputEMCSMISHTTPSPorts ).val();
          this.prevValEMCSMISHTTPPorts  = $( this.inputEMCSMISHTTPPorts  ).val();
          
          $( this.inputEMCSMISHTTPSPorts ).val( '' );
          $( this.inputEMCSMISHTTPPorts  ).val( '' );
        }
        else
        {
          $( this.inputEMCSMISHTTPSPorts ).val( this.prevValEMCSMISHTTPSPorts );
          $( this.inputEMCSMISHTTPPorts  ).val( this.prevValEMCSMISHTTPPorts  );
        }
        
        tsa.changesCanBeLost( true );
        tsa.clearSysMessages();
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Validate port list.
     * @private
     * @param  {string} list The port list.
     * @return {?strung}     If the given list is valid, a sanitized version of it
     *                       is returned; otherwise <code>null</code> is returned.
     */
    tsa.cmd.handlers.ConnsettingsHandler.validatePortList = function( list )
    {
      /** @type {boolean}        */ var valid  = TV.typeIsString( typeof list );
      /** @type {Array.<number>} */ var ports  = [];
      /** @type {RegExp}         */ var regexp = /^[0-9]+$/;
      /** @type {?string}        */ var result = null;
      
      if( valid && ( ( trim = $.trim( list ) ).length > 0 ) )
      {
        /** @type {Array.<string>} */ var chunks = $.trim( list ).split( ',' );
        
        $.each( chunks, function( chunkIndex, chunk )
        {
          if( valid = regexp.test( chunk = $.trim( chunk ) ) )
          {
            /** @type {number} */ var port = parseInt( chunk );
            
            valid = ( !isNaN( port ) && ( port > 0 ) && ( port <= 0xFFFF ) && ( ports.indexOf( port ) === -1 ) );
            
            if( valid ) ports.push( port );
          }
          
          return valid;
        } );
      }
      
      if( valid ) {
        result = ports.join( ',' );
      }
      
      return result;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Validate EMC Form.
     * @private
     * @return {boolean}
     */
    tsa.cmd.handlers.ConnsettingsHandler.prototype.validateEMC = function()
    {
      /** @type {boolean} */ var valid = true;
      
      $.each( [ this.inputEMCSMISHTTPSPorts
              , this.inputEMCSMISHTTPPorts
              ]
            , function( portInputIndex, portInput )
      {
        /** @type {?string} */ var value = $( portInput ).val();
        
        // Validate and sanitize...
        value = tsa.cmd.handlers.ConnsettingsHandler.validatePortList( value );
        valid = ( value != null );
        
        if( valid )
        {
          $( portInput ).val( value );
        }
        
        return valid;
      } );
      
      if( !valid )
      {
        window.alert( tsa.i18n.labelConnSettingsInvalidPorts );
      }
      
      return valid;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show summary screen.
     */
    tsa.cmd.handlers.ConnsettingsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = null;
      /** @type {tsa.widgets.FormPanel}   */ var emcFormPanel    = null;
      /** @type {tsa.cmd.Command}         */ var getValuesCmd    = null;
      
      contentArea = new ContentArea ( { title         : i18n.labelConnSettingsHeading
                                      , helpDoc       : 'sshelp_connsettings_panel.html'
                                      , showBackToTop : true } );
      
      this.buildEMCFields();
      
      emcFormPanel = new FormPanel( { title       : i18n.labelConnSettingsSLPandEMC
                                    , description : i18n.labelConnSettingsSLPandEMCDsc
                                    , action      : 'connsettings/putSensorSettings'
                                    } );
      
      emcFormPanel.appendFieldRow( { label   : i18n.labelConnSettings_SLP_DISCOVERY
                                   , element : this.divSLP
                                   , hint    : i18n.labelConnSettings_SLP_DISCOVERY_hint } );
      
      this.rowEMCSMISHTTPSPorts = emcFormPanel.appendFieldRow( { label   : i18n.labelConnSettings_EMC_SMIS_HTTPS_PORTS
                                                               , element : this.inputEMCSMISHTTPSPorts
                                                               , hint    : i18n.labelConnSettings_EMC_SMIS_HTTPS_PORTS_hint } );
      
      this.rowEMCSMISHTTPPorts  = emcFormPanel.appendFieldRow( { label   : i18n.labelConnSettings_EMC_SMIS_HTTP_PORTS
                                                               , element : this.inputEMCSMISHTTPPorts
                                                               , hint    : i18n.labelConnSettings_EMC_SMIS_HTTP_PORTS_hint } );
      
      emcFormPanel.appendButton( { type    : 'arrow'
                                 , label   : i18n.buttonSave
                                 , onclick : $.proxy( function( event ) { 
                                   if ( this.validateEMC() ) {
                                     /** @type {string} */ var splDiscovery = ($(this.radioSLPEnabled).is(':checked') ) ? "enabled" : "disabled";
                                     emcFormPanel.putHiddenInput("SLP_DISCOVERY", splDiscovery)
                                     emcFormPanel.submit();
                                   }
                                   }, this ) } );
      
      getValuesCmd = new Command( { action              : 'connsettings/getSensorSettings'
                                  , clearSystemMessages : false } );
      
      // Retrieve values from server, and use them to initialize the form fields:
      $( getValuesCmd ).on( 'executionSucceeded', $.proxy( function( event )
      {
        /** @type {Object} */ var ss = event.response.payload.sensorSettings;
        
        $( this.inputEMCSMISHTTPSPorts ).val( ( ss.EMC_SMIS_HTTPS_PORTS .effectiveValue !== null ) ? ss.EMC_SMIS_HTTPS_PORTS .effectiveValue : "" );
        $( this.inputEMCSMISHTTPPorts  ).val( ( ss.EMC_SMIS_HTTP_PORTS  .effectiveValue !== null ) ? ss.EMC_SMIS_HTTP_PORTS  .effectiveValue : "" );
        
        $( this.radioSLPEnabled  ).prop( 'checked', ( ss.SLP_DISCOVERY.effectiveValue !== "disabled" ) );
        $( this.radioSLPDisabled ).prop( 'checked', ( ss.SLP_DISCOVERY.effectiveValue === "disabled" ) );
        
        this.onSLPChange( null );
      }
      , this ) );
      
      // Reload the form after a successful submission:
      $( emcFormPanel ).on( 'executionSucceeded', $.proxy( function( event )
      {
        /** @type {tsa.cmd.ShowCommand} */ var showCmd = new ShowCommand( { action              : 'connsettings/index'
                                                                          , clearSystemMessages : false } );
        tsa.printSysMessage( new tsa.cmd.SystemMessage( { level   : 'info'
                                                        , message : tsa.i18n.labelConnSettingsSaved } ) );
        showCmd.execute();
      }
      , this ) );
      
      contentArea.append( emcFormPanel.getDomNode() );
      this.show( contentArea.getDomNode() );
      
      getValuesCmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ConnsettingsHandler;
  }
);