/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ButtonFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , Command
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ConsoleHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ConsoleHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ConsoleHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ConsoleHandler.prototype ).constructor = tsa.cmd.handlers.ConsoleHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ConsoleHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n        		= tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea 		= null;
      /** @type {tsa.widgets.FormPanel}     */ var formPanel	 	= null;
      /** @type {tsa.cmd.Command}           */ var getHelpCmd		= new Command( { action  : 'console/getHelp' } );

      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelConsoleHeading
                                      , helpDoc : 'sshelp_console_panel.html' } );
      
      contentArea.appendMessage( { text : i18n.labelConsoleCaption } );

      formPanel	= new FormPanel( { title: i18n.labelConsoleCommandHeading 
    	      					 , description : i18n.labelConsoleCommandCaption 
    	      					 , action : 'console/getCmdOutput'} );
      
      var p 	  = document.createElement( 'p' );
      var label   = $( '<label for="command">' + i18n.labelConsoleCommand + ': </label>').get(0);
      var cmdline = $( '<input type="text" class="ibm-text" size="68" maxlength="250" name="command" id="command" value="" />' ).get(0);

      $( p ).css( {'font-size' : 'small', 'padding' : '0' } );
      $( p ).append( label );
      $( p ).append( cmdline );
      formPanel.appendJointRow ( p );

      formPanel.appendHR();
      
      var button = ButtonFactory.buildButton( { type : 'arrow' , label : i18n.buttonEnter } );
	  formPanel.appendJointRow( button );
	  
	   $( button ).on( 'click', function( event ){
     	  formPanel.submit();
       } );
	   
	   $( formPanel ).on( 'executionSucceeded', function( event ){
		   $( '#results' ).text( event.response.payload.results );
	   });
	  
	   contentArea.append( formPanel.getDomNode() );
      
      //OUTPUT
      var div1 = document.createElement( 'div' );
      $( div1 ).addClass( 'ibm-container' );
      
      var h2 = document.createElement( 'h2' );
      $( h2 ).text( i18n.labelOutputHeading );
      $( div1 ).append( h2 );
      
      var div2 = document.createElement( 'div' );
      $( div2 ).addClass( 'ibm-container-body' );
      
      var output = $("<div class='ibm-code-sample' style='background:#ffffff !important; height:30em; width:522px; border:0px; margin: 0px'></>").get(0);
      var pre = document.createElement( 'pre' );
	  $( pre ).attr('id' , 'results');
	  
	  $( output ).append( pre );
	  $( div2 ).append( output );
	  $( div1 ).append( div2 );
	  contentArea.append( div1 );
	  
	  //BACK TO TOP 
	  var backPar = document.createElement( 'p' );
	  var backLink = $( '<a class="ibm-anchor-up-link" href="#ibm-content">'+i18n.buttonBackToTop+'</a>' ).get(0);
	  $( backPar ).append( backLink );
	  contentArea.append( backPar );
      
      //get properties file
      $( getHelpCmd ).on( 'executionSucceeded', function( event ){
    	  $( '#results' ).text( event.response.payload.results );
      });
      
      //get usage info
      getHelpCmd.execute();
      
      this.show( contentArea.getDomNode() );
      
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ConsoleHandler;
  }
);