/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'CredentialsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.CredentialsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
      
      // Initializing members...
      this.scopeSets = {};
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.CredentialsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CredentialsHandler.prototype ).constructor = tsa.cmd.handlers.CredentialsHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * The scope sets (mapped by PID).
     * @type {Array.<Object>}
     */
    tsa.cmd.handlers.CredentialsHandler.prototype.scopeSets = null;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen (list the credentials).
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._index = function()
    {
      this._list();
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show the credentials list screen.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._list = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelDiscoCredsHeading
                                                                                    , helpDoc       : 'sshelp_discovery_credentials_panel.html'
                                                                                    , showBackToTop : true } );
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}         */ var getCredCommand  = null;
      
      tsa.setLayout( 'wide' );
      
      contentArea.appendMessage( { text : i18n.labelDiscoCredsCaption } );
      contentArea.appendMessage( { text : i18n.labelDiscoCredsCaption2 } );
      
      var addCredClicked = $.proxy( function( event )
      {
        /** @type {tsa.cmd.ShowCommand} */ var showCmd = new ShowCommand( { action : 'credentials/new' } );
        
        showCmd.execute();
      }
      , this );
      
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonDiscoCredsAdd
                                                              , onclick : addCredClicked } ) );
      
      this.show( contentArea.getDomNode() );
      tsa.setLayout( 'wide' );
      
      // Retrieving the credentials from server...
      getCredCommand = new Command( { action              : 'credentials/getCredentials'
                                    , clearSystemMessages : false } );
      
      // Credentials will be used to populate the credentials table...
      $( getCredCommand ).on( 'executionSucceeded', $.proxy( function( event )
      {
        /** @type {Array.<Object>}         */ var credentials     = event.response.payload.credentials;
        /** @type {tsa.widgets.TablePanel} */ var tablePanel      = null;
        /** @type {Object}                 */ var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };
        
        if( TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 ) )
        {
          /** @type {number}         */ var scopeSetsMissing = 0;
          /** @type {Array.<Object>} */ var columnData       = [];
          
          $.each( credentials, function( credIndex, cred )
          {
            scopeSetsMissing += ( cred.scopeSetMissing ? 1 : 0 );
          } );
          
          columnData.push( { id : 'name'          , label : i18n.labelDiscoCredsName                , width : '25%' , sortable : false }
                         , { id : 'type'          , label : i18n.labelDiscoCredsType                , width : '15%' , sortable : false }
                         , { id : 'authType'      , label : i18n.labelDiscoCredsAuthenticationType  , width : '15%' , sortable : false }
                         , { id : 'userName'      , label : i18n.labelDiscoCredsUserName            , width : '10%' , sortable : false }
                         , { id : 'lastPwdChange' , label : i18n.labelDiscoCredsPasswordChangedDate , width : '10%' , sortable : false }
                         , { id : 'restrictions'  , label : i18n.labelDiscoCredsRestrictions        , width : '15%' , sortable : false }
                         , { id : 'actions'       , label : i18n.labelActionsColumn                 , width : '10%' , sortable : false } );
          
          if( scopeSetsMissing > 0 )
          {
            columnData[ columnData.length - 1 ].width = '10%';
            
            columnData.push( { id       : 'scopeSetMissing'
                             , label    : ''
                             , width    : '5%'
                             , sortable : false } );
          }
          
          tablePanelProps.columns = columnData;
        }
        else
        {
          tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }
        
        tablePanel = new TablePanel( tablePanelProps );
        
        // Iterating credentials, building rows...
        $.each( credentials, function( credIndex, cred )
        {
          /** @type {Array.<Element>} */ var elements     = [];
          /** @type {Element}         */ var nameSpan     = document.createElement( 'span' );
          /** @type {Element}         */ var nameAnchor   = document.createElement( 'a' );
          /** @type {Element}         */ var scopeSetSpan = document.createElement( 'span' );
          /** @type {Element}         */ var actionsSpan  = document.createElement( 'span' );
          /** @type {Element}         */ var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit     + ' "' + cred.name + '"' } );
          /** @type {Element}         */ var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete   + ' "' + cred.name + '"' } );
          /** @type {Element}         */ var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsDown } );
          /** @type {Element}         */ var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsUp   } );
          /** @type {boolean}         */ var isFirst      = ( credIndex === 0 );
          /** @type {boolean}         */ var isLast       = ( credIndex === ( credentials.length - 1 ) );
          
          // Column 1 - Error mark:
          if( scopeSetsMissing > 0 )
          {
            elements.push( IconFactory.buildSeverityIcon( { severity : 'ERROR' } ) );
          }
          
          // Column 2 - Name:
          nameAnchor.href = '#';
          $( nameAnchor ).addClass( 'ibm-feature-link' );
          $( nameAnchor ).css( { 'font-weight' : 'bold' } );
          $( nameAnchor ).text( cred.name );
          $( nameAnchor ).on( 'click', function( event )
          {
            tsa.cmd.handlers.CredentialsHandler.showCredentialClicked( cred.pid );
            
            try
            {
              event.stopPropagation();
            } catch ( error ) { /* Ignore */ }
            
            return false;
          } );
          
          $( nameSpan ).append( nameAnchor );
          $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
          elements.push( nameSpan );
          
          // Column 3 - Class label:
          elements.push( cred.authClassLabel );
          
         // Column 4 - password type:
          elements.push( tsa.cmd.handlers.CredentialsHandler.i18nAuthType(cred));
          
          
          // Column 5 - User name:
          elements.push( ( cred.userName !== null ) ? cred.userName : '' );
          
          // Column 6 - Password last changed:
          elements.push( ( cred.passwordLastChanged !== null ) ? cred.passwordLastChanged : i18n.labelDiscoCredsPasswordChangeNotAvailable );
          
          // Column 7 - Scope Set:
          $( scopeSetSpan ).text( ( cred.scopeSetName !== null ) ? cred.scopeSetName : '' );
          $( scopeSetSpan ).css( { 'color' : ( cred.scopeSetMissing ? 'red' : 'black' ) } );
          elements.push( scopeSetSpan );
          
          // Column 8 - Actions:
          $( actionsSpan ).append( editIcon   );
          $( actionsSpan ).append( deleteIcon );
          
          if( !isLast  ) $( actionsSpan ).append( moveDownIcon );
          if( !isFirst ) $( actionsSpan ).append( moveUpIcon   );
          
          $.each( [ editIcon, deleteIcon, moveDownIcon, moveUpIcon ], function( iconIndex, icon )
          {
            $( icon ).css( { 'cursor'       : 'pointer'
                           , 'margin-right' : '0.5em' } );
          } );
          
          $( editIcon     ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.editCredentialClicked( cred.pid );              } );
          $( deleteIcon   ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.deleteCredentialClicked( cred.name, cred.pid ); } );
          $( moveDownIcon ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked( cred.pid,  1 );          } );
          $( moveUpIcon   ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked( cred.pid, -1 );          } );
          
          elements.push( actionsSpan );
          
          // Append the row...
          tablePanel.appendRow( elements );
        } );
        
        contentArea.append( tablePanel.getDomNode() );
        contentArea.append( buttonContainer );
      }
      , this ) );
      
      getCredCommand.execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.showCredentialClicked = function( credentialPID )
    {
      ( new ShowCommand( { action : 'credentials/show/' + credentialPID } ) ).execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Edit credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.editCredentialClicked = function( credentialPID )
    {
      ( new ShowCommand( { action : 'credentials/edit/' + credentialPID } ) ).execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Delete credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.deleteCredentialClicked = function( credentialName, credentialPID )
    {
      if( TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 ) )
      {
        if( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + credentialName + '"?' ) )
        {
          /** @type {tsa.cmd.Command} */ var deleteCmd = new Command( { action     : 'credentials/deleteCredential'
                                                                      , parameters : { 'pid' : credentialPID } } );
          
          $( deleteCmd ).on( 'executionSucceeded', function( event )
          {
            ( new ShowCommand( { action              : 'credentials/index'
                               , clearSystemMessages : false } ) ).execute();
          } );
          
          deleteCmd.execute();
        }
      }
      else
      {
        logger.warn( 'Invalid credential PID. Deletion request did not proceed.' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Move credential - event handler.
     * @param {string} credentialPID The credential PID.
     * @param {number} offset        Negative to move up, positive to move down.
     */
    tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked = function( credentialPID, offset )
    {
      if( TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 ) )
      {
        if( TV.typeIsNumber( typeof offset ) && !isNaN( offset ) && ( offset !== 0 ) )
        {
          /** @type {tsa.cmd.Command} */ var moveCmd = new Command( { action     : 'credentials/ajaxMoveCredential'
                                                                    , parameters : { 'pid'    : credentialPID
                                                                                   , 'offset' : offset } } );
          
          $( moveCmd ).on( 'executionSucceeded', function( event )
          {
            ( new ShowCommand( { action              : 'credentials/index'
                               , clearSystemMessages : false } ) ).execute();
          } );
          
          moveCmd.execute();
        }
        else
        {
          logger.error( 'Invalid offset. Movement request did not proceed.' );
        }
      }
      else
      {
        logger.error( 'Invalid credential PID. Movement request did not proceed.' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show the credential details.
     * The credential PID is provided as the entity ID (last element in the URL)
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._show = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {string}                  */ var pid             = this.getParameter( 'entityId' );
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoCredsHeading
                                                                                    , helpDoc : 'sshelp_discovery_credential_panel.html' } );
      /** @type {tsa.widgets.TablePanel}  */ var generalTable    = new TablePanel( { title   : i18n.labelDiscoCredsGeneralHeading
                                                                                   , columns : [ { id: 'name'  , label: '' , width: '30%' , sortable: false }
                                                                                               , { id: 'value' , label: '' , width: '70%' , sortable: false } ] } );
      /** @type {tsa.widgets.TablePanel}  */ var propsTable      = new TablePanel( { title   : i18n.labelDiscoCredsPropsHeading
                                                                                   , columns : [ { id: 'name'  , label: i18n.labelDiscoCredsPropsName  , width: '40%' , sortable: false }
                                                                                               , { id: 'value' , label: i18n.labelDiscoCredsPropsValue , width: '60%' , sortable: false } ] } );
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      
      var getCredentialCmd = new Command( { action              : 'credentials/getCredential'
                                          , parameters          : { 'pid' : pid }
                                          , clearSystemMessages : false } );
      
      $( getCredentialCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object} */ var credential = event.response.payload.credential;
        
        if( TV.typeIsObject( typeof credential ) && ( credential !== null ) )
        {
          /** @type {Element} */ var scopeSetSpan      = document.createElement( 'span' );
          /** @type {?string} */ var userName          = credential.userName;
          /** @type {?string} */ var scopeSetName      = credential.scopeSetName;
          /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof scopeSetName ) && ( scopeSetName.length > 0 ) );
          /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
          /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );
          
          if( userName     === null ) userName     = '';
          if( scopeSetName === null ) scopeSetName = '';
          
          $( scopeSetSpan ).text( scopeSetName );
          $( scopeSetSpan ).css( { 'color' : ( credential.scopeSetMissing ? 'red' : 'black' ) } );
          
          generalTable.appendRow( [ i18n.labelDiscoCredsGeneralName     , credential.name           ] );
          generalTable.appendRow( [ i18n.labelDiscoCredsGeneralType     , credential.authClassLabel ] );
          generalTable.appendRow( [ i18n.labelDiscoCredsGeneralUserName , userName                  ] );
          generalTable.appendRow( [ i18n.labelDiscoCredsGeneralScopeSet , scopeSetSpan              ] );
          
          if( tsa.isDebugModeOn() )
          {
            generalTable.appendRow( [ i18n.labelDiscoCredsGeneralPidDebug   , credential.pid           ] );
            generalTable.appendRow( [ i18n.labelDiscoCredsGeneralClassDebug , credential.authClassName ] );
          }
          
          if( TV.typeIsObject( typeof credential.properties ) && ( credential.properties !== null ) )
          {
            for( /** @type {string} */ var propertyName in credential.properties )
            {
              /** @type {string} */ var propertyValue = credential.properties[ propertyName ];
              
              if( propertyName === 'authtype' )
              {
                /** @type {boolean} */ var isPKI = ( propertyValue.indexOf( 'authType_pki' ) === 0 );
                
                if( isPKI )
                {
                  propertyValue += scopeSetHasSSHKey  ? '_scoped' : '' ;
                }
                propertyValue  = tsa.cmd.handlers.CredentialsHandler.i18nAuthType( credential );
              }
              
              propsTable.appendRow( [ propertyName, propertyValue ] );
            }
          }
        }
        
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'back'
                                                                , grayout : true
                                                                , label   : i18n.buttonGoBack
                                                                , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonDiscoCredsEdit
                                                                , onclick : function( event ) { tsa.cmd.handlers.CredentialsHandler.editCredentialClicked( credential.pid ); } } ) );
        
        contentArea.append( generalTable.getDomNode() );
        contentArea.append( propsTable.getDomNode() );
        contentArea.append( buttonContainer );
      } );
      
      getCredentialCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show the credential creation screen.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._new = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoCredsNewCredsHeading
                                                                                    , helpDoc : 'sshelp_new_discovery_credential_panel.html' } );
      /** @type {tsa.widgets.FormPanel}   */ var credForm        = this.createCredentialForm();
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      
      credForm.setActionAttribute( 'credentials/postCredential' );
      
      // Buttons...
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonSave
                                                              , onclick : function( event ) { credForm.putHiddenInput( 'testCredential', null );
                                                                                              credForm.submit(); } } ) );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                              , label   : i18n.buttonCancel
                                                              , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );
      
      // Appending content...
      contentArea.append( credForm.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
      
      // What happens when creation succeeds? Print success message, then show the credentials list.
      $( credForm ).on( 'executionSucceeded', function( event )
      {
        if (credForm.getHiddenInput("testCredential") == null) 
        {
          /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();
          tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoCredsCreated, credentialName ) } );
          ( new ShowCommand( { action              : 'credentials/index'
                             , clearSystemMessages : false } ) ).execute();
        }
      } );
      $( credForm ).on( 'executionFailed', function( event )
          {
          if ( TV.typeIsObject( typeof event.response.payload.errors ) ) {
            for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
              credForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
            }
            delete event.response.payload.errors;
          }
          } );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show the credential edition screen.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._edit = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {String}                  */ var credentialId    = this.getParameter( 'entityId' );
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelDiscoEditCredsHeading
                                                                                    , helpDoc       : 'sshelp_edit_discovery_credential_panel.html' } );
      /** @type {tsa.widgets.FormPanel}   */ var credForm        = null;
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}         */ var getCredCmd      = new Command( { action              : 'credentials/getCredential'
                                                                                , parameters          : { 'pid' : credentialId }
                                                                                , clearSystemMessages : false } );
      $( getCredCmd ).on( 'executionSucceeded', $.proxy( function( event )
      {
        credForm = this.createCredentialForm( event.response.payload.credential );
        credForm.setActionAttribute( 'credentials/putCredential' );
        
        // What happens when edition succeeds? Print success message, then show the credentials list.
        $( credForm ).on( 'executionSucceeded', function( event )
        {
          if (credForm.getHiddenInput("testCredential") == null) 
          {
            /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();
            
            tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoCredsUpdated, credentialName ) } );
            
            ( new ShowCommand( { action              : 'credentials/index'
                               , clearSystemMessages : false } ) ).execute();
          }
        } );
        
        // Appending content...
        contentArea.append( credForm.getDomNode() );
        contentArea.append( buttonContainer );
      }
      , this ) );
      
      // Buttons...
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonSave
                                                              , onclick : function( event ) { credForm.putHiddenInput( 'testCredential', null );
                                                                                              credForm.submit(); } } ) );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                              , label   : i18n.buttonCancel
                                                              , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );
      
      // Show content area now...
      this.show( contentArea.getDomNode() );
      
      // Get the credential...
      getCredCmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build the creation/edition form.
     * @param {Object=} credential The credential data to initialize the form. Optional.
     * @return {tsa.widgets.FormPanel} The resulting form panel.
     * @private
     */
    tsa.cmd.handlers.CredentialsHandler.prototype.createCredentialForm = function( credential )
    {
      /** @type {Object}                */ var i18n            = tsa.i18n;
      /** @type {boolean}               */ var isEditionMode   = TV.typeIsObject( typeof credential ) && ( credential !== null );
      /** @type {tsa.widgets.FormPanel} */ var generalForm     = null;
      /** @type {tsa.widgets.FormPanel} */ var nameForm        = new FormPanel( { title       : i18n.labelDiscoCredsNameHeading
                                                                                , description : i18n.labelDiscoCredsNameDesc } );
      /** @type {tsa.widgets.FormPanel} */ var typeForm        = new FormPanel( { title       : i18n.labelDiscoCredsSelectHeading
                                                                                , description : i18n.labelDiscoCredsSelectDesc } );
      /** @type {tsa.widgets.FormPanel} */ var restrictionForm = new FormPanel( { title       : i18n.labelDiscoCredsScopeRestrictHeading
                                                                                , description : i18n.labelDiscoCredsScopeRestrictDesc } );
      /** @type {tsa.widgets.FormPanel} */ var scopeSetForm    = new FormPanel( { title       : i18n.labelDiscoCredsScopeSetHeading
                                                                                , description : i18n.labelDiscoCredsScopeSetDesc } );
      /** @type {tsa.widgets.FormPanel} */ var testForm        = new FormPanel( { title       : i18n.labelDiscoCredsTestHeading
                                                                                , description : i18n.labelDiscoCredsTestDesc } );
      /** @type {tsa.widgets.FormPanel} */ var firstAccessForm = null;
      /** @type {tsa.widgets.FormPanel} */ var lastAccessForm  = null;
      /** @type {tsa.widgets.FormPanel} */ var firstForm       = nameForm;
      /** @type {tsa.cmd.Command}       */ var getScopeSetsCmd = new Command( { action              : 'scopesets/getScopeSets'
                                                                              , parameters          : { 'withCredentials' : 'true' }
                                                                              , clearSystemMessages : false } );
      
      /** @type {Object.<string,tsa.widgets.FormPanel>} */ var accessForms =
      { HostAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsHostAuthEnterDesc               } )
      , WindowsHostAuth     : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWindowsHostAuthEnterDesc        } )
    //, JBossAuth           : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsJBossAuthEnterDesc              } )
    //, OracleAppAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsOracleAppAuthEnterDesc          } )
    //, WeblogicAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWeblogicAuthEnterDesc           } )
    //, WebSphereAuth1      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWebSphereAuthEnterDesc          } )
    //, WebSphereAuth2      : new FormPanel( { title: i18n.labelDiscoCredsWebsphereKeyStoreHeading   , description: i18n.labelDiscoCredsWebSphereAuthKeyStoreEditDesc   } )
    //, WebSphereAuth3      : new FormPanel( { title: i18n.labelDiscoCredsWebsphereTrustStoreHeading , description: i18n.labelDiscoCredsWebSphereAuthTrustStoreEditDesc } )
    //, WasSshAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWasSshAuthEnterDesc             } )
    //, Db2Auth             : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsDb2AuthEnterDesc                } )
    //, OracleAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsOracleAuthEnterDesc             } )
    //, SybaseAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSybaseAuthEnterDesc             } )
    //, SqlServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSqlServerAuthEnterDesc          } )
    //, DominoServerAuth    : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsDominoServerAuthEnterDesc       } )
    //, MQServerAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsMQServerAuthEnterDesc           } )
    //, ExchangeServerAuth  : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsExchangeServerAuthEnterDesc     } )
      , SnmpAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSnmpAuthEnterDesc               } )
      , SnmpV3Auth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSnmpV3AuthEnterDesc             } )
    //, CCMSServerAuth      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCCMSServerAuthEnterDesc         } )
      , CiscoAuth           : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCiscoAuthEnterDesc              } )
      , CiscoWorksAuth      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCiscoWorksAuthEnterDesc         } )
    //, LdapAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsLdapAuthEnterDesc               } )
    //, SLDServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSLDServerAuthEnterDesc          } )
    //, CSMServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCSMServerAuthEnterDesc          } )
    //, VeritasClusterAuth  : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsVeritasClusterAuthEnterDesc     } )
    //, ITMScopeAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsITMScopeAuthEnterDesc           } )
    //, SysImagerServerAuth : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSysImagerServerAuthEnterDesc    } )
    //, VCAuth              : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsVCAuthEnterDesc                 } )
      };
      
      // Linking access forms...
      for( /** @type {string} */ var name in accessForms )
      {
        if( lastAccessForm !== null )
        {
          lastAccessForm.linkTo( accessForms[ name ] );
        }
        
        if( firstAccessForm === null )
        {
          firstAccessForm = accessForms[ name ];
        }
        
        lastAccessForm = accessForms[ name ];
      }
      
      // Linking all forms together...
      nameForm        .linkTo( typeForm        );
      typeForm        .linkTo( firstAccessForm );
      lastAccessForm  .linkTo( restrictionForm );
      restrictionForm .linkTo( scopeSetForm    );
      scopeSetForm    .linkTo( testForm        );
      
      // Fields (common):
      /** @type {Element} */ var nameField     = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var typeField     = $( '<select name="CredentialType" id="CredentialType" >'
                                                  +   '<option value="HostAuth" selected="selected"' + '>' + i18n.labelDiscoCredsTypeHostAuth        + '</option>'
                                                  +   '<option value="WindowsHostAuth"'              + '>' + i18n.labelDiscoCredsTypeWindowsHostAuth + '</option>'
                                                  +   '<option value="SnmpAuth"'                     + '>' + i18n.labelDiscoCredsTypeSnmpAuth        + '</option>'
                                                  +   '<option value="SnmpV3Auth"'                   + '>' + i18n.labelDiscoCredsTypeSnmpV3Auth      + '</option>'
                                                  +   '<option value="CiscoAuth"'                    + '>' + i18n.labelDiscoCredsTypeCiscoAuth       + '</option>'
                                                  +   '<option value="CiscoWorksAuth"'               + '>' + i18n.labelDiscoCredsTypeCiscoWorksAuth  + '</option>'
                                                  + '</select>' ).get(0);
      /** @type {Element} */ var resTypeField  = $( '<select name="RestrictionType" id="RestrictionType">'
                                                  +   '<option value="all" selected="selected"'   + '>' + i18n.labelDiscoCredsScopeRestrictAll   + '</option>'
                                                  +   '<option value="limit"'                     + '>' + i18n.labelDiscoCredsScopeRestrictLimit + '</option>'
                                                  + '</select>' ).get(0);
      /** @type {Element} */ var scopeSetField = $( '<select name="scopename" id="scopename"></select>' ).get(0);
      /** @type {Element} */ var testhostField = $( '<input type="text" id="ipAddress" name="ipAddress" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var testButton    = ButtonFactory.buildButton( { type: 'arrow', label: i18n.labelDiscoCredsTestButton } );
      /** @type {Element} */ var testButtonCon = ButtonFactory.buildButtonContainer();
      
      $( testButtonCon ).append( testButton );
      
      nameForm        .appendFieldRow( { label: i18n.labelDiscoCredsName          , required: true  , element: nameField     } );
      typeForm        .appendFieldRow( { label: i18n.labelDiscoCredsSelectType    , required: true  , element: typeField     } );
      restrictionForm .appendFieldRow( { label: i18n.labelDiscoCredsScopeRestrict , required: true  , element: resTypeField  } );
      scopeSetForm    .appendFieldRow( { label: i18n.labelDiscoCredsScopeSetName  , required: true  , element: scopeSetField } );
      testForm        .appendFieldRow( { label: i18n.labelDiscoCredsTestIP        , required: false , element: testhostField } );
      testForm        .appendHR();
      testForm        .appendJointRow( testButtonCon );
      
      if( isEditionMode )
      {
        /** @type {Element} */ var nameSpan = document.createElement( 'span' );
        /** @type {Element} */ var typeSpan = document.createElement( 'span' );
        
        generalForm = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );
        
        $( nameSpan ).css( { 'font-weight' : 'bold' } );
        $( nameSpan ).text( credential.name           );
        $( typeSpan ).text( credential.authClassLabel );
        
        generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralName, element: nameSpan } );
        generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralType, element: typeSpan } );
        
        generalForm.linkTo( nameForm );
        firstForm = generalForm;
        
        firstForm.putHiddenInput( 'pid'            , credential.pid                 );
        firstForm.putHiddenInput( 'name'           , credential.name                );
        firstForm.putHiddenInput( 'CredentialType' , credential.authClassSimpleName );
        
        $( nameField ).val( credential.name                );
        $( typeField ).val( credential.authClassSimpleName );
        
        nameForm.hide();
        typeForm.hide();
        
        if( TV.typeIsObject( typeof credential.scopeSet ) && ( credential.scopeSet !== null ) )
        {
          $( resTypeField ).val( 'limit' );
        }
      }
      
      // Fields for HostAuth:
      /** @type {Element} */ var hostAuthUserName      = $( '<input type="text"   id="HostAuth_username"  name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hostAuthPassword1     = $( '<input type="password" id="HostAuth_password" name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hostAuthPassword2     = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var hostAuthAuthType      = $( '<select name="HostAuth_authtype" id="HostAuth_authtype" >'
                                                          +   '<option value="authType_default" selected="selected"' + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_default'    ) + '</option>'
                                                          +   '<option value="authType_password"'                    + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_password'   ) + '</option>'
                                                          +   '<option value="authType_pki_scoped"'                  + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                          +   '<option value="authType_pki"'                         + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_pki'        ) + '</option>'
                                                          + '</select>' ).get(0);
      /** @type {Element} */ var hostAuthUploadKey     = $( '<input type="file" id="HostAuth_sshKey" name="HostAuth_sshKey" />' ).get(0);
      /** @type {Element} */ var hostAuthUploadKeyType = document.createElement( 'p' );
      /** @type {Element} */ var hostAuthUploadKeyWarn = document.createElement( 'p' );
      /** @type {Element} */ var hostAuthUploadKeyDiv  = document.createElement( 'div' );
      
      $( hostAuthUploadKeyType ).text( i18n.labelDiscoCredsInternalSSHKey );
      
      $( hostAuthUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( hostAuthUploadKeyWarn ).css( { 'margin-top'       : '0.80em'
                                      , 'border'           : '1px solid #c0c000'
                                      , 'border-radius'    : '5px'
                                      , 'background-color' : '#ffffc0' } );
      
      $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyType );
      $( hostAuthUploadKeyDiv ).append( hostAuthUploadKey     );
      $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyWarn );
      
      accessForms.HostAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser                , required: true  , element: hostAuthUserName     } );
      accessForms.HostAuth.appendFieldRow( { label: i18n.labelPasswordField                 , required: false , element: hostAuthPassword1    } );
      accessForms.HostAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField          , required: false , element: hostAuthPassword2    } );
      accessForms.HostAuth.appendFieldRow( { label: i18n.labelCredentialsAuthenticationType , required: false , element: hostAuthAuthType     } );
      accessForms.HostAuth.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey              , required: true  , element: hostAuthUploadKeyDiv } );
      
      // Fields for WindowsHostAuth:
      /** @type {Element} */ var windowsHostAuthUserName        = $( '<input type="text"   id="WindowsHostAuth_username"  name="WindowsHostAuth_username"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var windowsHostAuthPassword1       = $( '<input type="password"  id="WindowsHostAuth_password" name="WindowsHostAuth_password"         size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var windowsHostAuthPassword2       = $( '<input type="password" id="WindowsHostAuth_password_confirm" name="WindowsHostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var windowsHostAuthAuthType        = $( '<select name="WindowsHostAuth_authtype" id="WindowsHostAuth_authtype">'
      /* Windows PKI is unsupported */ //                          +   '<option value="authType_default" selected="selected"' + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_default'    ) + '</option>'
                                                                   +   '<option value="authType_password"'                    + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_password'   ) + '</option>'
      /* Windows PKI is unsupported */ //                          +   '<option value="authType_pki"'                         + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_pki'        ) + '</option>'
      /* Windows PKI is unsupported */ //                          +   '<option value="authType_pki_scoped"'                  + '>' + tsa.cmd.handlers.CredentialsHandler.i18nAuthType( 'authType_pki_scoped' ) + '</option>'
                                                                   + '</select>' ).get(0);
      /** @type {Element} */ var windowsHostAuthUploadKey       = $( '<input type="file" id="WindowsHostAuth_sshKey" name="WindowsHostAuth_sshKey" />' ).get(0);
      /** @type {Element} */ var windowsHostAuthUploadKeyType   = document.createElement( 'p' );
      /** @type {Element} */ var windowsHostAuthUploadKeyWarn   = document.createElement( 'p' );
      /** @type {Element} */ var windowsHostAuthUploadKeyDiv    = document.createElement( 'div' );
      
      $( windowsHostAuthUploadKeyType ).text( i18n.labelDiscoCredsInternalSSHKey );
      
      $( windowsHostAuthUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
      $( windowsHostAuthUploadKeyWarn ).css( { 'margin-top'       : '0.80em'
                                             , 'border'           : '1px solid #c0c000'
                                             , 'border-radius'    : '5px'
                                             , 'background-color' : '#ffffc0' } );
      
      $( windowsHostAuthUploadKeyDiv ).append( windowsHostAuthUploadKeyType );
      $( windowsHostAuthUploadKeyDiv ).append( windowsHostAuthUploadKey     );
      $( windowsHostAuthUploadKeyDiv ).append( windowsHostAuthUploadKeyWarn );
      
      accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser                , required: true  , element: windowsHostAuthUserName     } );
      accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelPasswordField                 , required: false , element: windowsHostAuthPassword1    } );
      accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField          , required: false , element: windowsHostAuthPassword2    } );
      accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelCredentialsAuthenticationType , required: false , element: windowsHostAuthAuthType     } );
      accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey              , required: true  , element: windowsHostAuthUploadKeyDiv } );
      
      // Common value initialization for fields of HostAuth and WindowsHostAuth:
      if( isEditionMode )
      {
        /** @type {boolean} */ var isHostAuth    = ( credential.authClassSimpleName === 'HostAuth'        );
        /** @type {boolean} */ var isWinHostAuth = ( credential.authClassSimpleName === 'WindowsHostAuth' );
        
        if( isHostAuth || isWinHostAuth )
        {
          /** @type {Element} */ var usernameField     = ( isWinHostAuth ? windowsHostAuthUserName : hostAuthUserName );
          /** @type {Element} */ var authtypeField     = ( isWinHostAuth ? windowsHostAuthAuthType : hostAuthAuthType );
          /** @type {boolean} */ var isPKI             = ( credential.properties.authtype.indexOf( 'authType_pki' ) === 0 );
          /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                           ( credential.scopeSetName.length > 0 ) );
          /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
          /** @type {string}  */ var authType          = credential.properties.authtype;
          /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );
          if( isPKI )
          {
            authType +=   scopeSetHasSSHKey  ? '_scoped' : '' ;
          }
          
          
          $( usernameField ).val( credential.properties.username );
          $( authtypeField ).val( isHostAuth ? authType : /* Windows only supports: */ 'authType_password' );
        }
      }
      
      // Fields for SnmpAuth:
      /** @type {Element} */ var snmpAuthCommString1 = $( '<input type="password" name="SnmpAuth_communitystring" id="SnmpAuth_communitystring"        size="50" maxlength="250" required/>' ).get(0);
      /** @type {Element} */ var snmpAuthCommString2 = $( '<input type="password" name="SnmpAuth_communitystring_confirm" id="SnmpAuth_communitystring_confirm" size="50" maxlength="250" required/>' ).get(0);
      
      accessForms.SnmpAuth.appendFieldRow( { label: i18n.labelDiscoCredsCommunity1 , required: true  , element: snmpAuthCommString1 } );
      accessForms.SnmpAuth.appendFieldRow( { label: i18n.labelDiscoCredsCommunity2 , required: true  , element: snmpAuthCommString2 } );
      
      // Fields for SnmpV3Auth:
      /** @type {Element} */ var snmpV3AuthUserName  = $( '<input type="text"     name="SnmpV3Auth_username" id="SnmpV3Auth_username"            size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var snmpV3AuthPassword1 = $( '<input type="password" name="SnmpV3Auth_password" id="SnmpV3Auth_password"            size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var snmpV3AuthPassword2 = $( '<input type="password" name="SnmpV3Auth_password_confirm"  id="SnmpV3Auth_password_confirm"    size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var snmpV3AuthPrivPwd1  = $( '<input type="password" name="SnmpV3Auth_privpassword"  id="SnmpV3Auth_privpassword"       size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var snmpV3AuthPrivPwd2  = $( '<input type="password" name="SnmpV3Auth_privpassword_confirm"  id="SnmpV3Auth_privpassword_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var snmpV3AuthAuthProto = $( '<select name="SnmpV3Auth_authprotocol" id="SnmpV3Auth_authprotocol" >'
                                                        +   '<option value="None"' + '>' + i18n.labelDiscoCredsAuthProtoNone + '</option>'
                                                        +   '<option value="MD5"'  + '>' + i18n.labelDiscoCredsAuthProtoMd5  + '</option>'
                                                        +   '<option value="SHA"'  + '>' + i18n.labelDiscoCredsAuthProtoSha  + '</option>'
                                                        + '</select>' ).get(0);
      
      if( isEditionMode )
      {
        $( snmpV3AuthUserName  ).val( credential.properties.username     );
        $( snmpV3AuthAuthProto ).val( credential.properties.authprotocol );
        if( credential.properties.authprotocol === 'MD5' || credential.properties.authprotocol === 'SHA' )
        {
        	accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: true , element: snmpV3AuthPassword1 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: true , element: snmpV3AuthPassword2 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: true , element: snmpV3AuthPrivPwd1  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: true , element: snmpV3AuthPrivPwd2  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );
        }
        else{
        	accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: false , element: snmpV3AuthPassword1 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: false , element: snmpV3AuthPassword2 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: false , element: snmpV3AuthPrivPwd1  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: false , element: snmpV3AuthPrivPwd2  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );
        }
      }
      else
      {
    	  accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
          accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: false , element: snmpV3AuthPassword1 } );
          accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: false , element: snmpV3AuthPassword2 } );
          accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: false , element: snmpV3AuthPrivPwd1  } );
          accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: false , element: snmpV3AuthPrivPwd2  } );
          accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );
           
      }
      
      // Fields for CiscoAuth:
      /** @type {Element} */ var ciscoAuthUserName   = $( '<input type="text"     name="CiscoAuth_username"   id="CiscoAuth_username"            size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoAuthPassword1  = $( '<input type="password" name="CiscoAuth_password"    id="CiscoAuth_password"           size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoAuthPassword2  = $( '<input type="password" name="CiscoAuth_password_confirm" id="CiscoAuth_password_confirm"      size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoAuthEnablePwd1 = $( '<input type="password" name="CiscoAuth_enablepassword"   id="CiscoAuth_enablepassword"      size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoAuthEnablePwd2 = $( '<input type="password" name="CiscoAuth_enablepassword_confirm" id="CiscoAuth_enablepassword_confirm"  size="50" maxlength="250" />' ).get(0);
      
      accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser         , required: true  , element: ciscoAuthUserName   } );
      accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelPasswordField          , required: true  , element: ciscoAuthPassword1  } );
      accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField   , required: true  , element: ciscoAuthPassword2  } );
      accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsEnablePassw1 , required: false , element: ciscoAuthEnablePwd1 } );
      accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsEnablePassw2 , required: false , element: ciscoAuthEnablePwd2 } );
      
      
      $(ciscoAuthPassword1).prop('required',true);
      $(ciscoAuthPassword2).prop('required',true);
      
      if( isEditionMode )
      {
        $( ciscoAuthUserName  ).val( credential.properties.username );
      }
      
      // Fields for CiscoWorksAuth:
      /** @type {Element} */ var ciscoWorksAuthUserName   = $( '<input type="text"     name="CiscoWorksAuth_username"  id="CiscoWorksAuth_username"       size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoWorksAuthPassword1  = $( '<input type="password" name="CiscoWorksAuth_password"  id="CiscoWorksAuth_password"       size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var ciscoWorksAuthPassword2  = $( '<input type="password" name="CiscoWorksAuth_password_confirm" id="CiscoWorksAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
      
      accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser       , required: true , element: ciscoWorksAuthUserName  } );
      accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelPasswordField        , required: true , element: ciscoWorksAuthPassword1 } );
      accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField , required: true , element: ciscoWorksAuthPassword2 } );
      
      $(ciscoWorksAuthPassword1).prop('required',true);
      $(ciscoWorksAuthPassword2).prop('required',true);
      
      if( isEditionMode )
      {
        $( ciscoWorksAuthUserName  ).val( credential.properties.username );
      }
      
      // Event handler - type changed:
      var onTypeFieldChanged = function( event )
      {
        /** @type {string}  */ var selected  = $( typeField ).val();
        /** @type {string}  */ var className = '';
        /** @type {boolean} */ var testable  = false;
        
        for( /** @type {string} */ var name in accessForms )
        {
          accessForms[ name ][ ( name === selected ) ? 'show' : 'hide' ]();
        }
        
        switch( selected )
        {
          case 'HostAuth'            : testable = true ; className = 'com.collation.platform.security.auth.HostAuth'                            ; break;
          case 'WindowsHostAuth'     : testable = true ; className = 'com.collation.platform.security.auth.WindowsHostAuth'                     ; break;
          case 'JBossAuth'           : testable = false; className = 'com.collation.platform.security.auth.JBossAuth'                           ; break;
          case 'OracleAppAuth'       : testable = false; className = 'com.collation.platform.security.auth.OracleAppAuth'                       ; break;
          case 'WeblogicAuth'        : testable = false; className = 'com.collation.platform.security.auth.WeblogicAuth'                        ; break;
          case 'WebSphereAuth1'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
          case 'WebSphereAuth2'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
          case 'WebSphereAuth3'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
          case 'WasSshAuth'          : testable = false; className = 'com.collation.platform.security.auth.WasSshAuth'                          ; break;
          case 'Db2Auth'             : testable = false; className = 'com.collation.platform.security.auth.Db2Auth'                             ; break;
          case 'OracleAuth'          : testable = false; className = 'com.collation.platform.security.auth.OracleAuth'                          ; break;
          case 'SybaseAuth'          : testable = false; className = 'com.collation.platform.security.auth.SybaseAuth'                          ; break;
          case 'SqlServerAuth'       : testable = false; className = 'com.collation.platform.security.auth.SqlServerAuth'                       ; break;
          case 'DominoServerAuth'    : testable = false; className = 'com.collation.platform.security.auth.DominoServerAuth'                    ; break;
          case 'MQServerAuth'        : testable = false; className = 'com.collation.platform.security.auth.MQServerAuth'                        ; break;
          case 'ExchangeServerAuth'  : testable = false; className = 'com.collation.platform.security.auth.ExchangeServerAuth'                  ; break;
          case 'SnmpAuth'            : testable = true ; className = 'com.collation.platform.security.auth.SnmpAuth'                            ; break;
          case 'SnmpV3Auth'          : testable = true ; className = 'com.collation.platform.security.auth.SnmpV3Auth'                          ; break;
          case 'CCMSServerAuth'      : testable = false; className = 'com.collation.platform.security.auth.CCMSServerAuth'                      ; break;
          case 'CiscoAuth'           : testable = false; className = 'com.collation.platform.security.auth.CiscoAuth'                           ; break;
          case 'CiscoWorksAuth'      : testable = false; className = 'com.collation.platform.security.auth.CiscoWorksAuth'                      ; break;
          case 'LdapAuth'            : testable = false; className = 'com.collation.platform.security.auth.LdapAuth'                            ; break;
          case 'SLDServerAuth'       : testable = false; className = 'com.collation.platform.security.auth.SLDServerAuth'                       ; break;
          case 'CSMServerAuth'       : testable = false; className = 'com.ibm.cdb.discover.auth.sys.hpc.cm.csmserver.CSMServerAuth'             ; break;
          case 'VeritasClusterAuth'  : testable = false; className = 'com.collation.platform.security.auth.VeritasClusterAuth'                  ; break;
          case 'ITMScopeAuth'        : testable = false; className = 'com.ibm.cdb.discover.auth.net.itmscope.ITMScopeAuth'                      ; break;
          case 'SysImagerServerAuth' : testable = false; className = 'com.ibm.cdb.discover.auth.sys.hpc.cm.sysimagerserver.SysImagerServerAuth' ; break;
          case 'VCAuth'              : testable = false; className = 'com.collation.platform.security.auth.VCAuth'                              ; break;
        }
        
        testForm[ testable ? 'show' : 'hide' ]();
        typeForm.putHiddenInput( 'ClassName', className );
        
        if( ( selected === 'HostAuth'        ) ||
            ( selected === 'WindowsHostAuth' ) )
        {
          onHostAuthAuthTypeChanged( null );
        }
      };
      
      $( typeField ).on( 'change', onTypeFieldChanged );
      
      // Event handler - scope set restriction type changed:
      var onRestrictionTypeFieldChanged = $.proxy( function( event )
      {
        /** @type {boolean}        */ var isHostAuth          = !accessForms.HostAuth.isHidden();
        /** @type {boolean}        */ var isWindowsHostAuth   = !accessForms.WindowsHostAuth.isHidden();
        /** @type {boolean}        */ var authSupportsPKI     = ( isHostAuth || isWindowsHostAuth );
        /** @type {Element}        */ var authTypeField       = ( authSupportsPKI ? ( isWindowsHostAuth ? windowsHostAuthAuthType      : hostAuthAuthType      ) : null );
        /** @type {Element}        */ var warnMsgElement      = ( authSupportsPKI ? ( isWindowsHostAuth ? windowsHostAuthUploadKeyWarn : hostAuthUploadKeyWarn ) : null );
        /** @type {boolean}        */ var pkiGlobal           = ( authSupportsPKI && ( $( authTypeField ).val() === 'authType_pki'        ) );
        /** @type {boolean}        */ var pkiScoped           = ( authSupportsPKI && ( $( authTypeField ).val() === 'authType_pki_scoped' ) );
        /** @type {boolean}        */ var isPKI               = ( authSupportsPKI && ( pkiGlobal || pkiScoped ) );
        /** @type {boolean}        */ var restricted          = ( $( resTypeField ).val() === 'limit' );
        /** @type {?string}        */ var selectedScopeSetPID = ( ( restricted && ( scopeSetField.options.length > 0 ) ) ? $( scopeSetField ).val() : null );
        /** @type {Object}         */ var scopeSet            = ( ( TV.typeIsString( typeof selectedScopeSetPID                   ) &&
                                                                    TV.typeIsObject( typeof this.scopeSets[ selectedScopeSetPID ] ) )
                                                                  ? this.scopeSets[ selectedScopeSetPID ] : null );
        /** @type {Array.<Object>} */ var scopeSetPkiCreds    = ( ( isPKI && ( scopeSet !== null ) ) ? tsa.cmd.handlers.CredentialsHandler.retrievePKICredentialsFor( scopeSet ) : [] );
        /** @type {boolean}        */ var scopeSetHasPkiCreds = ( ( scopeSetPkiCreds !== null ) && ( scopeSetPkiCreds.length > 0 ) );
        /** @type {string}         */ var pkiCredsAsString    = '';
        /** @type {boolean}        */ var showWarningMessage  = ( isPKI && scopeSetHasPkiCreds && ( !isEditionMode || ( selectedScopeSetPID !== credential.scopeSetPID ) ) );
        
        $( warnMsgElement ).empty();
        $( warnMsgElement )[ showWarningMessage ? 'show' : 'hide' ]();
        
        if( authSupportsPKI )
        {
          if( scopeSetHasPkiCreds )
          {
            /** @type {string} */ var warnMsg = '';
            
            $.each( scopeSetPkiCreds, function( credIndex, cred )
            {
              pkiCredsAsString += ( ( credIndex > 0 ) ? ', ' : '' ) + cred.name;
            } );
            
            warnMsg = tsa.bind( i18n.labelDiscoCredsPKICredExists, [ pkiCredsAsString, scopeSet.name ] );
            
            $( warnMsgElement ).append( IconFactory.buildSeverityIcon( { severity : 'WARN' } )
                                      , document.createTextNode( '\xa0' + warnMsg ) );
          }
          
          if( pkiScoped && !restricted )
          {
            $( resTypeField ).val( 'limit' );
            restricted = true;
          }
        }
        
        scopeSetField.disabled = !restricted;
        
        // If the restriction type field is disabled, we need to send its value using a hidden field...
        resTypeField.disabled = pkiScoped;
        firstForm.putHiddenInput( resTypeField.name, ( pkiScoped ? 'limit' : null ) );
      }
      , this );
      
      $( resTypeField  ).on( 'change', onRestrictionTypeFieldChanged );
      $( scopeSetField ).on( 'change', onRestrictionTypeFieldChanged );
      
      // Event handler - HostAuth and WindowHostAuth AuthType changed:
      var onHostAuthAuthTypeChanged = function( event )
      {
        /** @type {boolean} */ var isHostAuth          = !accessForms.HostAuth.isHidden();
        /** @type {boolean} */ var isWindowsHostAuth   = !accessForms.WindowsHostAuth.isHidden();
        /** @type {Element} */ var authTypeField       = ( isWindowsHostAuth ? windowsHostAuthAuthType      : hostAuthAuthType      );
        /** @type {Element} */ var keyTypeMessage      = ( isWindowsHostAuth ? windowsHostAuthUploadKeyType : hostAuthUploadKeyType );
        /** @type {Element} */ var uploadKeyField      = ( isWindowsHostAuth ? windowsHostAuthUploadKey     : hostAuthUploadKey     );
        /** @type {boolean} */ var pkiGlobal           = ( $( authTypeField ).val() === 'authType_pki'        );
        /** @type {boolean} */ var pkiScoped           = ( $( authTypeField ).val() === 'authType_pki_scoped' );
        /** @type {boolean} */ var isPKI               = (pkiGlobal || pkiScoped);
        /** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
        /** @type {boolean} */ var hideSSHKeyUpload    = ( hideSSHKeyRow || pkiGlobal || isEditionMode );
        /** @type {boolean} */ var disableSSHKeyUpload = !pkiScoped;
        /** @type {boolean} */ var hideKeyTypeMessage  = pkiScoped && !isEditionMode;
        /** @type {Element} */ var replaceButton       = document.createElement( 'button' );
        /** @type {Element} */ var row                 = $( uploadKeyField ).closest( 'tr' ).get(0);
        
        $( row            )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
        $( uploadKeyField )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
        $( keyTypeMessage )[ hideKeyTypeMessage ? 'hide' : 'show' ]();
        
        // Password or Passphare? use the right label...
        $( hostAuthPassword1        ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelPassphraseField        : i18n.labelPasswordField );
        $( hostAuthPassword2        ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField );
        $( windowsHostAuthPassword1 ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelPassphraseField        : i18n.labelPasswordField );
        $( windowsHostAuthPassword2 ).closest( 'tr' ).find( 'label' ).text( isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField );
        
        if( isPKI )
        {
          $( keyTypeMessage ).text( pkiGlobal ? i18n.labelDiscoCredsInternalSSHKey
                                              : i18n.labelDiscoCredsExternalSSHKey );
          
          if( pkiScoped )
          {
            $( resTypeField ).val( 'limit' );
            
            if( isEditionMode )
            {
              replaceButton.type = 'button';
              $( replaceButton ).text( i18n.labelDiscoCredsExternalSSHKeyReplace );
              $( replaceButton ).on( 'click', function( event )
                                              {
                                                $( keyTypeMessage ).hide();
                                                $( uploadKeyField ).show();
                                              } );
              
              $( keyTypeMessage ).append( document.createTextNode( /* nbsp */ '\xa0' ) );
              $( keyTypeMessage ).append( replaceButton );
            }
          }
        }
        
        onRestrictionTypeFieldChanged();
        uploadKeyField.disabled = disableSSHKeyUpload;
      };
      
      $( hostAuthAuthType ).on( 'change', onHostAuthAuthTypeChanged );
      
      // Event handler Authentication protocol changed:
      var onAuthProtoChanged = function( event )
      {
    	  /** @type {string}  */ var selected   = $( snmpV3AuthAuthProto ).val();    	  
    	  
    	  if( selected === 'MD5' || selected === 'SHA' )
    	  {
    		  var labelNode = $(snmpV3AuthPassword1).closest('tr').find('label');
    		  
    		  accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPassword1).attr('id'), accessForms.SnmpV3Auth.validationRuleRequired );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelPasswordField, $("<span class='ibm-required'> *</span>").get(0));
    		  
    		  labelNode = $(snmpV3AuthPassword2).closest('tr').find('label');
    		  accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPassword2).attr('id'), accessForms.SnmpV3Auth.validationRuleRequired );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelConfirmPasswordField, $("<span class='ibm-required'> *</span>").get(0));
    		  
    		  labelNode = $(snmpV3AuthPrivPwd1).closest('tr').find('label');
    		  accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPrivPwd1).attr('id'), accessForms.SnmpV3Auth.validationRuleRequired );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelDiscoCredsPrivatePassw1, $("<span class='ibm-required'> *</span>").get(0));
    		  
    		  labelNode = $(snmpV3AuthPrivPwd2).closest('tr').find('label');
    		  accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPrivPwd2).attr('id'), accessForms.SnmpV3Auth.validationRuleRequired );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelDiscoCredsPrivatePassw2, $("<span class='ibm-required'> *</span>").get(0));
    		  
    	  }
    	  else if( selected === 'None' )
    	  {
    		  var labelNode = $(snmpV3AuthPassword1).closest('tr').find('label');
    		  
    		  accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPassword1).attr('id') );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelPasswordField);

    		  labelNode = $(snmpV3AuthPassword2).closest('tr').find('label');
    		  accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPassword2).attr('id') );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelConfirmPasswordField);
    		  
              labelNode = $(snmpV3AuthPrivPwd1).closest('tr').find('label');
              accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPrivPwd1).attr('id') );
              labelNode.empty();
              labelNode.append(i18n.labelDiscoCredsPrivatePassw1);
              
              labelNode = $(snmpV3AuthPrivPwd2).closest('tr').find('label');
    		  accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPrivPwd2).attr('id') );
    		  labelNode.empty();
    		  labelNode.append(i18n.labelDiscoCredsPrivatePassw2);
    	  }
    	  
      };
      
      $( snmpV3AuthAuthProto ).on( 'change', onAuthProtoChanged );
      
      // Event handler - WindowsHostAuth's AuthType changed:
      $( windowsHostAuthAuthType ).on( 'change', onHostAuthAuthTypeChanged );
      
      // Event handler - Test Button:
      $( testButton ).on( 'click', function( event )
      {
        /** @type {string} */ var action = firstForm.getActionAttribute();
        
        firstForm.putHiddenInput( 'testCredential', 'true' );
        firstForm.setActionAttribute( 'credentials/ajaxTestCredential' );
        firstForm.submit();
        firstForm.setActionAttribute( action );
      } );
      
      // Triggering the event handlers for the first time, for initialization purposes:
      onTypeFieldChanged( null );
      onRestrictionTypeFieldChanged( null );
      onHostAuthAuthTypeChanged( null );
      
      // Loading the Scope Sets in the Scope Set Selector...
      $( getScopeSetsCmd ).on( 'executionSucceeded', $.proxy( function( event )
      {
        if( TV.typeIsObject( typeof event.response.payload.scopeSets ) )
        {
          /** @type {Array.<Object>} */ var scopeSets = event.response.payload.scopeSets;
          
          if( TV.typeIsObject( typeof scopeSets ) && TV.isInstanceOf( scopeSets, Array ) )
          {
            logger.info( 'Populating scope set selector (' + scopeSets.length + ' scope set(s))...' );
            $( scopeSetField ).empty();
            this.scopeSets = {};
            
            $.each( scopeSets, $.proxy( function( scopeSetIndex, scopeSet )
            {
              if( TV.typeIsString( typeof scopeSet.displayName ) &&
                  TV.typeIsString( typeof scopeSet.pid         ) )
              {
                /** @type {Element} */ var option = document.createElement( 'option' );
                
                $( option ).text( scopeSet.displayName );
                $( option ).prop( 'value', scopeSet.pid );
                $( scopeSetField ).append( option );
                
                this.scopeSets[ scopeSet.pid ] = scopeSet;
              }
              else
              {
                logger.warn( 'Skipping invalid scope set (' + 'index'       + ': ' + scopeSetIndex        + '; '
                                                            + 'pid'         + ': ' + scopeSet.pid         + '; '
                                                            + 'displayName' + ': ' + scopeSet.displayName + ').' );
              }
            }
            , this ) );
            
            if( isEditionMode && TV.typeIsObject( typeof credential.scopeSet ) && ( credential.scopeSet !== null ) )
            {
              $( scopeSetField ).val( credential.scopeSet.pid );
            }
          }
        }
      }
      , this ) );
      
      // Retrieving Scope Sets from server...
      getScopeSetsCmd.execute();
      
      // Return the form...
      return firstForm;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Get all the PKI credentials for the given scope set.
     * @param {Object} scopeSetData Scope set data (as retrieved from server).
     * @return {Array.<Object>}     The PKI credential data entries (as retrieved from server).
     */
    tsa.cmd.handlers.CredentialsHandler.retrievePKICredentialsFor = function( scopeSetData )
    {
      /** @type {Array.<string>} */ var pkiCredentials = [];
      
      if( TV.typeIsObject( typeof scopeSetData             ) && ( scopeSetData !== null ) &&
          TV.typeIsObject( typeof scopeSetData.credentials ) && TV.isInstanceOf( scopeSetData.credentials, Array ) )
      {
        $.each( scopeSetData.credentials, function( credIndex, cred )
        {
          if( TV.typeIsObject( typeof cred                     ) && ( cred                     !== null           ) &&
              TV.typeIsObject( typeof cred.properties          ) && ( cred.properties          !== null           ) &&
//              TV.typeIsString( typeof cred.properties.authtype ) && ( cred.properties.authtype === 'authType_pki' ) )
              TV.typeIsString( typeof cred.properties.authtype ) && ( cred.properties.authtype === 'authType_pki_scoped' ) )
          {
            pkiCredentials.push( cred );
          }
        } );
      }
      
      return pkiCredentials;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Convert authentication-type value to i18n label.
     * @param {string} authtype Authentication type value.
     * @return {string}         The i18n label, or the authentication type value
     *                          unchanged if an i18n label was not found for it.
     */
    tsa.cmd.handlers.CredentialsHandler.i18nAuthType = function ( credential )
    {
      /** @type {string} */  var i18nLabel = '';
      /** @type {string} */  var authType  = '';
      if ( TV.typeIsString (typeof credential) )
      {
        authType = credential;
      }
      else if( !TV.typeIsUndefined( typeof credential.properties.authtype ) )
      {
       
        /** @type {boolean} */ var isPKI             = ( credential.properties.authtype.indexOf( 'authType_pki' ) === 0 );
        /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                         ( credential.scopeSetName.length > 0 ) );
        /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
        /** @type {string}  */ var authType          = credential.properties.authtype;
        /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );
        if( isPKI )
        {
          authType +=   scopeSetHasSSHKey  ? '_scoped' : '' ;
        }
      }
      else 
      {
        authType = credential.authClassLabel;
      }
      
      switch( authType )
      {
        case 'authType_default'    : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypeDefault   ; break;
        case 'authType_password'   : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
        case 'authType_pki'        : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePki       ; break;
        case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
        default                    : i18nLabel = authType;
      }
      
      return i18nLabel;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.CredentialsHandler;
  }
);
