/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 1
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery' ,
    'tsa/TypeVerificator' ,
    'tsa/cmd/handlers/AbstractHandler' ,
    'tsa/cmd/Command' ,
    'tsa/cmd/ShowCommand' ,
    'tsa/widgets/ButtonFactory' ,
    'tsa/widgets/ContentArea' ,
    'tsa/widgets/FormPanel' ,
    'tsa/widgets/IconFactory' ,
    'tsa/widgets/TablePanel'
  ],

  function( $ ,
            TV ,
            AbstractHandler ,
            Command ,
            ShowCommand ,
            ButtonFactory ,
            ContentArea ,
            FormPanel ,
            IconFactory ,
            TablePanel )
{
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger('CredentialsvmwareHandler');

    //-------------------------------------------------------------------------

    /**
     *  @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters.
     */
    tsa.cmd.handlers.CredentialsvmwareHandler = function(parameters) {
        if ((!TV.typeIsObject(typeof parameters)) || (parameters === null)) {
            parameters = /** @type {?} */ ( {} );
        }

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call(this, parameters);
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CredentialsvmwareHandler.prototype ).constructor = tsa.cmd.handlers.CredentialsvmwareHandler;

    //-------------------------------------------------------------------------


    /**
     * Show main screen (list the credentials).  NOT USED.  Credentials viewed via the VMware Dynamic Scope edit
     * function (ScopsetvmwareHandler.js).
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype._index = function() {

    };



    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** ADD CREDENTIAL SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.CredentialsvmwareHandler.prototype._new = function() {
        var i18n = tsa.i18n;

        var pid            = this.getParameter('pid');
        var credentialType = this.getParameter('type');
        var scopeSetName   = this.getParameter('scopeSetName');

        var contentArea    = new ContentArea( { title   : i18n.labelDiscoVmwareCredsNewCredsHeading,
                                                helpDoc : 'sshelp_discovery_vmware_credentials_new_panel.html' } );

        // Check with TSA server if internal SSH key has been defined or not.  This impacts what options are available for
        // the Linux credentials "Authentication Type" field.
        //
        // To keep from having the code be indented too much, the bulk of the processing is done in the createEditScreen() function
        // immediately below.
        var getApplianceSshKeyCommand = new Command( { action              : 'credentials/ajaxCheckInternalSshKey',
                                                       clearSystemMessages : false } );

        $(getApplianceSshKeyCommand).on('executionSucceeded', $.proxy(function() {
            this.createAddScreen(contentArea, pid, credentialType, scopeSetName, true);
        }, this ) );

        $(getApplianceSshKeyCommand).on('executionFailed', $.proxy( function() {
            this.createAddScreen(contentArea, pid, credentialType, scopeSetName, false);
        }, this ) );

        getApplianceSshKeyCommand.execute();

        this.show(contentArea.getDomNode());
    };


    //
    // Main function the creates the content for the new scope set screen.
    //
    // @param contentArea Main panel area where the new scope set panes are added to.
    // @param pid
    // @param credentialType Indicate whether credential is for VMware, Linux, or Windows.
    // @param scopeSetName
    // @param isPkiDefined If true, indicates that the TSA appliance SSH key has been configured; otherwise false.
    //                     NOTE: By default, TSA is shipped with no TSA appliance SSH key.  The TSA administrator
    //                           has to create one in order for it to exist.
    //
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.createAddScreen = function(contentArea, pid, credentialType, scopeSetName, isPkiDefined) {
        var i18n = tsa.i18n;

        var credForm        = this.createCredentialForm(credentialType, scopeSetName, null, pid, isPkiDefined);
        var buttonContainer = ButtonFactory.buildButtonContainer();

        if (credentialType === tsa.i18n.labelDiscoVmwareCredsVmwareType) {
            credForm.setActionAttribute( 'credentialsvmware/postVmwareCredential' );
        } else if (credentialType == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
            credForm.setActionAttribute( 'credentialsvmware/postLinuxCredential' );
        } else if (credentialType == tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            credForm.setActionAttribute( 'credentialsvmware/postWindowsCredential' );
        } else {
            logger.warn("Invalid credential type: " + credentialType);
        }

        /*
         * Create "save" and "cancel" buttons.
         */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonSave } );

        $(saveButton).on('click', function () {
            // Indicate normal save request (not a test credential).
            credForm.putHiddenInput('testCredential', null);

            var inputValue = $("input[name='name']").val();
            if (inputValue.length === 0) {
                credForm.validate();
            } else {
                $("input[name='name']").val(inputValue);
                credForm.submit();
            }
        } );

        var cancelButton = ButtonFactory.buildButton( { type    : 'cancel',
                                                        label   : i18n.buttonCancel } );

        $(cancelButton).on('click', function () {
            ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                 clearSystemMessages : true,
                                 parameters          : { pid : pid } } ) ).execute();
        } );

        $(buttonContainer).append(saveButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
        $(buttonContainer).append(cancelButton);

        // Appending content...
        contentArea.append( credForm.getDomNode() );
        contentArea.append( buttonContainer );

        /*
         * Add successful:
         *     - When save button was clicked, print success message, then show the credentials list.
         *     - When test credential button was clicked, stay at current panel so user can later save the credential if desired.
         */
        $(credForm).on('executionSucceeded', function() {
            if (credForm.getHiddenInput("testCredential") === null) {
                var credentialName = $("input[name='name']").val();
                tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoVmwareCredsCreated, credentialName ) } );

                ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                     clearSystemMessages : false,
                                     parameters          : { pid : pid } } ) ).execute();
            }
        } );

        /*
         * Add / Test failed.  Show which fields are in error.
         */
        $(credForm).on('executionFailed', function(event) {
            if (TV.typeIsObject(typeof event.response.payload.errors)) {
                for (var fieldName in event.response.payload.errors) {
                    credForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                }
                delete event.response.payload.errors;
            }
        } );
    };




    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** SHOW CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.CredentialsvmwareHandler.prototype._show = function() {
        var i18n = tsa.i18n;

        var pid          = this.getParameter('entityId');
        var sepid        = this.getParameter('pid');
        var contentArea  = new ContentArea( { title   : i18n.labelDiscoVmwareCredsHeading,
                                              helpDoc : 'sshelp_discovery_vmware_credentials_show_panel.html' } );
        var generalTable = new TablePanel( { title   : i18n.labelGeneralPaneHeading,
                                             columns : [ { id: 'name'  , label: '' , width: '30%' , sortable: false },
                                                         { id: 'value' , label: '' , width: '70%' , sortable: false } ] } );
        var buttonContainer = ButtonFactory.buildButtonContainer();

        /*
         * Get specific credential.
         */
        var getCredentialCmd = new Command( { action              : 'credentials/getCredential',
                                              parameters          : { 'pid' : pid },
                                              clearSystemMessages : false } );

        $(getCredentialCmd).on('executionSucceeded',  $.proxy(function(event) {
            var credential = event.response.payload.credential;

            if ((TV.typeIsObject(typeof credential)) && (credential !== null)) {
                var scopeSetSpan = document.createElement('span');
                var userName     = credential.userName;
                var scopeSetName = credential.scopeSetName;

                if (userName     === null) userName     = '';
                if (scopeSetName === null) scopeSetName = '';

                // Trim off hidden portion of scope set name
                var displayScopeSetName = this.getDisplayableScopeSetName(scopeSetName);

                $(scopeSetSpan).text(displayScopeSetName);
                $(scopeSetSpan).css( { 'color' : ( credential.scopeSetMissing ? 'red' : 'black' ) } );

                var credentialTarget = this.i18nCredentialType(credential.name);
                var credentialName   = credential.name.substring(0, credential.name.indexOf("_SCOPESET_"));

                /*
                 * Populate the rows of the show panel.
                 */
                generalTable.appendRow( [ i18n.labelNameField          , credentialName   ] );
                generalTable.appendRow( [ i18n.labelTypeField          , credentialTarget ] );
                generalTable.appendRow( [ i18n.labelUserNameFieldColon , userName         ] );
                generalTable.appendRow( [ i18n.labelScopeSetField      , scopeSetSpan     ] );

                // Authentication type only shown for Linux.
                if (credentialTarget == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
                    generalTable.appendRow( [ i18n.labelAuthenticationTypeField , this.i18nAuthType(credential.properties.authtype) ] );
                }

                // If TSA web UI is in debug mode, show a few more fields.
                if (tsa.isDebugModeOn()) {
                    generalTable.appendRow( [ i18n.labelPidDebugField   , credential.pid           ] );
                    generalTable.appendRow( [ i18n.labelClassDebugField , credential.authClassName ] );
                }

                // Go through credential properties, looking for authentication type
                if ((TV.typeIsObject(typeof credential.properties)) && (credential.properties !== null)) {
                    for (var propertyName in credential.properties ) {
                        var propertyValue = credential.properties[ propertyName ];

                        if (propertyName === 'authtype') {
                            var isPKI = (propertyValue.indexOf('authType_pki') === 0);

                            if (isPKI) {
                                propertyValue = this.i18nAuthType( propertyValue );
                            }
                        }
                    }
                }
            }

            /*
             * Create "back" and "edit" buttons.
             */
            var backButton = ButtonFactory.buildButton( { type    : 'back',
                                                          grayout : true,
                                                          label   : i18n.buttonGoBack } );

            $(backButton).on('click', function () {
                ( new ShowCommand( {
                    action : 'scopesetsvmware/show',
                    parameters : { pid : sepid   }
                } ) ).execute();
            } );

            var editButton = ButtonFactory.buildButton( { type  : 'arrow',
                                                          label : i18n.buttonDiscoCredsEdit } );

            $(editButton).on('click', function () {
                tsa.cmd.handlers.CredentialsvmwareHandler.prototype.editCredentialClicked( credential.pid, sepid );
            } );

            $(buttonContainer).append(backButton);
            $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
            $(buttonContainer).append(editButton);

            contentArea.append( generalTable.getDomNode() );
            contentArea.append( buttonContainer );
        }, this ) );

        getCredentialCmd.execute();

        this.show( contentArea.getDomNode() );
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.CredentialsvmwareHandler.prototype._edit = function() {
        var i18n = tsa.i18n;

        var contentArea  = new ContentArea( { title   : i18n.labelDiscoVmwareEditCredsHeading,
                                              helpDoc : 'sshelp_discovery_vmware_credentials_edit_panel.html' } );
        var pid          = this.getParameter('pid');
        var credentialId = this.getParameter('entityId');

        // Check with TSA server if internal SSH key has been defined or not.  This impacts what options are available for
        // the Linux credentials "Authentication Type" field.
        //
        // To keep from having the code be indented too much, the bulk of the processing is done in the createEditScreen() function
        // immediately below.
        var getApplianceSshKeyCommand = new Command( { action              : 'credentials/ajaxCheckInternalSshKey',
                                                       clearSystemMessages : false } );

        $(getApplianceSshKeyCommand).on('executionSucceeded', $.proxy(function() {
            this.createEditScreen(contentArea, pid, credentialId, true);
        }, this ) );

        $(getApplianceSshKeyCommand).on('executionFailed', $.proxy(function() {
            this.createEditScreen(contentArea, pid, credentialId, false);
        }, this ) );

        getApplianceSshKeyCommand.execute();

        this.show(contentArea.getDomNode());
    };


    //
    // Main function the creates the content for the edit scope set screen.
    //
    // @param contentArea Main panel area where the edit scope set panes are added to.
    // @param pid
    // @param credentialId
    // @param isPkiDefined If true, indicates that the TSA appliance SSH key has been configured; otherwise false.
    //                     NOTE: By default, TSA is shipped with no TSA appliance SSH key.  The TSA administrator
    //                           has to create one in order for it to exist.
    //
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.createEditScreen = function(contentArea, pid, credentialId, isPkiDefined) {
        var i18n = tsa.i18n;

        var credentialForm  = null;
        var buttonContainer = ButtonFactory.buildButtonContainer();

        var getCredentialCommand = new Command( { action              : 'credentials/getCredential',
                                                  parameters          : { 'pid' : credentialId },
                                                  clearSystemMessages : false } );

        $(getCredentialCommand).on('executionSucceeded', $.proxy(function(event) {
            var credentialType = this.i18nCredentialType(event.response.payload.credential.name);
            var scopeSetName   = this.getDisplayableScopeSetName(event.response.payload.credential.scopeSetName);

            credentialForm = this.createCredentialForm( credentialType, scopeSetName, event.response.payload.credential, pid, isPkiDefined );

            if (credentialType === tsa.i18n.labelDiscoVmwareCredsVmwareType) {
                credentialForm.setActionAttribute( 'credentialsvmware/putVmwareCredential' );
            } else if (credentialType == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
                credentialForm.setActionAttribute( 'credentialsvmware/putLinuxCredential' );
            } else if (credentialType == tsa.i18n.labelDiscoVmwareCredsWindowsType) {
                credentialForm.setActionAttribute( 'credentialsvmware/putWindowsCredential' );
            } else {
                logger.warn("Invalid credential type: " + credentialType);
            }

            // If edit succeeds, print success message, then show the credentials list.
            $(credentialForm).on('executionSucceeded', function() {
                if (credentialForm.getHiddenInput("testCredential") === null) {
                    var credentialName = $(credentialForm.getDomNode()).find( '[name=\'name\']' ).val();
                    tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoVmwareCredsUpdated, credentialName.substring(0, credentialName.indexOf("_SCOPESET_")) ) } );

                    ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                         parameters          : { pid : pid },
                                         clearSystemMessages : false } ) ).execute();
                }
            } );

            // Appending content...
            contentArea.append(credentialForm.getDomNode());
            contentArea.append(buttonContainer);
        } , this ) );

        /*
         * Create "save" and "cancel" buttons.
         */
        var saveEditButton = ButtonFactory.buildButton( { type  : 'arrow',
                                                          label : i18n.buttonSave } );

        $(saveEditButton).on('click', function () {
            credentialForm.putHiddenInput('testCredential', null);
            credentialForm.submit();
        } );

        var cancelEditButton = ButtonFactory.buildButton( { type  : 'cancel',
                                                            label : i18n.buttonCancel} );

        $(cancelEditButton).on('click', function() {
            ( new ShowCommand( { action : 'scopesetsvmware/show',
                                 parameters : { pid : pid}
                               } ) ).execute();
        } );

        $(buttonContainer).append(saveEditButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
        $(buttonContainer).append(cancelEditButton);

        // Get the credential...
        getCredentialCommand.execute();
    };


    /**
     * Edit credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.editCredentialClicked = function(credentialPID , pid) {
        ( new ShowCommand( { action : 'credentialsvmware/edit/' + credentialPID + '/',
                             parameters : { pid : pid } } ) ).execute();
    };


    //-------------------------------------------------------------------------

    /*****************************************************************************/
    /* CREATE CREDENTIAL FORM:                                                   */
    /*                                                                           */
    /* Used for both adding a new credential or editing an existing credential.  */
    /*                                                                           */
    /* The credential panel consists of 3 panes:                                 */
    /*    - General                                                              */
    /*    - Enter Access Information                                             */
    /*    - Test Access Credentials                                              */
    /*                                                                           */
    /* Both add and edit panels have the same fields:                            */
    /*    - for add,  "Name" field is input                                      */
    /*    - for edit, "Name" field has human readable version of credential      */
    /*      name.  The value of "Name" cannot be changed.                        */
    /*                                                                           */
    /* @param type Indicates whether credential is VMware, Linux, or Windows.    */
    /* @param scopeSetName - Display name of the VMware Dynamic Scope Set.       */
    /* @param credential -                                                       */
    /* @param pid -                                                              */
    /* @param isPkiDefined -                                                     */
    /*****************************************************************************/

    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.createCredentialForm = function(type, scopeSetName, credential, pid, isPkiDefined) {
        var i18n = tsa.i18n;

        var isEditMode      = TV.typeIsObject(typeof credential) && (credential !== null);
        var generalForm     = null;
        var nameForm        = new FormPanel( { title       : i18n.labelNamePaneHeading,
                                               description : i18n.labelDiscoVmwareCredsNameDesc } );
        var testForm        = new FormPanel( { title       : i18n.labelDiscoCredsTestHeading,
                                               description : i18n.labelDiscoCredsTestDesc } );
        var accessInfoForm  = new FormPanel( { title      : i18n.labelDiscoCredsEnterAccessHeading,
                                               description: i18n.labelDiscoCredsHostAuthEnterDesc} );

        var linuxScopeSet    = null;
        var credentialOsName = null;

        /*
         * Populate Access Information pane.  Hidden fields for Windows are different from Linux / VMware.
         */
        accessInfoForm.putHiddenInput("RestrictionType", "limit");
        accessInfoForm.putHiddenInput("dynamicScopeSetName", scopeSetName);
        if (type == tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            credentialOsName = "_WINDOWS";

            accessInfoForm.putHiddenInput('ClassName', 'com.collation.platform.security.auth.WindowsHostAuth');
            accessInfoForm.putHiddenInput("CredentialType", "WindowsHostAuth");
            accessInfoForm.putHiddenInput("authtype", "authType_password");

        } else {
            accessInfoForm.putHiddenInput('ClassName', 'com.collation.platform.security.auth.HostAuth');
            accessInfoForm.putHiddenInput("CredentialType", "HostAuth");

            if (type === tsa.i18n.labelDiscoVmwareCredsVmwareType) {
                credentialOsName = "_VMWARE";

                accessInfoForm.putHiddenInput("authtype", "authType_password");
            } else {
                credentialOsName = "_LINUX";
            }
        }

        accessInfoForm.putHiddenInput("credentialOsName", credentialOsName);

        var firstForm = nameForm;

        nameForm.linkTo(accessInfoForm);
        accessInfoForm.linkTo(testForm);

        if (isEditMode) {
            nameForm.putHiddenInput("scopename", credential.scopeSet.pid);

            generalForm = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );
            generalForm.linkTo( nameForm );
            firstForm = generalForm;
        }

        // Fields (common):
        var nameField = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
        var typeSpan  = document.createElement('span');

        var testhostField = $( '<input type="text" id="ipAddress" name="ipAddress" size="50" maxlength="250" />' ).get(0);

        /*
         * Set-up test credential button based on the type of credential being tested (VMware, Linux, Windows).
         */
        var testButton = null;

        if (type === tsa.i18n.labelDiscoVmwareCredsVmwareType) {
            testButton = this.createTestButton(firstForm, 'credentialsvmware/ajaxTestVmwareCredential');

        } else if (type === tsa.i18n.labelDiscoVmwareCredsLinuxType) {
            testButton = this.createTestButton(firstForm, 'credentialsvmware/ajaxTestLinuxCredential');

        } else if (type === tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            testButton = this.createTestButton(firstForm, 'credentialsvmware/ajaxTestWindowsCredential');

        } else {
            /*
             * Should not occur.....
             */
            logger.warn("Invalid credential type: " + type);
        }

        var testButtonCon = ButtonFactory.buildButtonContainer();

        $(testButtonCon).append(testButton);

        $(typeSpan).text(type);

        nameForm.appendFieldRow( { label: i18n.labelDiscoCredsName, required: true, element: nameField } );
        nameForm.appendFieldRow( { label: i18n.labelDiscoCredsType,                 element: typeSpan } );

        testForm.appendFieldRow( { label: i18n.labelDiscoCredsTestIP, required: false , element: testhostField } );
        testForm.appendHR();
        testForm.appendJointRow(testButtonCon);

        var dynamicScopeSetNameSpan = document.createElement('span');
        $(dynamicScopeSetNameSpan).css( { 'font-weight' : 'bold' } );

        if (isEditMode) {
            var nameSpan = document.createElement('span');

            $(nameSpan).css( { 'font-weight' : 'bold' } );

            $(nameSpan).text(credential.name.substring(0, credential.name.indexOf("_SCOPESET_")));

            $(dynamicScopeSetNameSpan).text(scopeSetName);

            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralName, element: nameSpan } );
            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralType, element: typeSpan } );
            generalForm.appendFieldRow( { label: i18n.labelScopeSetField,         element: dynamicScopeSetNameSpan } );

            firstForm.putHiddenInput('pid' , credential.pid);
            firstForm.putHiddenInput('name', credential.name);

            $(nameField).val(credential.name);

            nameForm.hide();

        } else {
            $(dynamicScopeSetNameSpan).text(scopeSetName);
            nameForm.appendFieldRow( { label: i18n.labelScopeSetField, element: dynamicScopeSetNameSpan } );
        }


        /*
         * Fields for HostAuth:
         *
         * NOTE: Values for "Authentication type" pull-down is dependent upon whether a (global) TSA Server SSH Key has been defined.
         */
        var hostAuthUserName  = $( '<input type="text"     id="HostAuth_username"         name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        var hostAuthPassword1 = $( '<input type="password" id="HostAuth_password"         name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        var hostAuthPassword2 = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        var hostAuthAuthType = null;
        if (isPkiDefined) {
            hostAuthAuthType = $( '<select name="HostAuth_authtype" id="HostAuth_authtype" >' +
                                      '<option value="authType_default" selected="selected"' + '>' + this.i18nAuthType( 'authType_default'    ) + '</option>' +
                                      '<option value="authType_password"'                    + '>' + this.i18nAuthType( 'authType_password'   ) + '</option>' +
                                      '<option value="authType_pki_scoped"'                  + '>' + this.i18nAuthType( 'authType_pki_scoped' ) + '</option>' +
                                      '<option value="authType_pki"'                         + '>' + this.i18nAuthType( 'authType_pki'        ) + '</option>' +
                                  '</select>' ).get(0);
        } else {
            hostAuthAuthType = $( '<select name="HostAuth_authtype" id="HostAuth_authtype" >' +
                                      '<option value="authType_default" selected="selected"' + '>' + this.i18nAuthType( 'authType_default'    ) + '</option>' +
                                      '<option value="authType_password"'                    + '>' + this.i18nAuthType( 'authType_password'   ) + '</option>' +
                                      '<option value="authType_pki_scoped"'                  + '>' + this.i18nAuthType( 'authType_pki_scoped' ) + '</option>' +
                                  '</select>' ).get(0);
        }

        var hostAuthUploadKey     = $( '<input type="file" id="HostAuthLinux_sshKey" name="HostAuthLinux_sshKey" />' ).get(0);
        var hostAuthUploadKeyType = document.createElement('p');
        var hostAuthUploadKeyWarn = document.createElement('p');
        var hostAuthUploadKeyDiv  = document.createElement('div');

        // Initially hide PKI exists warning since on add credential the authentication type is initially set to "Default".
        $( hostAuthUploadKeyWarn ).empty();
        $( hostAuthUploadKeyWarn )[ 'hide' ]();

        if (type !== tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            /*
             * VMware and Linux credentials use HostAuth.
             */
            $(hostAuthUploadKeyType).text(i18n.labelDiscoCredsInternalSSHKey);

            $(hostAuthUploadKeyDiv ).css( { 'font-size'        : '1.33em' } );
            $(hostAuthUploadKeyWarn).css( { 'margin-top'       : '0.80em',
                                            'border'           : '1px solid #c0c000',
                                            'border-radius'    : '5px',
                                            'background-color' : '#ffffc0' } );

            $(hostAuthUploadKeyDiv).append(hostAuthUploadKeyType);
            $(hostAuthUploadKeyDiv).append(hostAuthUploadKey    );
            $(hostAuthUploadKeyDiv).append(hostAuthUploadKeyWarn);

            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsUser  , required: true, element: hostAuthUserName  } );
            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw1, required: true, element: hostAuthPassword1 } );
            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw2, required: true, element: hostAuthPassword2 } );

            if (type == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
                accessInfoForm.appendFieldRow( { label: i18n.labelCredentialsAuthenticationType, required: false , element: hostAuthAuthType     } );
                accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey             , required: true  , element: hostAuthUploadKeyDiv } );
            }
        }

        // Fields (Elements) for WindowsHostAuth:
        var windowsHostAuthUserName  = $( '<input type="text"     id="WindowsHostAuth_username" name="WindowsHostAuth_username"                 size="50" maxlength="250" />' ).get(0);
        var windowsHostAuthPassword1 = $( '<input type="password" id="WindowsHostAuth_password" name="WindowsHostAuth_password"                 size="50" maxlength="250" />' ).get(0);
        var windowsHostAuthPassword2 = $( '<input type="password" id="WindowsHostAuth_password_confirm" name="WindowsHostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        var windowsHostAuthAuthType  = $( '<select name="WindowsHostAuth_authtype" id="WindowsHostAuth_authtype">' +
        /* Windows PKI unsupported */ //      '<option value="authType_default" selected="selected"' + '>' + this.i18nAuthType( 'authType_default'    ) + '</option>' +
                                              '<option value="authType_password"'                    + '>' + this.i18nAuthType( 'authType_password'   ) + '</option>' +
        /* Windows PKI unsupported */ //      '<option value="authType_pki"'                         + '>' + this.i18nAuthType( 'authType_pki'        ) + '</option>' +
        /* Windows PKI unsupported */ //      '<option value="authType_pki_scoped"'                  + '>' + this.i18nAuthType( 'authType_pki_scoped' ) + '</option>' +
                                          '</select>' ).get(0);
        var windowsHostAuthUploadKey     = $( '<input type="file" id="WindowsHostAuth_sshKey" name="WindowsHostAuth_sshKey" />' ).get(0);

        var windowsHostAuthUploadKeyType = document.createElement('p');
        var windowsHostAuthUploadKeyWarn = document.createElement('p');
        var windowsHostAuthUploadKeyDiv  = document.createElement('div');

        if (type === tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            /*
             * Windows credentials use WindowsHostAuth.
             */
            $(windowsHostAuthUploadKeyType).text(i18n.labelDiscoCredsInternalSSHKey);

            $(windowsHostAuthUploadKeyDiv ).css( { 'font-size'        : '1.33em' } );
            $(windowsHostAuthUploadKeyWarn).css( { 'margin-top'       : '0.80em',
                                                   'border'           : '1px solid #c0c000',
                                                   'border-radius'    : '5px',
                                                   'background-color' : '#ffffc0' } );

            $(windowsHostAuthUploadKeyDiv).append(windowsHostAuthUploadKeyType);
            $(windowsHostAuthUploadKeyDiv).append(windowsHostAuthUploadKey    );
            $(windowsHostAuthUploadKeyDiv).append(windowsHostAuthUploadKeyWarn);

            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsUser  , required: true, element: windowsHostAuthUserName  } );
            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw1, required: true, element: windowsHostAuthPassword1 } );
            accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw2, required: true, element: windowsHostAuthPassword2 } );
        }


        /*
         * Label password fields for VMware or Windows.  Linux is handled separately via an event handler below.
         */
        var labelNode = null;
        if (type == tsa.i18n.labelDiscoVmwareCredsVmwareType) {
            labelNode  = $(hostAuthPassword1).closest('tr').find('label');
            labelNode.empty();
            labelNode.append(i18n.labelPasswordField, $("<span class='ibm-required'> *</span>").get(0));

            labelNode  = $(hostAuthPassword2).closest('tr').find('label');
            labelNode.empty();
            labelNode.append(i18n.labelConfirmPasswordField, $("<span class='ibm-required'> *</span>").get(0));

        } else if (type == tsa.i18n.labelDiscoVmwareCredsWindowsType) {
            labelNode  = $(windowsHostAuthPassword1).closest('tr').find('label');
            labelNode.empty();
            labelNode.append(i18n.labelPasswordField, $("<span class='ibm-required'> *</span>").get(0));

            labelNode  = $(windowsHostAuthPassword2).closest('tr').find('label');
            labelNode.empty();
            labelNode.append(i18n.labelConfirmPasswordField, $("<span class='ibm-required'> *</span>").get(0));
        }

        // Common value initialization for fields of HostAuth and WindowsHostAuth:
        if (isEditMode) {
            var isHostAuth = (credential.authClassSimpleName === 'HostAuth');

            var usernameField     = (isHostAuth ? hostAuthUserName : windowsHostAuthUserName);
            var authtypeField     = (isHostAuth ? hostAuthAuthType : windowsHostAuthAuthType);
            var scopeSet          = ((TV.typeIsObject( typeof credential.scopeSet)) ? credential.scopeSet : null);
            var authType          = credential.properties.authtype;
            var scopeSetHasSSHKey = ((scopeSet !== null ) && (scopeSet.hasSSHKey === true));

            var isPKI = false;
            if (type == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
                // Only Linux can have PKI.
                if ((credential.properties.authtype !== null) && (credential.properties.authtype.indexOf('authType_pki') === 0)) {
                    isPKI = true;
                }
            }

            if (isPKI) {
                authType += scopeSetHasSSHKey  ? '_scoped' : '' ;
            }

            $(usernameField).val(credential.properties.username);
            $(authtypeField).val(isHostAuth ? authType : /* Windows only supports: */ 'authType_password');
        }

        /*
         * Event handler - Linux Authorization Type has changed.
         *     - Display the right fields for the authorization type.
         *     - Check if the new or edited credential will cause a conflicting PKI credential.  TSA only allows one
         *       PKI credential per scope set, so the conflicting PKI must be identified.
         */
        var onLinuxAuthTypeChanged = function( ) {

            // boolean conditions
            var pkiGlobal           = ($(hostAuthAuthType).val() === 'authType_pki'       );
            var pkiScoped           = ($(hostAuthAuthType).val() === 'authType_pki_scoped');
            var isPKI               = (pkiGlobal || pkiScoped);
            var hideSSHKeyRow       = !isPKI;
            var hideSSHKeyUpload    = (hideSSHKeyRow || pkiGlobal);
            var disableSSHKeyUpload = !pkiScoped;
            var hideKeyTypeMessage  = pkiScoped;

            // panel elements
            var row                 = $(hostAuthUploadKey).closest('tr').get(0);

            $(row)                  [ hideSSHKeyRow      ? 'hide' : 'show' ]();
            $(hostAuthUploadKey)    [ hideSSHKeyUpload   ? 'hide' : 'show' ]();
            $(hostAuthUploadKeyType)[ hideKeyTypeMessage ? 'hide' : 'show' ]();

            $(hostAuthUploadKeyType).text(i18n.labelDiscoCredsInternalSSHKey);

            /*
             * Choose the right field label based on authentication type -- either "password" or "passphrase"
             */
            var labelValue = isPKI ? i18n.labelPassphraseField : i18n.labelPasswordField ;
            var labelNode  = $(hostAuthPassword1).closest('tr').find('label');
            labelNode.empty();
            if (!pkiScoped) {
                labelNode.append(labelValue,$("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }

            labelValue = isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField ;
            labelNode  = $(hostAuthPassword2).closest('tr').find('label');
            labelNode.empty();
            if (!pkiScoped) {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }

            /*
             * Set whether password / passphrase field is optional or required
             *    - passphrase is optional for PKI authentication type (scope set restricted).
             *    - password / passphrase is required for all other authentication types.
             */
            if (pkiScoped) {
                accessInfoForm.removeValidation($(hostAuthPassword1).attr('id'));
                accessInfoForm.removeValidation($(hostAuthPassword2).attr('id'));
            } else {
                accessInfoForm.addValidationRule($(hostAuthPassword1).attr('id'), 'required');
                accessInfoForm.addValidationRule($(hostAuthPassword2).attr('id'), 'required');
            }

            /*
             * Function to get list of PKI credentials for the Linux scope set.
             */
            var retrievePKICredentialsFor = function(scopeSetData) {
                // Array of credentials with PKI (should only be a max of 1).
                var pkiCredentials = [];

                if ((TV.typeIsObject(typeof scopeSetData)) && (scopeSetData !== null) &&
                        (TV.typeIsObject(typeof scopeSetData.credentials)) && (TV.isInstanceOf(scopeSetData.credentials, Array))) {
                    $.each(scopeSetData.credentials, function(credIndex, cred) {
                        if ((TV.typeIsObject(typeof cred)) && (cred !== null) &&
                                (TV.typeIsObject(typeof cred.properties)) && (cred.properties !== null) &&
                                (TV.typeIsString(typeof cred.properties.authtype)) &&
                                ((cred.properties.authtype === 'authType_pki_scoped') || (cred.properties.authtype === 'authType_pki'))) {
                            /*
                             * PKI credential.  Skip current credential when in edit mode.
                             */
                            if (isEditMode && credential.pid == cred.pid) {
                                logger.info("skipping pid: " + cred.pid);
                            } else {
                                pkiCredentials.push(cred);
                            }
                        }
                    } );
                }

                return pkiCredentials;
            };

            /*
             * Determine if a warning message for conflicting PKI credentials is required.  If a conflict is detected, adding
             * the new or edited credential will cause the conflicting credential to be deleted.
             */
            var isHostAuth      = !accessInfoForm.isHidden();
            var authSupportsPKI = isHostAuth;
            var warnMsgElement  = hostAuthUploadKeyWarn ;

            // Array of Linux PKI credentials
            var scopeSetPkiCreds = ((isPKI && (linuxScopeSet !== null)) ? retrievePKICredentialsFor(linuxScopeSet) : []);

            var scopeSetHasPkiCreds = ((scopeSetPkiCreds !== null) && (scopeSetPkiCreds.length > 0));

            var pkiCredsAsString = '';

            $(warnMsgElement).empty();
            $(warnMsgElement)[ scopeSetHasPkiCreds ? 'show' : 'hide' ]();

            if (authSupportsPKI) {
                if (scopeSetHasPkiCreds) {
                    var warnMsg = '';

                    $.each(scopeSetPkiCreds, function(credIndex, cred) {
                        var displayCredName = cred.name.substring(0, cred.name.indexOf("_SCOPESET_"));
                        pkiCredsAsString += ((credIndex > 0) ? ', ' : '') + displayCredName;
                    } );

                    var displayLinuxScopeSetName = linuxScopeSet.name.replace("_LINUX", "");

                    warnMsg = tsa.bind(i18n.labelDiscoCredsPKICredExists, [ pkiCredsAsString, displayLinuxScopeSetName ]);

                    $(warnMsgElement).append(IconFactory.buildSeverityIcon( { severity : 'WARN' } ), document.createTextNode('\xa0' + warnMsg));
                }

            }

            hostAuthUploadKey.disabled = disableSSHKeyUpload;
        };

        // Only care about authentication type changes for Linux credentials.
        if (type == tsa.i18n.labelDiscoVmwareCredsLinuxType) {
            $(hostAuthAuthType).on('change', onLinuxAuthTypeChanged);
            onLinuxAuthTypeChanged( null );
        }

        /*
         * Use the get Scope Sets action to find the scope set for this dynamic scope set.
         */
        var getScopeSetsCmd = new Command( { action     : 'scopesets/getScopeSets',
                                             parameters : { 'withCredentials' : 'true',
                                                            'filter'          : 'Vmware' },
                                             clearSystemMessages : false } );

        $(getScopeSetsCmd).on('executionSucceeded', $.proxy(function(event) {

            logger.info("execute succeeded for get scope sets command for Linux");

            var vmwareScopeSet = null;

            if (TV.typeIsObject(typeof event.response.payload.scopeSets)) {
                // Array of VMware Dynamic Scope Set related scope sets.
                var scopeSets = event.response.payload.scopeSets;

                if (TV.typeIsObject(typeof scopeSets) && TV.isInstanceOf(scopeSets, Array)) {
                    logger.info('Populating scope set selector (' + scopeSets.length + ' scope set(s))...');

                    /*
                     * Find the VMware scope set (which is the base scope set for a dynamic scope set) in order to get its name.
                     *
                     * NOTE: Even though JSHint thinks that "scopeSetIndex" is an unused variable, removing it causes this
                     *       function to not work as intended.
                     */
                    $.each(scopeSets, function(scopeSetIndex, scopeSet) {
                        if (pid !== null && pid == scopeSet.pid) {
                            vmwareScopeSet = scopeSet;

                            logger.info("vmware scope set: " + scopeSet.name);

                            return false;
                        }
                    });

                    /*
                     * Find the associated for the target device type (VMware, Linux, or Windows).
                     */
                    var matchingLinuxScopeSetName = vmwareScopeSet.displayName;
                    if ((credentialOsName === "_LINUX") || (credentialOsName === "_WINDOWS")) {
                        matchingLinuxScopeSetName = vmwareScopeSet.displayName + credentialOsName;
                    }

                    $.each(scopeSets, $.proxy(function(scopeSetIndex, scopeSet) {
                        if (TV.typeIsString(typeof scopeSet.displayName) &&
                                TV.typeIsString(typeof scopeSet.pid) &&
                                scopeSet.displayName === matchingLinuxScopeSetName ) {
                            logger.info('Matching scope set (' + 'index'       + ': ' + scopeSetIndex        + '; ' +
                                                                 'pid'         + ': ' + scopeSet.pid         + '; ' +
                                                                 'name'        + ': ' + scopeSet.name        + '; ' +
                                                                 'displayName' + ': ' + scopeSet.displayName + ').' );

                            nameForm.putHiddenInput("scopename", scopeSet.pid);
                            linuxScopeSet = scopeSet;

                            logger.info("Matching scope set: " + scopeSet.name);

                            return false;
                        }
                    }, this ) );
                }
            }
        } , this ) );

        // Retrieving Scope Sets from server...

        getScopeSetsCmd.execute();

        // Return the form...
        return firstForm;
    };


    /*
     * Set-up test credential button based on the type of credential being tested (VMware, Linux, Windows).
     *
     * @param firstForm Form that will contain the test credential button.
     * @param actionAttribute Action taken when test credential button is clicked.
     *
     * @return Test credential button.
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.createTestButton = function(firstForm, actionAttribute) {
        var i18n = tsa.i18n;
        var testButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonTestCredential } );

        $(testButton).on('click', function( ) {
            tsa.clearSysMessages();

            var action = firstForm.getActionAttribute();  // Save current value and restore after panel is submitted.

            // Add flag to let completion of request know that this was a test and not a save request
            firstForm.putHiddenInput('testCredential', 'true');

            firstForm.setActionAttribute(actionAttribute);
            firstForm.submit();
            firstForm.setActionAttribute(action);

            $(firstForm).on( 'executionFailed', function(event) {
                if (TV.typeIsObject(typeof event.response.payload.errors))  {
                    for (var fieldName in event.response.payload.errors) {
                        firstForm.showFieldError(fieldName, event.response.payload.errors[fieldName]);
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );

        return testButton;
    };



    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== GENERAL FUNCTIONS                                                      ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/


    /*
     * Convert internal representation of authentication type to displayable text version.
     *
     * @param authtype Internal authentication type representation.
     *
     * @return Displayable string.
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.i18nAuthType = function(authtype) {
        var i18nLabel = '';

        if (!TV.typeIsString(typeof authtype)) {
            authtype = '' + authtype;
        }

        switch( authtype ) {
            case 'authType_default'    : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypeDefault   ; break;
            case 'authType_password'   : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
            case 'authType_pki'        : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePki       ; break;
            case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
            default                    : i18nLabel = authtype;
        }

        return i18nLabel;
    };


    /*
     * Determine credential type (ie, VMware, Linux, or Windows) based on internal name of a scope set.
     *
     * @param credentialName Internal name of credential.
     *
     * @return Target type of credential (ie, VMware, Linux, or Windows).
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.i18nCredentialType = function ( credentialName ) {
        var i18nLabel = '';

        if (credentialName.endsWith("_VMWARE_VmwareCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoVmwareCredsVmwareType;
        } else if (credentialName.endsWith("_LINUX_VmwareCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoVmwareCredsLinuxType;
        } else if (credentialName.endsWith("_WINDOWS_VmwareCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoVmwareCredsWindowsType;
        }

        return i18nLabel;
    };


    /*
     * Convert hidden scope set name into displayable scope set name.
     *
     * @param hiddenScopeSetName Hidden scope set name.
     *
     * @return Displayable scope set name.
     */
    tsa.cmd.handlers.CredentialsvmwareHandler.prototype.getDisplayableScopeSetName = function ( hiddenScopeSetName ) {
        var displayScopeSetName = hiddenScopeSetName.replace("_LINUX_VmwareCrt_","");
        if (displayScopeSetName === hiddenScopeSetName) {
            displayScopeSetName = hiddenScopeSetName.replace("_WINDOWS_VmwareCrt_","");
            if (displayScopeSetName === hiddenScopeSetName) {
                displayScopeSetName = hiddenScopeSetName.replace("_VmwareCrt_","");
            }
        }

        return displayScopeSetName;
    }


    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.CredentialsvmwareHandler;
  }
);
