/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
    [ 'jquery'
    , 'tsa/TypeVerificator'
    , 'tsa/cmd/handlers/AbstractHandler'
    , 'tsa/widgets/ContentArea'
    , 'tsa/widgets/TablePanel'
    , 'tsa/widgets/ButtonFactory'
    , 'tsa/cmd/Command'
    , 'tsa/cmd/ShowCommand'
    ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ButtonFactory
          , Command
          , ShowCommand
          )
    {
      //-------------------------------------------------------------------------

      var logger = tsa.getLogger( 'DatasnapshotHandler' );
      var archiveFile = null;
      //-------------------------------------------------------------------------

      /** @constructor
       *  @extends tsa.cmd.AbstractHandler
       *  Parameters added by this class (in addition to super-class' parameters):
       *    none.
       *
       *  @param {Object} parameters The parameters. */

      tsa.cmd.handlers.DatasnapshotHandler =  function( parameters )
      {
        if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call( this, parameters );
      };

      //-------------------------------------------------------------------------

      // Inheriting:
      tsa.cmd.handlers.DatasnapshotHandler.prototype = new AbstractHandler( {} );
      /** @type {?} */ ( tsa.cmd.handlers.DatasnapshotHandler.prototype ).constructor = tsa.cmd.handlers.DatasnapshotHandler;

      //-------------------------------------------------------------------------

      /**
       * Show main screen.
       */
      tsa.cmd.handlers.DatasnapshotHandler.prototype._index = function()
      {
         /** @type {Object}                    */ var i18n                      = tsa.i18n;
         /** @type {tsa.widgets.ContentArea}   */ var contentArea               = null;
         /** @type {Element}                   */ var dataSnapShotButton        = null;
         /** @type {Element}                   */ var downloadCollectionButton  = null;
         /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer           = ButtonFactory.buildButtonContainer();
         /** @type {boolean}                   */ var downloadLast              = false;
         /** @type {boolean}                   */ var isTsaRegistered           = false;
         /** @type {string}                   */  var lastModified              = '';
         /** @type {tsa.widgets.TablePanel} */    var tablePanel                = null;

         /** @type {tsa.cmd.Command}           */ var getJobInfoCmd       = new Command( { action              : 'datasnapshot/getJobInfo'
        , clearSystemMessages : false } );

         
         // Initialize the content area (its title and its help document)...
         contentArea = new ContentArea ( { title   : i18n.labelDataSnapshotHeading
                                              , helpDoc : 'sshelp_datasnapshot_panel.html' } );
  
         // Appending messages to the content area...
         //Hyper Link to IBM TSA site
         var redirectLink = document.createElement("a");
         redirectLink.href = "../transmission/index";
         redirectLink.target = '_blank';
         redirectLink.innerHTML = "Transmission Schedule";

         var text = document.createTextNode(i18n.labelDataSnapshotCaptionStart);

         var redirectContent = document.createElement("p");
         redirectContent.appendChild(text);
         redirectContent.appendChild(redirectLink);
         text = document.createTextNode(i18n.labelDataSnapshotCaptionEnd);
         redirectContent.appendChild(text);

         contentArea.append(redirectContent);
         

         tablePanel = new TablePanel( { title    : i18n.labelLastDataSnapshotDateHeading
                        , summary  : 'DataSnapshot form'
                        , columns  : [ { id : '' } ] } );
      
  
         // Appending button to content area...
         dataSnapShotButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonDataSnapshot ,
                                                          onclick : function( event ){
           if ( isTsaRegistered )
             {
             ( new Command( { action     : 'datasnapshot/ajaxRunCollection'
                           , parameters : { 'tx' : 'false' } } ) ).execute();
             }
          else
          {
            if ( window.confirm(i18n.labelCollectionSchedIsTsaRegistered) ) 
	        {
	          ( new ShowCommand( { action              : 'registration/index'
	                           , clearSystemMessages : false } ) ).execute();
	        }
          }
         } 
        });
         // Appending button to content area...
         downloadCollectionButton = ButtonFactory.buildButton ({
            type : 'download'
               , grayout : false
               , label   : i18n.buttonCollectionSchedDownloadLast
               , onclick : function( event ){
                 var viewCmd = new Command({ action : 'datasnapshot/ajaxIsCollectionJobRunning', clearSystemMessages : false,  blockUI : false });
                 viewCmd.execute();
                 $( viewCmd ).on( 'executionSucceeded', function( event ){
                 var isJobRunning = event.response.payload.jobRunning;
                 if  ( isJobRunning )
                 {
                   if (downloadLast)
                   {
                     if ( window.confirm(tsa.bind(i18n.labellastDataSnapshotAlert,[lastModified]) ))
                     {
                       tsa.cmd.handlers.DatasnapshotHandler.downloadLastCollection();
                     }
                   } 
                 }
                 else
                 {
                   if ( window.confirm(tsa.bind(i18n.labellastDataSnapshotDownloadAlert,[lastModified]) ))
                   {
                     tsa.cmd.handlers.DatasnapshotHandler.downloadLastCollection();
                   }  
                 }
              });
            }
           });  
         // Performing the job information retrieval...
         getJobInfoCmd.execute();          
         // Content area must be shown...
         this.show( contentArea.getDomNode() );
      // Preparing the retrieval of the transmission job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object}  */ var jobInfo  = event.response.payload.jobInfo;
        /** @type {Object}  */ var schedule = jobInfo.schedule;
        /** @type {Object}  */ var history  = jobInfo.jobHistory;
        
        archiveFile     = event.response.payload.lastArchiveFile;
        downloadLast    = event.response.payload.downloadLast;
        isTsaRegistered = event.response.payload.isTsaRegistered;
        lastModified = event.response.payload.lastModified;
        if(downloadLast)
        {
          tablePanel.appendRow( [ tsa.bind(i18n.labelLastDataSnapshotDateMessage,[lastModified]) ] );
          $( downloadCollectionButton ).css( { 'opacity' : '1.0' } );
          $( downloadCollectionButton ).prop('disabled' , false  );
        }
        else
        {
          $( tablePanel.getDomNode() ).hide();
          $( downloadCollectionButton ).css( { 'opacity' : '0.5' } );
          $( downloadCollectionButton ).prop('disabled' , true  );
        }
        contentArea.append( tablePanel.getDomNode() ); 
        $( buttonContainer ).append( downloadCollectionButton );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( dataSnapShotButton );
        contentArea.append( buttonContainer );

      } );
      }
    //-------------------------------------------------------------------------

    /**
     * Request (for download) of last collection.
     */
    tsa.cmd.handlers.DatasnapshotHandler.downloadLastCollection = function()
    {
      /** @type {tsa.cmd.Command} */ var cmd = null;

      cmd = new Command( { action  : archiveFile
                         , baseURL : Command.URL_SERVLET_CONTEXT
                         , blockUI : false
                         , ajax    : false } );
      cmd.execute();
    };
    return tsa.cmd.handlers.DatasnapshotHandler;
    }
    
  );