/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DbtoolsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DbtoolsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DbtoolsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DbtoolsHandler.prototype ).constructor = tsa.cmd.handlers.DbtoolsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.DbtoolsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n         = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea  = new ContentArea ( { title   : i18n.labelDBToolsHeading
                                                                                  , helpDoc : 'sshelp_dbtools_panel.html' } );
      /** @type {tsa.widgets.FormPanel}   */ var recreateForm = new FormPanel( { title       : i18n.labelRebuildDBHeading
                                                                               , description : i18n.labelDBRebuildCaption
                                                                               , confirmMsg  : i18n.labelRebuildConfirm
                                                                               , action      : 'dbtools/ajaxRebuild' } );
      /** @type {tsa.widgets.FormPanel}   */ var runstatsForm = new FormPanel( { title       : i18n.labelRunstatsDBHeading
                                                                               , description : i18n.labelDBRunstatsCaption
                                                                               , confirmMsg  : i18n.labelRunstatsConfirm
                                                                               , action      : 'dbtools/ajaxRunstats' } );
      /** @type {tsa.widgets.FormPanel}   */ var reorgForm    = new FormPanel( { title       : i18n.labelReorgDBHeading
                                                                               , description : i18n.labelDBReorgCaption
                                                                               , confirmMsg  : i18n.labelReorgConfirm
                                                                               , action      : 'dbtools/ajaxReorg' } );
      
      /** @type {HTMLElement} */ var presCredCheckbox = document.createElement( 'input' );
      /** @type {HTMLElement} */ var presCredLabel    = document.createElement( 'label' );
      
      presCredCheckbox.type    = 'checkbox';
      presCredCheckbox.name    = 'preserveCredentials';
      presCredCheckbox.value   = 'PRESERVE_CREDENTIALS';
      presCredCheckbox.checked = true;
      
      $( presCredLabel ).text( i18n.labelPreserveCredentials );
      $( presCredLabel ).prepend( presCredCheckbox );
      
      recreateForm.appendJointRow( presCredLabel );
      recreateForm.appendHR();
      recreateForm.appendJointRow( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonlabelRebuildDB
                                                              , onclick : function( event ) { recreateForm.submit(); } } ) );
      
      runstatsForm.appendJointRow( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonlabelRunstats
                                                              , onclick : function( event ) { runstatsForm.submit(); } } ) );
      
      reorgForm.appendJointRow( ButtonFactory.buildButton( { type    : 'arrow'
                                                           , label   : i18n.buttonlabelReorg
                                                           , onclick : function( event ) { reorgForm.submit(); } } ) );
      
      contentArea.appendMessage( { text : i18n.labelDBToolsCaption } );
      $(recreateForm.form).attr("aria-label", i18n.labelRebuildDBHeading);
      $(runstatsForm.form).attr("aria-label", i18n.labelRunstatsDBHeading);
      $(reorgForm.form   ).attr("aria-label", i18n.labelReorgDBHeading);
      contentArea.append( recreateForm.getDomNode() );
      contentArea.append( runstatsForm.getDomNode() );
      contentArea.append( reorgForm.getDomNode() );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DbtoolsHandler;
  }
);