/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DevUITestsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DevUITestsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DevUITestsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DevUITestsHandler.prototype ).constructor = tsa.cmd.handlers.DevUITestsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show Development UI Tests index screen.
     */
    tsa.cmd.handlers.DevUITestsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea( { title : i18n.labelDevUITestHeading } );
      
      // ---------- Long Wait Form: ---------------------------------
      ( $.proxy( function()
      {
        /** @type {tsa.widgets.FormPanel} */ var longWaitForm = new FormPanel( { title  : i18n.labelDevUITestLongWaitHeading
                                                                               , action : 'dev/ajaxTestLongWait' } );
        /** @type {Element} */ var submitButton = ButtonFactory.buildButton( { type  : 'arrow'
                                                                             , label : i18n.buttonSubmit } );
        
        $( longWaitForm ).on( 'executionSucceeded', function( event )
        {
          window.alert( i18n.labelOK );
        } );
        
        $( longWaitForm ).on( 'executionFailed', function( event )
        {
          window.alert( i18n.labelError );
        } );
        
        $( submitButton ).on( 'click', function( event )
        {
          longWaitForm.submit();
        } );
        
        longWaitForm.appendHeaderRow( '', i18n.labelDevUITestLongWaitCaption );
        longWaitForm.appendFieldRow( { element: submitButton } );
        contentArea.append( longWaitForm.getDomNode() );
      }
      , this ) )();
      
      // ---------- Layout Form: ------------------------------------
      ( $.proxy( function()
      {
        /** @type {tsa.widgets.FormPanel} */ var layoutForm = new FormPanel( { title  : i18n.labelDevUITestLayoutHeading
                                                                             , action : 'dev/dummy' } );
        /** @type {Element} */ var normalButton = ButtonFactory.buildButton( { type  : 'arrow'
                                                                             , label : i18n.buttonDevUITestLayoutNormal } );
        /** @type {Element} */ var wideButton   = ButtonFactory.buildButton( { type  : 'arrow'
                                                                             , label : i18n.buttonDevUITestLayoutWide   } );
        /** @type {Element} */ var widerButton  = ButtonFactory.buildButton( { type  : 'arrow'
                                                                             , label : i18n.buttonDevUITestLayoutWider  } );
        /** @type {Element} */ var buttonContainer = ButtonFactory.buildButtonContainer();
        
        $( buttonContainer ).append( normalButton );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( wideButton );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( widerButton );
        
        $( normalButton ).on( 'click', function( event ) { tsa.setLayout( 'normal' ); } );
        $( wideButton   ).on( 'click', function( event ) { tsa.setLayout( 'wide'   ); } );
        $( widerButton  ).on( 'click', function( event ) { tsa.setLayout( 'wider'  ); } );
        
        layoutForm.appendHeaderRow( '', i18n.labelDevUITestLayoutCaption );
        layoutForm.appendFieldRow( { element: buttonContainer } );
        
        $( layoutForm.getDomNode() ).css( { 'margin-top' : '28px' } );
        
        contentArea.append( layoutForm.getDomNode() );
      }
      , this ) )();
      
      // ------------------------------------------------------------
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DevUITestsHandler;
  }
);