/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DevUpdateHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DevUpdateHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DevUpdateHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DevUpdateHandler.prototype ).constructor = tsa.cmd.handlers.DevUpdateHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show Development UI Tests index screen.
     */
    tsa.cmd.handlers.DevUpdateHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea( { title : 'Update: Technical Details' } );
      /** @type {tsa.widgets.FormPanel}   */ var form        = new FormPanel( { title  : 'Retrieve Update Technical Details'
                                                                              , action : 'dev/ajaxUpdateAnalysis' } );
      /** @type {Element}                 */ var runButton   = ButtonFactory.buildButton( { type  : 'arrow'
                                                                                          , label : i18n.buttonSubmit } );
      /** @type {Array.<tsa.widgets.TablePanel>} */ var tables = [];
      /** @type {Array.<Element>}                */ var msgs   = [];
      
      $( form ).on( 'executionSucceeded', function( event )
      {
        /** @type {tsa.cmd.AjaxResponse} */ var response   = ( ( ( event    !== null ) && TV.typeIsObject( typeof event.response     ) ) ? event.response     : null );
        /** @type {Object}               */ var payload    = ( ( ( response !== null ) && TV.typeIsObject( typeof response.payload   ) ) ? response.payload   : null );
        /** @type {Object}               */ var updateInfo = ( ( ( payload  !== null ) && TV.typeIsObject( typeof payload.updateInfo ) ) ? payload.updateInfo : null );
        /** @type {Object}               */ var currentVer = ( ( ( payload  !== null ) && TV.typeIsObject( typeof payload.currentVer ) ) ? payload.currentVer : null );
        
        /** @type {tsa.widgets.TablePanel} */ var settingsTable   = new TablePanel( { title: 'Update settings'  , columns: [ { width : '33%' }, { width : '67%' } ] } );
        /** @type {tsa.widgets.TablePanel} */ var currentVerTable = new TablePanel( { title: 'Current versions' , columns: [ { width : '33%' }, { width : '67%' } ] } );
        
        tables.push( settingsTable   );
        tables.push( currentVerTable );
        
        settingsTable.appendRow( [ 'Development mode' , ( TV.typeIsBoolean( typeof payload.devMode     ) ? payload.devMode     : '?' ) ] );
        
        if( currentVer != null )
        {
          /** @type {Array.<string>} */ var components = [];
          
          for( /** @type {string} */ var component in currentVer )
          {
            components.push( component );
          }
          
          components.sort();
          
          $.each( components, function( i, component )
          {
            currentVerTable.appendRow( [ component, currentVer[ component ] ] );
          } );
        }
        
        contentArea.append( settingsTable   .getDomNode() );
        contentArea.append( currentVerTable .getDomNode() );
        
        if( updateInfo != null )
        {
          for( /** @type {string} */ var key in updateInfo )
          {
            /** @type {Object}                 */ var info  = updateInfo[ key ];
            /** @type {tsa.widgets.TablePanel} */ var table = new TablePanel( { columns : [ { width : '33%' }
                                                                                          , { width : '67%' } ] } );
            tables.push( table );
            table.setTitle( 'Update Available: ' + key );
            
            if( TV.typeIsString( typeof info.version    ) && ( info.version    .length > 0 ) ) table.setSubtitle( "v." + info.version );
            if( TV.typeIsString( typeof info.dateTS     ) && ( info.dateTS     .length > 0 ) ) table.appendRow( [ 'Date:'        , info.dateTS          ] );
            if( TV.typeIsString( typeof info.id         ) && ( info.id         .length > 0 ) ) table.appendRow( [ 'ID:'          , info.id              ] );
            if( TV.typeIsString( typeof info.productKey ) && ( info.productKey .length > 0 ) ) table.appendRow( [ 'Product Key:' , info.productKey      ] );
            if( TV.typeIsNumber( typeof info.size       ) && ( info.size               > 0 ) ) table.appendRow( [ 'Size:'        , info.size + " bytes" ] );
            
            // if( TV.typeIsObject( typeof info.updateContent ) && ( info.updateContent !== null ) )
            // {
            //   /** @type {Element} */ var ucList       = document.createElement( 'ul' );
            //   /** @type {Element} */ var vrmListItem  = document.createElement( 'li' );
            //   /** @type {Element} */ var dateListItem = document.createElement( 'li' );
            //   /** @type {Element} */ var rgpListItem  = document.createElement( 'li' );
            //   
            //   $( vrmListItem  ).append( $( '<strong>VRM:  </strong>' ).get(0), document.createTextNode( info.updateContent.vrm    ) );
            //   $( dateListItem ).append( $( '<strong>Date: </strong>' ).get(0), document.createTextNode( info.updateContent.dateTS ) );
            //   
            //   if( TV.typeIsObject( typeof info.updateContent.readmeGroupDetails ) &&
            //       TV.isInstanceOf( info.updateContent.readmeGroupDetails, Array ) )
            //   {
            //     /** @type {Element} */ var rgdList = document.createElement( 'ul' );
            //     
            //     $.each( info.updateContent.readmeGroupDetails, function( rgdIndex, rgd )
            //     {
            //       /** @type {Element} */ var rgdListItem = document.createElement( 'li' );
            //       
            //       $( rgdListItem ).text( rgd );
            //       $( rgdList ).append( rgdListItem );
            //     } );
            //     
            //     $( rgpListItem  ).append( $( '<strong>Readme Group Details:</strong>' ).get(0), rgdList );
            //   }
            //   
            //   $( ucList ).append( vrmListItem, dateListItem, rgpListItem );
            //   table.appendRow( [ 'Update Content:', ucList ] );
            // }
            
            contentArea.append( table.getDomNode() );
          }
        }
      } );
      
      $( form ).on( 'executionFailed', function( event )
      {
        window.alert( i18n.labelError );
      } );
      
      $( runButton ).on( 'click', function( event )
      {
        if( tables !== null )
        {
          $.each( tables, function( tableIndex, table )
          {
            $( table.getDomNode() ).remove();
          } );
          
          tables = [];
        }
        
        if( msgs !== null )
        {
          $.each( msgs, function( msgIndex, msg )
          {
            $( msg ).remove();
          } );
          
          msgs = [];
        }
        
        form.submit();
      } );
      
      form.appendHeaderRow( '', 'Click on the button below to retrieve the current Update Technical Details. '
                              + 'The server will return a list of components that need to be updated, as well as some settings. '
                              + 'This operation could take some minutes to perform.' );
      form.appendFieldRow( { element: runButton } );
      contentArea.append( form.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DevUpdateHandler;
  }
);