/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TabsPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , TablePanel
          , IconFactory
          , TabsPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DevWidgetsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DevWidgetsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DevWidgetsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DevWidgetsHandler.prototype ).constructor = tsa.cmd.handlers.DevWidgetsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show development description screen.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelDevWidgetsHeading } );
      /** @type {Array.<string>}          */ var widgetNames = [ 'ContentArea', 'TablePanel', 'FormPanel', 'TabsPanel', 'IconFactory', 'ButtonFactory' ];
      
      /** @type {tsa.widgets.TablePanel} */ var tablePanel =
        new TablePanel( { title    : 'TSA Widgets'
                        , summary  : 'TSA Widgets'
                        , columns  : [ { label : 'Class'       , width : '30%' }
                                     , { label : 'Description' , width : '70%' } ] } );
      
      contentArea.appendMessage( { text : i18n.labelDevWidgetsCaption } );
      contentArea.append( tablePanel.getDomNode() );
      
      $.each( widgetNames, $.proxy( function( widgetIndex, widgetName )
      {
        /** @type {Element} */ var link     = document.createElement( 'a' );
        /** @type {Element} */ var descText = '';
        
        $( link ).attr( 'href', '#' );
        $( link ).text( widgetName );
        
        switch( widgetName )
        {
          case 'ContentArea'   : descText = 'The root container element of any screen.'; break;
          case 'TablePanel'    : descText = 'General purpose table. Can retrieve data from the server using AJAX.'; break;
          case 'FormPanel'     : descText = 'Panel containing a form. Its fields\' labels and inputs are organized using a table layout.'; break;
          case 'TabsPanel'     : descText = 'Multiple tab panel. A single tab is shown at a time.'; break;
          case 'IconFactory'   : descText = 'Convenience methods for building icons.'; break;
          case 'ButtonFactory' : descText = 'Convenience methods for building buttons.'; break;
        }
        
        tablePanel.appendRow( [ link, descText ] );
        
        $( link ).on( 'click', $.proxy( function( event )
        {
          /** @type {string} */ var methodName = 'show' + widgetName;
          
          if( !TV.typeIsUndefined( typeof this[ methodName ] ) )
          {
            try
            {
              this[ methodName ]();
            }
            catch( error )
            {
              logger.error( error );
              logger.error( 'Unexpected error occurred during method execution (' + methodName + ').' );
            }
          }
          else
          {
            logger.error( 'Not implemented (' + methodName + ').' );
          }
          
          try { event.stopPropagation(); } catch( error ) { /* Ignore. */ }
          
          return false;
        }
        , this ) );
      }
      , this ) );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: ContentArea.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showContentArea = function()
    {
      /** @type {Element}                 */ var paragraph = document.createElement( 'p' );
      /** @type {tsa.widgets.ContentArea} */ var contentArea =
        new ContentArea( { title         : 'This is the title'
                         , showBackToTop : true } );
      
      contentArea.appendMessage( { level : 'HELP'  , text : 'This is a "HELP" message.'          } );
      contentArea.appendMessage( { level : 'INFO'  , text : 'This is an "INFO" message.'         } );
      contentArea.appendMessage( { level : 'WARN'  , text : 'This is a "WARN" message.'          } );
      contentArea.appendMessage( { level : 'ERROR' , text : 'This is an "ERROR" message.'        } );
      contentArea.appendMessage( {                   text : 'This is a message having no level.' } );
      
      contentArea.appendMessage( { level : 'HELP'  , border : true , text : 'This is a "HELP" message, with border.'          } );
      contentArea.appendMessage( { level : 'INFO'  , border : true , text : 'This is an "INFO" message, with border.'         } );
      contentArea.appendMessage( { level : 'WARN'  , border : true , text : 'This is a "WARN" message, with border.'          } );
      contentArea.appendMessage( { level : 'ERROR' , border : true , text : 'This is an "ERROR" message, with border.'        } );
      contentArea.appendMessage( {                   border : true , text : 'This is a message having no level, with border.' } );
      
      $( paragraph ).css( { 'border'           : '1px solid #c0c0c0'
                          , 'border-radius'    : '4px'
                          , 'padding'          : '4px'
                          , 'background-color' : '#e0e0e0' } );
      $( paragraph ).text( 'This is a custom HTML Element.' );
      contentArea.append( paragraph );
      
      // Side-bar accordion panel:
      /** @type {Element} */ var firstParagraph  = document.createElement( 'p' );
      /** @type {Element} */ var secondParagraph = document.createElement( 'p' );
      /** @type {Element} */ var thirdParagraph  = document.createElement( 'p' );
      /** @type {Date}    */ var now             = new Date();
      
      $( firstParagraph  ).text( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed'
                               + ' do eiusmod tempor incididunt ut labore et dolore magna aliqua.'
                               + ' Ut enim ad minim veniam, quis nostrud exercitation ullamc'
                               + ' laboris nisi ut aliquip ex ea commodo consequat.' );
      $( secondParagraph ).text( 'Duis aute irure dolor in reprehenderit in voluptate velit esse'
                               + ' cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat'
                               + ' cupidatat non proident, sunt in culpa qui officia deserunt'
                               + ' mollit anim id est laborum.' );
      $( thirdParagraph  ).text( 'Time: ' + ( ( now.getHours()   < 10 ) ? '0' : '' ) + now.getHours()   + ':'
                               +            ( ( now.getMinutes() < 10 ) ? '0' : '' ) + now.getMinutes() + ':'
                               +            ( ( now.getSeconds() < 10 ) ? '0' : '' ) + now.getSeconds() );
      
      contentArea.appendSidebarPanel( { title: 'Testing', body : $( '<p>This is a Sidebar Panel.</p>' ).get( 0 ) } );
      
      contentArea.appendSidebarAccordionPanel( [ { title : 'Accordion (First)'  , body : firstParagraph  }
                                               , { title : 'Accordion (Second)' , body : secondParagraph }
                                               , { title : 'Accordion (Third)'  , body : thirdParagraph  } ] );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: TablePanel.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showTablePanel = function()
    {
      /** @type {tsa.widgets.ContentArea} */ var contentArea =
        new ContentArea( { title : 'Demo: TablePanel' } );
      /** @type {tsa.widgets.TablePanel} */ var tablePanelA =
        new TablePanel( { title    : 'Countries ranked by surface area'
                        , subtitle : 'Year: 2015 (non-AJAX version)'
                        , summary  : 'Countries ranked by surface area (area in sq. Km)'
                        , columns  : [ { id : 'country' , label : 'Country'       , width : '60%' }
                                     , { id : 'area'    , label : 'Area (sq. Km)' , width : '40%' } ] } );
      /** @type {tsa.widgets.TablePanel} */ var tablePanelB =
        new TablePanel( { title        : 'Countries ranked by surface area'
                        , subtitle     : 'Year: 2015 (AJAX version)'
                        , summary      : 'Countries ranked by surface area (area in sq. Km; AJAX version)'
                        , ajaxAction   : 'dev/getCountryAreas'
                        , entryHandler : function( data ) { return [ data.country, data.area ]; }
                        , pageSize     : 10
                        , columns      : [ { id : 'country' , label : 'Country'       , width : '60%' , initSort : true }
                                         , { id : 'area'    , label : 'Area (sq. Km)' , width : '40%' } ] } );
      
      // Populating the non-AJAX version:
      tablePanelA.appendRow( [ 'Rusia'           , 17098250 ] );
      tablePanelA.appendRow( [ 'Canada'          ,  9984670 ] );
      tablePanelA.appendRow( [ 'United States'   ,  9831510 ] );
      tablePanelA.appendRow( [ 'China'           ,  9562911 ] );
      tablePanelA.appendRow( [ 'Brazil'          ,  8515770 ] );
      tablePanelA.appendRow( [ 'Australia'       ,  7741220 ] );
      tablePanelA.appendRow( [ 'India'           ,  3287260 ] );
      tablePanelA.appendRow( [ 'Argentina'       ,  2780400 ] );
      tablePanelA.appendRow( [ 'Kazakhstan'      ,  2724902 ] );
      tablePanelA.appendRow( [ 'Algeria'         ,  2381740 ] );
      tablePanelA.appendRow( [ 'Dem. Rep. Congo' ,  2344860 ] );
      tablePanelA.appendRow( [ 'Saudi Arabia'    ,  2149690 ] );
      tablePanelA.appendRow( [ 'Mexico'          ,  1964380 ] );
      tablePanelA.appendRow( [ 'Indonesia'       ,  1910930 ] );
      tablePanelA.appendRow( [ 'Sudan'           ,  1879358 ] );
      
      contentArea.appendMessage( { level: 'INFO', border: true, text: 'The following table is AJAX driven.' } );
      contentArea.append( tablePanelB.getDomNode() );
      contentArea.appendMessage( { level: 'INFO', border: true, text: 'The following table is static (it doesn\'t use AJAX).' } );
      contentArea.append( tablePanelA.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: TabsPanel.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showTabsPanel = function()
    {
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea( { title : 'Demo: TabsPanel' } );
      /** @type {tsa.widgets.TablePanel}  */ var tabsPanel   = new TabsPanel( { tabs: [ { title : 'TAB A', body : $( '<p>BODY A</p>' ).get(0) }
                                                                                      , { title : 'TAB B', body : $( '<p>BODY B</p>' ).get(0) } ] } );
      /** @type {tsa.widgets.FormPanel}   */ var formPanel   = new FormPanel( { title    : 'Sum a couple numbers...'
                                                                              , action   : 'dev/ajaxMath' } );
      /** @type {Element} */ var bodyForTabA      = document.createElement( 'p' );
      /** @type {Element} */ var bodyForTabB      = document.createElement( 'p' );
      /** @type {Element} */ var bodyForTabC      = document.createElement( 'p' );
      /** @type {Element} */ var elAdditionResult = document.createElement( 'span' );
      
      $( bodyForTabA ).text( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed'
                           + ' do eiusmod tempor incididunt ut labore et dolore magna aliqua.'
                           + ' Ut enim ad minim veniam, quis nostrud exercitation ullamc'
                           + ' laboris nisi ut aliquip ex ea commodo consequat.' );
      $( bodyForTabB ).text( 'Duis aute irure dolor in reprehenderit in voluptate velit esse'
                           + ' cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat'
                           + ' cupidatat non proident, sunt in culpa qui officia deserunt'
                           + ' mollit anim id est laborum.' );
      $( bodyForTabC ).text( 'Use the following form to sum numbers.' );
      
      formPanel.appendFieldRow( { label    : 'Operand 1'
                                , required : true
                                , element  : $( '<input type="text" size="8" name="op1" value="0" />' ).get(0)
                                , hint     : 'The first operand' } );
      formPanel.appendFieldRow( { label    : 'Operand 2'
                                , required : true
                                , element  : $( '<input type="text" size="8" name="op2" value="0" />' ).get(0)
                                , hint     : 'The second operand' } );
      formPanel.appendHR();
      formPanel.appendFieldRow( { label: 'Addition result' , element: elAdditionResult } );
      formPanel.appendHR();
      formPanel.appendFieldRow( { label   : ''
                                , element : ButtonFactory.buildButton( { type: 'arrow', label: 'Submit', onclick: function( event ) { formPanel.submit(); } } ) } );
      formPanel.hide();
      
      $( formPanel ).on( 'executionSucceeded', function( event ) { $( elAdditionResult ).text( event.response.payload.additionResult ); } );
      $( formPanel ).on( 'executionFailed'   , function( event ) { $( elAdditionResult ).text( 'ERROR' ); } );
      
      tabsPanel = new TabsPanel( { tabs: [ { title : 'Lorem ipsum' , body : bodyForTabA }
                                         , { title : 'Duis aute'   , body : bodyForTabB }
                                         , { title : 'Sum numbers' , body : bodyForTabC } ] } );
      
      $( tabsPanel ).on( 'tabChanged', function( event )
      {
        formPanel[ ( event.tabIndex === 2 ) ? 'show' : 'hide' ]();
      } );
      
      contentArea.appendMessage( { border: false, text: 'This is a demonstration of the TabsPanel Widget.' } );
      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( formPanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: FormPanel.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showFormPanel = function()
    {
      /** @type {tsa.widgets.ContentArea} */ var contentArea =
        new ContentArea( { title : 'Demo: FormPanel' } );
      /** @type {tsa.widgets.FormPanel} */ var formPanel =
        new FormPanel( { title    : 'Arithmetic operations'
                       , subtitle : 'AJAX driven'
                       , action   : 'dev/ajaxMath' } );
      
      /** @type {Element} */ var elAdditionResult       = document.createElement( 'span' );
      /** @type {Element} */ var elSubtractionResult    = document.createElement( 'span' );
      /** @type {Element} */ var elMultiplicationResult = document.createElement( 'span' );
      /** @type {Element} */ var elDivisionResult       = document.createElement( 'span' );
      
      formPanel.appendHeaderRow( 'Enter the operands', 'They will be sent to server, from which the results will be retrieved.' );
      formPanel.appendFieldRow( { label    : 'Operand 1'
                                , required : true
                                , element  : $( '<input type="text" size="8" name="op1" value="0" />' ).get(0)
                                , hint     : 'The first operand' } );
      formPanel.appendFieldRow( { label    : 'Operand 2'
                                , required : true
                                , element  : $( '<input type="text" size="8" name="op2" value="0" />' ).get(0)
                                , hint     : 'The second operand' } );
      formPanel.appendHR();
      formPanel.appendFieldRow( { label: 'Addition result'       , element: elAdditionResult       } );
      formPanel.appendFieldRow( { label: 'Subtraction result'    , element: elSubtractionResult    } );
      formPanel.appendFieldRow( { label: 'Multiplication result' , element: elMultiplicationResult } );
      formPanel.appendFieldRow( { label: 'Division result'       , element: elDivisionResult       } );
      
      formPanel.appendButton( { type    : 'arrow'
                              , label   : 'Submit'
                              , onclick : function( event )
                                          {
                                            formPanel.submit();
                                          }
                              } );
      
      $( formPanel ).on( 'executionSucceeded', function( event )
      {
        $( elAdditionResult       ).text( event.response.payload.additionResult       );
        $( elSubtractionResult    ).text( event.response.payload.subtractionResult    );
        $( elMultiplicationResult ).text( event.response.payload.multiplicationResult );
        $( elDivisionResult       ).text( event.response.payload.divisionResult       );
      } );
      
      $( formPanel ).on( 'executionFailed', function( event )
      {
        $( elAdditionResult       ).text( '-' );
        $( elSubtractionResult    ).text( '-' );
        $( elMultiplicationResult ).text( '-' );
        $( elDivisionResult       ).text( '-' );
        
        alert( 'Execution failed. Please check operands, they must be integers.' );
      } );
      
      contentArea.append( formPanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: IconFactory.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showIconFactory = function()
    {
      /** @type {tsa.widgets.ContentArea} */ var contentArea =
        new ContentArea( { title : 'Demo: IconFactory' } );
      /** @type {tsa.widgets.TablePanel} */ var tablePanel =
        new TablePanel( { title    : 'Icons'
                        , subtitle : 'Severity'
                        , summary  : 'Icon Factory Demo (Severity icons)'
                        , columns  : [ { label : 'Icon only'  , width : '50%' }
                                     , { label : 'With label' , width : '50%' } ] } );
      
      /** @type {Array.<string>} */ var severities = [ 'OK', 'INFO', 'WARN', 'ERROR', 'CANC' ];
      
      $.each( severities, function( sevIndex, severity )
      {
        tablePanel.appendRow( [ IconFactory.buildSeverityIcon( { severity : severity , withText : false } )
                              , IconFactory.buildSeverityIcon( { severity : severity , withText : true  } ) ] );
      } );
      
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show demo for widget: ButtonFactory.
     */
    tsa.cmd.handlers.DevWidgetsHandler.prototype.showButtonFactory = function()
    {
      /** @type {tsa.widgets.ContentArea} */ var contentArea =
        new ContentArea( { title : 'Demo: ButtonFactory' } );
      /** @type {tsa.widgets.TablePanel} */ var tablePanel =
        new TablePanel( { title    : 'Buttons'
                        , summary  : 'Button Factory Demo'
                        , columns  : [ { label : 'Standard'   , width : '50%' }
                                     , { label : 'Grayed-out' , width : '50%' } ] } );
      
      /** @type {Array.<string>} */ var types = [ 'cart'     , 'arrow'    , 'cancel'   , 'customize' ,
                                                  'download' , 'view'     , 'add'      , 'back'      ,
                                                  'phone'    , 'chat'     , 'email'    , 'refresh'   ,
                                                  'remove'   , 'undo'     , 'video'    , 'pda'       ];
      
      $.each( types, function( typeIndex, type )
      {
        tablePanel.appendRow( [ ButtonFactory.buildButton( { type : type , grayout : false , label : type } )
                              , ButtonFactory.buildButton( { type : type , grayout : true  , label : type } ) ] );
      } );
      
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DevWidgetsHandler;
  }
);