/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DiscorunsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DiscorunsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DiscorunsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DiscorunsHandler.prototype ).constructor = tsa.cmd.handlers.DiscorunsHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Show summary screen.
     */
    tsa.cmd.handlers.DiscorunsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n           = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea    = null;
      /** @type {tsa.widgets.TablePanel}  */ var mainTablePanel = null;
      /** @type {Element}                 */ var contentAreaMsg = null;
      
      contentArea = new ContentArea ( { title         : i18n.labelDiscoRunsHeading
                                      , helpDoc       : 'sshelp_discovery_runs_panel.html'
                                      , showBackToTop : true } );
      
      /*
       * Process each one of the JSON entries from server.
       * Must return an array containing strings or elements for an HTML Table Row.
       */
      var entryHandler = function( entry )
      {
        /** @type {Array.<(string|Element)>} */ var result          = null;
        /** @type {string}                   */ var strForStatus    = '';
        /** @type {Element}                  */ var elForName       = document.createElement( 'a' );
        /** @type {string}                   */ var strForStartTime = '';
        /** @type {string}                   */ var strForEndTime   = '';
        /** @type {Element}                  */ var elForActions    = document.createElement( 'div' );
        /** @type {Element}                  */ var iconForDelete   = null;
        /** @type {Element}                  */ var iconForDownload = null;
        
        iconForDelete   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + ': ' + entry.displayName } );
        iconForDownload = IconFactory.buildActionIcon( { type : 'download' , caption : i18n.labelDiscoRunsDownloadDiagnosticsLogFile } );
        
        $( iconForDelete   ).css( { 'margin-right' : '0.5em', 'cursor' : 'pointer' } );
        $( iconForDownload ).css( { 'margin-right' : '0.5em', 'cursor' : 'pointer' } );
        
        if( !entry.complete )
        {
          $( iconForDelete   ).css( { 'display' : 'none' } );
          $( iconForDownload ).css( { 'display' : 'none' } );
        }
        else if( entry.profileName === 'Level 1 Discovery' )
        {
          $( iconForDownload ).css( { 'display' : 'none' } );
        }
        
        $( iconForDelete ).on( 'click', function( event )
        {
          tsa.clearSysMessages();
          
          if( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + entry.displayName + '"?' ) )
          {
            /** @type {tsa.cmd.Command} */ var cmd = new Command( { action              : 'discoruns/deleteEntry'
                                                                  , parameters          : { 'id' : entry.pid }
                                                                  , printSystemMessages : false } );
            
            $( cmd ).on( 'executionSucceeded', function( event )
            {
              /** @type {tsa.cmd.ShowCommand} */ var showCmd = new ShowCommand( { action              : 'discoruns/index'
                                                                                , clearSystemMessages : false } );
              
              tsa.printSysMessage( { level   : 'info'
                                   , message : tsa.bind( tsa.i18n.labelDiscoRunsDeleted, entry.displayName ) } );
              
              showCmd.execute();
            } );
            
            $( cmd ).on( 'executionFailed', function( event )
            {
              tsa.printSysMessage( { level   : 'error'
                                   , message : tsa.bind( tsa.i18n.labelDiscoRunsNotDeleted, entry.displayName ) } );
            } );
            
            cmd.execute();
          }
        } );
        
        $( iconForDownload ).on( 'click', function( event )
        {
          tsa.clearSysMessages();
          
          window.location = tsa.cmd.Command.URL_SERVLET_CONTEXT
                          + 'discoruns/download/'
                          + entry.pid + '_' + entry.runId + '_diagnostics.log';
        } );
        
        strForStatus    = ( TV.typeIsString( typeof entry.statusLabel ) ? entry.statusLabel : '' );
        strForStartTime = ( TV.typeIsString( typeof entry.startTS     ) ? entry.startTS     : '' );
        strForEndTime   = ( TV.typeIsString( typeof entry.endTS       ) ? entry.endTS       : '' );
        
        // Status could need to be converted from "${i18n.label}" to the actual label...
        if( ( strForStatus.indexOf( '${i18n.' ) === 0 ) &&
            ( strForStatus.lastIndexOf( '}' ) === ( strForStatus.length - 1 ) ) )
        {
          strForStatus = tsa.i18n[ strForStatus.substring( 7, strForStatus.length - 1 ) ];
        }
        
        $( elForName ).attr( 'href', '#' );
        $( elForName ).text( TV.typeIsString( typeof entry.displayName ) ? entry.displayName : '' );
        $( elForName ).addClass( 'ibm-feature-link' );
        $( elForName ).on( 'click', function( event )
        {
          /** @type {Element} */ var elementForEntry      = tsa.cmd.handlers.DiscorunsHandler.buildDetailsFor( entry );
          /** @type {Element} */ var entryButtonContainer = document.createElement( 'p' );
          /** @type {Element} */ var backButton           = ButtonFactory.buildButton(
            { type    : 'back'
            , grayout : true
            , label   : tsa.i18n.buttonGoBack
            , onclick : function( event )
                        {
                          $( elementForEntry ).remove();
                          $( entryButtonContainer ).remove();
                          $( mainTablePanel.getDomNode() ).show();
                          $( contentAreaMsg ).show();
                          contentArea.setTitle( i18n.labelDiscoRunsHeading );
                        }
            }
          );
          
          tsa.clearSysMessages();
          $( mainTablePanel.getDomNode() ).hide();
          $( contentAreaMsg ).hide();
          contentArea.setTitle( i18n.labelDiscoRunEntryHeading );
          $( entryButtonContainer ).append( backButton );
          contentArea.append( elementForEntry );
          contentArea.append( entryButtonContainer );
          
          try
          {
            event.stopPropagation();
          } catch ( error ) { /* Ignore */ }
          
          return false;
        } );
        
        $( elForActions ).append( iconForDelete   );
        $( elForActions ).append( iconForDownload );
        
        result = [ strForStatus, elForName, strForStartTime, strForEndTime, elForActions ];
        
        return result;
      };
      
      contentAreaMsg = contentArea.appendMessage( { text : i18n.labelDiscoRunsCaption } );
      
      mainTablePanel  = new TablePanel  ( { title        : i18n.labelDiscoRunsEntriesHeading
                                          , summary      : i18n.labelDiscoRunsSummary
                                          , emptyMsg     : i18n.labelDiscoRunsNoEntries
                                          , ajaxAction   : 'discoruns/getEntries'
                                          , entryHandler : entryHandler
                                          , pageSize     : 100
                                          , columns      : [ { id       : 'status'
                                                             , label    : i18n.labelDiscoRunsStatus
                                                             , width    : '15%'
                                                             , sortable : false }
                                                           , { id       : 'name'
                                                             , label    : i18n.labelDiscoRunsRunName
                                                             , width    : '25%'
                                                             , sortable : false }
                                                           , { id       : 'start_time'
                                                             , label    : i18n.labelDiscoRunsStartTime
                                                             , width    : '25%'
                                                             , sortable : false }
                                                           , { id       : 'end_time'
                                                             , label    : i18n.labelDiscoRunsEndTime
                                                             , width    : '25%'
                                                             , sortable : false }
                                                           , { id       : 'action'
                                                             , label    : i18n.labelActionsColumn
                                                             , width    : '10%'
                                                             , sortable : false } ] } );
      
      contentArea.append( mainTablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * @param  {Object}  entry Discovery History data entry.
     * @return {Element}       HTML Element containing details for a particular discovery
     *                         history entry.
     */
    tsa.cmd.handlers.DiscorunsHandler.buildDetailsFor = function( entry )
    {
      /** @type {Object}                */ var i18n       = tsa.i18n;
      /** @type {Element}               */ var container  = document.createElement( 'div' );
      /** @type {tsa.widgets.FormPanel} */ var infoPanel  = null;
      /** @type {tsa.widgets.FormPanel} */ var scopePanel = null;
      
      infoPanel  = new FormPanel( { title   : i18n.labelDiscoRunsInfoHeading   } );
      scopePanel = new FormPanel( { title   : i18n.labelDiscoRunsScopesHeading } );
      
      var asSpan = function( text )
      {
        return $( '<span />' ).text( text ).get(0);
      }
      
      if( tsa.isDebugModeOn() )
      {
        infoPanel.appendFieldRow( { label   : i18n.labelDiscoRunsPID
                                  , element : asSpan( entry.pid ) } );
      }
      
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsRunId           , element : asSpan( entry.runId           ) } );
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsRunType         , element : asSpan( entry.runType         ) } );
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsStatus          , element : asSpan( entry.status          ) } );
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsTerminationType , element : asSpan( entry.terminationType ) } );
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsServerId        , element : asSpan( entry.serverId        ) } );
      infoPanel.appendFieldRow( { label : i18n.labelDiscoRunsProfileName     , element : asSpan( entry.profileName     ) } );
      
      $( container ).append( infoPanel.getDomNode() );
      
      // Scopes are not sent by server, and seemingly extracting scope info for each discovery is expensive...
      // scopePanel.appendJointRow( asSpan( entry.scopes.join( ', ' ) ) );
      // $( container ).append( scopePanel.getDomNode() );
      
      return container;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DiscorunsHandler;
  }
);