/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DiscoveryHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DiscoveryHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DiscoveryHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DiscoveryHandler.prototype ).constructor = tsa.cmd.handlers.DiscoveryHandler;
    
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.DiscoveryHandler.prototype.scopeSets= null;
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.DiscoveryHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n               = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea        = null;
      /** @type {tsa.widgets.TablePanel}    */ var scheduleTablePanel = null;
      /** @type {tsa.widgets.TablePanel}    */ var historyTablePanel  = null;
      /** @type {Element}                   */ var editScheduleButton = null;
      /** @type {Element}                   */ var runDiscoveryButton = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer    = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd      = new Command( { action              : 'discovery/getJobInfo'
                                                                                     , clearSystemMessages : false } );
      /** @type {Object}                    */ var scopeSetList  = [];
      scopeSetList.size = 0;
      
      /** @type {tsa.cmd.Command} */ var getScopesetsCmd = new Command( {  action               : 'scopesets/getScopeSets'
                                                                          , parameters          : { 'filter' : 'ALL' }
                                                                          , clearSystemMessages : false });
      
      
      // make wide the layout
      tsa.setLayout("wide"); 
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelDiscoSchedHeading
                                      , helpDoc : 'sshelp_discovery_schedule_panel.html' } );
      
      // Appending messages to the content area...
      contentArea.appendMessage( { level : 'INFO'  , border : true , text : i18n.labelDiscoSchedInfo } );
      
      
      // Building the schedule information table...
      scheduleTablePanel = new TablePanel( { title    : i18n.labelDiscoSchedScheduleHeading
                                           , emptyMsg : i18n.labelDiscoSchedStatusDisabled
                                           , summary  : "Summary for Discovery Schedule"
                                           , columns  : [ { id : 'name'    , label: i18n.labelDiscoSchedColumnName , width : '20%' }
                                                        , { id : 'nextRun' , label: i18n.labelDiscoSchedNextRun    , width : '20%' } 
                                                        , { id : 'runsAt'  , label: i18n.labelDiscoSchedRunsAt     , width : '45%' } 
                                                        , { id : 'Actions' , label: i18n.labelActionsColumn        , width : '15%' } ] } );
      
      // Create Expand all and collapse all actions
      /** @type {Element} */ var expandIcon    = IconFactory.buildActionIcon( { type : 'expand'     , caption : i18n.labelDiscoSchedExpandAll } );
      
      /** @type {Element} */ var collapseIcon  = IconFactory.buildActionIcon( { type : 'collapse'        , caption : i18n.labelDiscoSchedCollapseAll} );
      /** @type {Element} */ var divInfoAction = document.createElement("div");
      /** @type {Element} */ var aRef    = document.createElement("a");
      /** @type {function} */
      var toogleScopeSets = function(message, imageSRC) {
        $(".scheduleListAction").attr("src", tsa.attributes.imagesPath + "/" + imageSRC);
        $(".scheduleListAction").attr("alt", message);
        $(".scheduleListAction").attr("title", message);
      }
      
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( expandIcon );
      $( aRef ).append( i18n.labelDiscoSchedExpandAllScopes);
      $( divInfoAction ).append( aRef );
      $( divInfoAction ).append(tsa.NBSP);
      $(aRef).on('click',function(){
        $( '.scopeSets' ).show();
        toogleScopeSets(i18n.labelDiscoSchedCollapse,"expand16.png");
      });
      aRef    = document.createElement("a");
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( collapseIcon );
      $( aRef ).append( i18n.labelDiscoSchedCollapseAllSchedules);
      $( divInfoAction ).append( aRef );
      $( divInfoAction ).append(tsa.NBSP);
      $(aRef).on('click',function(){
        $( '.scopeSets' ).hide();
        toogleScopeSets(i18n.labelDiscoSchedExpand,"expandb16.png");
      });
      
      $( divInfoAction ).css( { 'float' : 'right' } );
      contentArea.append(divInfoAction);
      
      //here add a div
      $(scheduleTablePanel.headerCaption).append(divInfoAction);
      
      
      // Building the history table...
      historyTablePanel = new TablePanel( { title    : i18n.labelDiscoSchedHistoryHeading
                                          , emptyMsg : i18n.labelDiscoSchedNoJobHistory
                                          , summary  : "Summary for Discovery Schedule"
                                          , columns  : [ { id: 'status'   , label: i18n.labelDiscoSchedStatus   , width: '10%' }
                                                       , { id: 'schedule' , label: i18n.labelDiscoSchedName     , width: '15%' }
                                                       , { id: 'name'     , label: i18n.labelDiscoSchedInstance , width: '20%' }
                                                       , { id: 'state'    , label: i18n.labelDiscoSchedState    , width: '15%' }
                                                       , { id: 'comments' , label: i18n.labelDiscoSchedComments , width: '40%' } ] } );
      
      // Preparing the retrieval of the discovery job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', $.proxy(function( event )
      {
        /** @type {Object} */ var jobInfo      = event.response.payload.jobInfo;
        /** @type {Object} */ var scheduleList = jobInfo.scheduleList;
        /** @type {String} */ var scheduleId   = event.response.payload.scheduleId;
        
        /** @type {Object} */ var history  = jobInfo.jobHistory;
        // Populating schedule table (if available; if not, the empty-message will be shown)...
        if( scheduleList !== null )
        {
          //First fill the full discovery
          $.each( scheduleList, $.proxy(function( entryIndex, schedule ) 
          { 
             if (schedule.scheduleId == "fulldiscovery") 
             {
              
              //first check if we have Scope Sets
              var count = 0;
              for (key in scopeSetList) 
              {
                  if (key == 'size') 
                  {
                    continue;
                  }
                  if( !TV.typeIsUndefined( typeof scopeSetList[key] ) )
                  {
                    count++;
                  }
              }
              
              
              //if we have, then procedd
              if(count>0)
              { 
                var row = this.addScheduleRow(schedule, scheduleTablePanel,true);
                for (key in scopeSetList) 
                {
                    if (key == 'size') 
                    {
                      continue;
                    }
                    if( !TV.typeIsUndefined( typeof scopeSetList[key] ) )
                    {
                      this.createScopeSetRow(scheduleTablePanel,scopeSetList[key],schedule.scheduleId);
                      count++;
                    }
                }
             } 
             else
             {
               //create row without collapse image
               var row = this.addScheduleRow(schedule, scheduleTablePanel,false);
               
             }
              return false;
            }  
          }, this));

          //fill any other discovery schedule
          $.each( scheduleList, $.proxy(function( entryIndex, schedule ) 
          {
            if (schedule.scheduleId != "fulldiscovery") 
            {
              //look for the total of schedules
              var count = 0;
              if ( scopeSetList.size>0 ) 
              {
                $.each( schedule.scopeSets.split(","), $.proxy(function( entryIndex, entry ) 
                {
                  if( !TV.typeIsUndefined( typeof scopeSetList[entry] ) )
                  {
                    count++;
                  }
                }, this));
              }
              
              if (count > 0) 
              {
                this.addScheduleRow(schedule, scheduleTablePanel, true);
                $.each( schedule.scopeSets.split(","), $.proxy(function( entryIndex, entry ) {
                      if( !TV.typeIsUndefined( typeof scopeSetList[entry] ) )
                      {
                        this.createScopeSetRow(scheduleTablePanel, scopeSetList[entry], schedule.scheduleId);
                        count++;
                      }
                }, this));
              } else {
                this.addScheduleRow(schedule, scheduleTablePanel, false);
              }
              
            }
           }, this));
          
          
          
          
         }
        // Populating history table (if available; if not, the empty-message will be shown)...
        if( history !== null )
        {
          $.each( history, function( entryIndex, entry )
          {
            /** @type {Element} */ var commentList = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get(0);
            
            $.each( entry.comments, function( commentIndex, comment )
            {
              /** @type {Element} */ var commentElement = $( '<li style="padding-top: 0px;"></li>' ).get(0);
              
              $( commentElement ).text( comment );
              $( commentList ).append( commentElement );
            } );
            
            historyTablePanel.appendRow( [ IconFactory.buildSeverityIcon( { severity : entry.statusCode } )
                                         , entry.name
                                         , entry.startTS
                                         , entry.stateText
                                         , commentList ] );
          } );
        }
       
      }, this));
      // Buttons...
      editScheduleButton = ButtonFactory.buildButton( { type : 'add'   , grayout : true , label : i18n.buttonDiscoSchedAddDiscoSchedule } );
      runDiscoveryButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : true  , label : i18n.buttonDiscoSchedRunNow       } );
      
      $( editScheduleButton ).on( 'click', function( event )
      {
        ( new ShowCommand( { action : 'discovery/new' } ) ).execute();
      } );
      
      $( runDiscoveryButton ).on( 'click', function( event )
      {
        ( new Command( { action     : 'discovery/ajaxRunFullDiscovery'
                       , parameters : { 'tx' : 'false' } } ) ).execute();
      } );
      
      $( buttonContainer ).append( editScheduleButton );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( runDiscoveryButton );
      
      // Appending content...
      contentArea.append( scheduleTablePanel.getDomNode() );
      contentArea.append( buttonContainer );
      contentArea.append( historyTablePanel.getDomNode() );
      
      getScopesetsCmd.execute();
      
      $(getScopesetsCmd).on("executionSucceeded",function(event){
     // Performing the job information retrieval...
        getJobInfoCmd.execute();
        if( !TV.typeIsUndefined( typeof event.response.payload.scopeSets ) )
        {
          $.each(event.response.payload.scopeSets, function(idxScopeset, element){
            scopeSetList[element.pid]=element.displayName;
            scopeSetList.size+=1;
          });
        }
      }); 
      
      
      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Edit screen.
     */
    tsa.cmd.handlers.DiscoveryHandler.prototype._new = function()
    { 
      this._edit();
    };
    /**
     * Show edit screen.
     */
    tsa.cmd.handlers.DiscoveryHandler.prototype._edit = function()
    {
      /** @type {Object}                    */ var i18n                = tsa.i18n;
      /** @type {string}                    */ var scheduleId          = this.getParameter('scheduleId');
      /** @type {string}                    */ var scheduleName        = this.getParameter('scheduleName');
      /** @type {tsa.widgets.ContentArea}   */ var contentArea         = null;
      /** @type {tsa.widgets.FormPanel}     */ var discoveryScheduleFormPanel = null;
      /** @type {tsa.widgets.FormPanel}     */ var scheduleFormPanel   = null;
      /** @type {Element}                   */ var hourSelector        = document.createElement( 'select' );
      /** @type {Element}                   */ var minuteSelector      = document.createElement( 'select' );
      /** @type {Element}                   */ var weekModeRadio       = document.createElement( 'input' );
      /** @type {Element}                   */ var monthModeRadio      = document.createElement( 'input' );
      /** @type {Array.<Element>}           */ var weekDayCheckboxes   = [];
      /** @type {Element}                   */ var weekDaysRow         = null;
      /** @type {Array.<Element>}           */ var monthDayCheckboxes  = [];
      /** @type {Element}                   */ var monthDaysRow        = null;
      /** @type {Element}                   */ var saveButton          = null;
      /** @type {Element}                   */ var cancelButton        = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer     = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd       = new Command( { action              : 'discovery/getJobInfo'
                                                                                     , clearSystemMessages : false 
                                                                                     , parameters : { scheduleId : scheduleId}} );
      /** @type {tsa.cmd.Command}           */ var getUnassignedCmd    = new Command( { action              : 'discovery/getUnassignedScopesets'
                                                                                      , clearSystemMessages : false } );
      
       $( hourSelector       ).attr("id", "hour"      ); 
       $( minuteSelector     ).attr("id", "minute"    );
       $( weekModeRadio      ).attr("id", "weekMode"  );
       $( monthModeRadio     ).attr("id", "monthMode" );
      // Initializing content area...
      /** @type {string}*/ var title           = null;
      /** @type {string}*/ var scheduleCaption = null;
      /** @type {string}*/ var sshelp          = null;
      
      if(scheduleId == null)
      {
        sshelp          = 'sshelp_add_discovery_schedule_panel.html';
        title           = i18n.labelDiscoSchedHeadingAdd;
        scheduleCaption = i18n.labelDiscoSchedHeadingAddCaption;
      } 
      else 
      {
        sshelp          = 'sshelp_edit_discovery_schedule_panel.html'
        title           = i18n.labelDiscoSchedHeadingEdit;
        scheduleCaption = i18n.labelDiscoSchedHeadingEditCaption;
      }
        
        //TODO CHANGE THE HELP PAGE
      contentArea = new ContentArea( { title   : title
                                     , helpDoc : sshelp } );
      
      
      
      // Initializing form panels...
      discoveryScheduleFormPanel = new FormPanel( { title       : i18n.labelDiscoSchedHeading
                                           , description : scheduleCaption
                                           , action      : 'discovery/putJobSchedule' } );
      if (scheduleId != null){
        discoveryScheduleFormPanel.putHiddenInput("scheduleId", scheduleId);
      }
      discoveryScheduleFormPanel.putHiddenInput("enabled", true);
      scheduleFormPanel = new FormPanel( { title       : i18n.labelDiscoSchedScheduleHeading
                                         , description : i18n.labelDiscoSchedScheduleCaption } );
      
      $( discoveryScheduleFormPanel ).on( 'executionSucceeded', function( event )
      {
        tsa.printSysMessage( { level: 'info', message: i18n.disco_job_updated } );
        
        ( new ShowCommand( { action              : 'discovery/index'
                           , clearSystemMessages : false} ) ).execute();
      } );
      
      // Multiple form panels, but submitted as a single form...
      discoveryScheduleFormPanel.linkTo( scheduleFormPanel );

      //add list of scopesets
      discoveryScheduleFormPanel.appendFieldRow( { label    : i18n.labelDiscoSchedName
                                          , required : true
                                          , element  : $('<input size="50" name="scheduleName" id="scheduleName" value="" type="text">').get(0) } );
      
      var scopeSetGroup = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);
      /** @type {tsa.cmd.Command} */ var getScopesetsCmd = new Command( {  action               : 'scopesets/getScopeSets'
                                                                          , parameters          : { 'filter' : 'ALL' }
                                                                          , clearSystemMessages : false });
      getScopesetsCmd.execute();
      $(getScopesetsCmd).on("executionSucceeded",function(event)
      {
        if( !TV.typeIsUndefined( typeof event.response.payload.scopeSets ) )
        { 
          $.each(event.response.payload.scopeSets, function(idxScopeset, element)
          {
            var scopesetId    = "scopeset" + idxScopeset;
            var scopesetInput = $("<input name='scopesets' class='checkBoxClass' type='checkbox'>").get(0);
            $(scopesetInput).attr("id", scopesetId);
            $(scopesetInput).attr("value", element.pid);
            var scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
            $(scopeLabel).text(element.displayName);
            $(scopeLabel).attr("for", scopesetId);
            $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
          });
          if( scheduleId !== "fulldiscovery") 
          {
            var unassignedGroup       = $("<span class='ibm-input-group' role='group' aria-label='UnassignedScopes'></span>").get(0);
            var unassignedScopeInput  = $("<input id='unassignedScopeset' type='checkbox'>").get(0);
            var unassignedScopesLabel = document.createElement("label");
            $(unassignedScopesLabel).text(i18n.labelDiscoSchedUnassignedScopeset);
            $(unassignedScopesLabel).attr("for", "unassignedScopeset");
            $(unassignedGroup).append(unassignedScopeInput,unassignedScopesLabel);
            //add list of scopesets
            discoveryScheduleFormPanel.appendFieldRow( { label    : i18n.labelDiscoSchedScopesets
                                                , required : false
                                                , aRequire : false
                                                , element  : unassignedGroup } );
            discoveryScheduleFormPanel.appendFieldRow( {  label    : i18n.labelDiscoSchedSelectScopesets
                                                        , required : true
                                                        , aRequire : false
                                                        , element  : scopeSetGroup } );
            //create button check all and uncheck all if is not full discovery
            var buttonContainerSchedule  = ButtonFactory.buildButtonContainer();
            var checkAllButton           = ButtonFactory.buildButton( { type : 'add'    , grayout : true , label : i18n.labelDiscoSchedSelectAll   , onclick : function( event ) {  $('.checkBoxClass').prop('checked', true); } } );
            var uncheckAllButton         = ButtonFactory.buildButton( { type : 'remove' , grayout : true , label : i18n.labelDiscoSchedDeselectAll , onclick : function( event ) {  $('.checkBoxClass').prop('checked', false); } } );
            $( buttonContainerSchedule ).append( checkAllButton );
            $( buttonContainerSchedule ).append( ButtonFactory.buildButtonSeparator() );
            $( buttonContainerSchedule ).append( uncheckAllButton );
            var row = discoveryScheduleFormPanel.layoutTableBody.insertRow(-1);
            var td = document.createElement("td");
            $(td).attr("colspan", 2);
            $(td).append(buttonContainerSchedule);
            $(row).append(td);
          } 
          else 
          {
            //  set Scope Set and schedules for Full Discovery.
            discoveryScheduleFormPanel.putHiddenInput("scheduleName", scheduleName);
            discoveryScheduleFormPanel.putHiddenInput("scopesets", scheduleName);
          }
          //execute getJobInfo after we create the the scopesets, if scheduleName exists, that means that we are editing and not just creating 
          if( scheduleId != null){
            getJobInfoCmd.execute();
          }
         
          
          $( getUnassignedCmd ).on( 'executionSucceeded', function( event )
          {
            var unassignedScopeSets = [];
            var count = event.response.payload.unassigned.length;
            $.each( event.response.payload.unassigned, function( idxScopeSet, scopeSet )
              {
                 unassignedScopeSets[scopeSet] = scopeSet;
                 
              });
            $("#unassignedScopeset").change(function()
            {
              if(this.checked) 
              {
                if(count>0)
                {
                  $(".checkBoxClass").each( function( idxScopeSet, check )
                  {
                    if(TV.typeIsUndefined( typeof unassignedScopeSets[$(check).val()]))
                    {
                      //hide nodes
                      $(check).hide();
                      $(check).next().hide();
                      $(check).next().next().hide();
                    }
                  });
                }
                else
                {
                  alert(i18n.labelDiscoSchedUnassignedScopesetAlert);
                }
              }
              else
              {
                $(".checkBoxClass").show();
                $(".checkBoxLabelClass").show();
                $("br").show();
                
              }
            });
            
          }); 
         //retrieve unassigned scopeSets after creating the list of scope sets
          getUnassignedCmd.execute(); 
        } else {
          if( scheduleId !== "fulldiscovery" ) 
          {
            alert(i18n.labelDiscoSchedNoScopeSets);
            ( new ShowCommand( { action : 'discovery/index' } ) ).execute();  
          }
          else
          {
            // set Scope Set and schedule name for Full Discovery.
            discoveryScheduleFormPanel.putHiddenInput("scheduleName", scheduleName);
            discoveryScheduleFormPanel.putHiddenInput("scopesets", scheduleName);
           // get job info for full discovery
            getJobInfoCmd.execute();
          }
        }
      });
      
      // Hour selector...
      hourSelector.name = 'hour';
      
      for( /** @type {number} */ var hour=0; hour<24; hour++)
      {
        $( hourSelector ).append( $( '<option value="' + hour + '">' + ( ( hour < 10 ) ? '0' : '' ) + hour + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtHour
                                        , required : true
                                        , element  : hourSelector } );
      
      // Minute selector...
      minuteSelector.name = 'minute';
      
      for( /** @type {number} */ var minute=0; minute<60; minute+=5)
      {
        $( minuteSelector ).append( $( '<option value="' + minute + '">' + ( ( minute < 10 ) ? '0' : '' ) + minute + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtMinute
                                        , required : true
                                        , element  : minuteSelector } );
      
      // Day mode radio buttons...
      weekModeRadio.type    = 'radio';
      weekModeRadio.name    = 'daymode';
      weekModeRadio.value   = 'week';
      weekModeRadio.checked = true;
      monthModeRadio.type   = 'radio';
      monthModeRadio.name   = 'daymode';
      monthModeRadio.value  = 'month';
      
      scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleDayMode
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span       = document.createElement( 'span' );
                         /** @type {Element} */ var weekLabel  = document.createElement( 'label' );
                         /** @type {Element} */ var monthLabel = document.createElement( 'label' );
                         
                         $( weekLabel  ).append( weekModeRadio  , document.createTextNode( i18n.scheduleDayModeWeek  ) );
                         $( monthLabel ).append( monthModeRadio , document.createTextNode( i18n.scheduleDayModeMonth ) );
                         $( span       ).append( weekLabel, document.createElement( 'br' ), monthLabel );
                         $( span       ).addClass( 'ibm-input-group' );
                         $( span       ).attr( 'role', 'radiogroup' );
                         $( span       ).attr( 'aria-label', i18n.scheduleDayMode );
                         
                         return span;
                       } )()
      } );
      
      // Days of week checkboxes...
      weekDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnDays
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span = document.createElement( 'span' );
                         
                         $( span ).addClass( 'ibm-input-group' );
                         $( span ).attr( 'role', 'group' );
                         $( span ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         $.each( [ 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ], function( wdIndex, wd )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="weekday" value="' + ( wdIndex + 1 ) + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label>' + i18n[ 'schedule' + wd.charAt(0).toUpperCase() + wd.substring(1) ] + '</label>' ).get(0);
                           
                           $( label ).prepend( checkbox );
                           weekDayCheckboxes.push( checkbox );
                           $( span ).append( label, document.createElement( 'br' ) );
                         } );
                         
                         return span;
                       } )()
      } );
      
      // Days of month checkboxes...
      monthDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnMonthDays
        , hint     : i18n.scheduleOnMonthDaysDesc
        , required : true
        , element  : ( function()
                       {
                         /** @type {Element} */ var div = document.createElement( 'div' );
                         
                         $( div ).addClass( 'ibm-input-group' );
                         $( div ).attr( 'role', 'group' );
                         $( div ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         for( /** @type {number} */ var md=1; md<=31; md++ )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="monthday" value="' + md + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label style="display: inline-block; font-weight: normal; margin-right: 0.8em">'
                                                                  + ( ( md < 10 ) ? '0' : '' ) + md
                                                                  + '</label>' ).get(0);
                           $( label ).prepend( checkbox );
                           monthDayCheckboxes.push( checkbox );
                           $( div ).append( label );
                           
                           if( ( md % 7 ) === 0 )
                           {
                             $( div ).append( document.createElement( 'br' ) );
                           }
                         };
                         
                         return div;
                       } )()
      } );
      
      // Event handler: day mode changed...
      /** @type {function(Event)} */
      var dayModeChanged = function( event )
      {
        logger.info( 'Day mode changed: processing started...' );
        
        $( weekDaysRow  ).css( { 'display' : ( weekModeRadio  .checked ? '' : 'none' ) } );
        $( monthDaysRow ).css( { 'display' : ( monthModeRadio .checked ? '' : 'none' ) } );
        
        // Selecting a mode will enable all of the checkboxes of the other mode...
        if ( weekModeRadio  .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', true ); } );
        if ( monthModeRadio .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', true ); } );
        
        // If the user triggered the event (event is an object, and it is not null), clear the check-boxes of the mode he selected...
        if( TV.typeIsObject( typeof event ) && ( event !== null ) )
        {
          if( weekModeRadio  .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', false ); } );
          if( monthModeRadio .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', false ); } );
        }
        
        logger.info( 'Day mode changed: processing finished.' );
      };
      
     
      
              
      $( weekModeRadio  ).add( monthModeRadio ).on( 'change', dayModeChanged );
      
      $(weekModeRadio).trigger("change");
      
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
            if( scheduleId === "fulldiscovery") 
            {
              $("#scheduleName").prop('disabled', true);
            }
            // saving old name
            discoveryScheduleFormPanel.putHiddenInput("oldScheduleName", scheduleName);
            discoveryScheduleFormPanel.putHiddenInput("scheduleId", scheduleId);
            /** @type {Object} */ var schedule   = event.response.payload.jobInfo;
            /** @type {Object} */ var scopeSets = event.response.payload.pids;
            if(!TV.typeIsUndefined( typeof schedule.enabled))
            {
              discoveryScheduleFormPanel.putHiddenInput("enabled", schedule.enabled);  
            }
            // Populating schedule table (if available; if not, the empty-message will be shown)...
            $( '[name="scheduleName"]' ).val( scheduleName );
            
            
            $( hourSelector       ).val( schedule.hour   );
            $( minuteSelector     ).val( schedule.minute );
            
            weekModeRadio  .checked = ( schedule.dayOfWeek.length !== 7);
            monthModeRadio .checked = ( schedule.dayOfWeek.length === 7);
            
            $(weekModeRadio).trigger("change"); 
            
            $.each( weekDayCheckboxes, function( checkboxIndex, checkbox )
            {
              checkbox.checked = ( schedule.dayOfWeek.indexOf( checkboxIndex + 1 ) >= 0 );
            } );
            
            $.each( monthDayCheckboxes, function( checkboxIndex, checkbox )
            {
              checkbox.checked = ( schedule.dayOfMonth.indexOf( parseInt( $( checkbox ).val(), 10 ) ) >= 0 );
            } );
            //populate Scopesets checkbox
            if(scheduleId !== "fulldiscovery")
            {
              $.each( scopeSets, function( scopeSetIndex, scopeSet )
                  {
                    $('input:checkbox[value='+scopeSet.pid+']').attr("checked", true);
                  } );
              
            }
            
            
          } );
      
      
      // Buttons...
      saveButton   = ButtonFactory.buildButton( { type : 'arrow'  , label : i18n.buttonSave   } );
      cancelButton = ButtonFactory.buildButton( { type : 'cancel' , label : i18n.buttonCancel } );
      
      $( saveButton ).on( 'click', function( event )
      {
        discoveryScheduleFormPanel.submit();
      } );
      
      $( cancelButton ).on( 'click', function( event )
      {
        ( new ShowCommand( { action : 'discovery/index' } ) ).execute();
      } );
      
      $( buttonContainer ).append( saveButton );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( cancelButton );
      
      // Appending content...
      contentArea.append( discoveryScheduleFormPanel.getDomNode() );
      contentArea.append( scheduleFormPanel.getDomNode() );
      contentArea.append( buttonContainer );
      
      
      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.DiscoveryHandler.prototype.createScopeSetRow = function( tablePanel, scopeSetName,scopeSetId)
    { 
      var rowScopesets = tablePanel.tableBody.insertRow(-1);
      $(rowScopesets).addClass("scopeSets")
      $(rowScopesets).attr("name", scopeSetId)
      var td = document.createElement("td");
      $(td).attr('colspan', 3 );
      $(td).append(scopeSetName);
      $(rowScopesets).append(document.createElement("td"), td);
      $(rowScopesets).hide();
      
    };
  //-------------------------------------------------------------------------
    tsa.cmd.handlers.DiscoveryHandler.prototype.addScheduleRow = function(schedule, tablePanel, createCollapse){
      i18n = tsa.i18n;
      /**@type function() */
      var clickDisplayHandler = function ( event ) 
      {
        (new ShowCommand( {
            action : event.data.URL
          , clearSystemMessages : false 
          , parameters          : { scheduleName : event.data.scheduleName
                                   , scheduleId   : event.data.scheduleId
                                    } } )  
         ).execute();
      }
      /**@type function() */
      var clickTriggerHandler = function ( event ) 
      {
        tsa.clearSysMessages();
        var proceed = true;
        var scopeSets = "";
        var parameters = {};
        if(!TV.typeIsUndefined( typeof event.data.confirm ))
        {
          proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.scheduleName + "' ?")
        }
        if(!TV.typeIsUndefined( typeof event.data.scopeSets ))
        {
         parameters.scopeSets = event.data.scopeSets;
        }
        if(!TV.typeIsUndefined( typeof event.data.scheduleId ))
        {
          parameters.scheduleId = event.data.scheduleId;
        }
        if(!TV.typeIsUndefined( typeof event.data.scheduleName ))
        {
          parameters.scheduleName = event.data.scheduleName;
        }
        if(!TV.typeIsUndefined( typeof event.data.enabled ))
        {
          parameters.enabled = event.data.enabled;
        }
        if(proceed)
        {
          var actionCmd = new Command( { action : event.data.URL
                                              , clearSystemMessages : false 
                                              , parameters : parameters } );
          actionCmd.execute();
          $( actionCmd ).on( 'executionSucceeded', function( event )
          {
                (new ShowCommand( {
                   action : 'discovery/index'
                     , clearSystemMessages : false} 
                  )).execute(); 
                } );
          }
      };
      /** @type {string} */  var scheduleName = schedule.scheduleName;
      /** @type {Element} */ var divActions   = document.createElement("div");
      /** @type {Element} */ var statusIcon   = null;
      if (schedule.enabled){
        statusIcon = IconFactory.buildActionIcon( { type : 'active'   , caption :  "'" + scheduleName +"'" + i18n.labelCollectionEnabled} );
      } else {
        statusIcon = IconFactory.buildActionIcon( { type : 'offline'   , caption : "'" + scheduleName +"'" + i18n.labelCollectionDisabled} );  
      }
      /** @type {Element} */ var editIcon   = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit   + " '" + scheduleName + "'"} );
      /** @type {Element} */ var runIcon    = IconFactory.buildActionIcon( { type : 'runb'     , caption : i18n.labelDiscoStart   + " '" + scheduleName + "'" + i18n.labelDiscoDiscovery } );
      /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + " '" + scheduleName + "'"} );

      
      $( editIcon   ).on( 'click', { URL : 'discovery/edit'                     , scheduleId : schedule.scheduleId , scheduleName : scheduleName   } , clickDisplayHandler );
      
      var actionEnable = !schedule.enabled;
      $( statusIcon ).on( 'click', { URL : 'discovery/ajaxToggleJobSchedule'    , scheduleId : schedule.scheduleId , scheduleName : scheduleName , enabled : actionEnable  } , clickTriggerHandler );
      $( runIcon    ).on( 'click', { URL : 'discovery/ajaxRunDiscoverySchedule' , scheduleId : schedule.scheduleId , scheduleName : scheduleName , scopeSets : schedule.scopeSets } , clickTriggerHandler );
      $( deleteIcon ).on( 'click', { URL : 'discovery/ajaxDeleteJobSchedule'    , scheduleId : schedule.scheduleId , scheduleName : scheduleName , confirm : true} , clickTriggerHandler );

      /** @type {Element} */ var aRef    = document.createElement("a");
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( editIcon );
      $( divActions ).append( aRef );
      $( divActions ).append(tsa.NBSP);
      if (schedule.scheduleId !== "fulldiscovery") {
        aRef    = document.createElement("a");
        $( aRef ).attr( 'href', '#' );
        $( aRef ).append( deleteIcon );
        $( divActions ).append( aRef );
        $( divActions ).append(tsa.NBSP);
      }
      aRef    = document.createElement("a");
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( statusIcon );
      $( divActions ).append( aRef );
      $( divActions ).append(tsa.NBSP);
      
      
      aRef    = document.createElement("a");
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( runIcon );
      $( divActions ).append( aRef );
      $( divActions ).append(tsa.NBSP);
      
    var divScheduleName = document.createElement("div");
    $(divScheduleName).text(scheduleName);
    
    if(createCollapse){
      /** @type {Element} */ var expandScheduleIcon = IconFactory.buildActionIcon( { type : 'expandb'   , caption : i18n.labelDiscoSchedExpand } );
      $(expandScheduleIcon).children("img").addClass("scheduleListAction");
      $(expandScheduleIcon).on('click',function(){
        $('[name="'+schedule.scheduleId+'"]').toggle();
        var img = $(expandScheduleIcon).children("img");
        var imageSrc = $(img).attr("src");
        imageSrc = "" + imageSrc;
        if(imageSrc.indexOf("expandb")>-1)
        {
          $(img).attr("src",   tsa.attributes.imagesPath + "/" +"expand16.png");
          $(img).attr("alt",   i18n.labelDiscoSchedCollapse);
          $(img).attr("title", i18n.labelDiscoSchedCollapse);
        }
        else
        {
          $(img).attr("src",  tsa.attributes.imagesPath + "/" +"expandb16.png");
          $(img).attr("alt",   i18n.labelDiscoSchedExpand);
          $(img).attr("title", i18n.labelDiscoSchedExpand);
        }
        
      });
      aRef    = document.createElement("a");
      $( aRef ).attr( 'href', '#' );
      $( aRef ).append( expandScheduleIcon );
      $( divScheduleName ).prepend(tsa.NBSP);
      $( divScheduleName ).prepend( aRef );
    }
    return tablePanel.appendRow( [divScheduleName, schedule.nextPerformanceTS ,schedule.formatted , divActions] );
  };
    
    return tsa.cmd.handlers.DiscoveryHandler;
  }
);