/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ShowCommand
          , Command
          , IconFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'DiscoverysettingsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.DiscoverysettingsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.DiscoverysettingsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DiscoverysettingsHandler.prototype ).constructor = tsa.cmd.handlers.DiscoverysettingsHandler;
    
//-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.DiscoverysettingsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n         = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea  = new ContentArea ( { title : i18n.labelDiscoverySettingsHeading
                                                                                , helpDoc : 'sshelp_discoverysettings_panel.html' } );
      /** @type {tsa.widgets.ContentArea} */ var tablePanel   = new TablePanel  ( { title: i18n.labelDiscoverySettingsInfoCaption 
                                                                                  , columns: [ { id : "settings" , label : i18n.labelDiscoverySettings, width : '50%' }
                                                                                             , { id : "actions"  , label : i18n.labelActionsColumn,     width : '50%' }] 
                                                                                   , summary : "Summary InventorySummary"} );
      /** @type {tsa.cmd.Command} */         var cmdDiscoInfo = new Command( { action : 'discoverysettings/getDiscoverySettingsInfo' } );
      contentArea.appendMessage( { text : i18n.labelDisoverySettingsSupportCaption } );
      contentArea.appendMessage( { text : i18n.labelDiscoverySettingsCaption } );
      cmdDiscoInfo.execute();
      $( cmdDiscoInfo ).on( 'executionSucceeded', function( event ) {
        /**@type function() */
        var clickTriggerHandler = function ( event ) {
          tsa.clearSysMessages();
          var actionCmd = new Command( { action : event.data.URL
                                       , clearSystemMessages : false  } );
          actionCmd.execute();
          $( actionCmd ).on( 'executionSucceeded', function( event )
               {
                (new ShowCommand( {
                   action : 'discoverysettings/index'
                     , clearSystemMessages : false} 
                  )).execute(); 
                } );
        };
        $.each(event.response.payload.provider , function( igsIndex, provider ) {
              /** @type {Element} */ var statusIcon  = null;
              /** @type {Element} */ var aStatusIcon = document.createElement("a");
              if (provider.providerEnable){
                statusIcon = IconFactory.buildActionIcon( { type : 'active'   , caption :  "'" + provider.providerName +"' " + i18n.labelCollectionEnabled} );
                $( statusIcon ).on( 'click', { URL : 'discoverysettings/ajaxDisableInventoryCollection/'   + provider.providerName   } , clickTriggerHandler );
              } else {
                statusIcon = IconFactory.buildActionIcon( { type : 'offline'   , caption : "'" + provider.providerName +"' " + i18n.labelCollectionDisabled} );
                $( statusIcon ).on( 'click', { URL : 'discoverysettings/ajaxEnableInventoryCollection/'   + provider.providerName   } , clickTriggerHandler );
              }
              
              $( aStatusIcon ).attr( 'href', '#' );
              $( aStatusIcon ).append(statusIcon);
              tablePanel.appendRow( [ provider.providerName,aStatusIcon] );
              });
          } );
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.DiscoverysettingsHandler;
  }
);