/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
         )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'DocumentationHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.DocumentationHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.DocumentationHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.DocumentationHandler.prototype ).constructor = tsa.cmd.handlers.DocumentationHandler;

    //-------------------------------------------------------------------------

    /**
     * Show main screen.
     */
    tsa.cmd.handlers.DocumentationHandler.prototype._index = function()
    {
    	 /** @type {Object}                    */ var i18n                    = tsa.i18n;
    	 /** @type {tsa.widgets.ContentArea}   */ var contentArea             = null;
    	 /** @type {Element}                   */ var documentationButton     = null;
    	 /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer         = ButtonFactory.buildButtonContainer();

    	 // Initialize the content area (its title and its help document)...
    	 contentArea = new ContentArea ( { title   : i18n.labelDocumentationHeading
    	                                    , helpDoc : 'sshelp_documentation_panel.html' } );

    	 // Appending messages to the content area...
    	 contentArea.appendMessage( { text : i18n.labelDocumentationCaption } );
    	 contentArea.appendMessage( { text : i18n.labelDocumentationDesc } );
    	 contentArea.appendMessage( { text : i18n.labelDocumentationDescSecond } );

    	 //Hyper Link to IBM TSA site
    	 var redirectLink = document.createElement("a");
    	 redirectLink.href = "https://ibm.biz/TSAdemo";
    	 redirectLink.target = '_blank';
    	 redirectLink.innerHTML = "https://ibm.biz/TSAdemo";

    	 var text = document.createTextNode(i18n.labelRedirectTSASite);

    	 var redirectContent = document.createElement("p");
    	 redirectContent.appendChild(text);
    	 redirectContent.appendChild(redirectLink);

    	 contentArea.append(redirectContent);


    	 // Appending button to content area...
    	 documentationButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonRedirectDocumentation ,
														    onclick : function( event ){tsa.cmd.handlers.DocumentationHandler.redirectDocumentation();} })


    	 $( buttonContainer ).append( documentationButton );
         contentArea.append( buttonContainer );
    	 // Content area must be shown...
         this.show( contentArea.getDomNode() );

    }

    //-------------------------------------------------------------------------

    /**
     * Request to redirect to IBM TSA Documentation URL.
     */
    tsa.cmd.handlers.DocumentationHandler.redirectDocumentation = function()
    {
      var redirectWindow = window.open('https://ibm.biz/BdzQiE', '_blank');
      redirectWindow.location;
    };

    return tsa.cmd.handlers.DocumentationHandler;
  }
);