/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ShowCommand
          , Command
          , IconFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'FirewallHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.FirewallHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.FirewallHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.FirewallHandler.prototype ).constructor = tsa.cmd.handlers.FirewallHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.FirewallHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n         = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea  = new ContentArea ( { title : i18n.labelFirewallHeading 
                                                                                , helpDoc : 'sshelp_firewall_panel.html' } );
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel   = new TablePanel  ( { title: i18n.labelFirewallPortsHeading 
                                                                                  , columns: [ { id : "port" , label : i18n.labelFirewallPort       , width : '50%' }
                                                                                             , { id : "actions"  , label : i18n.labelActionsColumn  , width : '50%' }] 
                                                                                   , summary : "Firewall info"} );
      /** @type {tsa.cmd.Command} */         var cmdFirewallPorts = new Command( { action : 'firewall/getFirewallPorts' } );
      contentArea.appendMessage( { text : i18n.labelFirewallCaption } );
      cmdFirewallPorts.execute();
      $( cmdFirewallPorts ).on( 'executionSucceeded', function( event ) {
        /**@type function() */
        var clickTriggerHandler = function ( event ) {
          tsa.clearSysMessages();
          var actionCmd = new Command( { action : event.data.URL
                                       , clearSystemMessages : false  } );
          actionCmd.execute();
          $( actionCmd ).on( 'executionSucceeded', function( event )
               {
                (new ShowCommand( {
                   action : 'firewall/index'
                     , clearSystemMessages : false} 
                  )).execute(); 
                } );
        };
        $.each(event.response.payload.portInfoList , function( idx, firewallPort ) {
              /** @type {Element} */ var statusIcon  = null;
              /** @type {Element} */ var aStatusIcon = document.createElement("a");
              if (firewallPort.firewallOn){
                statusIcon = IconFactory.buildActionIcon( { type : 'active'   , caption : i18n.labelFirewallOff } );
                $( statusIcon ).on( 'click', { URL : 'firewall/ajaxDisableFirewallPort/'   + firewallPort.port   } , clickTriggerHandler );
              } else {
                statusIcon = IconFactory.buildActionIcon( { type : 'offline'   , caption : i18n.labelFirewallOn } );
                $( statusIcon ).on( 'click', { URL : 'firewall/ajaxEnableFirewallPort/'   + firewallPort.port   } , clickTriggerHandler );
              }
              
              $( aStatusIcon ).attr( 'href', '#' );
              $(aStatusIcon).append(statusIcon);
              tablePanel.appendRow( [ firewallPort.port,aStatusIcon] );
              });
          } );
      $( cmdFirewallPorts ).on( 'executionFailed', function( event )
          {
            if( TV.typeIsObject( typeof event.response.payload.redirect ) )
            {
              (new ShowCommand( {
                action : 'main/index'
                  , clearSystemMessages : false} 
               )).execute(); 
            }
          } );
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
  
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.FirewallHandler;
  }
);