/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/Command'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , ButtonFactory
          , TablePanel
          , Command )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'InventorycleanuparchiveHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.InventorycleanuparchiveHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype ).constructor = tsa.cmd.handlers.InventorycleanuparchiveHandler;
    
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.fieldsPerRow        = 2;
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.sortProperties      = [];
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store               = [];
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.friendlyNames       = null;
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.propertyOrder       = [ 'displayName', 'lastModified', 'name', 'cleanedUp', 'xsi:type', 'manufacturer', 'scope', 'model', 'contextIp', 'serialNumber' ];
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.compactProperties   = [ 'displayName', 'contextIp', 'manufacturer', 'model' ];
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.headerCounter       = 0;
    
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype._index = function()
    {
      
      
      /** @type {Object}                    */   var i18n             = tsa.i18n;
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer  = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ContentArea}   */   var contentArea      = new ContentArea ( { title : i18n.labelInventoryCleanupArchivesHeading
                                                                                          , helpDoc       : 'sshelp_inventory_cleanup_archive_panel.html'} );
      /** @type {tsa.cmd.Command}           */   var getArchiveList   = new Command( { action : 'inventorycleanuparchive/getArchiveList'
                                                                                      , clearSystemMessages:false} );
      
      /** @type {tsa.widgets.TablePanel}   */    var tablePanel       = new TablePanel( { title    : i18n.labelInventoryCleanupArchivesEntriesHeading
                                                                                              , emptyMsg : i18n.labelInventoryCleanupArchivesNoEntries } );
      /** @type {element}                   */   var divOptions       = document.createElement("div");
      /** @type {element}                   */   var divDownload      = document.createElement("div");
      /** @type {element}                   */   var tableGroup       = document.createElement("table");
      
      contentArea.appendMessage( { text : i18n.labelInventoryCleanupArchivesCaption } );      

      //creates the content of div options
      /** @type {element}                   */    var pOrderBy      = document.createElement("p");
      /** @type {element}                   */    var labelOrderBy  = document.createElement("label");
      /** @type {element}                   */    var selectOrderBy = $("<select id='ica_sort_selector' style='width:100%; margin-top: 4px;'></select>").get(0); 
      $(labelOrderBy).attr("for","ica_sort_selector");
      $(labelOrderBy).text(i18n.labelSortBy + ":");
      $(pOrderBy).append(labelOrderBy);
      $(pOrderBy).append(selectOrderBy);
      $(divOptions).append(pOrderBy);
      
      
      /** @type {element}    */    var pSortReverse     = document.createElement("p");
      /** @type {element}    */    var labelSortReverse = document.createElement("label");
      
      $( pSortReverse ).css( { 'position'  : 'relative' } );
      $( pSortReverse ).append($("<input id='ica_sort_inverse' type='checkbox' style='margin-right:0.3em; vertical-align:middle;' />"));
      $(labelSortReverse).attr("for","ica_sort_inverse");
      $(labelSortReverse).text(i18n.labelInverse);
      /** @type {element}    */    var buttonSortReverse = $("<button style='position:absolute; right: 4px; bottom: 4px; padding: 0px;' >Apply</button>").get(0);
      $( buttonSortReverse ).on( 'click', function( event )
          {
            tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.refreshTable();
          } );
      

      /** @type {element}    */    var buttonCompactView = $("<input id='ica_compact_view' type='checkbox' style='margin-right:0.3em; vertical-align:middle;'  />").get(0);
      $( buttonCompactView ).on( 'click', function( event )
          {
        tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.refreshTable();
          } );
      /** @type {element}                   */    var labelCompactView  = document.createElement("label");
      /** @type {element}                   */    var pCompactView      = document.createElement("p");
      
      $(labelCompactView).attr("for","ica_compact_view");
      $(labelCompactView).text(i18n.labelCompactView);
      $(pCompactView).append(buttonCompactView)
      $(pCompactView).append(labelCompactView)
      
      $( pSortReverse ).append(labelSortReverse);
      $( pSortReverse ).append(buttonSortReverse);
     
      $( divOptions   ).append(pSortReverse);
      $( divOptions   ).append(pCompactView);
      
      
      //creates the Download div on the right bar
      /** @type {element}    */   var pDownload         = document.createElement("p");   
      /** @type {element}    */   var buttonDownloadTXT = ButtonFactory.buildButton(
            { type    : 'download'
            , grayout : false
            , label   : i18n.labelInventoryCleanupArchiveDownloadSmallTxt
            , onclick : function( event )
              {    
                tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.download('txt');
              }
            }
          );
      /** @type {element}    */   var buttonDownloadCVS = ButtonFactory.buildButton(
           { type    : 'download'
           , grayout : false
           , label   : i18n.labelInventoryCleanupArchiveDownloadCsv
           , onclick : function( event )
             { 
               tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.download('csv');
             }
           }
         );
        
      
      $(pDownload).append(buttonDownloadTXT);
      $(pDownload).append(buttonDownloadCVS);
      $(divDownload).append(pDownload);
      contentArea.appendSidebarPanel( { title: i18n.labelOptions , body: divOptions } );
      contentArea.appendSidebarPanel( { title: i18n.labelInventoryCleanupArchiveDownload  , body: divDownload } );
      
      
      contentArea.append(tablePanel.getDomNode())
      
      getArchiveList.execute();
      $(getArchiveList).on('executionSucceeded', $.proxy(function(event){
        /** @type {Object}   */ var icaList = event.response.payload.icaList;
        /** @type {element}  */ var tbody   = $("<tbody></tbody>").get(0);
        $(tbody).attr( "id" , "ica_tbody" );
        $(tableGroup).append(tbody);
        $( tableGroup ).css( { 'width'           : '100%'     } );
        $( tableGroup ).css( { 'border-collapse' : 'collapse' } ); 
        tablePanel.appendRow([tableGroup]);
        $.each( icaList, function( icaIdx, list )
            {
                if(list.report.length>0){
                  /** @type {element}  */ var hiddenInput  = $("<input type='hidden' />").get(0);
                  $(hiddenInput).attr( "id"    , "json_"+list.pid );
                  $(hiddenInput).attr( "value" , JSON.stringify(list) );
                  contentArea.append(hiddenInput);
                }
            });
        tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.addReportEntries(icaList);
      }, this));
      this.show( contentArea.getDomNode() ); 
    };
    //-------------------------------------------------------------------------
    
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.download = function( format )
    {
      window.location = '/appliance/inventorycleanuparchive/download/ica_' + ( new Date() ).getTime() + '.' + format;
    };
    
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.buildComputerSystemHtmlRows = function( csData, compact )
    {
      /** @type {Array.<element>} */ var rows         = [];
      /** @type {element}         */ var row          = null;
      /** @type {Array.<element>} */ var fieldsDone   = [];
      /** @type {number}          */ var fieldsPerRow = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.fieldsPerRow;
      
      var addField = function( name, value )
      {
        /** @type {element} */ var cell1        = document.createElement( 'th' );
        $(cell1).css({ 'background-color' : 'transparent', 'background-image' : 'none' });
        /** @type {element} */ var cell2        = document.createElement( 'td' );
        /** @type {number}  */ var fieldCount   = fieldsDone.length;
        /** @type {number}  */ var first        = ( ( fieldCount % fieldsPerRow ) === 0 );
        /** @type {number}  */ var last         = ( ( fieldCount % fieldsPerRow ) === ( fieldsPerRow - 1 ) );
        /** @type {string}  */ var friendlyName = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.friendlyNames[ name ];
        /** @type {string}  */ var headerId     = 'ica_th_' + ( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.headerCounter++ );
        
        if( ( typeof friendlyName ) !== 'string' )
        {
          friendlyName = name;
        }
        
        $( cell1 ).css( { 'font-weight'  : 'bold'
                             , 'padding'      : '0px 0px 2px ' + ( first ? 0 : 12 ) + 'px'
                             , 'border-width' : '0px' } );
        $( cell2 ).css( { 'padding'      : '0px 0px 2px 6px'
                             , 'border-width' : '0px' } );
        
        $( cell1 ).text( friendlyName + ( ( name.length > 0 ) ? ':' : '' ) );
        $( cell2 ).text( value );
        
        cell1.setAttribute( 'id'      , headerId );
        cell2.setAttribute( 'headers' , headerId );
        
        if( row === null )
        {
          row = document.createElement( 'tr' );
          rows.push( row );
        }
        
        row.appendChild( cell1 );
        row.appendChild( cell2 );
        
        if( last ) row = null;
        
        fieldsDone.push( name );
      };
      
      if( ( ( typeof csData ) === 'object' ) && ( csData !== null ) )
      {
        /** @type {boolean} */ var isCompactProperty = function( propName ) { return ( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.compactProperties.indexOf( propName ) >= 0 ); };
        
        $( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.propertyOrder ).each( function( propIndex, propName )
        {
          if( ( ( typeof csData[ propName ] ) !== 'undefined' ) &&
              ( ( typeof csData[ propName ] ) !== 'object'    ) &&
              ( !compact || isCompactProperty( propName ) ) )
          {
            addField( propName, csData[ propName ] );
          }
        } );
        
        // Include the properties not appearing in the ordered properties array...
        
        for( var propName in csData )
        {
          if( ( ( typeof csData[ propName ] )  !== 'undefined' ) &&
              ( ( typeof csData[ propName ] )  !== 'object'    ) &&
              ( fieldsDone.indexOf( propName ) <   0           ) &&
              ( !compact || isCompactProperty( propName ) ) )
          {
            addField( propName, csData[ propName ] );
          }
        }
        
        while( ( fieldsDone.length % fieldsPerRow ) > 0 ) addField( '', '' );
      }
      
      return rows;
    };
    //---------------------------------------------------------------------
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.sortStore = function( sortBy, inverse )
    {
      if( ( ( typeof sortBy  ) === 'string'  ) &&
          ( ( typeof inverse ) === 'boolean' ) )
      {
        /** @type {Array.<element>}  */ var sortArray = [];
        /** @type {Array.<element>}  */ var newStore  = [];
        
        $( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store ).each( function( index, entry )
        {
          if( ( typeof entry[ sortBy ] ) !== 'string' )
          {
            entry[ sortBy ] = '';
          }
          
          sortArray.push( entry[ sortBy ] );
        } );
        
        try
        {
          sortArray.sort( function( a, b )
          {
            return a.toLowerCase().localeCompare( b.toLowerCase() );
          } );
        }
        catch( error ) // just sort; not perfect (as it is case sensitive) but it is better than nothing.
        {
          sortArray.sort();
        }
        
        while( sortArray.length > 0 )
        {
          /** @type {element}  */ var value = ( inverse ? sortArray.pop() : sortArray.shift() );
          
          $( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store ).each( function( entryIndex, entry )
          {
            /** @type {boolean}  */ var found = ( ( entry !== null ) && ( entry[ sortBy ] === value ) );
            
            if( found )
            {
              newStore.push( entry );
              tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store[ entryIndex ] = null;
            }
            
            return !found;
          } );
        }
        
        tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store = newStore;
      }
    };
    //---------------------------------------------------------------------
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.refreshTable = function()
    {
      /** @type {Array.<element>}  */ var sortBy   = $( '#ica_sort_selector' ).val();
      /** @type {Array.<element>}  */ var inverse  = $( '#ica_sort_inverse'  ).get(0).checked;
      /** @type {Array.<element>}  */ var tbody    = $( '#ica_tbody'         ).get(0);
      /** @type {Array.<element>}  */ var compact  = $( '#ica_compact_view'  ).get(0).checked;
      /** @type {Array.<element>}  */ var htmlRows = null;
      //aqui1
      tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.sortStore( sortBy, inverse );
      $( tbody ).empty();
      
      for( var i=0; i<tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store.length; i++ )
      {
        /** @type {element}  */ var computerSystemData = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store[i];
        
        if( compact && ( ( ( typeof computerSystemData.compactHtmlRows ) !== 'object' ) || ( computerSystemData.compactHtmlRows === null ) ) )
        {
          computerSystemData.compactHtmlRows = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.buildComputerSystemHtmlRows( computerSystemData, true );
        }
        else if( !compact && ( ( ( typeof computerSystemData.htmlRows ) !== 'object' ) || ( computerSystemData.htmlRows === null ) ) )
        {
          computerSystemData.htmlRows = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.buildComputerSystemHtmlRows( computerSystemData, false );
        }
        
        /** @type {element}  */ htmlRows = ( compact ? computerSystemData.compactHtmlRows : computerSystemData.htmlRows );
        
        $( htmlRows ).each( function( htmlRowIndex, htmlRow )
        {
          /** @type {boolean}  */ var isFirstRow = ( htmlRowIndex === 0 );
          /** @type {boolean}  */ var isLastRow  = ( htmlRowIndex === ( htmlRows.length - 1 ) );
          
          $( htmlRow ).children().each( function( htmlCellIndex, htmlCell )
          {
            if( isFirstRow )
            {
              $( htmlCell ).css( { 'padding-top' : '6px' } );
              
              if( tbody.children.length > 0 )
              {
                $( htmlCell ).css( { 'border-top' : '1px solid #ccc' } );
                
              }
            }
            else if( isLastRow )
            {
              $( htmlCell ).css( { 'padding-bottom' : '6px' } );
            }
          } );
          
          tbody.appendChild( htmlRow );
        } );
      }
    };
    //-------------------------------------------------------------------
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.addReportEntry = function( icaId )
    {
      /** @type {element}  */ var jsonInput = document.getElementById( 'json_' + icaId );
      if( ( ( typeof jsonInput ) === 'object' ) && ( jsonInput !== null ) )
      {
        /** @type {string}   */  var json          = jsonInput.value;
        /** @type {object}   */  var enclosure     = JSON.parse( json );
        /** @type {object}   */  var report        = enclosure.report;
        /** @type {object}   */  var friendlyNames = enclosure.friendlyNames;
        
        if( ( ( typeof report ) === 'object' ) && ( report !== null ) )
        {
          if( ( ( typeof friendlyNames ) !== 'object' ) || ( friendlyNames === null ) )
          {
            friendlyNames = {};
          }
          tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.friendlyNames = friendlyNames;
          
          // Iterating report entries...
          
          $( report ).each( function( index, entry )
          {
            // Some properties might be not required to show...
            
            if( ( typeof entry[ 'type' ] ) !== 'undefined' ) delete entry[ 'type' ];
            
            // Adding the entry...
            
            tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.store.push( entry );
            
            // Updating the known property names...
            
            for( var key in entry )
            {
              if( ( key.length > 0 ) && ( tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.sortProperties.indexOf( key ) === -1 ) )
              {
                /** @type {element}  */ var sortSelector = document.getElementById( 'ica_sort_selector' );
                /** @type {element}  */ var option       = document.createElement( 'option' );
                /** @type {string}  */  var friendlyName = tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.friendlyNames[ key ];
                
                if( ( typeof friendlyName ) !== 'string' ) friendlyName = key;
                
                option.value = key;
                option.appendChild( document.createTextNode( friendlyName ) );
                tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.sortProperties.push( key );
                
                sortSelector.appendChild( option );
              }
            }
          } );
        }
      }
    };
    //-------------------------------------------------------------------
    tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.addReportEntries = function( icaList )
    {      
        $.each( icaList, function( icaIdx, list )
            {
              if(list.report.length>0){
                tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.addReportEntry( list.pid );
              }
            });
      
        tsa.cmd.handlers.InventorycleanuparchiveHandler.prototype.refreshTable();
    };
    //-------------------------------------------------------------------
    return tsa.cmd.handlers.InventorycleanuparchiveHandler;
  }
);