/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'   
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , IconFactory
          , TablePanel
          , ButtonFactory
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'InventorysettingsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.InventorysettingsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.InventorysettingsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.InventorysettingsHandler.prototype ).constructor = tsa.cmd.handlers.InventorysettingsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.InventorysettingsHandler.prototype._index = function()
    {
      /** @type {Object}                  */   var i18n                     = tsa.i18n;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer          = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ContentArea} */   var contentArea              = new ContentArea ( { title : i18n.labelInventorySchedHeading
                                                                                  , helpDoc       : 'sshelp_inv_report_settings_panel.html'} );
      /** @type {tsa.widgets.TablePanel} */    var scheduleTablePanel       = new TablePanel( { title: i18n.labelInventorySummaryHeading 
                                                                                  , columns: [ { id    : "id" , width : '40%'  }
                                                                                              ,{ value : "id" , width : '60%'  }] } );
      /** @type {tsa.cmd.ShowCommand} */        var getInventorySettings    = new Command( { action : 'inventorysettings/getSettings'
                                                                                              , clearSystemMessages : false} );
      /** @type {tsa.widgets.TablePanel} */     var historyTablePanel       = new TablePanel( { title: i18n.labelCollectionSchedHistoryHeading 
                                                                                          , emptyMsg : i18n.labelInventorySchedNoJobHistory
                                                                                          , columns: [ {  id  : "status"   ,label : i18n.labelInventorySchedStatus  , width : '8%'  }
                                                                                                      , { id  : "instance" ,label : i18n.labelInventorySchedInstance, width : '28%'  }
                                                                                                      , { id  : "state"    ,label : i18n.labelInventorySchedState   , width : '14%'  }
                                                                                                      , { id  : "comments" ,label : i18n.labelInventorySchedComments, width : '50%'  }] } );
      
      getInventorySettings.execute();
      
      $( getInventorySettings ).on( 'executionSucceeded', function( event )
          {

          
          /** @type {string}*/   var scheduleDesc = event.response.payload.scheduleDesc;
          /** @type {object}*/   var history      = event.response.payload.history;
          /** @type {Object}*/   var jobInfo      = event.response.payload.job;
          
          
          if( TV.typeIsUndefined( typeof scheduleDesc ) ){
            scheduleTablePanel.appendRow([i18n.labelInventoryCleanupScheduleStatus, i18n.labelDisabled              ]);
            scheduleTablePanel.appendRow([i18n.labelInventoryExpiryAge            , event.response.payload.expiryAge ]);
          }else{
            scheduleTablePanel.appendRow([i18n.labelInventorySchedNextRun  , event.response.payload.nextPerformance ]);
            scheduleTablePanel.appendRow([i18n.labelInventorySchedSchedule , event.response.payload.scheduleDesc    ]);
            scheduleTablePanel.appendRow([i18n.labelInventoryExpiryAge     , event.response.payload.expiryAge       ]);
          }
          
          
          //Table panel for history 
          if(! TV.typeIsUndefined( typeof history ) ){
            
            $.each( history, function( entryIndex, entry )
                {
                /** @type {Element} */ var commentList = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get(0);
                $.each( entry.comments, function( commentIndex, comment )
                    {
                      /** @type {Element} */ var commentElement = $( '<li style="padding-top: 0px;"></li>' ).get(0);
                      $( commentElement ).text( comment );
                      $( commentList    ).append( commentElement );
                      
                    });
                historyTablePanel.appendRow( [ IconFactory.buildSeverityIcon( { severity : entry.severity } )
                                                , entry.name
                                                , entry.stateText
                                                , commentList ] );
                });
            
          }
          if(jobInfo.editAllowed){
            $( buttonContainer ).append(
                ButtonFactory.buildButton(
                  { type    : 'arrow'
                  , grayout : false
                  , label   : i18n.buttonInventorySchedEditSchedule
                  , onclick : function( event )
                    {
                      ( new ShowCommand( { action : 'inventorysettings/edit' } ) ).execute();
                    }
                  }
                )
              );
              
              $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
          }
          
          if(jobInfo.runAllowed){
            $( buttonContainer ).append(
                ButtonFactory.buildButton(
                  { type    : 'arrow'
                  , grayout : false
                  , label   : i18n.buttonInventorySchedRunNow
                  , onclick : function( event )
                    {
                      /** @type {tsa.cmd.ShowCommand} */        var runInventoryCleanup = new Command( { action : 'inventorysettings/ajaxRunInventoryCleanUp'
                                                                                                          , clearSystemMessages : false } );
                      runInventoryCleanup.execute();
                      
                      
                      
                    }
                  }
                )
              );
              
              $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
            
          }
          $( buttonContainer ).append(
              ButtonFactory.buildButton(
                { type    : 'arrow'
                , grayout : false
                , label   : i18n.buttonInventorySchedShowCleanupArchive
                , onclick : function( event )
                  {
                  ( new ShowCommand( { action : 'inventorycleanuparchive/index' } ) ).execute();
                  }
                }
              )
            );
          
          
          
          
          
          
          });
      
     
      contentArea.appendMessage( { text : i18n.labelInventorySchedOverview } );
      contentArea.append( scheduleTablePanel.getDomNode() );
      contentArea.append( historyTablePanel.getDomNode() );
      contentArea.append( buttonContainer );
      
      this.show( contentArea.getDomNode());
    };
    
    
    /**
     * Show archive screem
     */

    //-------------------------------------------------------------------------
    tsa.cmd.handlers.InventorysettingsHandler.prototype._edit = function()
    {
      /** @type {Object}                    */ var i18n                 = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea          = null;
      /** @type {tsa.widgets.FormPanel}     */ var enablementFormPanel  = null;
      /** @type {tsa.widgets.FormPanel}     */ var scheduleFormPanel    = null;
      /** @type {tsa.widgets.FormPanel}     */ var scheduleExpFormPanel = null;
      /** @type {Element}                   */ var enablementSelector   = document.createElement( 'select' );
      /** @type {Element}                   */ var hourSelector         = document.createElement( 'select' );
      /** @type {Element}                   */ var minuteSelector       = document.createElement( 'select' );
      /** @type {Element}                   */ var ageSelector          = document.createElement( 'select' );
      /** @type {Element}                   */ var weekModeRadio        = document.createElement( 'input' );
      /** @type {Element}                   */ var monthModeRadio       = document.createElement( 'input' );
      /** @type {Array.<Element>}           */ var weekDayCheckboxes    = [];
      /** @type {Element}                   */ var weekDaysRow          = null;
      /** @type {Array.<Element>}           */ var monthDayCheckboxes   = [];
      /** @type {Element}                   */ var monthDaysRow         = null;
      /** @type {Element}                   */ var saveButton           = null;
      /** @type {Element}                   */ var cancelButton         = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer      = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd        = new Command( { action              : 'inventorysettings/getJobInfo'
                                                                                       , clearSystemMessages : false } );
      $( enablementSelector ).attr("id", "enabled"   );
      $( hourSelector       ).attr("id", "hour"      ); 
      $( minuteSelector     ).attr("id", "minute"    );
      $( weekModeRadio      ).attr("id", "weekMode"  );
      $( monthModeRadio     ).attr("id", "monthMode" );
      $( ageSelector        ).attr("id", "expiryAge");
      // Initializing content area...
      contentArea = new ContentArea( { title   : i18n.labelInventorySettingsHeading
                                     , helpDoc : 'sshelp_edit_inv_schedule_settings.html' } );
      
      contentArea.appendMessage( { text : i18n.labelInventorySettingsCaption } );
      // Initializing form panels...
      enablementFormPanel = new FormPanel( { title       : i18n.labelInventoryCleanupSchedEnableHeading
                                           , description : i18n.labelInventoryCleanupEnableCaption
                                           , action      : 'inventorysettings/ajaxUpdate' } );
      
      scheduleFormPanel    = new FormPanel( { title       : i18n.labelInventorySchedScheduleHeading
                                         , description : i18n.labelInventorySchedScheduleCaption } );
      
      scheduleExpFormPanel = new FormPanel( { title       : i18n.labelInventorySettingsExpirationHeading
                                              , description : i18n.labelInventorySettingsExpirationDesc } );
      
      
      
      $( enablementFormPanel ).on( 'executionSucceeded', function( event )
      {
        ( new ShowCommand( { action              : 'inventorysettings/index'
                           , clearSystemMessages : false} ) ).execute();
      } );
      
      // Multiple form panels, but submitted as a single form...
      enablementFormPanel.linkTo( scheduleFormPanel );
      scheduleFormPanel.linkTo( scheduleExpFormPanel );
      
      
      // Enablement selector...
      enablementSelector.name = 'enable';
      
      $.each( [ { value: 'true'  , label: i18n.labelInventoryCleanupSchedEnabled  }
              , { value: 'false' , label: i18n.labelInventoryCleanupSchedDisabled } ]
            , function( optionDataIndex, optionData )
              {
                /** @type {Element} */ var option = document.createElement( 'option' );
                
                $( option ).attr( 'value', optionData.value );
                $( option ).text( optionData.label );
                $( enablementSelector ).append( option );
              } );
      
      enablementFormPanel.appendFieldRow( { label    : i18n.labelInventoryCleanupSchedEnable
                                          , required : true
                                          , element  : enablementSelector } );
      
      /** @type {function(Event)} */
      var enablementChanged = function( event )
      {
        /** @type {boolean} */ var disabled = ( $( enablementSelector ).val() === 'false' );
        
        hourSelector   .disabled = disabled;
        minuteSelector .disabled = disabled;
        //ageSelector    .disabled = disabled;
        weekModeRadio  .disabled = disabled;
        monthModeRadio .disabled = disabled;
        
        
        $.each( weekDayCheckboxes.concat( monthDayCheckboxes ), function( checkboxIndex, checkbox )
        {
          checkbox.disabled = disabled;
        } );
      };
      
      $( enablementSelector ).on( 'change', enablementChanged );
      
      // Hour selector...
      hourSelector.name = 'hour';
      
      for( /** @type {number} */ var hour=0; hour<24; hour++)
      {
        $( hourSelector ).append( $( '<option value="' + hour + '">' + ( ( hour < 10 ) ? '0' : '' ) + hour + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtHour
                                        , required : true
                                        , element  : hourSelector } );
      
      // Minute selector...
      minuteSelector.name = 'minute';
      
      for( /** @type {number} */ var minute=0; minute<60; minute+=5)
      {
        $( minuteSelector ).append( $( '<option value="' + minute + '">' + ( ( minute < 10 ) ? '0' : '' ) + minute + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtMinute
                                        , required : true
                                        , element  : minuteSelector } );
   // Minute selector...
      ageSelector.name = 'expiryAge';
      
      for( /** @type {number} */ var expiryAge=30; expiryAge<100; expiryAge+=30)
      {
        $( ageSelector ).append( $( '<option value="' + expiryAge + '">' + expiryAge + '</option>' ) );
      }
      
      scheduleExpFormPanel.appendFieldRow( { label    : i18n.labelInventoryExpiryAge
                                        , required : true
                                        , element  : ageSelector 
                                        , hint      : i18n.labelInventoryMaxAgeDesc} );
      
      // Day mode radio buttons...
      weekModeRadio.type    = 'radio';
      weekModeRadio.name    = 'daymode';
      weekModeRadio.value   = 'week';
      weekModeRadio.checked = true;
      monthModeRadio.type   = 'radio';
      monthModeRadio.name   = 'daymode';
      monthModeRadio.value  = 'month';
      
      scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleDayMode
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span       = document.createElement( 'span' );
                         /** @type {Element} */ var weekLabel  = document.createElement( 'label' );
                         /** @type {Element} */ var monthLabel = document.createElement( 'label' );
                         
                         $( weekLabel  ).append( weekModeRadio  , document.createTextNode( i18n.scheduleDayModeWeek  ) );
                         $( monthLabel ).append( monthModeRadio , document.createTextNode( i18n.scheduleDayModeMonth ) );
                         $( span       ).append( weekLabel, document.createElement( 'br' ), monthLabel );
                         $( span       ).addClass( 'ibm-input-group' );
                         $( span       ).attr( 'role', 'radiogroup' );
                         $( span       ).attr( 'aria-label', i18n.scheduleDayMode );
                         
                         return span;
                       } )()
      } );
      
      // Days of week checkboxes...
      weekDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnDays
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span = document.createElement( 'span' );
                         
                         $( span ).addClass( 'ibm-input-group' );
                         $( span ).attr( 'role', 'group' );
                         $( span ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         $.each( [ 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ], function( wdIndex, wd )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="weekday" value="' + ( wdIndex + 1 ) + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label>' + i18n[ 'schedule' + wd.charAt(0).toUpperCase() + wd.substring(1) ] + '</label>' ).get(0);
                           
                           $( label ).prepend( checkbox );
                           weekDayCheckboxes.push( checkbox );
                           $( span ).append( label, document.createElement( 'br' ) );
                         } );
                         
                         return span;
                       } )()
      } );
      
      // Days of month checkboxes...
      monthDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnMonthDays
        , hint     : i18n.scheduleOnMonthDaysDesc
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var div = document.createElement( 'div' );
                         
                         $( div ).addClass( 'ibm-input-group' );
                         $( div ).attr( 'role', 'group' );
                         $( div ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         for( /** @type {number} */ var md=1; md<=31; md++ )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="monthday" value="' + md + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label style="display: inline-block; font-weight: normal; margin-right: 0.8em">'
                                                                  + ( ( md < 10 ) ? '0' : '' ) + md
                                                                  + '</label>' ).get(0);
                           $( label ).prepend( checkbox );
                           monthDayCheckboxes.push( checkbox );
                           $( div ).append( label );
                           
                           if( ( md % 7 ) === 0 )
                           {
                             $( div ).append( document.createElement( 'br' ) );
                           }
                         };
                         
                         return div;
                       } )()
      } );
      
      // Event handler: day mode changed...
      /** @type {function(Event)} */
      var dayModeChanged = function( event )
      {
        logger.info( 'Day mode changed: processing started...' );
        
        $( weekDaysRow  ).css( { 'display' : ( weekModeRadio  .checked ? '' : 'none' ) } );
        $( monthDaysRow ).css( { 'display' : ( monthModeRadio .checked ? '' : 'none' ) } );
        
        // Selecting a mode will enable all of the checkboxes of the other mode...
        if ( weekModeRadio  .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', true ); } );
        if ( monthModeRadio .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', true ); } );
        
        // If the user triggered the event (event is an object, and it is not null), clear the check-boxes of the mode he selected...
        if( TV.typeIsObject( typeof event ) && ( event !== null ) )
        {
          if( weekModeRadio  .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', false ); } );
          if( monthModeRadio .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', false ); } );
        }
        
        logger.info( 'Day mode changed: processing finished.' );
      };
      
      $( weekModeRadio  ).add( monthModeRadio ).on( 'change', dayModeChanged );
      
      // Preparing the retrieval of the discovery job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object} */ var jobInfo   = event.response.payload.jobInfo;
        /** @type {Object} */ var schedule  = jobInfo.schedule;
        /** @type {Object} */ var isEnabled = ( schedule !== null ); 
      
        // Populating schedule table (if available; if not, the empty-message will be shown)...
        if( isEnabled ) 
        {
          $( enablementSelector ).val( 'true' );
          $( hourSelector       ).val( schedule.hour   );
          $( minuteSelector     ).val( schedule.minute );
          
          
          weekModeRadio  .checked = ( schedule.dayOfWeek.length !== 7);
          monthModeRadio .checked = ( schedule.dayOfWeek.length === 7);
          
          $.each( weekDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfWeek.indexOf( checkboxIndex + 1 ) >= 0 );
          } );
          
          $.each( monthDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfMonth.indexOf( parseInt( $( checkbox ).val(), 10 ) ) >= 0 );
          } );
        }
        else
        {
          $( enablementSelector ).val( 'false' );
          
        }
        $( ageSelector        ).val( jobInfo.expiryAge );
        enablementChanged( null );
        dayModeChanged( null );
      } );
      
      // Buttons...
      saveButton   = ButtonFactory.buildButton( { type : 'arrow'  , label : i18n.buttonSave   } );
      cancelButton = ButtonFactory.buildButton( { type : 'cancel' , label : i18n.buttonCancel } );
      
      $( saveButton ).on( 'click', function( event )
      {
        enablementFormPanel.submit();
      } );
      
      $( cancelButton ).on( 'click', function( event )
      {
        ( new ShowCommand( { action : 'inventorysettings/index' } ) ).execute();
      } );
      
      $( buttonContainer ).append( saveButton );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( cancelButton );
      
      // Appending content...
      contentArea.append( enablementFormPanel.getDomNode() );
      contentArea.append( scheduleFormPanel.getDomNode() );
      contentArea.append( buttonContainer );
      
      // Performing the job information retrieval...
      getJobInfoCmd.execute();
      
      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    
      
      
  };
    return tsa.cmd.handlers.InventorysettingsHandler;     
  }
);