/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ContentArea
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'JvmHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.JvmHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.JvmHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.JvmHandler.prototype ).constructor = tsa.cmd.handlers.JvmHandler;

    //-------------------------------------------------------------------------
//    /**
//     * Show main screen.
//     */
    tsa.cmd.handlers.JvmHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title          : i18n.labelJVMHeading
                                                                                  , showBackToTop : true 
                                                                                  , helpDoc : 'sshelp_jvm_panel.html'} );

      contentArea.appendMessage( { text : i18n.labelJVMCaption } );

      /** @type {tsa.cmd.Command} */ var getJvmInfoCommand = new Command ( { action             : 'jvm/getJvmInfo'
                                                                           , clearSystemMessages: false   });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMMemory  = new TablePanel( { title: i18n.labelJVMMemoryHeading
          , emptyMsg: i18n.labelJVMNoMemory
          , columns: [ { id : "consumedMemory"  , label : i18n.labelJVMMemoryConsumed , width : '31%' }
                     , { id : "totalMemory"     , label : i18n.labelJVMMemoryTotal    , width : '23%' }
                     , { id : "freeMemory"      , label : i18n.labelJVMMemoryFree     , width : '23%' }
                     , { id : "maxMemory"       , label : i18n.labelJVMMemoryMax      , width : '23%' }] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMPlatform  = new TablePanel( { title: i18n.labelJVMPlatformHeading
        , emptyMsg: i18n.labelJVMNoPlatform
        , columns: [] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMLocations  = new TablePanel( { title: i18n.labelJVMLocationsHeading
        , emptyMsg: i18n.labelJVMNoLocation
        , columns: [ { id : "name"      , label : i18n.labelJVMLocationsName    , width : '40%' }
                     , { id : "path"      , label : i18n.labelJVMLocationsPath    , width : '60%' }] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMThreads  = new TablePanel( { title: i18n.labelJVMThreadsHeading
        , emptyMsg: i18n.labelJVMNoThreads
        , columns: [ { id : "name"      , label : i18n.labelJVMThreadsName      , width : '50%' }
                     , { id : "priority"  , label : i18n.labelJVMThreadsPriority  , width : '5%' }
                     , { id : "group"     , label : i18n.labelJVMThreadsGroup     , width : '25%' }
                     , { id : "alive"     , label : i18n.labelJVMThreadsAlive     , width : '10%' }
                     , { id : "daemon"    , label : i18n.labelJVMThreadsDaemon    , width : '10%' }] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMNetwork  = new TablePanel( { title: i18n.labelJVMNetworkHeading
        , emptyMsg: i18n.labelJVMNoNetifs
        , columns: [ { id : "address"   , label : i18n.labelJVMNetworkAddress    , width : '50%' }
                   , { id : "hostname"  , label : i18n.labelJVMNetworkHostname   , width : '5%' }
                   , { id : "loopback"  , label : i18n.labelJVMNetworkLoopback   , width : '25%' }] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMEclipse  = new TablePanel( { title: i18n.labelJVMEclipseHeading
        , emptyMsg: i18n.labelJVMNoJobs
        , columns: [ { id : "name"      , label : i18n.labelJVMEclipseName      , width : '50%' }
                     , { id : "priority"  , label : i18n.labelJVMEclipsePriority  , width : '5%' }
                     , { id : "state"     , label : i18n.labelJVMEclipseState     , width : '25%' }
                     , { id : "class"     , label : i18n.labelJVMEclipseClass     , width : '10%' }
                     , { id : "system"    , label : i18n.labelJVMEclipseSystem    , width : '10%' }
                     , { id : "user"      , label : i18n.labelJVMEclipseUser      , width : '10%' }] });

      /** @type {tsa.widgets.TablePanel} */ var tablePanelJVMSystem  = new TablePanel( { title: i18n.labelJVMSystemHeading
        , emptyMsg: i18n.labelJVMNoSysProps
        , columns: [ { id : "name"      , label : i18n.labelJVMSystemName    , width : '50%' }
                     , { id : "value"     , label : i18n.labelJVMSystemValue   , width : '5%' }] });

      contentArea.append(tablePanelJVMMemory.getDomNode());
      contentArea.append(tablePanelJVMPlatform.getDomNode());
      contentArea.append(tablePanelJVMLocations.getDomNode());
      contentArea.append(tablePanelJVMThreads.getDomNode());
      contentArea.append(tablePanelJVMNetwork.getDomNode());
      contentArea.append(tablePanelJVMEclipse.getDomNode());
      contentArea.append(tablePanelJVMSystem.getDomNode());

      getJvmInfoCommand.execute();
      $( getJvmInfoCommand ).on( 'executionSucceeded', function( e ){
        /** @type {JvmBean} */ var jvm = e.response.payload.jvm;
        /** @type {PlatformBean} */ var platform = e.response.payload.platform;

        tablePanelJVMMemory.appendRow([jvm.usedMemoryMb   + " Mb()"
                                      , jvm.totalMemoryMb + " Mb()"
                                      , jvm.freeMemoryMb  + " Mb()"
                                      , jvm.maxMemoryMb   + " Mb()"
                                      ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformOSType
                                        , platform.os
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformOSArch
                                        , platform.osArch
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformOSName
                                        , platform.osName
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformOSVersion
                                        , platform.osVersion
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformProcessor
                                        , platform.processor
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformWindowing
                                        , platform.ws
                                        ]);

        tablePanelJVMPlatform.appendRow([i18n.labelJVMPlatformFramework
                                        , platform.frameworkVersion
                                        ]);

        $.each(platform.locations, function(index, location){
          tablePanelJVMLocations.appendRow([location.label
                                           , location.path
                                           ])});

        $.each(jvm.threads, function(index, thread){
          tablePanelJVMThreads.appendRow([thread.name
                                         , thread.priority
                                         , thread.group
                                         , thread.alive
                                         , thread.daemon
                                         ])});

        $.each(jvm.jobs, function(index, job){
          tablePanelJVMEclipse.appendRow([job.name
                                         , job.priority
                                         , job.state
                                         , job.classname
                                         , job.system
                                         , job.user
                                         ])});

        $.each(jvm.netifs, function(index, netif){
          tablePanelJVMNetwork.appendRow([netif.address
                                         , netif.hostname
                                         , netif.loopback
                                         ])});

        $.each(jvm.sysProperties, function(index, sysProperty){
          tablePanelJVMSystem.appendRow([sysProperty.name
                                         , sysProperty.value
                                         ])});
      });

      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.JvmHandler;
  }
);