/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/TablePanel'
  ],

  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ContentArea
          , ButtonFactory
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'LicenseviewHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.LicenseviewHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.LicenseviewHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.LicenseviewHandler.prototype ).constructor = tsa.cmd.handlers.LicenseviewHandler;

    //-------------------------------------------------------------------------

    /**
     * Buffer of entries retrieved from server. Key: ID.
     * @private
     * @type {Object.<string,Object>}
     */
    tsa.cmd.handlers.LicenseviewHandler.entryBuffer = {};





    tsa.cmd.handlers.LicenseviewHandler.prototype._index = function(){
      /** @type {Object}                    */     var i18n                        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */     var contentArea                 = new ContentArea ( { title : i18n.labelLicenseHeading } );
      /** @type {element}                   */     var divBaseLicense              = $('<div id="baseLicensePanel" class="ibm-container" style="display: block;"></div>').get(0);
      /** @type {String}                    */     var separatelyLicenseText       = null;
      /** @type {String}                    */     var licenseTextInfo             = null;
      /** @type {String}                    */     var noticeInfoText              = null;
      /** @type {String}                    */     var nonIBMNoticesText           = null;
      /** @type {element}                   */     var overLayWindowState          = this.createOverLayWindows("stateWindow", i18n.labelibmLicenseState);
      /** @type {element}                   */     var overLayWindowNotices        = this.createOverLayWindows("noticeInfo", i18n.labelibmLicenseNoticeHeading);
      /** @type {element}                   */     var overLayNonIBMNotices        = this.createOverLayWindows("nonIBMNotices", i18n.labelibmLicenseNonIBMNoticeHeading);
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerStateofWork  = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerNoticeInfo   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerNonIBMNotices= ButtonFactory.buildButtonContainer();


      /** @type {tsa.cmd.Command           */     var getLicenseData   = new Command({action:'licenseview/getLicenseData'
                                                                                         ,clearSystemMessages:false
                                                                                          });

      /** @type {tsa.widgets.TablePanel} */      var tableNoticeInfo   = new TablePanel( { title     : i18n.labelibmLicenseNoticeHeading
                                                                                          , summary  : 'License statement info'
                                                                                          , columns  : [ { content : '' } ] } );

      /** @type {tsa.widgets.TablePanel} */      var tableStatemAndWork = new TablePanel( { title    : i18n.labelibmLicenseHeading
                                                                                          , summary  : 'License statement work'
                                                                                          , columns  : [ { id : '' } ] } );
      
      /** @type {tsa.widgets.TablePanel} */      var tableNonIBMNotice = new TablePanel( {  title    : i18n.labelibmLicenseNonIBMNoticeHeading
                                                                                          , summary  : 'License statement work'
                                                                                          , columns  : [ { id : '' } ] } );

      contentArea.appendMessage( { text : i18n.labelLicenseAcceptedCaption } );
      getLicenseData.execute();

      $( buttonContainerStateofWork ).append( ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.labelibmLicenseStateButton
            , onclick : function( event )
              {
                $("#ibmLicenseTextstateWindow").text(licenseTextInfo);
                $(overLayWindowState).show();
              }
            }
          )
        );
      $( buttonContainerNoticeInfo ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.labelibmLicenseNoticeButton
            , onclick : function( event )
              {
              $("#ibmLicenseTextnoticeInfo").text(noticeInfoText);
              $(overLayWindowNotices).show();
              }
            }
          )
        );
      $( buttonContainerNonIBMNotices ).append(
              ButtonFactory.buildButton(
                { type    : 'arrow'
                , grayout : false
                , label   : i18n.labelibmLicenseNonIBMNoticeButton
                , onclick : function( event )
                  {
                  $("#ibmLicenseTextnonIBMNotices").text(nonIBMNoticesText);
                  $(overLayNonIBMNotices).show();
                  }
                }
              )
            );

      tableStatemAndWork.appendRow([ buttonContainerStateofWork ]);
      tableNoticeInfo.appendRow([ buttonContainerNoticeInfo ]);
      tableNonIBMNotice.appendRow([ buttonContainerNonIBMNotices ]);

      $(getLicenseData).on('executionSucceeded', $.proxy(function(event) {
        noticeInfoText  = event.response.payload.noticesInfo;
        licenseTextInfo = event.response.payload.ibmLicenseText;
        nonIBMNoticesText  = event.response.payload.nonIBMNoticesText;
        contentArea.append( divBaseLicense );
        contentArea.append( tableStatemAndWork.getDomNode() );
        contentArea.append( tableNonIBMNotice.getDomNode() );
        contentArea.append( tableNoticeInfo.getDomNode() );
        contentArea.append( overLayWindowState );
        contentArea.append( overLayWindowNotices );
        contentArea.append( overLayNonIBMNotices );
        this.createBaseLicensePanel(divBaseLicense);
        var divLicense = $('<div id="baseLicense" style="background:#ffffff" class="ibm-container-body ibm-license-code-sample ibm-license-wordwrap"></div>').get(0);
        var code = document.createElement("code");
        $(code).append(event.response.payload.baseLicense);
        $(divLicense).append(code);
        $(divBaseLicense).append(divLicense);
       }, this));



      this.show( contentArea.getDomNode() );

    };
    tsa.cmd.handlers.LicenseviewHandler.prototype.createBaseLicensePanel = function(divBaseLicense){
      // Manual create panel for BaseLicensePanel, license is handled different
      var spanLicenseHeading = $('<span id="baseLicenseDesc"></span>').get(0);
      var aSpanLicenseHeading = document.createElement("a");
      $(aSpanLicenseHeading).text(tsa.i18n.baseLicenseHeading);
      var hSpanLicenseHeading = document.createElement("h2");
      $(hSpanLicenseHeading).append(aSpanLicenseHeading);
      $(spanLicenseHeading).append(hSpanLicenseHeading);
      $(divBaseLicense).append(spanLicenseHeading);
      return divBaseLicense;
    };
    tsa.cmd.handlers.LicenseviewHandler.prototype.createOverLayWindows = function(id, headingInfo){

      // Since License is hardly use, a manual creation of elements to avoid effort time consuming widget that will be hardly use,  this page only is used one time by version.
      var divOverLay = $('<div class="ibm-common-overlay ibm-overlay-alt-two" style="display: block; top: 75px; left: -120.5px;" ></div>').get(0);
      $(divOverLay).attr("id", id);
      $(divOverLay).append('<div class="ibm-head"><p><a class="ibm-common-overlay-close" onclick="jQuery(\'#'+id+'\').hide();"  href="#close">'+tsa.i18n.buttonOverlayClose+'</a></p></div>');
      $(divOverLay).append('<div class="ibm-body">'
                          + '<div class="ibm-main">'
                          + '<div id="licenseHeading" class="ibm-title">'
                          + ' <h2><div >'+ headingInfo +'</div></h2>'
                          + '</div>'
                          + '<div id="ibmLicensePanelOverlay" class="ibm-container ibm-alternate">'
                          + '    <div id="ibmLicense" class="ibm-container-body ibm-license-code-sample ibm-license-wordwrap" aria-label="'+ tsa.i18n.labelseparatelyLicenseHeading +' ">'
                          + '        <code >'
                          + '        <div id="ibmLicenseText' + id + '"></div>'
                          + '        </code>'
                          + '    </div>'
                          + '</div>'
                          + '</div>'
                          + '</div>');
      $(divOverLay).append('<div class="ibm-footer"></div>');
      $(divOverLay).hide();

      return divOverLay;
    };


    //-------------------------------------------------------------------------
    return tsa.cmd.handlers.LicenseviewHandler;
  }
);
