/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ContentArea
          , ButtonFactory
          , IconFactory
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'LogsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.LogsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.LogsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.LogsHandler.prototype ).constructor = tsa.cmd.handlers.LogsHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Buffer of entries retrieved from server. Key: ID.
     * @private
     * @type {Object.<string,Object>}
     */
    tsa.cmd.handlers.LogsHandler.entryBuffer = {};
    
    //-------------------------------------------------------------------------
    
    /**
     * Show summary screen.
     */
    tsa.cmd.handlers.LogsHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = null;
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel      = null;
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      
      contentArea = new ContentArea ( { title         : i18n.labelActivityLogHeading
                                      , helpDoc       : 'sshelp_activity_log_panel.html'
                                      , showBackToTop : true } );
      
      var entryHandler = function( entry )
      {
        /** @type {Array.<(string|Element)>} */ var result        = null;
        /** @type {Element}                  */ var elForSeverity = null;
        /** @type {string}                   */ var strForTime    = '';
        /** @type {Element}                  */ var elForMessage  = document.createElement( 'a' );
        
        tsa.cmd.handlers.LogsHandler.entryBuffer[ entry.id ] = entry;
        
        elForSeverity = IconFactory.buildSeverityIcon( { severity : ( TV.typeIsNumber( typeof entry.severity ) ? entry.severity : -1 ) } );
        strForTime    = ( TV.typeIsString( typeof entry.time ) ? entry.time : '?' );
        
        $( elForMessage ).attr( 'href', '#' );
        $( elForMessage ).text( TV.typeIsString( typeof entry.message ) ? entry.message : '?' );
        $( elForMessage ).addClass( 'ibm-feature-link' );
        $( elForMessage ).on( 'click', function( event )
        {
          /** @type {Element} */ var elementForEntry     = tsa.cmd.handlers.LogsHandler.buildElementForLogEntry( entry );
          /** @type {Element} */ var entryButtonContainer = document.createElement( 'p' );
          /** @type {Element} */ var backButton           = ButtonFactory.buildButton(
            { type    : 'back'
            , grayout : true
            , label   : tsa.i18n.buttonGoBack
            , onclick : function( event )
                        {
                          $( elementForEntry ).remove();
                          $( entryButtonContainer ).remove();
                          $( tablePanel.getDomNode() ).show();
                          $( buttonContainer ).show();
                          contentArea.setTitle( i18n.labelActivityLogHeading );
                        }
            }
          );
          
          $( tablePanel.getDomNode() ).hide();
          $( buttonContainer ).hide();
          contentArea.setTitle( i18n.labelActivityLogEntryDetailsHeading );
          $( entryButtonContainer ).append( backButton );
          contentArea.append( elementForEntry );
          contentArea.append( entryButtonContainer );
          
          try
          {
            event.stopPropagation();
          } catch ( error ) { /* Ignore */ }
          
          return false;
        } );
        
        result = [ elForSeverity, strForTime, elForMessage ];
        
        return result;
      };
      
      tablePanel  = new TablePanel  ( { title        : i18n.labelActivityLogEntriesHeading
                                      , summary      : i18n.labelActivityLogTableSummary
                                      , emptyMsg     : i18n.labelActivityLogNoEntries
                                      , ajaxAction   : 'logs/getLogEntries'
                                      , entryHandler : entryHandler
                                      , columns      : [ { id       : 'severity'
                                                         , label    : i18n.labelActivityLogSeverity
                                                         , width    : '15%'
                                                         , sortable : true }
                                                       , { id       : 'time'
                                                         , label    : i18n.labelActivityLogTime
                                                         , width    : '25%'
                                                         , sortable : true
                                                         , initSort : 'desc' }
                                                       , { id       : 'message'
                                                         , label    : i18n.labelActivityLogMessage
                                                         , width    : '60%'
                                                         , sortable : true } ] } );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'download'
          , grayout : false
          , label   : i18n.buttonActivityLogDownloadAll
          , onclick : function( event )
            {
              if( window.confirm( tsa.i18n.labelActivityLogDownloadAllConfirm ) )
              {
                logger.info( 'All logs will be downloaded...' );
                tsa.cmd.handlers.LogsHandler.downloadAllLogs();
                try
                {
                  $( event.target ).css( { 'opacity' : '0.5' } );
                  $( event.target ).attr( 'disabled', 'disabled' );
                }
                catch( error ) { /* Ignore. */ }
              }
            }
          }
        )
      );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : true
          , label   : i18n.buttonActivityLogClear
          , onclick : function( event )
            {
              if( window.confirm( tsa.i18n.labelActivityLogClearConfirm ) )
              {
                logger.info( 'Audit log will be cleared...' );
                tsa.cmd.handlers.LogsHandler.clearLog();
              }
            }
          }
        )
      );
      
      contentArea.append( tablePanel.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show log entry.
     * Entry will be retrieved from the current buffer.
     * Entry ID retrieved from ShowCommand parameters ('id').
     */
    tsa.cmd.handlers.LogsHandler.prototype._show = function()
    {
      /** @type {string}                  */ var id = '';
      /** @type {tsa.widgets.ContentArea} */ var contentArea = null;
      
      contentArea = new ContentArea ( { title         : tsa.i18n.labelActivityLogEntryDetailsHeading
                                      , helpDoc       : 'sshelp_activity_log_panel.html'
                                      , showBackToTop : true } );
      
      if( TV.typeIsString( typeof this.getParameter( 'id' ) ) )
      {
        id = this.getParameter( 'id' );
      }
      else if( TV.typeIsString( typeof this.getParameter( 'entityId' ) ) )
      {
        id = this.getParameter( 'entityId' );
      }
      
      if( id.length > 0 )
      {
        var showEntry = $.proxy( function( entry )
        {
          
          if( entry !== null )
          {
            contentArea.append( tsa.cmd.handlers.LogsHandler.buildElementForLogEntry( entry ) );
          }
        }
        , this );
        
        if( TV.typeIsObject( typeof tsa.cmd.handlers.LogsHandler.entryBuffer[ id ] ) )
        {
          showEntry( tsa.cmd.handlers.LogsHandler.entryBuffer[ id ] );
        }
        else
        {
          // Try to retrieve from server? (in case client doesn't have it yet)...
          /** @type {tsa.cmd.Command} */ var cmd = new Command( { action              : 'logs/getLogEntry'
                                                                , parameters          : { 'id' : id }
                                                                , clearSystemMessages : false } );
          $( cmd ).on( 'executionSucceeded', function( event )
          {
            showEntry( event.response.payload.entry );
          } );
          
          cmd.execute();
        }
      }
      else
      {
        logger.warn( 'Couldn\'t show entry: ID was not specified.' );
      }
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Auxiliary method for building an HTML Element to represent a log entry,
     * when the data is already available.
     * @private
     * @param {Object} entry The object containing the entry information.
     * @return {Element}
     */
    tsa.cmd.handlers.LogsHandler.buildElementForLogEntry = function( entry )
    {
      /** @type {Element} */ var result = null;
      
      if( TV.typeIsObject( typeof entry ) && ( entry !== null ) )
      {
        /** @type {tsa.widgets.TablePanel} */ var tp = null;
        
        /** @type {Object}  */ var i18n            = tsa.i18n;
        /** @type {Element} */ var elForSeverity   = null;
        /** @type {string}  */ var strForTime      = ( TV.typeIsString( typeof entry.time ) ? entry.time : '?' );
        /** @type {string}  */ var strForMessage   = ( TV.typeIsString( typeof entry.message ) ? entry.message : '?' );
        /** @type {string}  */ var strForThrowable = '';
        
        elForSeverity = IconFactory.buildSeverityIcon( { severity : ( TV.typeIsNumber( typeof entry.severity ) ? entry.severity : -1 )
                                                       , withText : true } );
        
        if( TV.typeIsObject( typeof entry.throwable ) && ( entry.throwable !== null ) )
        {
          /** @type {Object} */ var cause = entry.throwable;
          
          do
          {
            strForThrowable += ( ( strForThrowable.length > 0 ) ? ' caused by ' : '' )
                            +  cause.name + ' (' + cause.message + ')'
          } while( TV.typeIsObject( typeof cause.cause ) && ( ( cause = cause.cause ) !== null ) );
        }
        
        tp = new TablePanel( { title    : ( entry.isChild ? i18n.labelActivityLogXDetailsHeading
                                                          : i18n.labelActivityLogDetailsHeading )
                             , emptyMsg : i18n.labelActivityLogNoDetails
                             , columns  : [ { width : '40%' }
                                          , { width : '60%' } ]
                             } );
        
        tp.appendRow( [ i18n.labelActivityLogMessage   + ':' , strForMessage   ] );
        tp.appendRow( [ i18n.labelActivityLogSeverity  + ':' , elForSeverity   ] );
        tp.appendRow( [ i18n.labelActivityLogTime      + ':' , strForTime      ] );
        tp.appendRow( [ i18n.labelActivityLogException + ':' , strForThrowable ] );
        
        if( TV.typeIsObject( typeof entry.children ) && ( entry.children !== null ) &&
            TV.isInstanceOf( entry.children, Array ) && ( entry.children.length > 0 ) )
        {
          result = document.createElement( 'div' );
          $( result ).append( tp.getDomNode() );
          
          $.each( entry.children, function( childIndex, child )
          {
            $( result ).append( tsa.cmd.handlers.LogsHandler.buildElementForLogEntry( child ) );
          } );
        }
        else
        {
          result = tp.getDomNode();
        }
      }
      else
      {
        logger.error( 'Couldn\'t build HTML Element for Log Entry: invalid entry data.' );
      }
      
      return result;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Request (for download) a file containing all the logs entries.
     */
    tsa.cmd.handlers.LogsHandler.downloadAllLogs = function()
    {
      /** @type {tsa.cmd.Command} */ var cmd = null;
      
      cmd = new Command( { action              : 'logs/download/download.zip'
                         , baseURL             : Command.URL_SERVLET_CONTEXT
                         , blockUI             : false
                         , ajax                : false
                         , clearSystemMessages : false } );
      cmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Request for clearing the log.
     */
    tsa.cmd.handlers.LogsHandler.clearLog = function()
    {
      /** @type {tsa.cmd.Command} */ var cmd = null;
      
      cmd = new Command( { action : 'logs/ajaxClear' } );
      cmd.execute();
      $( cmd ).on( 'executionSucceeded', function( event )
          {
            (new ShowCommand( {  
                action : 'logs/index'
              , clearSystemMessages : false } )  
            ).execute();
          } );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.LogsHandler;
  }
);