/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/handlers/SetupwizardHandler'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ContentArea
          , IconFactory
          , TablePanel 
          , SetupwizardHandler )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'MainHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.MainHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.MainHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.MainHandler.prototype ).constructor = tsa.cmd.handlers.MainHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * @override
     * @inheritDoc 
     */
    tsa.cmd.handlers.MainHandler.prototype.shouldAutoRefresh = function( commandName )
    {
      /** @type {?number} */ var seconds = null;
      
      switch( commandName )
      {
        case 'index' : seconds = 60; break;
        default      : seconds = AbstractHandler.prototype.shouldAutoRefresh.call( this, commandName );
      }
      
      return seconds;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Show summary screen.
     */
    tsa.cmd.handlers.MainHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n             = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea      = new ContentArea( { title   : i18n.labelSummaryHeading
                                                                                     , helpDoc : 'sshelp_summary_panel.html' } );
      /** @type {tsa.widgets.TablePanel}  */ var sysStatusPanel   = null;
      /** @type {tsa.widgets.TablePanel}  */ var jobSummaryPanel  = null;
      /** @type {tsa.widgets.TablePanel}  */ var invSummaryPanel  = null;
      /** @type {tsa.widgets.TablePanel}  */ var productInfoPanel = null;
      /** @type {tsa.cmd.Command}         */ var getSysSummaryCmd = new Command( { action              : 'main/getSysSummary'
                                                                                 , clearSystemMessages : false } );
      
      sysStatusPanel   = new TablePanel( { title        : i18n.labelSummaryStatusHeading
                                         , summary      : i18n.labelSummaryStatusDesc
                                         , emptyMsg     : i18n.labelSummaryNoStatus
                                         , columns      : [ { id : 'service'  , label : i18n.labelSummaryService  , width : '25%' , sortable : false }
                                                          , { id : 'status'   , label : i18n.labelSummaryStatus   , width : '10%' , sortable : false }
                                                          , { id : 'comments' , label : i18n.labelSummaryComments , width : '65%' , sortable : false } ] } );
      jobSummaryPanel  = new TablePanel( { title        : i18n.labelSummaryJobsHeading
                                         , description  : i18n.labelSummaryJobsCaption
                                         , summary      : i18n.labelSummaryJobsSummary
                                         , emptyMsg     : i18n.labelSummaryNoJobs
                                         , columns      : [ { id : 'name'     , label : i18n.labelSummaryName     , width : '30%' , sortable : false }
                                                          , { id : 'state'    , label : i18n.labelSummaryState    , width : '15%' , sortable : false }
                                                          , { id : 'comments' , label : i18n.labelSummaryComments , width : '40%' , sortable : false }
                                                          , { id : 'actions'  , label : i18n.labelActionsColumn   , width : '15%' , sortable : false } ] } );
      invSummaryPanel  = new TablePanel( { title        : i18n.labelSummaryInventoryHeading
                                         , description  : i18n.labelSummaryInventoryCaption
                                         , emptyMsg     : i18n.labelSummaryNoInventory
                                         , columns      : [ { id : 'unique'   , label : ''                        , width : '100%' , sortable : false } ] } );
      productInfoPanel = new TablePanel( { title        : i18n.labelSummaryProdInfoHeading
                                         , description  : i18n.labelSummaryProdInfoCaption
                                         , summary      : i18n.labelSummaryProductSummary
                                         , emptyMsg     : i18n.labelSummaryNoProdInfo
                                         , columns      : [ { id : 'property' , label : i18n.labelSummaryProperty , width : '45%' , sortable : false }
                                                          , { id : 'value'    , label : i18n.labelSummaryValue    , width : '55%' , sortable : false } ] } );
      
      $( getSysSummaryCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object}  */ var payload                      = event.response.payload;
        /** @type {Object}  */ var systemHealth                 = payload.systemHealth;
        /** @type {Object}  */ var jobs                         = payload.jobs;
        /** @type {Object}  */ var inventorySummary             = payload.inventorySummary;
        /** @type {boolean} */ var isInventorySummaryJobRunning = payload.isInventorySummaryJobRunning;
        /** @type {Object}  */ var productData                  = payload.productData;
        
        // Health...
        try
        {
          if( TV.typeIsObject( typeof systemHealth        ) && ( systemHealth !== null ) &&
              TV.typeIsObject( typeof systemHealth.issues ) && TV.isInstanceOf( systemHealth.issues, Array ) )
          {
            $.each( systemHealth.issues, function( issueIndex, issue )
            {
              if( TV.typeIsObject( typeof issue ) && ( issue !== null ) && !issue.isHidden )
              {
                /** @type {Array.<(string|Element)>} */ var sysHealthRow = [];
                /** @type {Element}                  */ var labelWrapper = document.createElement( 'strong' );
                /** @type {Element}                  */ var detailList   = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get(0);
                
                // Label...
                $( labelWrapper ).text( TV.typeIsString( typeof i18n[ issue.label ] ) ? i18n[ issue.label ] : issue.label );
                
                if( issue.isLink && TV.typeIsString( typeof issue.link ) )
                {
                  /** @type {Element} */ var link = document.createElement( 'a' );
                  
                  $( link ).append( labelWrapper );
                  $( link ).addClass( 'ibm-feature-link' );
                  $( link ).attr( 'href', '#' );
                  $( link ).on( 'click', function( event )
                  {
                    event.preventDefault();
                    ( new ShowCommand( { action : ( issue.link + '/index' ) } ) ).execute();
                    return false;
                  } );
                  
                  sysHealthRow.push( link );
                }
                else
                {
                  sysHealthRow.push( labelWrapper );
                }
                
                // Severity...
                sysHealthRow.push( IconFactory.buildSeverityIcon( { severity : issue.severity } ) );
                
                // Details (children)...
                if( TV.typeIsObject( typeof issue.children ) && TV.isInstanceOf( issue.children, Array ) )
                {
                  if( issue.children.length > 0 )
                  {
                    $.each( issue.children, function( detailIndex, detail )
                    {
                      if( TV.typeIsObject( typeof detail ) && ( detail !== null ) )
                      {
                        if( TV.typeIsString( typeof detail.message ) && ( detail.message.indexOf( 'TADDM' ) < 0 ) )
                        {
                          /** @type {Element} */ var detailListItem = $( '<li style="padding-top: 0px;"></li>' ).get(0);
                          
                          if( detail.severity === 'ERROR' )
                          {
                            $( detailListItem ).css( { 'color' : 'red' } );
                          }
                          
                          $( detailListItem ).text( detail.message );
                          $( detailList ).append( detailListItem );
                        }
                      }
                    } );
                  }
                  else
                  {
                    /** @type {Element} */ var noDetailsListItem = $( '<li style="padding-top: 0px;"></li>' ).get(0);
                    
                    $( noDetailsListItem ).text( issue.message );
                    $( detailList ).append( noDetailsListItem );
                  }
                }
                
                sysHealthRow.push( detailList );
                sysStatusPanel.appendRow( sysHealthRow );
              }
            } );
          }
        }
        catch( error )
        {
          logger.error( error );
        }
        
        // Jobs...
        try
        {
          if( TV.typeIsObject( typeof jobs ) && TV.isInstanceOf( jobs, Array ) )
          {
            $.each( jobs, function( jobIndex, job )
            {
              if( TV.typeIsObject( typeof job ) && ( job !== null ) )
              {
                /** @type {Element}                  */ var nameWrapper    = document.createElement( 'strong' );
                /** @type {Array.<(string|Element)>} */ var jobRow         = [];
                /** @type {Element}                  */ var statusMsgsList = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get( 0 );
                /** @type {Element}                  */ var cancelButton   = IconFactory.buildActionIcon( { type: 'cancel', caption: i18n.buttonCancel + ' ' + job.name } );
                
                $( nameWrapper ).text( job.name );
                
                if( TV.typeIsObject( typeof job.statusMessages ) && TV.isInstanceOf( job.statusMessages, Array ) )
                {
                  $.each( job.statusMessages, function( statusMsgIndex, statusMsg )
                  {
                    /** @type {Element} */ var statusMsgListItem = $( '<li style="padding-top: 0px;"></li>' ).get( 0 );
                    
                    $( statusMsgListItem ).text( statusMsg );
                    $( statusMsgsList ).append( statusMsgListItem );
                  } );
                }
                
                $( cancelButton ).css( { 'margin-right' : '0.5em'
                                       , 'cursor'       : 'pointer' } );
                
                if( job.allowCancel )
                {
                  $( cancelButton ).on( 'click', function( event )
                  {
                    /** @type {tsa.cmd.Command} */ var cancelJobCmd = null;
                    
                    cancelJobCmd = new Command( { action : 'main/ajaxStopJob/' + job.id } );
                    
                    $( cancelJobCmd ).on( 'executionSucceeded', function( event )
                    {
                      ( new ShowCommand( { action: 'main/index', clearSystemMessages: false } ) ).execute();
                    } );
                    
                    cancelJobCmd.execute();
                  } );
                }
                else
                {
                  $( cancelButton ).hide();
                }
                
                jobRow.push( nameWrapper    );
                jobRow.push( job.stateText  );
                jobRow.push( statusMsgsList );
                jobRow.push( cancelButton   );
                
                jobSummaryPanel.appendRow( jobRow );
              }
            } );
          }
        }
        catch( error )
        {
          logger.error( error );
        }
        
        // Inventory summary...
        try
        {
          if( TV.typeIsObject( typeof inventorySummary ) && ( inventorySummary !== null ) && ( inventorySummary.collectedOn >= 0 ) )
          {
            /** @type {Element}                  */ var inventorySummaryDiv = document.createElement( 'div' );
            /** @type {Array.<(string|Element)>} */ var inventorySummaryRow = [ inventorySummaryDiv ];
            /** @type {Element}                  */ var invSumList          = $( '<ul style="list-style-type: none; margin:0px; padding:0px;"></ul>' ).get( 0 );
            
            $( inventorySummaryDiv ).css( { 'white-space' : 'nowrap'
                                          , 'cursor'      : 'pointer' } );
            
            $( inventorySummaryDiv ).on( 'click', function( event )
            {
              ( new ShowCommand( { action : 'inventorysummary/index' } ) ).execute();
            } );
            
            if( TV.typeIsObject( typeof inventorySummary.groups ) && TV.isInstanceOf( inventorySummary.groups, Array ) )
            {
              $.each( inventorySummary.groups, function( groupIndex, group )
              {
                if( group !== null )
                {
                  /** @type {Element} */ var invSumListItem = $( '<li style="padding-top: 5px; padding-right: 15px; float: left; width: 30%;"></li>' ).get( 0 );
                  /** @type {Element} */ var groupIcon      = null;
                  /** @type {Element} */ var groupLabel     = document.createElement( 'span' );
                  
                  if( group.isResultsSuspect )
                  {
                    groupIcon = IconFactory.buildSeverityIcon( { severity : 'ERROR' } );
                  }
                  else if( !TV.typeIsString( typeof group.iconName ) )
                  {
                    groupIcon = IconFactory.buildSeverityIcon( { severity : 'WARN' } );
                  }
                  else
                  {
                    groupIcon = document.createElement( 'img' );
                    groupIcon.alt = "";
                    groupIcon.src = tsa.attributes.imagesPath + '/' + 'inventory' + '/' + group.iconName;
                  }
                  
                  if( group.isResultsSuspect )
                  {
                    $( groupLabel ).css( { 'color' : 'red' } );
                  }
                  
                  $( groupLabel ).text( i18n[ group.label ] + ' (' + ( TV.typeIsNumber( typeof group.count ) ? group.count : '?' ) + ')' );
                  
                  $( invSumListItem ).append( groupIcon );
                  $( invSumListItem ).append( document.createTextNode( tsa.NBSP ) );
                  $( invSumListItem ).append( groupLabel );
                  $( invSumList ).append( invSumListItem );
                }
              } );
            }
            
            $( inventorySummaryDiv ).append( invSumList );
            invSummaryPanel.appendRow( inventorySummaryRow );
          }
          else if( isInventorySummaryJobRunning )
          {
            /** @type {Element}                  */ var invJobRunningDiv = document.createElement( 'div' );
            /** @type {Array.<(string|Element)>} */ var invJobRunningRow = [ invJobRunningDiv ];
            
            $( invJobRunningDiv ).text( i18n.labelInventorySummaryGenerationInProgress );
            invSummaryPanel.appendRow( invJobRunningRow );
          }
        }
        catch( error )
        {
          logger.error( error );
        }
        
        // Product information...
        try
        {
          if( TV.typeIsObject( typeof productData            ) && ( productData !== null ) &&
              TV.typeIsObject( typeof productData.properties ) && TV.isInstanceOf( productData.properties, Array ) )
          {
            $.each( productData.properties, function( propIndex, prop )
            {
              if( !prop.isDebugOnly || tsa.isDebugModeOn() )
              {
                productInfoPanel.appendRow( [ i18n[ prop.label ], '' + prop.value ] );
              }
            } );
            
            if( tsa.isDebugModeOn() )
            {
              productInfoPanel.appendRow( [ 'Debug mode', tsa.isDebugModeOn() ] );
            }
          }
        }
        catch( error )
        {
          logger.error( error );
        }
      } );
      
      getSysSummaryCmd.execute();
      
      //delete the wizard menu
      /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
      setupwizardHandler.exitWizard();
      
      contentArea.append( sysStatusPanel   .getDomNode() );
      contentArea.append( jobSummaryPanel  .getDomNode() );
      contentArea.append( invSummaryPanel  .getDomNode() );
      contentArea.append( productInfoPanel .getDomNode() );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.MainHandler;
  }
);