/*
 * IBM Technical Support Appliance Version 2 Release 3 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
          , FormPanel
          , TabsPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NetdnsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NetdnsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NetdnsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NetdnsHandler.prototype ).constructor = tsa.cmd.handlers.NetdnsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NetdnsHandler.prototype._index = function()
    {
      this.initNetwork();
      /** @type {Object}                  */   var i18n              = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */   var contentArea       = new ContentArea ( { title : i18n.labelNetworkHeading
                                                                                       , helpDoc : 'sshelp_netdns_panel.html'} );
      /** @type {tsa.cmd.Command}           */ var getNetworkCommand = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.FormPanel}     */ var formNameServ      = new FormPanel( { title  : i18n.labelNetdnsDnsHeading 
                                                                                      , action : 'netdns/putUpdateNetwork'  });
      /** @type {tsa.widgets.FormPanel}     */ var formDHCP          = new FormPanel( { title  : i18n.labelNetdnsDhcpIfHeadingDesc });
      /** @type {tsa.widgets.FormPanel}     */ var formDNS           = new FormPanel( { title  : i18n.labelNetdnsServersHeading })
      /** @type {tsa.widgets.FormPanel}     */ var formDomainSSO     = new FormPanel( { title  : i18n.labelNetdnsDomainsHeading })
      
      formNameServ.linkTo( formDHCP );
      formDHCP.linkTo( formDNS );
      formDNS.linkTo( formDomainSSO );
      
      
      /** @type {Element} */ var bodyTabGlobal      = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNInterface  = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabDNSSettings = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNRoutes     = document.createElement( 'p' );
      $( bodyTabGlobal      ).text( i18n.labelNetworkCaption  );
      $( bodyTabNInterface  ).text( i18n.labelNetifCaption    );
      $( bodyTabDNSSettings ).text( i18n.labelNetdnsCaption   );
      $( bodyTabNRoutes     ).text( i18n.labelNetrouteCaption );
      /** @type {tsa.widgets.TabsPanel}  */ var tabsPanel    = new TabsPanel( { tabs: [ { title : i18n.labelNetworkGlobalHeading     , body : bodyTabGlobal
                                                                                         , action : "netglobal/index" }
                                                                                      , { title : i18n.labelNetworkInterfacesHeading , body : bodyTabNInterface
                                                                                          , action : "netif/index" }
                                                                                      , { title : i18n.labelNetworkDnsHeading        , body : bodyTabDNSSettings
                                                                                          , action : "netdns/index" }
                                                                                      , { title : i18n.labelNetrouteHeading          , body : bodyTabNRoutes
                                                                                          , action : "netroute/index"
                                                                                        }] } );
      //move to tab 2
      tabsPanel.goToTab(2);
      //form Name services
      /** @type {Element} */ var selectNameServ = document.createElement( 'select' );
      selectNameServ.name = "dns";
      selectNameServ.id = "dns";
      opt = document.createElement( 'option' );
      opt.value = "STATIC";
      $( opt ).text( i18n.labelNetdnsDnsStatic ); 
      $( selectNameServ ).append( opt );
      
      opt = document.createElement( 'option' );
      opt.value = "DHCP";
      $( opt ).text( i18n.labelNetdnsDnsDhcp ); 
      $( selectNameServ ).append( opt );
      
      formNameServ.setDescription(i18n.labelNetdnsDnsHeadingDesc);
      formNameServ.appendFieldRow( { label    : i18n.labelNetdnsDns
        , required : true
        , element  : selectNameServ
        } );
      //DHCP Interface form
      /** @type {Element} */ var selectDHCP = document.createElement( 'select' );
      selectDHCP.name = "dhcpIfName";
      selectDHCP.id   = "dhcpIfName";
      formDHCP.setDescription(i18n.labelNetdnsDhcpIfHeadingDesc);
      formDHCP.appendFieldRow( { label    : i18n.labelNetdnsDhcpIfName
        , required : true
        , element  : selectDHCP
        } );
      //Form DNS Server Search Order
      formDNS.setDescription(i18n.labelNetdnsServersDesc);
      var server1 = $( '<input size="50" maxlength="250" name="server1" id="server1" value="" type="text">' ).get(0);
      var server2 = $( '<input size="50" maxlength="250" name="server2" id="server2" value="" type="text">' ).get(0);
      var server3 = $( '<input size="50" maxlength="250" name="server3" id="server3" value="" type="text">' ).get(0)
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer1
                              , required : true
                              , hint     : i18n.labelNetdnsServer1Desc
                              , element  : server1
        } );
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer2
                              , hint     : i18n.labelNetdnsServer2Desc
                              , element  : server2
        } );
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer3
                              , hint     : i18n.labelNetdnsServer3Desc
                              , element  : server3
        } );
      // Domain suffix search order
      
      formDomainSSO.setDescription(i18n.labelNetdnsDomainsDesc);
      
      var domain1 = $( '<input size="50" maxlength="250" name="domain1" id="domain1" value="" type="text">' ).get(0);
      var domain2 = $( '<input size="50" maxlength="250" name="domain2" id="domain2" value="" type="text">' ).get(0)
      var domain3 = $( '<input size="50" maxlength="250" name="domain3" id="domain3" value="" type="text">' ).get(0)
      formDomainSSO.appendFieldRow( { label    : i18n.labelNetdnsDomain1
                              , hint     : i18n.labelNetdnsDomain1
                              , element  : domain1
        } );
      formDomainSSO.appendFieldRow( { label    : i18n.labelNetdnsDomain2
                              , hint     : i18n.labelNetdnsDomain2Desc
                              , element  : domain2
        } );
      formDomainSSO.appendFieldRow( { label    : i18n.labelNetdnsDomain3
                              , hint     : i18n.labelNetdnsDomain3Desc
                              , element  : domain3
        } );
      var changeHandler = function( event )
      {
        /** @type {boolean} */ var disabled = ( $( selectNameServ ).val() === 'DHCP' );
        server1.disabled = disabled;
        server2.disabled = disabled;
        server3.disabled = disabled;
        domain1.disabled = disabled;
        domain2.disabled = disabled;
        domain3.disabled = disabled;
        selectDHCP.disabled = !disabled;
      };
      $( selectNameServ ).on( 'change', changeHandler );
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
              {
              $( formNameServ ).on( 'executionSucceeded', function( e )
                  {
                   //if reboot needed ask user if restart now
                    if(e.response.payload["reboot"])
                    {
                      if ( confirm('System restart is required. Restart system now?') ) {
                        tsa.cmd.handlers.NetdnsHandler.prototype.rebootSystem('auto');
                      } else {
                        tsa.cmd.handlers.NetdnsHandler.prototype.rebootSystem('user');
                      }
                    }
                    ( new ShowCommand( { action : 'netdns/index' , clearSystemMessages : false  } ) ).execute();
                  } );
              // Show errors
              $( formNameServ ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        formNameServ.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );
              formNameServ.submit();
              }
            }
          )
        );
      getNetworkCommand = new Command( {
        action:'netdns/getNetworkData'
      , clearSystemMessages:false
    } );
    
    getNetworkCommand.execute();
    
    
    
    
    $( getNetworkCommand ).on( 'executionSucceeded', function(e) {
      for( var key in e.response.payload.networkData ) {
        $( '[name="' + key + '"]' ).val( e.response.payload.networkData[ key ] );
      }
      for( var key in e.response.payload.nifElements ) {
        /** @type {object} */  var nif = e.response.payload.nifElements[key] ;
        /** @type {element} */ var optIf = document.createElement("option");     
        optIf.value = nif.name ;
        $( optIf ).text( nif.pid ); 
        $(selectDHCP).append(optIf);
        if(( $( selectNameServ ).val() === 'STATIC' )){
          selectDHCP.disabled = true;
        } else {
            server1.disabled = true;
            server2.disabled = true;
            server3.disabled = true;
            domain1.disabled = true;
            domain2.disabled = true;
            domain3.disabled = true;
            
        }
      }
    } );
      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( formNameServ.getDomNode() );
      contentArea.append( formDHCP.getDomNode() );
      contentArea.append( formDNS.getDomNode() );
      contentArea.append( formDomainSSO.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.NetdnsHandler.prototype.initNetwork = function(){
      $("#ibm-secondary-navigation").show();  
      /** @type {element} */ var aNetwork     = $("#aNetworkID");
      $( aNetwork ).unbind( "click" );
      $(aNetwork).text(tsa.i18n.labelNavNetworkBasic );
      $(aNetwork).on("click", function(){
                                       ( new ShowCommand( { action : ( 'network/index') } ) ).execute();
                                        }); 
   };
   /**
    * Request for restarting system.
    * @param {string} restart Restart mode. Use either:
    *                         'auto' : Restart will be initiated by the system.
    *                         'user' : Restart should be initiated by the user.
    */
   tsa.cmd.handlers.NetdnsHandler.prototype.rebootSystem = function( restart )
   {
      /** @type {tsa.cmd.Command} */ var cmd = null;
      
      cmd = new Command( { action              : 'network/ajaxReboot'
                         , clearSystemMessages : false
                         , parameters          : { "reboot" : restart } 
                         } );
      cmd.execute();
   };
   //-------------------------------------------------------------------------
    return tsa.cmd.handlers.NetdnsHandler;
  }
);