/*
 * IBM Technical Support Appliance Version 2 Release 3 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
          , FormPanel
          , TabsPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NetglobalHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NetglobalHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NetglobalHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NetglobalHandler.prototype ).constructor = tsa.cmd.handlers.NetglobalHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NetglobalHandler.prototype._index = function()
    {
      this.initNetwork();
      /** @type {Object}                  */ var i18n          = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea   = new ContentArea ( { title : i18n.labelNetworkHeading 
                                                                                 , helpDoc : 'sshelp_netglobal_panel.html' } );
      /** @type {tsa.widgets.FormPanel}   */ var formIdentify  = new FormPanel( { title  : i18n.labelNetglobalIdentityHeading
                                                                                , action : 'netglobal/putUpdateNetwork'     } );
      /** @type {tsa.cmd.Command}           */ var getNetworkCommand = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
      /** @type {Element} */ var bodyTabGlobal      = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNInterface  = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabDNSSettings = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNRoutes     = document.createElement( 'p' );
      $( bodyTabGlobal      ).text( i18n.labelNetworkCaption  );
      $( bodyTabNInterface  ).text( i18n.labelNetifCaption    );
      $( bodyTabDNSSettings ).text( i18n.labelNetdnsCaption   );
      $( bodyTabNRoutes     ).text( i18n.labelNetrouteCaption );
      
      /** @type {tsa.widgets.TabsPanel}  */ var tabsPanel    = new TabsPanel( { tabs: [ { title : i18n.labelNetworkGlobalHeading     , body : bodyTabGlobal
                                                                                         , action : "netglobal/index" }
                                                                                      , { title : i18n.labelNetworkInterfacesHeading , body : bodyTabNInterface
                                                                                          , action : "netif/index" }
                                                                                      , { title : i18n.labelNetworkDnsHeading        , body : bodyTabDNSSettings
                                                                                          , action : "netdns/index" }
                                                                                      , { title : i18n.labelNetrouteHeading          , body : bodyTabNRoutes
                                                                                          , action : "netroute/index"
                                                                                        }] } );
                                                                                      
      formIdentify.appendFieldRow( { label    : i18n.labelNetglobalHost
        , required : true
        , hint     : i18n.labelNetglobalHostDesc
        , element  : $( '<input size="50" maxlength="250" name="hostname" id="hostname" value="" type="text">' ).get(0)
        } );
      formIdentify.appendFieldRow( { label    : i18n.labelNetglobalDomainName
        , required : true
        , hint     : i18n.labelNetglobalDomainNameDesc
        , element  : $( '<input size="50" maxlength="250" name="domain" id="domain" value="" type="text">' ).get(0)
        } );
      
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
              {
              $( formIdentify ).on( 'executionSucceeded', function( e )
                  {
                   //if reboot is needed then ask user if restart now
                    if(e.response.payload["reboot"])
                    {
                      if ( confirm('System restart is required. Restart system now?') ) {
                        tsa.cmd.handlers.NetglobalHandler.prototype.rebootSystem('auto');
                      } else {
                        tsa.cmd.handlers.NetglobalHandler.prototype.rebootSystem('user');
                      }
                    }
                    ( new ShowCommand( { action : 'netglobal/index' , clearSystemMessages : false  } ) ).execute();
                  } );
              // Show errors
              $( formIdentify ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        formIdentify.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );  
                formIdentify.submit();
              }
            }
          )
        );
      getNetworkCommand = new Command( {
        action:'netglobal/getNetworkData'
      , clearSystemMessages:false
    } );
    
    getNetworkCommand.execute();
   
    $( getNetworkCommand ).on( 'executionSucceeded', function(e) {
    
      $( '[name="hostname"]' ).val( e.response.payload.hostname );
      $( '[name="domain"]'   ).val( e.response.payload.domain   );
    } );
      
      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( formIdentify.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Enable secondary navigation
     */
    tsa.cmd.handlers.NetglobalHandler.prototype.initNetwork = function(){
       $("#ibm-secondary-navigation").show();  
       /** @type {element} */ var aNetwork     = $("#aNetworkID");
       $( aNetwork ).unbind( "click" );
       $(aNetwork).text(tsa.i18n.labelNavNetworkBasic );
       $(aNetwork).on("click", function(){
                                        ( new ShowCommand( { action : ( 'network/index') } ) ).execute();
                                         }); 
    };
   //-------------------------------------------------------------------------
    /**
     * Request for restarting system.
     * @param {string} restart Restart mode. Use either:
     *                         'auto' : Restart will be initiated by the system.
     *                         'user' : Restart should be initiated by the user.
     */
    tsa.cmd.handlers.NetglobalHandler.prototype.rebootSystem = function( restart )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;
       
       cmd = new Command( { action              : 'network/ajaxReboot'
                          , clearSystemMessages : false
                          , parameters          : { "reboot" : restart } 
                          } );
       cmd.execute();
    };
  //-------------------------------------------------------------------------
    return tsa.cmd.handlers.NetglobalHandler;
  }
);