/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
          , FormPanel
          , TabsPanel
          )
    {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NetifHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NetifHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NetifHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NetifHandler.prototype ).constructor = tsa.cmd.handlers.NetifHandler;
    //subnet values.
    tsa.cmd.handlers.NetifHandler.prototype.subnetArray= [ "subnetMask0000", "subnetMask128000",
        "subnetMask192000", "subnetMask224000", "subnetMask240000", "subnetMask248000", "subnetMask252000",
        "subnetMask254000", "subnetMask255000", "subnetMask25512800", "subnetMask25519200",
        "subnetMask25522400", "subnetMask25524000", "subnetMask25524800", "subnetMask25525200",
        "subnetMask25525400", "subnetMask25525500", "subnetMask2552551280", "subnetMask2552551920",
        "subnetMask2552552240", "subnetMask2552552400", "subnetMask2552552480", "subnetMask2552552520",
        "subnetMask2552552540", "subnetMask2552552550", "subnetMask255255255128",
        "subnetMask255255255192", "subnetMask255255255224", "subnetMask255255255240",
        "subnetMask255255255248", "subnetMask255255255252", "subnetMask255255255254",
        "subnetMask255255255255" ];
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NetifHandler.prototype._index = function()
    {
      this.initNetwork();
      /** @type {Object}                  */   var i18n                 = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */   var contentArea          = new ContentArea ( { title : i18n.labelNetworkHeading
                                                                                          , helpDoc : 'sshelp_netifs_panel.html'} );
      /** @type {tsa.cmd.Command}           */ var getNetworkCommand    = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer      = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.FormPanel}     */ var formIPAssign         = new FormPanel( { title  : i18n.labelNetifAssignHeading
                                                                                         , action : 'netif/putUpdateNetwork'     } );
      /** @type {tsa.widgets.FormPanel}     */ var formIPConfig       = new FormPanel( { title  : i18n.labelNetifStaticHeading });
      /** @type {tsa.widgets.FormPanel}     */ var formGatewayRoute   = new FormPanel( { title  : i18n.labelNetifDefGatewayIfHeading });
      /** @type {tsa.widgets.FormPanel}     */ var formDefaultGateway = new FormPanel( { title  : i18n.labelNetifDefGatewayHeading });
      
      
      //Link formPanel
      formIPAssign.linkTo(formIPConfig);
      formIPConfig.linkTo(formGatewayRoute);
      formGatewayRoute.linkTo(formDefaultGateway);
      
      /** @type {Element} */ var bodyTabGlobal      = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNInterface  = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabDNSSettings = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNRoutes     = document.createElement( 'p' );
      
      $( bodyTabGlobal      ).text( i18n.labelNetworkCaption  );
      $( bodyTabNInterface  ).text( i18n.labelNetifCaption    );
      $( bodyTabDNSSettings ).text( i18n.labelNetdnsCaption   );
      $( bodyTabNRoutes     ).text( i18n.labelNetrouteCaption );
      
      /** @type {tsa.widgets.TabsPanel}  */ var tabsPanel    = new TabsPanel( { tabs: [ {  title : i18n.labelNetworkGlobalHeading     
                                                                                        ,   body : bodyTabGlobal
                                                                                        , action : "netglobal/index" }
                                                                                      , { title : i18n.labelNetworkInterfacesHeading  , body : $('<div></div>').get(0)
                                                                                        , action : "netif/index" }
                                                                                      , {  title : i18n.labelNetworkDnsHeading        , body : bodyTabDNSSettings
                                                                                        , action : "netdns/index" }
                                                                                      , {  title : i18n.labelNetrouteHeading          , body : bodyTabNRoutes
                                                                                        , action : "netroute/index"
                                                                                        }] } );
      tabsPanel.goToTab(1);
      //panel IP Assignment
      /** @type {Element} */ var selectAssign = document.createElement( 'select' );
      selectAssign.name = "assignment" ;
      selectAssign.id   = "assignment" ;
      /** @type {Element} */ var optAssign = document.createElement( 'option' );
      optAssign.value = "STATIC";
      $( optAssign ).text( i18n.labelNetifAssignStatic ); 
      $( selectAssign ).append( optAssign );
      
      optAssign = document.createElement( 'option' );
      optAssign.value = "DYNAMIC";
      $( optAssign ).text( i18n.labelNetifAssignDynamic ); 
      $( selectAssign ).append( optAssign );
      
      formIPAssign.setDescription(i18n.labelNetifAssignDesc);
      formIPAssign.appendFieldRow( {  label    : i18n.labelNetifAssign
                                    , required : true
                                    , element  : selectAssign
                                    } );
      
      //Panel IP Configuration
      /** @type {Object} */ var inputStaticAdress = $( '<input size="50" maxlength="250" name="address" id="address" value="" type="text">' ).get(0);
      formIPConfig.setDescription(i18n.labelNetifStaticDesc);
      formIPConfig.appendFieldRow( {  label    : i18n.labelNetifStaticAddress
                                    , required : true
                                    , hint     : i18n.labelNetifStaticAddressDesc
                                    , element  : inputStaticAdress
                                   } );
      /** @type {Element} */ var selectSubnet = document.createElement( 'select' );
      /** @type {Element} */ var opt          = document.createElement( 'option' );
      selectSubnet.name = "subnet";
      selectSubnet.id   = "subnet";
      $.each( this.subnetArray, function( idx, element ) {
            opt = document.createElement( 'option' );
            opt.value = i18n[element];
            $( opt ).text( i18n[element] ); 
            $( selectSubnet ).append( opt );
        }  );
      
      formIPConfig.appendFieldRow( {  label    : i18n.labelNetifStaticSubnet
                                    , required : true
                                    , hint     : i18n.labelNetifStaticSubnetDesc
                                    , element  : selectSubnet
                                 } );
      
      // Panel Default Gateway Route
      /** @type {Element} */ var selectGatewayRoute = document.createElement( 'select' );
      selectGatewayRoute.name = "isDefGatewayIf" ;
      selectGatewayRoute.id   = "isDefGatewayIf" ;
      /** @type {Element} */ var optDefGatewayIf = document.createElement( 'option' );
      /** @type {Element} */ var optDefGatewayIfNot = document.createElement( 'option' );
      
      
      formGatewayRoute.setDescription(i18n.labelNetifDefGatewayIfHeadingDesc);
      formGatewayRoute.appendFieldRow( {  label    : i18n.labelNetifDefGatewayIfSelect
                                        , required : true
                                        , element  : selectGatewayRoute
                                     } );
      //Default gateway
      formDefaultGateway.setDescription(i18n.labelNetifDefGatewayHeadingDesc);
      /** @type {Element} */ inputGateway = $( '<input size="50" maxlength="250" name="gateway" id="gateway" value="" type="text">' ).get(0);
      formDefaultGateway.appendFieldRow( {  label    : i18n.labelNetifDefGatewayAddress
                                          , required : true
                                          , hint     : i18n.labelNetifDefGatewayAddressDesc
                                          , element  : inputGateway
                                       } );
      
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
              {
              
              $( formIPAssign ).on( 'executionSucceeded', function( e )
                  {
                   //if reboot needed then ask user if restart now
                    if(e.response.payload["reboot"])
                    {
                      if ( confirm('System restart is required. Restart system now?') ) {
                        tsa.cmd.handlers.NetifHandler.prototype.rebootSystem('auto');
                      } else {
                        tsa.cmd.handlers.NetifHandler.prototype.rebootSystem('user');
                      }
                    }
                    ( new ShowCommand( { action : 'netif/index' , clearSystemMessages : false  } ) ).execute();
                  } );
              // Show errors
              $( formIPAssign ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        formIPAssign.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );
              formIPAssign.submit();
              }
            }
          )
        );
      
      /** @type {Element} */ var URLNetworkData = 'netif/getNetworkData';
      if (this.getParameter( 'entityId' ) != null){
        URLNetworkData += '/' + this.getParameter( 'entityId' ) ;
      }
      getNetworkCommand = new Command( {
                                        action : URLNetworkData
                                      , clearSystemMessages:false
                                      } );
      
      getNetworkCommand.execute();
      
      $( getNetworkCommand ).on( 'executionSucceeded', function(e) {
        /** @type {Element} */ var ifName = null;
        if(!TV.typeIsUndefined( typeof e.response.payload.ifName )){
          ifName = e.response.payload.ifName;
        }
        
        for( var key in e.response.payload.networkData ) {
          $( '[name="' + key + '"]' ).val( e.response.payload.networkData[ key ] );
        }
        
        
        if(e.response.payload.networkData.assignment!=='STATIC'){
          //When assignment is not static we should disable panel static ip 
          selectSubnet      .disabled = true;
          inputStaticAdress .disabled = true;
        }
        
        //build interfaces
        if( !TV.typeIsUndefined( typeof e.response.payload.interfaces ) ){
          /** @type {Element} */ var tabSection      = $('<div class="ibm-tab-section ibm-text"> <h2 class="ibm-access">Tab navigation</h2></div>').get(0);
          /** @type {Element} */ var ulInterface     = $('<ul class="ibm-tabs"></ul>').get(0);
          /** @type {Element} */ var liInterface     = null;
          /** @type {Element} */ var aInterface      = null;
          $.each( e.response.payload.interfaces , function( idx, elementIF ){
            
            liInterface    = document.createElement("li");
            
            if ( idx+1 === e.response.payload.interfacesSize ) {
              $(liInterface).addClass('ibm-last-tab');
            } else if(idx === 0) {
              $(liInterface).addClass('ibm-first-tab');
            }
            
            if (ifName === elementIF.pid) {
              $(liInterface).addClass('ibm-highlight-tab');
            }
            
            aInterface     = $('<a href="#"></a>').get(0);
            $(aInterface).text(elementIF.name);
            $(aInterface).on("click",function() {
              /** @type {string} */ var URL='netif/index/'+elementIF.name;
              (new ShowCommand( {
                action : URL
                  , clearSystemMessages : false 
               })).execute();
            });
            
            $(liInterface).append( aInterface );
            $(liInterface).append($('<span class="ibm-sep">&nbsp;</span>').get(0));
            $(ulInterface).append( liInterface );
          
          } );
          $(tabSection).append(ulInterface);
          tabsPanel.appendElement(tabSection);
          tabsPanel.appendElement(bodyTabNInterface);
        }
        
        if(!TV.typeIsUndefined(e.response.payload.isDefGatewayIf) && e.response.payload.isDefGatewayIf ){
          $( selectGatewayRoute ).append( optDefGatewayIf );
          $( optDefGatewayIf ).text( i18n.labelNetifDefGatewayIfTrue );
          optDefGatewayIf.value = "true";
          $(optDefGatewayIf).prop("selected",true);
          if(!TV.typeIsUndefined( e.response.payload.networkData.assignment) && e.response.payload.networkData.assignment==='STATIC'){
            inputGateway.disabled = false;
          }
        } else {
          $( selectGatewayRoute ).append( optDefGatewayIf );
          $( selectGatewayRoute ).append( optDefGatewayIfNot );
          $( optDefGatewayIf ).text( i18n.labelNetifDefGatewayIfTrue );
          optDefGatewayIf.value = "true";
          $( optDefGatewayIfNot ).text( i18n.labelNetifDefGatewayIfFalse );
          optDefGatewayIfNot.value = "false";
          $(optDefGatewayIfNot).prop("selected",true);
          inputGateway.disabled = true;
          
        }
        /** @type {function(Event)} */
        var selectGatewayChangeHandler = function( event )
        {
          /** @type {boolean} */ var disabled = ( $( selectGatewayRoute ).val() === 'true' );
          inputGateway      .disabled = disabled;
        };
        $( selectGatewayRoute ).on( 'change', selectGatewayChangeHandler );
        
      } );
      
      /** @type {function(Event)} */
      var selectAssignChangeHandler = function( event )
      {
        /** @type {boolean} */ var disabled = ( $( selectAssign ).val() === 'DYNAMIC' );
        selectSubnet      .disabled = disabled;
        inputStaticAdress .disabled = disabled;
        inputGateway      .disabled = disabled;
      };

      $( selectAssign ).on( 'change', selectAssignChangeHandler );
      
      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( formIPAssign.getDomNode() );
      contentArea.append( formIPConfig.getDomNode() );
      contentArea.append( formGatewayRoute.getDomNode() );
      contentArea.append( formDefaultGateway.getDomNode() );
      contentArea.append( buttonContainer );
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Enable secondary navigation
     */
    tsa.cmd.handlers.NetifHandler.prototype.initNetwork = function(){
      $("#ibm-secondary-navigation").show();  
      /** @type {element} */ var aNetwork     = $("#aNetworkID");
      $( aNetwork ).unbind( "click" );
      $(aNetwork).text(tsa.i18n.labelNavNetworkBasic );
      $(aNetwork).on("click", function(){
                                       ( new ShowCommand( { action : ( 'network/index') } ) ).execute();
                                        }); 
   };
   //-------------------------------------------------------------------------
   /**
    * Request for restarting system.
    * @param {string} restart Restart mode. Use either:
    *                         'auto' : Restart will be initiated by the system.
    *                         'user' : Restart should be initiated by the user.
    */
   tsa.cmd.handlers.NetifHandler.prototype.rebootSystem = function( restart )
   {
      /** @type {tsa.cmd.Command} */ var cmd = null;
      
      cmd = new Command( { action              : 'network/ajaxReboot'
                         , clearSystemMessages : false
                         , parameters          : { "reboot" : restart } 
                         } );
      cmd.execute();
   };
   //-------------------------------------------------------------------------
    return tsa.cmd.handlers.NetifHandler;
  }
);