/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/IconFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , ButtonFactory
          , FormPanel
          , TabsPanel
          , TablePanel
          , IconFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NetrouteHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NetrouteHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NetrouteHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NetrouteHandler.prototype ).constructor = tsa.cmd.handlers.NetrouteHandler;
    tsa.cmd.handlers.NetrouteHandler.prototype.subnetArray= [ "subnetMask0000", "subnetMask128000",
        "subnetMask192000", "subnetMask224000", "subnetMask240000", "subnetMask248000", "subnetMask252000",
        "subnetMask254000", "subnetMask255000", "subnetMask25512800", "subnetMask25519200",
        "subnetMask25522400", "subnetMask25524000", "subnetMask25524800", "subnetMask25525200",
        "subnetMask25525400", "subnetMask25525500", "subnetMask2552551280", "subnetMask2552551920",
        "subnetMask2552552240", "subnetMask2552552400", "subnetMask2552552480", "subnetMask2552552520",
        "subnetMask2552552540", "subnetMask2552552550", "subnetMask255255255128",
        "subnetMask255255255192", "subnetMask255255255224", "subnetMask255255255240",
        "subnetMask255255255248", "subnetMask255255255252", "subnetMask255255255254",
        "subnetMask255255255255" ];
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NetrouteHandler.prototype._index = function()
    {
      this.initNetwork();
      /** @type {Object}                  */    var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */    var contentArea = new ContentArea ( { title : i18n.labelNetworkHeading 
                                                                                  , helpDoc : 'sshelp_netroutes_panel.html'} );
      
      /** @type {string}                  */    var pid               = this.getParameter( 'entityId' );
      /** @type {tsa.widgets.ButtonFactory} */  var buttonContainer   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.TablePanel} */     var tablePanel              = new TablePanel( { title        : i18n.labelInventorySummaryTypeHeading
                                                                                              , summary      : i18n.labelInventorySummaryTypeObjectsHeading
                                                                                              , ajaxAction   : 'netroute/getListNetworkData'
                                                                                              , entryHandler : $.proxy(this.infoEntryHandler,this)
                                                                                              , pageSize     : 10
                                                                                              , columns      : [ { id : "destination" , label : i18n.labelNetworkDestination  , width : '20%' , sortable : true  }
                                                                                                               , { id : "mask"        , label : i18n.labelNetworkMask         , width : '20%' , sortable : true  }
                                                                                                               , { id : "gateway"     , label : i18n.labelNetworkGateway      , width : '20%' , sortable : true  }
                                                                                                               , { id : "interface"   , label : i18n.labelNetworkIfName       , width : '20%' , sortable : true  }
                                                                                                               , { id : "actions"     , label : i18n.labelActions             , width : '20%' , sortable : false} ] } );
      /** @type {Element} */ var bodyTabGlobal      = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNInterface  = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabDNSSettings = document.createElement( 'p' );
      /** @type {Element} */ var bodyTabNRoutes     = document.createElement( 'p' );
      
      $( bodyTabGlobal      ).text( i18n.labelNetworkCaption );
      $( bodyTabNInterface  ).text( i18n.labelNetifCaption );
      $( bodyTabDNSSettings ).text( i18n.labelNetdnsCaption );
      $( bodyTabNRoutes     ).text( i18n.labelNetrouteCaption );
      
      /** @type {tsa.widgets.TabsPanel}  */ var tabsPanel    = new TabsPanel( { tabs: [ { title : i18n.labelNetworkGlobalHeading     , body : bodyTabGlobal
                                                                                         , action : "netglobal/index" }
                                                                                      , { title : i18n.labelNetworkInterfacesHeading , body : bodyTabNInterface
                                                                                          , action : "netif/index" }
                                                                                      , { title : i18n.labelNetworkDnsHeading        , body : bodyTabDNSSettings
                                                                                          , action : "netdns/index" }
                                                                                      , { title : i18n.labelNetrouteHeading          , body : bodyTabNRoutes
                                                                                          , action : "netroute/index"
                                                                                        }] } );
      tabsPanel.goToTab(3);
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonNetworkAddRoute
            , onclick : function( event )
              {
              ( new ShowCommand( { action : 'netroute/new' } ) ).execute();
              }
            }
          )
        );
      
      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( tablePanel.getDomNode() );
      
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Enable secondary navigation
     */
    tsa.cmd.handlers.NetrouteHandler.prototype.initNetwork = function(){
      $("#ibm-secondary-navigation").show();  
      /** @type {element} */ var aNetwork     = $("#aNetworkID");
      $( aNetwork ).unbind( "click" );
      $(aNetwork).text(tsa.i18n.labelNavNetworkBasic );
      $(aNetwork).on("click", function(){
                                       ( new ShowCommand( { action : ( 'network/index') } ) ).execute();
                                        }); 
   };
  //-------------------------------------------------------------------------
   tsa.cmd.handlers.NetrouteHandler.prototype.infoEntryHandler = function( entry )
   {
       
      /** @type {Array.<(string|Element)>} */ var result             = null;
      /** @type {Element}                  */ var dDestination       = document.createElement("div");
      /** @type {Element}                  */ var dMask              = document.createElement("div");
      /** @type {Element}                  */ var dGateway           = document.createElement("div");
      /** @type {Element}                  */ var dInterface         = document.createElement("div");
      /** @type {Element}                  */ var dActions           = document.createElement("div");
      
      /** @type {function()}               */ 
      var clickEventHandler  = $.proxy(function(event){
        /** @type {string}          */ var actionURL    = 'netroute/show/' + entry.pid;
        (new ShowCommand( {
          action : actionURL
          , clearSystemMessages : false
                           })).execute();
        
      },this);
      if(entry.staticRoute){
        /**@type function() */
        var clickTriggerHandler = function ( event ) {
          tsa.clearSysMessages();
          var proceed = true;
          if(!TV.typeIsUndefined( typeof event.data.confirm )){
            proceed = confirm(tsa.i18n.labelDeleteConfirmation + " '" + event.data.destination + "' ?")
          }
          if(proceed){
            var actionCmd = new Command( { action : event.data.URL
                                                , clearSystemMessages : false 
                                                } );
            actionCmd.execute();
            $( actionCmd ).on( 'executionSucceeded', function( event )
                 {
                  (new ShowCommand( {
                     action : 'netroute/index'
                       , clearSystemMessages : false} 
                    )).execute(); 
                  } );
          }
        };
        /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : tsa.i18n.labelButtonDelete + " '" + entry.destination + "'"} );
        $( deleteIcon ).on( 'click', { URL : 'netroute/ajaxDelete/' + entry.pid , destination : entry.destination, confirm : true} , clickTriggerHandler );
        $(dActions).append(deleteIcon);
      }
      
      $( dDestination ).on( 'click', clickEventHandler );
      $( dMask        ).on( 'click', clickEventHandler );
      $( dGateway     ).on( 'click', clickEventHandler );
      $( dInterface   ).on( 'click', clickEventHandler );
      /** @type {Element} */ var sDestination = document.createElement("strong");
      $( sDestination ).append( entry.destination   );
      $( dDestination ).append( sDestination   );
      $( dMask        ).append( entry.subnetMask    );
      $( dGateway     ).append( entry.gateway       );
      $( dInterface   ).append( entry.networkIfName );
      return [ dDestination , dMask , dGateway , dInterface , dActions ];
   };
   
   /**
   Show detailed route
   */
   tsa.cmd.handlers.NetrouteHandler.prototype._show = function()
   {
     this.initNetwork();
     /** @type {Object}                  */    var i18n              = tsa.i18n;
     /** @type {string}                  */    var pid               = this.getParameter( 'entityId' );
     /** @type {tsa.widgets.ContentArea} */    var contentArea       = new ContentArea ( { title : i18n.labelNetrouteHeading } );
     /** @type {tsa.cmd.Command}         */    var getNetworkCommand = new Command( {  action :'netroute/ajaxShowRoute/' + pid
                                                                                    , clearSystemMessages:false } );
     /** @type {tsa.widgets.ContentArea} */     var tablePanel               = new TablePanel( { title: i18n.labelNetrouteGeneralHeading 
                                                                                                 , columns: [ { id : "name"  , width : '35%'  }
                                                                                                            , { id : "value" , width : '65%' }] 
                                                                                                  , summary : "Summary InventorySummary"} );
     /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
     contentArea.appendMessage( { text : i18n.labelNetrouteCaption } );
     getNetworkCommand.execute();
     
     $( getNetworkCommand ).on( 'executionSucceeded', function(e) {
       /** @type {Object} */ var theRoute    = e.response.payload.theRoute;  
       tablePanel.appendRow([i18n.labelNetrouteType        , theRoute.routeType     !=null ? theRoute.routeType     : ""]);
       tablePanel.appendRow([i18n.labelNetrouteDestination , theRoute.destination   !=null ? theRoute.destination   : ""]);
       tablePanel.appendRow([i18n.labelNetrouteMask        , theRoute.mask          !=null ? theRoute.mask          : ""]);
       tablePanel.appendRow([i18n.labelNetrouteGateway     , theRoute.gateway       !=null ? theRoute.gateway       : ""]);
       tablePanel.appendRow([i18n.labelNetrouteStatic      , theRoute.isStaticRoute !=null ? theRoute.isStaticRoute : ""]);
       tablePanel.appendRow([i18n.labelNetrouteIfName      , theRoute.ifName        !=null ? theRoute.ifName        : ""]);
       
       if(!theRoute.isStaticRoute){
           $(buttonContainer).hide();
       }
     } );
     $( buttonContainer ).append(
         ButtonFactory.buildButton(
           { type    : 'arrow'
           , grayout : false
           , label   : i18n.buttonNetrouteEdit
           , onclick : function( event )
             {
             /** @type {string} */ var editURL  = (pid != null ? 'netroute/edit/'+pid : 'netroute/edit/');
             
             ( new ShowCommand( { action : ( editURL) } ) ).execute();
             }
           }
         )
       );
     
     contentArea.append(tablePanel.getDomNode());
     
     contentArea.append( buttonContainer );
     
     this.show( contentArea.getDomNode() );
     
     };
     
     
     
     /**
      * Edit
      * */
     tsa.cmd.handlers.NetrouteHandler.prototype._edit = function()
     {
       this._new();
     };
     /**
     New Route view
     */
     tsa.cmd.handlers.NetrouteHandler.prototype._new = function()
     {
       this.initNetwork();
       /** @type {Object}                  */    var i18n                 = tsa.i18n;
       /** @type {string}                  */    var pid                  = this.getParameter( 'entityId' );
       /** @type {tsa.widgets.ContentArea} */    var contentArea          = new ContentArea ( { title : i18n.labelNetworkRoutesHeading } );
       /** @type {tsa.cmd.Command}         */    var getNetworkInterfaces = new Command( {  action :'netroute/getInterfaces'
                                                                                                 , clearSystemMessages:false } );
       /** @type {tsa.cmd.Command}         */    var getNetworkRouteData  = null;
       /** @type {string}                  */    var actionURL= (pid!=null?'netroute/putUpdateNetworkData/'+pid : 'netroute/ajaxCreateRoute'  );
       /** @type {tsa.widgets.FormPanel}     */ var  formPanel             = new FormPanel( { title  : i18n.labelNetrouteDetailsHeading
                                                                                       , action : actionURL } );
       if(pid!=null){
         formPanel.putHiddenInput("pid",pid);
       }
       /** @type {tsa.widgets.ButtonFactory} */  var buttonContainer   = ButtonFactory.buildButtonContainer();
       formPanel.setDescription(i18n.labelNetrouteDetailsDesc);
       
       
       formPanel.appendFieldRow( {   label    : i18n.labelNetrouteDestination
                                   , required : true
                                   , hint     : i18n.labelNetrouteDestinationDesc
                                   , element  : $( '<input size="50" maxlength="250" name="destination" id="destination" value="" type="text">' ).get(0)
                                } );
       formPanel.appendFieldRow( {   label    : i18n.labelNetrouteGateway
                                   , required : true
                                   , hint     : i18n.labelNetrouteGatewayDesc
                                   , element  : $( '<input size="50" maxlength="250" name="gateway" id="gateway" value="" type="text">' ).get(0)
                                } );
       /** @type {Element} */ var selectSubnet = document.createElement( 'select' );
       /** @type {Element} */ var opt          = document.createElement( 'option' );
       selectSubnet.name = "subnetMask";
       selectSubnet.id   = "subnetMask";
       $.each( this.subnetArray, function( idx, element ) {
             opt = document.createElement( 'option' );
             opt.value = i18n[element];
             $( opt ).text( i18n[element] ); 
             $( selectSubnet ).append( opt );
         }  );
       formPanel.appendFieldRow( {   label    : i18n.labelNetrouteSubnetMask
                                   , required : true
                                   , hint     : i18n.labelNetrouteSubnetMaskDesc
                                   , element  : selectSubnet
                                } );
       /** @type {Element} */ var selectInterfaces = document.createElement( 'select' );
       
       formPanel.appendFieldRow( {   label    : i18n.labelNetrouteIfName
                                   , required : true
                                   , hint     : i18n.labelNetrouteIfNameDesc
                                   , element  : selectInterfaces
                                } );
       getNetworkInterfaces.execute();
       $( getNetworkInterfaces ).on( 'executionSucceeded', function(e) {
         selectInterfaces.name = "interfaceName";
         selectInterfaces.id   = "interfaceName";
         $.each( e.response.payload.interfaces, function( idx, element ) {
               opt = document.createElement( 'option' );
               opt.value = element.pid;
               $( opt ).text( element.name ); 
               $( selectInterfaces ).append( opt );
           }  );
       } );
       if(pid!=null){
         getNetworkRouteData=new Command( {  action :'netroute/getNetworkRouteData/'+pid
           , clearSystemMessages:false } );
         getNetworkRouteData.execute();
         $( getNetworkRouteData ).on( 'executionSucceeded', function(e) {
            for( var key in e.response.payload.networkData ) {
                $( '[name="' + key + '"]' ).val( e.response.payload.networkData[ key ] );
            }
            $( '[name="subnetMask"]' ).val( e.response.payload.networkData[ 'subnetMask' ] );
         } );
       }
       $( buttonContainer ).append(
           ButtonFactory.buildButton(
             { type    : 'arrow'
             , grayout : false
             , label   : i18n.buttonSave
             , onclick : function( event )
               {
               formPanel.submit();
               $( formPanel ).on( 'executionSucceeded', function(e) {
                 ( new ShowCommand( { action : 'netroute/index' } ) ).execute();  
               })
               
               
               
               }
             }
           )
         ); 
       $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
       $( buttonContainer ).append(
           ButtonFactory.buildButton(
             { type    : 'arrow'
             , grayout : false
             , label   : i18n.buttonCancel
             , onclick : function( event )
               {
               ( new ShowCommand( { action : 'netroute/index' } ) ).execute();
               }
             }
           )
         ); 
        
       
       contentArea.appendMessage( { text : i18n.labelNetrouteCaption } );
       
       contentArea.append(formPanel.getDomNode());
       contentArea.append( buttonContainer );
       this.show( contentArea.getDomNode() );
     };
 //-------------------------------------------------------------------------
    return tsa.cmd.handlers.NetrouteHandler;
  }
);