/*
 * IBM Technical Support Appliance Version 2 Release 3 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ButtonFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , TabsPanel
          , Command
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NettoolsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NettoolsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NettoolsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NettoolsHandler.prototype ).constructor = tsa.cmd.handlers.NettoolsHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NettoolsHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n           = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea    = null;
      /** @type {tsa.widgets.TabPanel}      */ var tabsPanel    = null
      /** @type {tsa.widgets.FormPanel}     */ var nettoolsFormPanel = new FormPanel( { title : i18n.labelNettoolsNetworkDiagnostic , action : 'nettools/getToolOutput' } );
      /** @type {tsa.cmd.Command}           */ var getToolsCmd   = new Command( { action  : 'nettools/getTools' } );
      
      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelNettoolsHeading
                                      , helpDoc : 'sshelp_nettools_panel.html' } );
      
      contentArea.appendMessage( { text : i18n.labelNettoolsCaption } );
      
      var currentTab = null;
      
      //get properties file
      $( getToolsCmd ).on( 'executionSucceeded', function( event ){

       //Setup tabs    
       var tabList  = event.response.payload.tabs;
       var tabInfo  = [];
       var idMap  = {};   
       var desc  = null;
       var label   = null;
       var subtitle =  null;

       currentTab = event.response.payload.selectedTabId; 

       $.each( tabList, function( tabIndex, tab )
         {
        idMap [ tabIndex ] = tab.id; //set map of tab ids

        switch(tab.id) {
        case "traceroute":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( i18n.labelNettoolsTracerouteDesc );
         label = i18n.labelNettoolsTracerouteTab
         break;
        case "testssh":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( i18n.labelNettoolsTestsshDesc );
         label = i18n.labelNettoolsTestsshTab
         break;
        case "interfaces":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsInterfacesTab
         break;  
        case "ethernet":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsEthernetTab
         break;
        case "address":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsAddressTab
         break;
        case "routes":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsRoutesTab
         break;  
        case "arp":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsArpTab
         break;
        case "sockets":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsSocketsTab
         break;
        case "iptabs":
         subtitle = document.createElement( 'p' );
         $( subtitle ).text( tsa.NBSP );
         label = i18n.labelNettoolsIptabsTab
         break;  
        default: 
         subtitle = document.createElement( 'p' );
           $( subtitle ).text( i18n.labelNettoolsPingDesc );
           label = i18n.labelNettoolsPingTab
           break;  
        }

        if(tabIndex == 0){
         tabInfo.push( { title : label, body : subtitle, isFirst : true } );

        }else if(tabIndex == tabList.length - 1) {
         tabInfo.push( { title : label, body : subtitle, isLast : true } );

        } else{
         tabInfo.push( { title : label, body : subtitle } );
        }

         });

       tabsPanel = new TabsPanel( { tabs: tabInfo } );

       //IP Address
        nettoolsFormPanel.putHiddenInput('selectedTabId', currentTab);
        var ip = document.createElement( 'div' );

        
        addressRow = nettoolsFormPanel.appendFieldRow( { label    : i18n.labelNettoolsPingAddr
                                                       , required : true
                                                       , element  : $( '<input size="50" maxlength="250" name="addr" id="addr" value="" aria-required="true" class="ibm-text"  type="text" style="margin: 0px 0px 0px 15px">' ).get(0)
              } );
        var addressHR = nettoolsFormPanel.appendHR();
        $(nettoolsFormPanel.descParagraph).hide();
        //RESULTS 
       var textArea = $("<div class='ibm-code-sample' style='background:#ffffff !important; height:30em; width:522px; border:0px; margin: 0px'></>").get(0);
       $( textArea ).hide();

       var pre = document.createElement( 'pre' );
       $( pre ).attr('id' , 'results');

       $( textArea ).append( pre );
       nettoolsFormPanel.appendJointRow( textArea );

       //BUTTON
       var toolButton = ButtonFactory.buildButton( { type : 'arrow' , label : i18n.buttonNettoolsPing } );
       nettoolsFormPanel.appendJointRow( toolButton );

       //BACK TO TOP 
       var backPar = document.createElement( 'p' );
       var backLink = $( '<a class="ibm-anchor-up-link" href="#ibm-content">'+i18n.buttonBackToTop+'</a>' ).get(0);
       $( backPar ).append( backLink );
       $( backPar ).hide();

       $( toolButton ).on( 'click', function( event ){
        nettoolsFormPanel.submit();
       } );


       //Command/tool execution
       $( nettoolsFormPanel ).on( 'executionSucceeded', function( event )
       { 
        var results = event.response.payload.results
        if(currentTab == 'testssh'){
         message = results.split("\n");
         $( '#results' ).text( message[0] + "\n" + message[1] + message[2] );
        }else {
         $( '#results' ).text( results );
        }
        $( '#addr' ).val( '' );
        $( textArea ).show();
        $( backPar ).show();

        if(currentTab != 'ping' && currentTab != 'traceroute' && currentTab != 'testssh'){
         $( addressRow ).hide();
         $( addressHR ).hide();
         $( toolButton ).hide();
        }
         } );  


       //When a tab is clicked
       $( tabsPanel ).on( 'tabChanged', function( event ){
        var tabIdx = event.tabIndex;
        var tabId = idMap[tabIdx];
        currentTab = tabId;

        nettoolsFormPanel.putHiddenInput('selectedTabId', tabId);

        if(tabId == 'ping' || tabId == 'traceroute' || tabId == 'testssh'){
         if(tabId == 'ping'){
          $( toolButton ).val( i18n.buttonNettoolsPing );

         }else if (tabId == 'traceroute'){
          $( toolButton ).val( i18n.buttonNettoolsTraceroute );

         }else{
          $( toolButton ).val( i18n.buttonNettoolsTestssh );
         }

         $( addressRow ).show();
         $( addressHR ).show();
         $( textArea ).hide();
         $( toolButton ).show();

        }else{
         nettoolsFormPanel.removeValidation("addr");
         nettoolsFormPanel.submit();
        }

       } );

       contentArea.append( tabsPanel.getDomNode() );
       contentArea.append( nettoolsFormPanel.getDomNode() );
       contentArea.append( backPar );

      });
          
    
      //Get tabs
      getToolsCmd.execute();
    
      this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.NettoolsHandler;
  }
);