/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , Command
          , ShowCommand
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'NetworkHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.NetworkHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.NetworkHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.NetworkHandler.prototype ).constructor = tsa.cmd.handlers.NetworkHandler;
    //subnet values.
    tsa.cmd.handlers.NetworkHandler.prototype.subnetArray= [ "subnetMask0000", "subnetMask128000",
      "subnetMask192000", "subnetMask224000", "subnetMask240000", "subnetMask248000", "subnetMask252000",
      "subnetMask254000", "subnetMask255000", "subnetMask25512800", "subnetMask25519200",
      "subnetMask25522400", "subnetMask25524000", "subnetMask25524800", "subnetMask25525200",
      "subnetMask25525400", "subnetMask25525500", "subnetMask2552551280", "subnetMask2552551920",
      "subnetMask2552552240", "subnetMask2552552400", "subnetMask2552552480", "subnetMask2552552520",
      "subnetMask2552552540", "subnetMask2552552550", "subnetMask255255255128",
      "subnetMask255255255192", "subnetMask255255255224", "subnetMask255255255240",
      "subnetMask255255255248", "subnetMask255255255252", "subnetMask255255255254",
      "subnetMask255255255255" ];
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.NetworkHandler.prototype._index = function()
    {
      this.initNetwork();
      
      /** @type {Object}                    */ var i18n              = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea       = new ContentArea ( { title : i18n.labelNetworkBasicHeading 
                                                                                       , helpDoc : 'sshelp_network_panel.html' } );
      /** @type {tsa.widgets.FormPanel}     */ var formIdentify      = new FormPanel( { title  : i18n.labelNetglobalIdentityHeading
                                                                                      , action : 'network/putUpdateNetwork'     } );
      /** @type {tsa.widgets.FormPanel}     */ var formIPAssign      = new FormPanel( { title  : i18n.labelNetifAssignHeading })
      /** @type {tsa.widgets.FormPanel}     */ var formIPConfig      = new FormPanel( { title  : i18n.labelNetifStaticHeading })
      /** @type {tsa.widgets.FormPanel}     */ var formNameServ      = new FormPanel( { title  : i18n.labelNetdnsDnsHeading })
      /** @type {tsa.widgets.FormPanel}     */ var formDNS           = new FormPanel( { title  : i18n.labelNetdnsServersHeading })
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getNetworkCommand = null;
      
      formIdentify.linkTo( formIPAssign );
      formIPAssign.linkTo( formIPConfig );
      formIPConfig.linkTo( formNameServ );
      formNameServ.linkTo( formDNS );
      
      contentArea.appendMessage( { text : i18n.labelNetworkBasicCaption } );
      
      formIdentify.setDescription(i18n.labelNetglobalIdentityHeadingDesc);
      
      formIdentify.appendFieldRow( {  label    : i18n.labelNetglobalHost
                                    , required : true
                                    , hint     : i18n.labelNetglobalHostDesc
                                    , element  : $( '<input size="50" maxlength="250" name="hostname" id="hostname" value="" type="text">' ).get(0)
                                   } );
      
      formIdentify.appendFieldRow( {  label    : i18n.labelNetglobalDomainName
                                    , required : true
                                    , hint     : i18n.labelNetglobalDomainNameDesc
                                    , element  : $( '<input size="50" maxlength="250" name="domain" id="domain" value="" type="text">' ).get(0)
                                  } );
      
      //Panel Assignment
      
      /** @type {Element} */ var selectAssign = document.createElement( 'select' );
      selectAssign.name="assignment";
      selectAssign.id="assignment";
      
      /** @type {Element} */ var opt = document.createElement( 'option' );
      opt.value = "STATIC";
      $( opt ).text( i18n.labelNetifAssignStatic ); 
      $( selectAssign ).append( opt );
      opt = document.createElement( 'option' );
      opt.value = "DYNAMIC";
      $( opt ).text( i18n.labelNetifAssignDynamic ); 
      $( selectAssign ).append( opt );
     
      formIPAssign.setDescription(i18n.labelNetifAssignDesc);
      formIPAssign.appendFieldRow( { label    : i18n.labelNetifAssign
        , required : true
        , element  : selectAssign
        } );
      
      //Panel IP Configuration 
      /** @type {Element} */ var selectSubnet = document.createElement( 'select' );
      /** @type {Element} */ var inputAddress = $( '<input size="50" maxlength="250" name="address" id="address" value="" type="text">' ).get(0)
      /** @type {Element} */ var inputGateway = $( '<input size="50" maxlength="250" name="gateway" id="gateway" value="" type="text">' ).get(0);
      
      formIPConfig.setDescription(i18n.labelNetifStaticDesc);
      
      formIPConfig.appendFieldRow( {  label    : i18n.labelNetifStaticAddress
                                    , required : true
                                    , hint     : i18n.labelNetifStaticAddressDesc
                                    , element  : inputAddress
                                    } );
      
      selectSubnet.name = "subnet" ;
      selectSubnet.id   = "subnet" ;
      $.each( this.subnetArray, function( idx, element )
          {
            opt = document.createElement( 'option' );
            opt.value = i18n[element];
            $( opt ).text( i18n[element] ); 
            $( selectSubnet ).append( opt );
          } );

      formIPConfig.appendFieldRow( { label    : i18n.labelNetifStaticSubnet
        , required : true
        , hint     : i18n.labelNetifStaticSubnetDesc
        , element  : selectSubnet
        } );
      formIPConfig.appendFieldRow( { label    : i18n.labelNetifStaticGateway
                                  , required : true
                                  , hint     : i18n.labelNetifStaticGatewayDesc
                                  , element  : inputGateway 
                                    } );
     
      /** @type {Element} */ var selectNameServ = document.createElement( 'select' );
      selectNameServ.name = "dns"
      selectNameServ.id = "dns"
      opt = document.createElement( 'option' );
      opt.value = "STATIC";
      $( opt ).text( i18n.labelNetdnsDnsStatic ); 
      $( selectNameServ ).append( opt );
      
      opt = document.createElement( 'option' );
      opt.value = "DHCP";
      $( opt ).text( i18n.labelNetdnsDnsDhcp ); 
      $( selectNameServ ).append( opt );
      // Panel Name Services
      formNameServ.setDescription(i18n.labelNetdnsDnsHeadingDesc);
      formNameServ.appendFieldRow( { label    : i18n.labelNetdnsDns
        , required : true
        , element  : selectNameServ
        } );

      // Panel DNS
      formDNS.setDescription(i18n.labelNetdnsServersDesc);
      /** @type {Element} */ var inputServer1 = $( '<input size="50" maxlength="250" name="server1" id="server1" value="" type="text">' ).get(0)
      /** @type {Element} */ var inputServer2 = $( '<input size="50" maxlength="250" name="server2" id="server2" value="" type="text">' ).get(0)
      /** @type {Element} */ var inputServer3 = $( '<input size="50" maxlength="250" name="server3" id="server3" value="" type="text">' ).get(0)
      
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer1
                              , hint     : i18n.labelNetdnsServer1Desc
                              , required : true
                              , element  : inputServer1
                               } );
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer2
                              , hint     : i18n.labelNetdnsServer2Desc
                              , element  : inputServer2 
                               } );
      formDNS.appendFieldRow( { label    : i18n.labelNetdnsServer3
                              , hint     : i18n.labelNetdnsServer3Desc
                              , element  : inputServer3
                               } );
      
      $( selectAssign ).on( 'change', function( event )
          {
          /** @type {boolean} */ var disabled    = ( $( selectAssign ).val() === 'DYNAMIC' );
          /** @type {boolean} */ var disabledSvr = ( $( selectNameServ ).val() === 'DHCP' && disabled);
          selectSubnet.disabled = disabled;
          inputAddress.disabled = disabled;
          inputGateway.disabled = disabled;
          inputServer1.disabled = disabledSvr;
          inputServer2.disabled = disabledSvr;
          inputServer3.disabled = disabledSvr;
         });
      
      $( selectNameServ ).on( 'change', function(event){
        /** @type {boolean} */ var disabled = ( $( selectAssign ).val() === 'DYNAMIC' ) && ( $( selectNameServ ).val() === 'DHCP' );
        inputServer1.disabled = disabled;
        inputServer2.disabled = disabled;
        inputServer3.disabled = disabled;
      });
      
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
             {
              $( formIdentify ).on( 'executionSucceeded', function( e )
                  {
                   //if reboot needed then ask user if restart now
                    if(e.response.payload["reboot"])
                    {
                      if ( confirm('System restart is required. Restart system now?') ) {
                        tsa.cmd.handlers.NetworkHandler.prototype.rebootSystem('auto');
                      } else {
                        tsa.cmd.handlers.NetworkHandler.prototype.rebootSystem('user');
                      }
                    }
                    ( new ShowCommand( { action : 'network/index' , clearSystemMessages : false  } ) ).execute();
                  } );
              // Show errors
              $( formIdentify ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        formIdentify.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );  
                formIdentify.submit();
              }
            }
          )
        );
        
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
              (new ShowCommand( {
                action : 'main/index'
                  , clearSystemMessages : false 
               })).execute();
              }
            }
          )
        );
      
      getNetworkCommand = new Command( {
          action:'network/getNetworkData'
        , clearSystemMessages:false
      } );
      
      getNetworkCommand.execute();
     
      $( getNetworkCommand ).on( 'executionSucceeded', function(e) {
        for( var key in e.response.payload.networkData ) {
          $( '[name="' + key + '"]' ).val( e.response.payload.networkData[ key ] );
        }
        /** @type {boolean} */ var disabled    = ( $( selectAssign ).val() === 'DYNAMIC' );
        /** @type {boolean} */ var disabledSvr = ( $( selectNameServ ).val() === 'DHCP' && disabled);
        selectSubnet.disabled = disabled;
        inputAddress.disabled = disabled;
        inputGateway.disabled = disabled;
        inputServer1.disabled = disabledSvr;
        inputServer2.disabled = disabledSvr;
        inputServer3.disabled = disabledSvr;
      } );
        

      contentArea.append( formIdentify.getDomNode() );
      contentArea.append( formIPAssign.getDomNode() );
      contentArea.append( formIPConfig.getDomNode() );
      contentArea.append( formNameServ.getDomNode() );
      contentArea.append( formDNS.getDomNode() );
      contentArea.append( buttonContainer );
            
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Enable secondary navigation
     */
    tsa.cmd.handlers.NetworkHandler.prototype.initNetwork = function()
    {
      $("#ibm-secondary-navigation").show();
      /** @type {element} */ var aNetwork     = $("#aNetworkID");
      $( aNetwork ).unbind( "click" );
      $(aNetwork).text(tsa.i18n.labelNavNetworkAdvanced );
      $(aNetwork).on("click",function(){
             ( new ShowCommand( { action : ( 'netglobal/index') } ) ).execute();
          });
    };
    
    /**
     * Request for restarting system.
     * @param {string} restart Restart mode. Use either:
     *                         'auto' : Restart will be initiated by the system.
     *                         'user' : Restart should be initiated by the user.
     */
    
    tsa.cmd.handlers.NetworkHandler.prototype.rebootSystem = function( restart )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;
       
       cmd = new Command( { action              : 'network/ajaxReboot'
                          , clearSystemMessages : false
                          , parameters          : { "reboot" : restart } 
                          } );
       cmd.execute();
    };
    return tsa.cmd.handlers.NetworkHandler;
  }
  
);