    /*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/IconFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , TablePanel
          , IconFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'OsgiHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.OsgiHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.OsgiHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.OsgiHandler.prototype ).constructor = tsa.cmd.handlers.OsgiHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.OsgiHandler.prototype._index = function()
    {
      tsa.setLayout( 'wide' );
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelOsgiHeading
                                                                                 , helpDoc : 'sshelp_osgi_panel.html'} );
      
      
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel = null;
      contentArea.appendMessage( { text : i18n.labelOsgiCaption } );
      var entryHandler = function( entry )
      {
        /** @type {Array.<(string|Element)>} */ var result        = null;
        /** @type {Object}  */ var bundle                      = entry;
          
        /**@type function() */
        var clickTriggerHandler = function ( event ) {
          tsa.clearSysMessages();
          var proceed = true;
          if(!TV.typeIsUndefined( typeof event.data.confirm )){
              proceed = confirm("Are you sure you want to uninstall bundle?");
          }
          if(proceed){
            var actionCmd = new Command( { action : event.data.URL
                                                , clearSystemMessages : false  } );
            actionCmd.execute();
            $( actionCmd ).on( 'executionSucceeded', function( event )
                 {
                  (new ShowCommand( {
                     action : 'osgi/index'
                       , clearSystemMessages : false} 
                    )).execute(); 
                  } );
          }
        };
        /** @type {Element}         */ var bundleIcon    = null;
        /** @type {Element}         */ var toogleIcon    = null;
        /** @type {Element}         */ var actionsSpan   = document.createElement( 'span' );
        if (bundle.active) { 
            toogleIcon= IconFactory.buildActionIcon( { type : 'stop'   , caption :'stop the bundle' } );
            $( toogleIcon ).on( 'click', { URL : 'osgi/ajaxStop/'   + bundle.bundleId   } , clickTriggerHandler );
        } else {
            toogleIcon = IconFactory.buildActionIcon( { type : 'runb'  , caption : 'start the bundle' } );
            $( toogleIcon ).on( 'click', { URL : 'osgi/ajaxStart/'   + bundle.bundleId    } , clickTriggerHandler );
        }
        /** @type {Element}         */ var updateIcon    = IconFactory.buildActionIcon( { type : 'refresh'   , caption : 'update the bundle' } );
        /** @type {Element}         */ var uninstallIcon = IconFactory.buildActionIcon( { type : 'delete' , caption : 'uninstall the bundle ' } );
        
        if( bundle.fragmentBundle) {
            bundleIcon = IconFactory.buildActionIcon( { type : 'bundlefrg'     , caption : bundle.statusText  } );
        } else if ( bundle.libraryBundle){
            bundleIcon = IconFactory.buildActionIcon( { type : 'bundlelib'     , caption : 'Bundle with no activator' } );
        } else {
            bundleIcon = IconFactory.buildActionIcon( { type : 'bundle'     , caption : bundle.statusText    } );
        }
        $( updateIcon    ).on( 'click', { URL : 'osgi/ajaxUpdate/'      + bundle.bundleId                 } , clickTriggerHandler );
        $( uninstallIcon ).on( 'click', { URL : 'osgi/ajaxUninstall/'   + bundle.bundleId , confirm : true} , clickTriggerHandler );
        
        $(actionsSpan).append(toogleIcon);
        $(actionsSpan).append(tsa.NBSP)
        $(actionsSpan).append(updateIcon);
        $(actionsSpan).append(tsa.NBSP)
        $(actionsSpan).append(tsa.NBSP)
        $(actionsSpan).append(uninstallIcon);
        
        /** @type {function()}*/
       var displayTrigger = function() {
            (new ShowCommand( {
                action : 'osgi/show/' + bundle.bundleId
                  , clearSystemMessages : false 
              })).execute();
        };
        
        /** @type {Element}         */ var divIcon   = document.createElement( 'div' );
        /** @type {Element}         */ var divName   = document.createElement( 'div' );
        /** @type {Element}         */ var divStatus = document.createElement( 'div' );
        /** @type {Element}         */ var divSName  = document.createElement( 'div' );
        
        $(divIcon).append( bundleIcon)
        $(divName).append(bundle.displayName);
        $(divStatus).append(bundle.statusText);
        $(divSName).append(bundle.symbolicName);
        $(divIcon  ).on( 'click'  , displayTrigger);
        $(divName  ).on( 'click'  , displayTrigger);
        $(divStatus).on( 'click'  , displayTrigger);
        $(divSName ).on( 'click'  , displayTrigger);
        
        result = [divIcon, divName, divStatus, divSName, actionsSpan];
        
        return result;
        
      };
      tablePanel   = new TablePanel( { title        : i18n.labelSummaryStatusHeading
          , summary      : i18n.labelSummaryStatusDesc
          , emptyMsg     : i18n.labelSummaryNoStatus
          , ajaxAction   : 'osgi/getList'
          , entryHandler : entryHandler
          , columns      : [ { id : 'type'     , label : i18n.labelOsgiBundleType           , width :  '5%' , sortable : true }
                           , { id : 'name'     , label : i18n.labelOsgiBundleName           , width : '45%' , sortable : true }
                           , { id : 'state'    , label : i18n.labelOsgiBundleState          , width : '15%' , sortable : true }
                           , { id : 'symbName' , label : i18n.labelOsgiBundleSymbolicName   , width : '15%' , sortable : true }
                           , { id : 'actions'  , label : i18n.labelActionsColumn            , width : '20%' , sortable : false } ] } );
      
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    /**
     * Show bundle screen.
     */
    tsa.cmd.handlers.OsgiHandler.prototype._show = function()
    {
      /** @type {string} */ var entityId = this.getParameter( 'entityId' );
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelOsgiBundleHeading } );
      /** @type {tsa.cmd.Command}         */ var getBundleCmd = new Command( { action              : 'osgi/getBundle/' +entityId
                                                                              , clearSystemMessages : false } ); 
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel   = new TablePanel( { title        : i18n.labelOsgiBundleDetailsHeading
                                                                                , summary      : i18n.labelSummaryStatusDesc
                                                                                , emptyMsg     : i18n.labelSummaryNoStatus
                                                                                , columns      : [ { id : 'name'  , width : '20%' , sortable : true }
                                                                                                 , { id : 'value' , width : '80%' , sortable : false } ] } );
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel2   = new TablePanel( {  title        : i18n.labelOsgiBundleRegServicesHeading
                                                                                  , summary      : i18n.labelSummaryStatusDesc
                                                                                  , emptyMsg     : i18n.labelOsgiBundleNoRegisteredServices
                                                                                  , columns      : [ { id : 'number'  , }
                                                                                                   , { id : 'pid'   , label : 'PID'   , width : '20%'  }
                                                                                                   , { id : 'description'   , label : 'Description'   , width : '60%'  }
                                                                                                   , { id : 'vendor' , label : 'Vendor' , width : '20%' } ] } );
      contentArea.append( tablePanel.getDomNode() );
      contentArea.append( tablePanel2.getDomNode() );
      
      $( getBundleCmd ).on( 'executionSucceeded', function( event )
              {
                  /** @type {Object}  */ var bundle                      = event.response.payload.theBundle;
                  tablePanel.appendRow(['Name'          , bundle.name != null ?bundle.name :'']);
                  tablePanel.appendRow(['Bundle Id'     , bundle.bundleId != null ? bundle.bundleId : '' ]);
                  tablePanel.appendRow(['Symbolic Name' , bundle.symbolicName !=null ? bundle.symbolicName : '']);
                  tablePanel.appendRow(['Location'      , bundle.location != null? bundle.location : '' ]);
                  tablePanel.appendRow(['Status'        , bundle.statusText !=null ?bundle.statusText : '']);
                  tablePanel.appendRow(['Description'   , bundle.description !=null ? bundle.description : '']);
                  tablePanel.appendRow(['Vendor'        , bundle.vendor != null ? bundle.vendor : '']);
                  tablePanel.appendRow(['Version'       , bundle.version != null ? bundle.version : '']);
                  tablePanel.appendRow(['Activator'     , bundle.activator != null ? bundle.activator :'']);
                  tablePanel.appendRow(['Data Root'     , bundle.dataRoot!= null ? bundle.dataRoot : '']);
                  tablePanel.appendRow(['State Location', bundle.stateLocation != null ? bundle.stateLocation  : '']);
                  $.each( bundle.registeredServices, function( index, service )
                          {
                              var pid         = service.servicePid != null ? service.servicePid : ""; 
                              var description = service.serviceDescription != null ? service.serviceDescription : "";
                              var vendor      = service.serviceVendor != null ? service.serviceVendor : "";
                              tablePanel2.appendRow([index+1, pid, description, vendor]);
                          })
                  
              });
      
      getBundleCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.OsgiHandler;
  }
);
