/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/SetupwizardHandler'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          ,TablePanel
          ,FormPanel
          ,ButtonFactory
          ,Command
          ,ShowCommand
          , SetupwizardHandler
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'PasswordHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.PasswordHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.PasswordHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.PasswordHandler.prototype ).constructor = tsa.cmd.handlers.PasswordHandler;

    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.PasswordHandler.prototype._index = function()
    {
     /** @type {Object}                  */     var i18n                    = tsa.i18n;
     /** @type {tsa.widgets.ContentArea} */     var contentArea             = null;
     /** @type {tsa.widgets.TablePanel} */      var tablePanel              = null;
     /** @type {tsa.widgets.FormPanel} */       var formPanel               = null;
     /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer         = ButtonFactory.buildButtonContainer();
     /** @type {tsa.cmd.Command} */             var getPasswordAgeCommand   = null;
     /** @type {tsa.cmd.ShowCommand} */         var showCmd                 = new ShowCommand( { action : 'main/index', clearSystemMessages : false  } );

    contentArea = new ContentArea ( { title         : i18n.labelPasswordHeading
                                   , helpDoc       : 'sshelp_password_panel.html'
                                   , showBackToTop : true } );
    contentArea.appendMessage( { text : i18n.labelPasswordCaption } );

    tablePanel = new TablePanel( { title    : i18n.labelPasswordAgeHeading
                        , summary  : 'Password form'
                        , columns  : [ { id : '' } ] } );

    contentArea.append( tablePanel.getDomNode() );

    formPanel= new FormPanel( { title  : i18n.labelPasswordChangeHeading
                              , action : 'password/ajaxUpdate' } );
    formPanel.setDescription( i18n.labelPasswordEnterDesc1);
    formPanel.appendFieldRow( { label    : i18n.labelPasswordEnterCurrentPasswd
                                , required : true
                                , element  : $( '<input type="password" size="50" name="currPassword" id="currPassword" value="" required/>' ).get(0)
                                } );
    formPanel.appendFieldRow( { label    : i18n.labelPasswordEnterPassw1
                                , required : true
                                , element  : $( '<input type="password" size="50" name="password" id="password" value="" required/>' ).get(0)
                                } );
    formPanel.appendFieldRow( { label    : i18n.labelPasswordEnterPassw2
                                , required : true
                                , element  : $( '<input type="password" size="50" name="password_confirm" id="password_confirm" value="" required/>' ).get(0)
                                } );
    //delete the wizard menu
    /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
    setupwizardHandler.exitWizard();

    $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonSave
          , onclick : function( event )
            {
              formPanel.submit();
        $( formPanel ).on( 'executionSucceeded', function( event )
        {
          showCmd.execute();
        } );

            }
          }
        )
      );

    $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

    $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'cancel'
          , grayout : true
          , label   : i18n.buttonCancel
          , onclick : function( event )
            {
              showCmd.execute();
            }
          }
        )
      );
    contentArea.append( formPanel.getDomNode() );
    contentArea.append( buttonContainer );
    this.show( contentArea.getDomNode() );

     //retrieving data from server
    getPasswordAgeCommand = new Command({
      action:'password/getPasswordAge'
     ,clearSystemMessages:false
      });

    $(getPasswordAgeCommand).on('executionSucceeded', function(event){
       /**@type {number} */ var passwordAge=event.response.payload.passwordAge;
        if(passwordAge<0){
          tablePanel.appendRow( [ i18n.labelPasswordIsNotChangedMessage] );
        }else if(passwordAge==0){
          tablePanel.appendRow( [ i18n.labelPasswordIsChangedTodayMessage] );
        }else{
          tablePanel.appendRow( [ tsa.bind(i18n.labelPasswordAgeMessage,[passwordAge]) ] );
        }
      })
      getPasswordAgeCommand.execute();
    };
//
    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.PasswordHandler;
  }
);