/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TabsPanel'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ButtonFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , TabsPanel
          , Command
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'PropertiesHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.PropertiesHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.PropertiesHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.PropertiesHandler.prototype ).constructor = tsa.cmd.handlers.PropertiesHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.PropertiesHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n				= tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea		= null;
      /** @type {tsa.widgets.TablePanel}    */ var tabsPanel		= null;
      /** @type {Element}                   */ var saveButton		= null;
      /** @type {Element}                   */ var undoButton		= null;
      /** @type {Element}                   */ var textArea			= null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer	= ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.FormPanel}     */ var textFormPanel	= new FormPanel( { action : 'properties/ajaxUpdate', border : false } );
      /** @type {tsa.cmd.Command}           */ var getPropCmd		= new Command( { action  : 'properties/ajaxEdit' } );
  
      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelPropertiesHeading
                                      , helpDoc : 'sshelp_advanced_properties_panel.html' } );
      
      contentArea.appendMessage( { text : i18n.labelPropertiesCaption } );
      
      var cautionText = tsa.NBSP + "<span style='font-weight: bold'>" + i18n.labelCaution + ":</span><br/>" + i18n.labelPropertiesDisclaimer;
      contentArea.appendMessage( { level : 'WARN'  , border : true , text : cautionText , textIsHtml : true } );
             
	  //get properties file
      $( getPropCmd ).on( 'executionSucceeded', function( event ){
    	  var idMap				= {};
    	  var textMap 			= {};
    	  var restartMap		= {};
    	  var currentId			= 0;
    	  var infoSrc		    = tsa.attributes.imagesPath + '/' + tsa.i18n[ 'imageStatusInfo16' ];
    	  var tabList		    = event.response.payload.tabs;
    	  var selectedId		= event.response.payload.selectedTabId;
    	  var restartRequired	= event.response.payload.restartRequired;
    	  var content			= event.response.payload.text;
    
    	  var infoText = tsa.NBSP + "<span style='font-weight: bold'>" + i18n.labelNote + ":</span><span id='restartSpan'>";
    	  if(restartRequired)
    		  infoText += tsa.NBSP + i18n.labelPropertiesRestartMessage;
		  else
			  infoText += tsa.NBSP + i18n.labelPropertiesImmediateMessage;
    	  
    	  infoText += "</span>";
    	  contentArea.appendMessage( { level : 'INFO'  , border : true , text : infoText , textIsHtml : true } );
         
   	   	  //set up tabs
   	   	  var tabInfo = [];
   	   	  
   		  $.each( tabList, function( tabIndex, tab )
    	  {
    		  idMap		[ tabIndex ] = tab.id;	//set map of tab ids
    		  textMap	[ tabIndex ] = tab.text; //set map of tab contents

        	  var subtitle = document.createElement( 'p' );
       	   	  $( subtitle ).text( i18n.labelPropertiesTabDesc );

    		  //set map of restartRequired	
    		  if(tab.restartRequired)
    			  restartMap[ tabIndex ] = " " + i18n.labelPropertiesRestartMessage;
    		  else
    			  restartMap[ tabIndex ] = " " + i18n.labelPropertiesImmediateMessage;
    		      
    		  if(tab.isFirst){
    			  tabInfo.push( { title : tab.label, body : subtitle, isFirst : true } );
    			
    		  }else if(tab.isLast) {
    			  tabInfo.push( { title : tab.label, body : subtitle, isLast : true } );
    		   
    		  } else{
    			  tabInfo.push( { title : tab.label, body : subtitle } );
    			  
    		  }
    		
    	  });
   		  
   		  tabsPanel = new TabsPanel( { tabs: tabInfo } );
    	  
   		  //FILE CONTENT AREA
    	  textFormPanel.putHiddenInput('selectedTabId', selectedId);
    	      	  
    	  textArea = document.createElement( 'textarea' );
    	  $( textArea ).attr( 'id', 'text' );
    	  $( textArea ).attr( 'name', 'text' );
    	  $( textArea ).attr( 'rows', '30' );
    	  $( textArea ).attr( 'cols', '130' );
    	  $( textArea ).css( { 'border-width' : '1px'} );
    	  $( textArea ).val( content );
    	  textFormPanel.appendJointRow( textArea );
    	 
    	  //Buttons
          saveButton  = ButtonFactory.buildButton( { type : 'arrow' , label : i18n.buttonSave } );
          undoButton  = ButtonFactory.buildButton( { type : 'undo'  , label : i18n.buttonUndo , grayout : true} );
    	  
          $( saveButton ).on( 'click', function( event ){
        	  textFormPanel.submit();
        	  textMap [ currentId ] =  $('#text').val();
           } );
          
          $( undoButton ).on( 'click', function( event ){
        	  $( textArea ).val( textMap[currentId] );
           } );
                    
          $( buttonContainer ).append( saveButton );
          $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
          $( buttonContainer ).append( undoButton );
          textFormPanel.appendJointRow( buttonContainer );
          
          //When a tab is clicked
          $( tabsPanel ).on( 'tabChanged', function( event ){
        	 var tabIdx = event.tabIndex;
           	 textFormPanel.putHiddenInput('selectedTabId', idMap[tabIdx]);
           	 $( textArea ).val( textMap[tabIdx] );
           	 $('#restartSpan').text(restartMap[tabIdx]);
           	 currentId = tabIdx;
           } );
            	  
      
          contentArea.append( tabsPanel.getDomNode() );
          contentArea.append( textFormPanel.getDomNode() );
      }); 
      
      //Get prop/tab info
      getPropCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.PropertiesHandler;
  }
);