/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/data/Countries'
  , 'tsa/cmd/handlers/SetupwizardHandler'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , FormPanel
          , ButtonFactory
          , Command
          , ShowCommand
          , Countries
          , SetupwizardHandler
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'RegistrationHandler' );
    
    //-------------------------------------------------------------------------

    /**
     * @constructor
     * @extends tsa.cmd.AbstractHandler
     * Parameters added by this class (in addition to super-class' parameters):
     *   none.
     *
     * @param {Object} parameters The parameters.
     */
    tsa.cmd.handlers.RegistrationHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.RegistrationHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.RegistrationHandler.prototype ).constructor = tsa.cmd.handlers.RegistrationHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.RegistrationHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                     = tsa.i18n;
      /** @type {tsa.widgets.FormPanel}     */ var formPanelContact         = new FormPanel( { title  : i18n.labelRegistrationContactHeading
                                                                                           , action : 'registration/ajaxUpdateRegistration' } );
      /** @type {tsa.widgets.FormPanel}     */ var formPanelLocation        = new FormPanel( { title  : i18n.labelRegistrationSyslocHeading} );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer          = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getRegistrationCommand   = null;
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdCancel            = new ShowCommand( { action : 'main/index' } );
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdSave              = new ShowCommand( { action : 'registration' , clearSystemMessages : false } );
      /** @type {Element}                   */ var selectCountries          = document.createElement( 'select' );
      /** @type {Element}                   */ var setupWizardEnabled       = this.getParameter("setupWizardEnabled");
      // Initialize the content area (its title and its help document)...
      if ( setupWizardEnabled || (setupWizardEnabled == false || setupWizardEnabled == 'false') ) {
          /** @type {tsa.widgets.ContentArea}   */ var contentArea              = new ContentArea ( { title   : i18n.labelRegistrationHeading
              ,  helpDoc : 'sshelp_wizard_registration_panel.html' } );
      } else {
          /** @type {tsa.widgets.ContentArea}   */ var contentArea              = new ContentArea ( { title   : i18n.labelRegistrationHeading
              ,  helpDoc : 'sshelp_registration_panel.html' } );
      }
      
      contentArea.appendMessage( { text : i18n.labelRegistrationCaption } );
      
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactCo
                                       , required : true
                                       , hint     : i18n.labelRegistrationContactCoDesc
                                       , element  : $( '<input type="text" size="50" name="contactCompany" id="contactCompany" value="" required/>' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactName
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactNameDesc
                                       , element  : $( '<input type="text" size="50" name="contactName" id="contactName" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactPhone
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactPhoneDesc
                                       , element  : $( '<input type="text" size="50" name="contactPhone" id="contactPhone" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactEmail
                                       , required : false
                                       , hint     : i18n.labelRegistrationContactEmailDesc 
                                       , element  : $( '<input type="text" size="50" name="contactEmail" id="contactEmail" value="" />' ).get(0)
                                       } );
      formPanelContact.appendFieldRow( { label    : i18n.labelRegistrationContactIBMid
                                       , required : false
                                       , element  : $( '<input type="text" size="50" name="contactIBMid" id="contactIBMid" value="" />' ).get(0)
                                       } );
      
      /** @type {Element} */ var infoIBMIdDiv  = document.createElement( 'div' );
      /** @type {Element} */ var aRefIBMidLink1 = document.createElement("a");
      /** @type {Element} */ var aRefIBMidLink2 = document.createElement("a");
      
      $( aRefIBMidLink1 ).attr( 'href', i18n.labelRegistrationContactIBMidInstructionsLink1 );
      $( aRefIBMidLink1 ).attr( 'target', '_blank' );
      $( aRefIBMidLink1 ).append( i18n.labelRegistrationContactIBMidInstructionsLink1Text );
      
      $( aRefIBMidLink2 ).attr( 'href', i18n.labelRegistrationContactIBMidInstructionsLink2 );
      $( aRefIBMidLink2 ).attr( 'target', '_blank' );
      $( aRefIBMidLink2 ).append( i18n.labelRegistrationContactIBMidInstructionsLink2Text );
      
      $( infoIBMIdDiv ).css( { 'color'     : '#666'
                             , 'padding'   : '2px'
                             , 'font-size' : '0.9em' } );
      
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions1 );
      $( infoIBMIdDiv ).append( aRefIBMidLink1 );
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions2 );
      $( infoIBMIdDiv ).append( aRefIBMidLink2 );
      $( infoIBMIdDiv ).append( i18n.labelRegistrationContactIBMidInstructions3 );
      
      formPanelContact.appendFieldRow( { required : false
                                       , element  : $( infoIBMIdDiv ).get(0)
                                       } );
      
      formPanelContact.setDescription( i18n.labelRegistrationContactDesc );
      formPanelLocation.setDescription( i18n.labelRegistrationSyslocDesc );
      
      selectCountries.name = 'locationCountry';
      selectCountries.id   = 'locationCountry';
      
      $.each( Countries.getCountries(), function( countryIndex, country )
      {
        /** @type {Element} */ var opt = document.createElement( 'option' );
        opt.value = country.id;
        $( opt ).text( i18n[country.name.label] ); 
        $( selectCountries ).append( opt );
      } );
      
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocCountry
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocCountryDesc 
                                        , element  : selectCountries
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocState
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocStateDesc 
                                        , element  : $( '<input type="text" size="50" name="locationState" id="locationState" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocZip
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocZipDesc 
                                        , element  : $( '<input type="text" size="50" name="locationZip" id="locationZip" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocCity
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocCityDesc 
                                        , element  : $( '<input type="text" size="50" name="locationCity" id="locationCity" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocAddress
                                        , required : true
                                        , hint     : i18n.labelRegistrationSyslocAddressDesc 
                                        , element  : $( '<input type="text" size="50" name="locationAddress" id="locationAddress" value="" required/>' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocPhone
                                        , hint     : i18n.labelRegistrationSyslocPhoneDesc 
                                        , element  : $( '<input type="text" size="50" name="locationPhone" id="locationPhone" value="" />' ).get(0)
                                        } );
      formPanelLocation.appendFieldRow( { label    : i18n.labelRegistrationSyslocBuilding
                                        , required : false
                                        , hint     : i18n.labelRegistrationSyslocBuildingDesc 
                                        , element  : $( '<input type="text" size="50" name="locationName" id="locationName" value=""/>' ).get(0)
                                        } );
      
      $( formPanelContact ).on( 'executionSucceeded', function( event )
      { 
          // Check if the payload has the setupWizardEnabled parameter = true
            if (!TV.typeIsUndefined( typeof event.response.payload.setupWizardEnabled ) && event.response.payload.setupWizardEnabled == 'true') {
                // hide alert panel
                /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                setupwizardHandler.hideAlertPanel();
                setupwizardHandler.removeWizardLetfMenu();
                
                if ((setupWizardEnabled == false || setupWizardEnabled == 'false')) {
                    tsa.cmd.handlers.RegistrationHandler.redirectWizard ('clock/index', "true" );
                } else {
                    tsa.cmd.handlers.RegistrationHandler.redirectWizard ('clock/index', "false" );
                }
            } else {
          	 showCmdSave.execute();
          }
      } );
      
      $( formPanelContact ).on( 'executionFailed', function( event )
      {
        if( TV.typeIsObject( typeof event.response.payload.errors ) )
        {
          for( /** @type {string} */ var fieldName in event.response.payload.errors )
          {
            formPanelContact.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
          }
        }
      } );
      
      /**
       * Check the setupWizardEnabled and setupWizardEnabled = false parameters to see if any is true and display the screen in wizard mode
       */
      if ( setupWizardEnabled || (setupWizardEnabled == false || setupWizardEnabled == 'false') ) {
          /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
         /** hide left menu* */
         setupwizardHandler.hideLeftMenu();
         
         productList = [ i18n.labelnameConnectivity, i18n.labelnameRegistration, i18n.labelnameClock, i18n.labelnameTransmition, i18n.labelnameIU ];
         
         
         setupwizardHandler.generateList( productList, 1 );
             
         // variable for indicate the wizard process
         formPanelContact.appendFieldRow( { label     : ''
                                           , required : false
                                           , element  : $( '<input type="hidden" size="50" name="setupWizardEnabled" id="setupWizardEnabled" value="true" />' ).get(0)
                                       } );
         // back button
         $( buttonContainer ).append(
              ButtonFactory.buildButton(
                { type    : 'back'
                , grayout : true
                , label   : i18n.buttonlabelBackSetupwizardPage
                , onclick : function( event )
                  {
                        if ((setupWizardEnabled == false || setupWizardEnabled == 'false')) {
                           tsa.cmd.handlers.RegistrationHandler.redirectWizard ('connectivity/index', "true" );
                          
                        } else {
                            tsa.cmd.handlers.RegistrationHandler.redirectWizard ('connectivity/index', "false" );
                        }
                  }
                }
              )
            );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        /**
         * show the next and save
         */        
        $( buttonContainer ).append(
                ButtonFactory.buildButton(
                  { type    : 'arrow'
                  , grayout : false
                  , label   : i18n.buttonlabelSaveContinueSetupwizardPage
                  , onclick : function( event )
                    {
                       formPanelContact.submit();
                    }
                  }
         ));  
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonlabelExitSetupwizardPage
            , onclick : function( event )
              {
              //remove wizard and left menu
                /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                setupwizardHandler.exitWizard();
                  ( new ShowCommand( { action : 'main/index' } ) ).execute();
              }
            }
          )
        );
        
      } else { 
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonSave
          , onclick : function( event )
            {
              formPanelContact.submit();
            }
          }
        )
      );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'cancel'
          , grayout : true
          , label   : i18n.buttonCancel
          , onclick : function( event )
            {
              showCmdCancel.execute();
            }
          }
        )
      );
      }
      contentArea.append( formPanelContact.getDomNode() );
      contentArea.append( formPanelLocation.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
      
      //retrieving data from server
      getRegistrationCommand = new Command( {
          action:'registration/getRegistration'
        , clearSystemMessages:false
      } );
      
      // linking, master form submits slaves...
      formPanelContact.linkTo( formPanelLocation );
      
      $( getRegistrationCommand ).on( 'executionSucceeded', function(e) {
        for( var key in e.response.payload ) {
          $( '[name="' + key + '"]' ).val( e.response.payload[ key ] );
        }
      } );
      
      //cancel behavior
      getRegistrationCommand.execute(); 
    };
    
    /**
     * Show Commands for wizard
     * @param {string} action Action for the ShowCommand.
     * @param {string} reboot Flag to reboot when wizard finished
     * @param {string} parameters Parameters for the Show Command
     */
    tsa.cmd.handlers.RegistrationHandler.redirectWizard = function( action, reboot  )
    {                
                      var parameters = {};
                      if (reboot == "true") {
                        parameters.setupWizardEnabled = false;  
                       } else {
                        parameters.setupWizardEnabled = true;
                       }
                       (new ShowCommand({
                           action              : action,
                           clearSystemMessages : true,
                           parameters          : parameters
                       })).execute();
    };

    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.RegistrationHandler;
  }
);