/*
 * IBM Technical Support Appliance Version 2 Release 5 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'SchedrestartHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.SchedrestartHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.SchedrestartHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SchedrestartHandler.prototype ).constructor = tsa.cmd.handlers.SchedrestartHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SchedrestartHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n               = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea        = null;
      /** @type {tsa.widgets.TablePanel}    */ var scheduleTablePanel = null;
      /** @type {tsa.widgets.TablePanel}    */ var historyTablePanel  = null;
      /** @type {Element}                   */ var editScheduleButton = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer    = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd      = new Command( { action              : 'schedrestart/getJobInfo'
                                                                                     , clearSystemMessages : false } );
      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelSchedRestartHeading
                                      , helpDoc : 'sshelp_scheduled_restart_panel.html' } );
      
      // Appending messages to the content area...
      contentArea.appendMessage( { text : i18n.labelSchedRestartOverview } );
      contentArea.appendMessage( { level : 'WARN' , border : true , text : i18n.labelCaution + ':' + i18n.msgSchedRestartCaution } );
      
      // Building the schedule information table...
      scheduleTablePanel = new TablePanel( { title    : i18n.labelSchedRestartScheduleHeading
                                           , emptyMsg : i18n.labelSchedRestartStatusDisabled
                                           , columns  : [ { id : 'propertyName'  , width : '30%' }
                                                        , { id : 'propertyValue' , width : '70%' } ] } );
      
      // Building the history table...
      historyTablePanel = new TablePanel( { title    : i18n.labelSchedRestartHistoryHeading
                                          , emptyMsg : i18n.labelSchedRestartNoJobHistory
                                          , columns  : [ { id: 'status'   , label: i18n.labelSchedRestartStatus   , width: '15%' }
                                                       , { id: 'name'     , label: i18n.labelSchedRestartInstance , width: '30%' }
                                                       , { id: 'state'    , label: i18n.labelSchedRestartState    , width: '15%' }
                                                       , { id: 'comments' , label: i18n.labelSchedRestartComments , width: '45%' } ] } );
      
      // Preparing the retrieval of the job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object} */ var jobInfo  = event.response.payload.jobInfo;
        /** @type {Object} */ var schedule = jobInfo.schedule;
        /** @type {Object} */ var history  = jobInfo.jobHistory;
        
        // Populating schedule table (if available; if not, the empty-message will be shown)...
        if( schedule !== null )
        {
          scheduleTablePanel.appendRow( [ i18n.labelSchedRestartNextRun  , jobInfo.nextPerformanceTS  ] );
          scheduleTablePanel.appendRow( [ i18n.labelSchedRestartSchedule , schedule.formatted         ] );
        }
        
        // Populating history table (if available; if not, the empty-message will be shown)...
        if( history !== null )
        {
          $.each( history, function( entryIndex, entry )
          {
            /** @type {Element} */ var commentList = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get(0);
            
            $.each( entry.comments, function( commentIndex, comment )
            {
              /** @type {Element} */ var commentElement = $( '<li style="padding-top: 0px;"></li>' ).get(0);
              
              $( commentElement ).text( comment );
              $( commentList ).append( commentElement );
            } );
            
            historyTablePanel.appendRow( [ IconFactory.buildSeverityIcon( { severity : entry.statusCode } )
                                         , entry.name
                                         , entry.stateText
                                         , commentList ] );
          } );
        }
      } );
      
      // Buttons...
      editScheduleButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonSchedRestartEditSchedule } );
      
      $( editScheduleButton ).on( 'click', function( event )
      {
        ( new ShowCommand( { action : 'schedrestart/edit/com.ibm.servicestation.schedrestart' } ) ).execute();
      } );
      
      $( buttonContainer ).append( editScheduleButton );
      
      // Appending content...
      contentArea.append( scheduleTablePanel.getDomNode() );
      contentArea.append( historyTablePanel.getDomNode() );
      contentArea.append( buttonContainer );
      
      // Performing the job information retrieval...
      getJobInfoCmd.execute();
      
      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SchedrestartHandler.prototype._edit = function()
    {
      /** @type {Object}                    */ var i18n                = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea         = null;
      /** @type {tsa.widgets.FormPanel}     */ var enablementFormPanel = null;
      /** @type {tsa.widgets.FormPanel}     */ var scheduleFormPanel   = null;
      /** @type {Element}                   */ var enablementSelector  = document.createElement( 'select' );
      /** @type {Element}                   */ var hourSelector        = document.createElement( 'select' );
      /** @type {Element}                   */ var minuteSelector      = document.createElement( 'select' );
      /** @type {Element}                   */ var weekModeRadio       = document.createElement( 'input' );
      /** @type {Element}                   */ var monthModeRadio      = document.createElement( 'input' );
      /** @type {Array.<Element>}           */ var weekDayCheckboxes   = [];
      /** @type {Element}                   */ var weekDaysRow         = null;
      /** @type {Array.<Element>}           */ var monthDayCheckboxes  = [];
      /** @type {Element}                   */ var monthDaysRow        = null;
      /** @type {Element}                   */ var saveButton          = null;
      /** @type {Element}                   */ var cancelButton        = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer     = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd       = new Command( { action              : 'schedrestart/getJobInfo'
                                                                                      , clearSystemMessages : false } );
      
      $( enablementSelector ).attr("id", "enabled"   );
      $( hourSelector       ).attr("id", "hour"      ); 
      $( minuteSelector     ).attr("id", "minute"    );
      $( weekModeRadio      ).attr("id", "weekMode"  );
      $( monthModeRadio     ).attr("id", "monthMode" );
      
      
      // Initializing content area...
      contentArea = new ContentArea( { title   : i18n.labelSchedRestartHeading
                                     , helpDoc : 'sshelp_edit_scheduled_restart_panel.html' } );
      
      // Initializing form panels...
      enablementFormPanel = new FormPanel( { title       : i18n.labelSchedRestartEnableHeading
                                           , description : i18n.labelSchedRestartEnableCaption
                                           , action      : 'schedrestart/putJobSchedule' } );
      
      scheduleFormPanel = new FormPanel( { title       : i18n.labelSchedRestartScheduleHeading
                                         , description : i18n.labelSchedRestartScheduleCaption } );
      
      $( enablementFormPanel ).on( 'executionSucceeded', function( event )
      {
        tsa.printSysMessage( { level: 'info', message: i18n.labelSchedRestartSettingsUpdated } );
        
        ( new ShowCommand( { action              : 'schedrestart/edit/com.ibm.servicestation.schedrestart'
                           , clearSystemMessages : false} ) ).execute();
      } );
      
      // Multiple form panels, but submitted as a single form...
      enablementFormPanel.linkTo( scheduleFormPanel );
      
      // Enablement selector...
      enablementSelector.name = 'enabled';
      
      $.each( [ { value: 'true'  , label: i18n.labelSchedRestartEnabled  }
              , { value: 'false' , label: i18n.labelSchedRestartDisabled } ]
            , function( optionDataIndex, optionData )
              {
                /** @type {Element} */ var option = document.createElement( 'option' );
                
                $( option ).attr( 'value', optionData.value );
                $( option ).text( optionData.label );
                $( enablementSelector ).append( option );
              } );
      
      enablementFormPanel.appendFieldRow( { label    : i18n.labelSchedRestartEnable
                                          , required : true
                                          , element  : enablementSelector } );
      
      /** @type {function(Event)} */
      var enablementChanged = function( event )
      {
        /** @type {boolean} */ var disabled = ( $( enablementSelector ).val() === 'false' );
        
        hourSelector   .disabled = disabled;
        minuteSelector .disabled = disabled;
        weekModeRadio  .disabled = disabled;
        monthModeRadio .disabled = disabled;
        
        $.each( weekDayCheckboxes.concat( monthDayCheckboxes ), function( checkboxIndex, checkbox )
        {
          checkbox.disabled = disabled;
        } );
      };
      
      $( enablementSelector ).on( 'change', enablementChanged );
      
      // Hour selector...
      hourSelector.name = 'hour';
      
      for( /** @type {number} */ var hour=0; hour<24; hour++)
      {
        $( hourSelector ).append( $( '<option value="' + hour + '">' + ( ( hour < 10 ) ? '0' : '' ) + hour + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtHour
                                        , required : true
                                        , element  : hourSelector } );
      
      // Minute selector...
      minuteSelector.name = 'minute';
      
      for( /** @type {number} */ var minute=0; minute<60; minute+=5)
      {
        $( minuteSelector ).append( $( '<option value="' + minute + '">' + ( ( minute < 10 ) ? '0' : '' ) + minute + '</option>' ) );
      }
      
      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtMinute
                                        , required : true
                                        , element  : minuteSelector } );
      
      // Day mode radio buttons...
      weekModeRadio.type    = 'radio';
      weekModeRadio.name    = 'daymode';
      weekModeRadio.value   = 'week';
      weekModeRadio.checked = true;
      monthModeRadio.type   = 'radio';
      monthModeRadio.name   = 'daymode';
      monthModeRadio.value  = 'month';
      
      scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleDayMode
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span       = document.createElement( 'span' );
                         /** @type {Element} */ var weekLabel  = document.createElement( 'label' );
                         /** @type {Element} */ var monthLabel = document.createElement( 'label' );
                         
                         $( weekLabel  ).append( weekModeRadio  , document.createTextNode( i18n.scheduleDayModeWeek  ) );
                         $( monthLabel ).append( monthModeRadio , document.createTextNode( i18n.scheduleDayModeMonth ) );
                         $( span       ).append( weekLabel, document.createElement( 'br' ), monthLabel );
                         $( span       ).addClass( 'ibm-input-group' );
                         $( span       ).attr( 'role', 'radiogroup' );
                         $( span       ).attr( 'aria-label', i18n.scheduleDayMode );
                         
                         return span;
                       } )()
      } );
      
      // Days of week checkboxes...
      weekDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnDays
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span = document.createElement( 'span' );
                         
                         $( span ).addClass( 'ibm-input-group' );
                         $( span ).attr( 'role', 'group' );
                         $( span ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         $.each( [ 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ], function( wdIndex, wd )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="weekday" value="' + ( wdIndex + 1 ) + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label>' + i18n[ 'schedule' + wd.charAt(0).toUpperCase() + wd.substring(1) ] + '</label>' ).get(0);
                           
                           $( label ).prepend( checkbox );
                           weekDayCheckboxes.push( checkbox );
                           $( span ).append( label, document.createElement( 'br' ) );
                         } );
                         
                         return span;
                       } )()
      } );
      
      // Days of month checkboxes...
      monthDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnMonthDays
        , hint     : i18n.scheduleOnMonthDaysDesc
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var div = document.createElement( 'div' );
                         
                         $( div ).addClass( 'ibm-input-group' );
                         $( div ).attr( 'role', 'group' );
                         $( div ).attr( 'aria-label', i18n.scheduleOnDays );
                         
                         for( /** @type {number} */ var md=1; md<=31; md++ )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="monthday" value="' + md + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label style="display: inline-block; font-weight: normal; margin-right: 0.8em">'
                                                                  + ( ( md < 10 ) ? '0' : '' ) + md
                                                                  + '</label>' ).get(0);
                           $( label ).prepend( checkbox );
                           monthDayCheckboxes.push( checkbox );
                           $( div ).append( label );
                           
                           if( ( md % 7 ) === 0 )
                           {
                             $( div ).append( document.createElement( 'br' ) );
                           }
                         };
                         
                         return div;
                       } )()
      } );
      
      // Event handler: day mode changed...
      /** @type {function(Event)} */
      var dayModeChanged = function( event )
      {
        logger.info( 'Day mode changed: processing started...' );
        
        $( weekDaysRow  ).css( { 'display' : ( weekModeRadio  .checked ? '' : 'none' ) } );
        $( monthDaysRow ).css( { 'display' : ( monthModeRadio .checked ? '' : 'none' ) } );
        
        // Selecting a mode will enable all of the checkboxes of the other mode...
        if ( weekModeRadio  .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', true ); } );
        if ( monthModeRadio .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', true ); } );
        
        // If the user triggered the event (event is an object, and it is not null), clear the check-boxes of the mode he selected...
        if( TV.typeIsObject( typeof event ) && ( event !== null ) )
        {
          if( weekModeRadio  .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', false ); } );
          if( monthModeRadio .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', false ); } );
        }
        
        logger.info( 'Day mode changed: processing finished.' );
      };
      
      $( weekModeRadio  ).add( monthModeRadio ).on( 'change', dayModeChanged );
      
      // Preparing the retrieval of the job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object} */ var jobInfo   = event.response.payload.jobInfo;
        /** @type {Object} */ var schedule  = jobInfo.schedule;
        /** @type {Object} */ var isEnabled = ( schedule !== null );
        
        // Populating schedule table (if available; if not, the empty-message will be shown)...
        if( isEnabled )
        {
          $( enablementSelector ).val( 'true' );
          $( hourSelector       ).val( schedule.hour   );
          $( minuteSelector     ).val( schedule.minute );
          
          weekModeRadio  .checked = ( schedule.dayOfWeek.length !== 7);
          monthModeRadio .checked = ( schedule.dayOfWeek.length === 7);
          
          $.each( weekDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfWeek.indexOf( checkboxIndex + 1 ) >= 0 );
          } );
          
          $.each( monthDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfMonth.indexOf( parseInt( $( checkbox ).val(), 10 ) ) >= 0 );
          } );
        }
        else
        {
          $( enablementSelector ).val( 'false' );
        }
        
        enablementChanged( null );
        dayModeChanged( null );
      } );
      
      // Buttons...
      saveButton   = ButtonFactory.buildButton( { type : 'arrow'  , label : i18n.buttonSave   } );
      cancelButton = ButtonFactory.buildButton( { type : 'cancel' , label : i18n.buttonCancel } );
      
      $( saveButton ).on( 'click', function( event )
      {
        enablementFormPanel.submit();
      } );
      
      $( cancelButton ).on( 'click', function( event )
      {
        ( new ShowCommand( { action : 'schedrestart/index' } ) ).execute();
      } );
      
      $( buttonContainer ).append( saveButton );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( cancelButton );
      
      // Appending content...
      contentArea.append( enablementFormPanel.getDomNode() );
      contentArea.append( scheduleFormPanel.getDomNode() );
      contentArea.append( buttonContainer );
      
      // Performing the job information retrieval...
      getJobInfoCmd.execute();
      
      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.SchedrestartHandler;
  }
);