/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'  
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ButtonFactory
          , FormPanel
          , ShowCommand
          , Command
          , IconFactory)
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ScopesHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ScopesHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ScopesHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopesHandler.prototype ).constructor = tsa.cmd.handlers.ScopesHandler;
    // Exclusion Form Panel
    tsa.cmd.handlers.ScopesHandler.prototype.fpExclusion         = null;
    
    tsa.cmd.handlers.ScopesHandler.prototype.noOfExclusionRows          = 0;
    
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype._new = function(){
      
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {boolean}                 */     var editEnabled             = true;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelDiscoScopesHeading
                                                                                , helpDoc       : 'sshelp_discovery_scope_new_edit_panel.html'
                                                                                , showBackToTop : true } );
      /** @type {tsa.widgets.FormPanel}   */     var fpDiscoveryOptions      = new FormPanel( { title  : i18n.labelDiscoScopesSelectHeading
                                                                                 , action : 'scopes/ajaxCreateScope' } );
      /** @type {tsa.widgets.FormPanel}   */     var fpAdresses              = new FormPanel( { title  : i18n.labelDiscoScopesAddressHeading
                                                                                                , description: i18n.labelDiscoScopesAddressDesc } );
      /** @type {string}                  */     var pid                     = this.getParameter('pid');
      /** @type {element}        */              var buttonException         = null;
      /** @type {element}        */              var buttonSeparator         = null;
      this.fpExclusion = new FormPanel( { title   : i18n.labelDiscoScopeExclusionHeading
                                    , description : i18n.labelDiscoScopeExclusionDesc } );
      // Do not show by default Exclusion Panel
      this.fpExclusion.hide();
      fpDiscoveryOptions.linkTo( fpAdresses );
      fpAdresses.linkTo( this.fpExclusion );
      fpAdresses.appendFieldRow( {  label    : i18n.labelDiscoScopesAddressAddress
                                  , required : true
                                  , element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0) 
                                  , hint     : i18n.labelSpecifyIpAddressFieldDescription } );
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {element}                   */   var selectIP        = $("<select name='option' id='option' size='1'></select>").get(0);
      /** @type {element}                   */   var option          = document.createElement( 'option' );
      /** @type {function()}        */   
      var changeHandler = function(e){
        tsa.clearSysMessages();
        // Remove validations
        $(fpAdresses.layoutTableBody).empty();
        fpAdresses.removeValidation("aaddr");
        fpAdresses.removeValidation("rstart");
        fpAdresses.removeValidation("rend");
        fpAdresses.removeValidation("rdesc");
        fpAdresses.removeValidation("naddr");
        fpAdresses.removeValidation("nmask");
        fpAdresses.removeValidation("ndesc");
        
        if (this.value === 'address'){
          $(buttonException).hide();
          $(buttonSeparator).hide();
          fpAdresses.setTitle(i18n.labelDiscoScopesAddressHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesAddressDesc);
          fpAdresses.appendFieldRow( {  label    : i18n.labelDiscoScopesAddressAddress
                                      , required : true
                                      , element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0) 
                                      , hint     : i18n.labelSpecifyIpAddressFieldDescription } );
          e.data.fpExclusion.hide();
        } else if (this.value === 'range'){
          $(buttonException).show();
          $(buttonSeparator).show();
          fpAdresses.setTitle(i18n.labelDiscoScopesRangeHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesRangeDesc);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeStart
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='rstart' id='rstart' value='' aria-required='true' class='ibm-text' aria-describedby='descrstart' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesRangeStartDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeEnd
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='rend' id='rend' value='' aria-required='true' class='ibm-text' aria-describedby='descrend' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesRangeEndDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  : $("<input size='50' maxlength='250' name='rdesc' id='rdesc' value='' class='ibm-text' aria-describedby='descrdesc' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
          e.data.fpExclusion.show();
          
        } else if (this.value === 'network'){
          $(buttonException).show();
          $(buttonSeparator).show();
          fpAdresses.setTitle(i18n.labelDiscoScopesNetworkHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesNetworkDesc);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkAddress
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='naddr' id='naddr' value='' aria-required='true' class='ibm-text' aria-describedby='descnaddr' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesNetworkAddressDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkMask
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='nmask' id='nmask' value='' aria-required='true' class='ibm-text' aria-describedby='descnmask' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesNetworkMaskDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  : $("<input size='50' maxlength='250' name='ndesc' id='ndesc' value='' class='ibm-text' aria-describedby='descndesc' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
          e.data.fpExclusion.show();
        }
      };
      $( selectIP ).on('change'   ,{ fpExclusion : this.fpExclusion }, changeHandler );
      $( option ).attr('value'    , 'address' );
      $( option ).attr('selected' , 'selected');
      $( option ).text(i18n.labelDiscoScopesSelectAddress );
      $( selectIP ).append(option);
      
      option   = document.createElement( 'option' );
      $( option ).attr( 'value', 'range' );
      $( option ).text( i18n.labelDiscoScopesSelectRange );
      $( selectIP ).append(option);
      
      option   = document.createElement( 'option' );
      $( option ).attr( 'value', 'network' );
      $( option ).text( i18n.labelDiscoScopesSelectNetwork );
      $(selectIP).append(option);
      
      fpDiscoveryOptions.setDescription(i18n.labelDiscoScopesSelectDescription);
      fpDiscoveryOptions.appendFieldRow( {  label    : i18n.labelDiscoScopesComboBoxLabel
                                          , required : true
                                          , element  : selectIP } );
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : $.proxy(function( event )
              {
              fpDiscoveryOptions.putHiddenInput("noOfRows",this.noOfExclusionRows);
              fpDiscoveryOptions.putHiddenInput("pid",pid);
              fpDiscoveryOptions.submit();
              $( fpDiscoveryOptions ).on( 'executionSucceeded', function( event )
              {
                (new ShowCommand( {
                  action : 'scopesets/edit'
                    , clearSystemMessages : false 
                    , parameters :  { pid : pid } 
                })).execute();
              } );
              // Show errors
              $( fpDiscoveryOptions ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        fpDiscoveryOptions.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );
              
              
              },this)
            }
          )
        );
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
      
      buttonException = ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : true
            , label   : i18n.buttonAddExclusion
            , onclick : $.proxy(function( event )
              {
                $.proxy(this.addExclusionRow(),this);
              },this)
            }
          )
        
      buttonSeparator=ButtonFactory.buildButtonSeparator ;
      
      $( buttonException ).hide();
      $( buttonSeparator ).hide();
      $( buttonContainer ).append( buttonException );
      $( buttonContainer ).append( buttonSeparator )
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
                (new ShowCommand( {
                  action : 'scopesets/index'
                    , clearSystemMessages : false} 
                 )).execute();
              }
            }
          )
        );
      $(fpDiscoveryOptions.form).attr("aria-label", i18n.labelDiscoScopesSelectHeading);
      $(fpAdresses.form).attr("aria-label", i18n.labelDiscoScopesAddressHeading);
      $(this.fpExclusion.form).attr("aria-label", i18n.labelDiscoScopeExclusionHeading);
      contentArea.append( fpDiscoveryOptions.getDomNode() );
      contentArea.append( fpAdresses.getDomNode() );
      contentArea.append( this.fpExclusion.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype._move = function(){
      tsa.clearSysMessages();
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {string}                  */     var pid                     = this.getParameter('pid');
      /** @type {string}                  */     var name                    = this.getParameter('name');
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title        : i18n.labelScopesMoveHeading
                                                                                                , helpDoc       : 'sshelp_edit_move_scope_to_scopeset_panel.html'} );
      /** @type {tsa.widgets.FormPanel} */       var formPanel               = new FormPanel( {        title  : i18n.labelScopesMoveSelectHeading
                                                                                                , description : tsa.bind(i18n.labelScopesMoveSelectDesc,name)
                                                                                                , action      : 'scopes/ajaxMoveScopes' } );
      /** @type {tsa.cmd.Command} */             var getScopesetsCmd         = new Command( {  action               : 'scopesets/getScopeSets'
                                                                                            , clearSystemMessages : false 
                                                                                            , parameters          : { name : pid } } );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer         = ButtonFactory.buildButtonContainer();
      formPanel.putHiddenInput("pid", pid);
      contentArea.appendMessage({ text : i18n.labelScopesMoveCaption }); 
      getScopesetsCmd.execute();
      $(getScopesetsCmd).on("executionSucceeded",function(event){
        /** @type {element}        */   var scopeIps   = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);
        /** @type {element}        */   var scopesDest = $("<select name='destScopeset' id='destScopeset'  size='1' required></select>").get(0);
        $.each( event.response.payload.scopeSets, function( idx, scope ){
         if(scope.pid === pid){
           /** @type {element} */ var scopeInput = null;
           /** @type {element} */ var scopeLabel = null;
           /** @type {string } */ var scopeId    = null;
           $.each(scope.scopeElements, function(idxScope, element){
             scopeId = "scopes" + idxScope;
             scopeInput = $("<input name='scopes' type='checkbox'>").get(0);
             $(scopeInput).attr("id", scopeId);
             $(scopeInput).attr("value", element.pid);
             scopeLabel = document.createElement("label");
             $(scopeLabel).text(element.externalForm);
             $(scopeLabel).attr("for", scopeId);
             $(scopeIps).append(scopeInput, scopeLabel, document.createElement("br"));
             });
         } else {
           /** @type {element} */ var scopeOption = document.createElement("option");
           $(scopeOption).attr("value", scope.pid);
           $(scopeOption).text(scope.displayName);
           $(scopesDest).append(scopeOption);
         }
        });
        formPanel.appendFieldRow( { label    : i18n.SummaryForScopesTable
                                  , required : true
                                  , aRequire : false
                                  , element  :  scopeIps } );
        formPanel.appendFieldRow( { label      : i18n.labelScopesMoveDestScopeSet
                                    , required : true
                                    , element  : scopesDest } );
      });
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonMove
            , onclick : function( event )
              {
                formPanel.submit();
                $( formPanel ).on( 'executionSucceeded', function( event )
                {
                  (new ShowCommand( {
                    action : 'scopesets/show'
                      , clearSystemMessages : false
                      , parameters : { pid : pid } }
                   )).execute();
                } );
              }
            }
          )
        );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
        $( buttonContainer ).append(
            ButtonFactory.buildButton(
              { type    : 'cancel'
              , grayout : true
              , label   : i18n.buttonCancel
              , onclick : function( event )
                {
                (new ShowCommand( {
                  action : 'scopesets/show'
                    , clearSystemMessages : false
                    , parameters: { pid : pid } } 
                 )).execute();
                }
              }
            )
          );
      contentArea.append( formPanel.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype.addExclusionRow = function(){
      $(this.fpExclusion.layoutTable).css('width', '');
      /** @type {Element} */ var row     = document.createElement( 'tr' );
      /** @type {Element} */ var cell    = document.createElement( 'td' );
      /** @type {Element} */ var cell2   = document.createElement( 'td' );
      /** @type {Element} */ var select  = document.createElement( 'select' );
      /** @type {Element} */ var aDel    = $("<a class ='deleteRow' href='#' style='text-decoration: none' ></a>").get(0);
      /** @type {Element} */ var option  = document.createElement( 'option' );
      $( option ).attr( 'selected', 'selected' );
      $( aDel ).on("click", function(e){
        tsa.clearSysMessages();
        $(this).closest('tr').remove();
        });
      /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete' , caption :  'Delete Exclusion'} );
      $(aDel).append(deleteIcon);
      $( select ).attr("href","#");
      $( option ).val("excAddress");
      $( option ).text("Host");
      $( select ).append(option);
      
      option     = document.createElement( 'option' );
      $(option).val("excRange");
      $(option).text("Range");
      $( select ).append(option);
      
      option     = document.createElement( 'option' );
      $(option).val("excNetwork");
      $(option).text("Subnet");
      $( select ).append(option);
      
      $( cell ).append( aDel     );
      $( cell ).append( tsa.NBSP );
      $( cell ).append( select   );
      $( row  ).append(  cell    );
      
      
      this.buildExclusionHost(cell2,this.noOfExclusionRows,this.fpExclusion,null,null);
      
      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopesHandler.prototype.ipRule;
      
      this.fpExclusion.addValidationRule('excAddr['+this.noOfExclusionRows+'][ip]',ipRule);
      $(select).on("change", { noOfExclusionRows : this.noOfExclusionRows , exclusionForm : this.fpExclusion} , function(e){
        $(cell2).empty();
        /** @type {Element} */ var input  = null;
        if (this.value === 'excAddress'){
          
          tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionHost(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);
          
        } else if (this.value === 'excRange'){
          tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionRange(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);
          
        } else if(this.value === 'excNetwork'){
          tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionSubnet(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,null);
        }
      });
      $(cell).append(aDel);
      $(cell).append(select);
      $(row).append( cell );
      $(row).append( cell2 );
      
      this.fpExclusion.layoutTableBody.appendChild( row );
      
      this.noOfExclusionRows+=1;
    
    };
  //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype.createExclusionElementsRows = function(exclusions , pid, sepid){
      $.each( exclusions , $.proxy(function( idx, exclusion ){

        $(this.fpExclusion.layoutTable).css('width', '');
        /** @type {Element} */ var row        = document.createElement( 'tr' );
        /** @type {Element} */ var cell       = document.createElement( 'td' );
        /** @type {Element} */ var cell2      = document.createElement( 'td' );
        /** @type {Element} */ var select     = document.createElement( 'select' );
        /** @type {Element} */ var aDel       = $("<a class ='deleteRow' href='#' style='text-decoration: none' ></a>").get(0);
        /** @type {Element} */ var option     = document.createElement( 'option' );
        /** @type {Element} */ var ipAddress  = null ;
        /** @type {Element} */ var excPid     = null ;
        /** @type {Element} */ var excGPid    = null ;
        $( option ).attr('selected', 'selected');
        $( aDel ).on("click" , function(){
          if( confirm(tsa.i18n.labelDeleteConfirmation+" '"+ exclusion.name+" '") ){
            $(this).closest('tr').remove();
            /** @type {tsa.cmd.Command} */ var actionCmd = new Command( { action : "scopes/ajaxDelete/" + exclusion.pid
                                                                         , clearSystemMessages : false });
            actionCmd.execute();
            $( actionCmd ).on( 'executionSucceeded', function( event )
                 {
                  (new ShowCommand( {
                     action : 'scopes/edit'
                     , clearSystemMessages : false
                     , parameters : { pid : pid
                                    , sepid : sepid}} 
                    )).execute(); 
                  } );
          }
        });
        /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete' , caption :  'Delete Exclusion'} );
        $(aDel).append(deleteIcon);
        $( select ).attr("href","#");
        $( option ).val("excAddress");
        $( option ).text("Host");
        $( select ).append(option);
        
        option     = document.createElement( 'option' );
        $(option).val("excRange");
        $(option).text("Range");
        $( select ).append(option);
        
        option     = document.createElement( 'option' );
        $(option).val("excNetwork");
        $(option).text("Subnet");
        $( select ).append(option);
        
        $( cell ).append( aDel     );
        $( cell ).append( tsa.NBSP );
        $( cell ).append( select   );
        $( row  ).append(  cell    );
        
        
        /** @type {Element} */ var input  = null;
        if(exclusion.type===0){
          $(select).find('[value="excAddress"]').attr('selected', 'selected');
          ipAddress = exclusion.ipAddress;
          excPid    = exclusion.pid;
          excGPid   = "guid["+(this.noOfExclusionRows+1)+"][ip]";
          this.fpExclusion.putHiddenInput(excGPid,exclusion.pid);
          
          this.buildExclusionHost(cell2,this.noOfExclusionRows,this.fpExclusion,exclusion,null);
          
        } else if(exclusion.type===1){
          $(select).find('[value="excRange"]').attr('selected', 'selected');
          ipAddress = exclusion.start;
          excPid    = exclusion.pid;
          excGPid   = "guid["+(this.noOfExclusionRows+1)+"][range]";
          this.fpExclusion.putHiddenInput(excGPid,exclusion.pid);
          this.buildExclusionRange(cell2,this.noOfExclusionRows,this.fpExclusion,exclusion,null);
          
        } else if(exclusion.type===2){
          $(select).find('[value="excNetwork"]').attr('selected', 'selected');
          ipAddress = exclusion.ipAddress;
          excPid    = exclusion.pid;
          excGPid   = "guid["+(this.noOfExclusionRows+1)+"][subnet]";
          this.fpExclusion.putHiddenInput(excGPid,exclusion.pid);
          this.buildExclusionSubnet(cell2,this.noOfExclusionRows,this.fpExclusion,exclusion, null);
          
        }
        // Create the change action for select
        $(select).on("change", { noOfExclusionRows : this.noOfExclusionRows , exclusionForm : this.fpExclusion} , function(e){
          $(cell2).empty();
          /** @type {Element} */ var input  = null;
          if (this.value === 'excAddress'){
            e.data.exclusionForm.putHiddenInput(excGPid,null);
            excGPid   = "guid["+(e.data.noOfExclusionRows+1)+"][ip]";
            e.data.exclusionForm.putHiddenInput(excGPid,excPid);
            tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionHost(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,ipAddress);
            
          } else if (this.value === 'excRange'){
            e.data.exclusionForm.putHiddenInput(excGPid,null);
            excGPid   = "guid["+(e.data.noOfExclusionRows+1)+"][range]";
            e.data.exclusionForm.putHiddenInput(excGPid,excPid);
            tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionRange(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,ipAddress);
            
          } else if(this.value === 'excNetwork'){
            e.data.exclusionForm.putHiddenInput(excGPid,null);
            excGPid   = "guid["+(e.data.noOfExclusionRows+1)+"][subnet]";
            e.data.exclusionForm.putHiddenInput(excGPid,excPid);
            tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionSubnet(cell2,e.data.noOfExclusionRows,e.data.exclusionForm,null,ipAddress);
          }
        });
        $(cell).append(aDel);
        $(cell).append(tsa.NBSP);
        $(cell).append(select);
        $( row ).append( cell );
        $( row ).append( cell2 );
        this.fpExclusion.layoutTableBody.appendChild( row );
        this.noOfExclusionRows+=1;
    },this));
    };
    //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype.createExclusionElementsRows_original = function(exclusions , pid, sepid){
      $.each( exclusions , $.proxy(function( idx, exclusion ){
        $(this.fpExclusion.layoutTable).css('width', '');
        /** @type {Element} */ var row    = document.createElement( 'tr' );
        /** @type {Element} */ var cell   = document.createElement( 'td' );
        /** @type {Element} */ var cell2  = document.createElement( 'td' );
        /** @type {Element} */ var select = document.createElement( 'select' );
        /** @type {Element} */ var aDel   = $("<a class ='deleteRow' href='#' style='text-decoration: none' ></a>").get(0);
        /** @type {Element} */ var option = document.createElement( 'option' );
        $( option ).attr('selected', 'selected');
        $( aDel ).on("click" , function(){
          if( confirm(tsa.i18n.labelDeleteConfirmation+" '"+ exclusion.name+" '") ){
            $(this).closest('tr').remove();
            /** @type {tsa.cmd.Command} */ var actionCmd = new Command( { action : "scopes/ajaxDelete/" + exclusion.pid
                                                                         , clearSystemMessages : false });
            actionCmd.execute();
            $( actionCmd ).on( 'executionSucceeded', function( event )
                 {
                  (new ShowCommand( {
                     action : 'scopes/edit'
                     , clearSystemMessages : false
                     , parameters : { pid : pid
                                    , sepid : sepid}} 
                    )).execute(); 
                  } );
          }
        });
        /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete' , caption :  'Delete Exclusion'} );
        $(aDel).append(deleteIcon);
        $( select ).attr("href","#");
        $(cell).append(aDel);
        $(cell).append(tsa.NBSP);
        $(cell).append(select);
        
        /** @type {Element} */ var input  = null;
        if(exclusion.type===0){
          $( option ).val("excAddress");
          $( option ).text("Host");
          $( select ).append(option);
          input = $("<input type='text' size='15' id='excAddr' placeholder='" + tsa.i18n.labelDiscoScopesExclusionIpAdress + "' data-index='' class='ibm-text ipscope' >").get(0);
          this.fpExclusion.putHiddenInput("guid["+(this.noOfExclusionRows+1)+"][ip]",exclusion.pid);
          $( input ).attr( 'value', exclusion.ipAddress);
          $( input ).attr( 'name', 'excAddr['+this.noOfExclusionRows+'][ip]' );
          $(cell2).append(input);
          
          input = $("<input type='hidden' name='dummy'>").get(0);
          $(cell2).append(input);
          
          input = $("<input  size='30' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' data-index='' class='ibm-text' type='text' >").get(0);
          $( input ).attr( 'name', 'excAddrDesc['+this.noOfExclusionRows+'][ip]' );
          $( input ).attr( 'value', exclusion.name );
          $(cell2).append(input);
        } else if(exclusion.type===1){
          this.fpExclusion.putHiddenInput("guid["+(this.noOfExclusionRows+1)+"][range]",exclusion.pid);
          option     = document.createElement( 'option' );
          $(option).val("excRange");
          $(option).text("Range");
          $( select ).append(option);
          input = $("<input size='15'  placeholder='" + tsa.i18n.labelDiscoScopesExclusionStarIp + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text' >").get(0);
          $( input ).attr( 'name', 'excRstart['+this.noOfExclusionRows+'][range]' );
          $( input ).attr( 'value', exclusion.start );
          $( cell2 ).append(input);
          
          input   = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionEndIp + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text' >").get(0);
          $( input ).attr( 'name', 'excRend['+this.noOfExclusionRows+'][range]' );
          $( input ).attr( 'value', exclusion.end );
          $(cell2).append(input);
          
          input   = $("<input size='25' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' class='ibm-text' type='text' >  ").get(0);
          $( input ).attr( 'name', 'excRdesc['+this.noOfExclusionRows+'][range]' );
          $( input ).attr( 'value', exclusion.name );
          $( cell2 ).append(input);
          
        } else if(exclusion.type===2){
          this.fpExclusion.putHiddenInput("guid["+(this.noOfExclusionRows+1)+"][subnet]",exclusion.pid);
          option = document.createElement( 'option' );
          $(option).val("excNetwork");
          $(option).text("Subnet");
          $( select ).append(option);
          input = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionSubnet + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text' >").get(0);
          $( input ).attr( 'name', 'excNaddr['+this.noOfExclusionRows+'][subnet]' );
          $( input ).attr( 'value', exclusion.ipAddress );
          $( cell2 ).append(input);
          
          input   = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionSubnetMask + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text' >").get(0);
          $( input ).attr( 'name', 'excNmask['+this.noOfExclusionRows+'][subnet]' );
          $( input ).attr( 'value', exclusion.subnetMask );
          $( cell2 ).append(input);
          
          input   = $("<input size='25' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' class='ibm-text' type='text' > ").get(0);
          $( input ).attr( 'name', 'excDesc['+this.noOfExclusionRows+'][subnet]' );
          $( input ).attr( 'value', exclusion.name );
          $( cell2 ).append(input);
        }
        $( row ).append( cell );
        $( row ).append( cell2 );
        this.fpExclusion.layoutTableBody.appendChild( row );
        this.noOfExclusionRows+=1;
    },this));
    };
    //--------------------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesHandler.prototype._edit = function(){
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelDiscoScopesHeading
                                                                                                 , helpDoc       : 'sshelp_discovery_scope_new_edit_panel.html'
                                                                                                 , showBackToTop : true } );
      /** @type {tsa.widgets.FormPanel}   */     var fpDiscoveryOptions      = new FormPanel( { title  : i18n.labelDiscoScopesSelectHeading
                                                                                              , action : 'scopes/ajaxEditScopeElement' } );
      /** @type {tsa.widgets.FormPanel}   */     var fpAdresses              = new FormPanel( { title  : i18n.labelDiscoScopesAddressHeading
                                                                                                , description: i18n.labelDiscoScopesAddressDesc } );
      /** @type {tsa.cmd.Command} */             var getScopeElementCmd       = new Command({  clearSystemMessages : false });
      /** @type {string}                  */     var pid                      = this.getParameter('pid');
      /** @type {string}                  */     var sepid                    = this.getParameter('sepid');
      /** @type {element}        */              var buttonException         = null;
      /** @type {element}        */              var buttonSeparator         = null;
      this.fpExclusion                                                       = new FormPanel( { title  : i18n.labelDiscoScopeExclusionHeading
                                                                                                  , description: i18n.labelDiscoScopeExclusionDesc } );
      /** @type {String}        */              var ipAdress               = null;
      /** @type {function()}        */
      
      var changeHandler = function(e){
        tsa.clearSysMessages();
        // Remove validations
        $(fpAdresses.layoutTableBody).empty();
        fpAdresses.removeValidation("aaddr");
        fpAdresses.removeValidation("rstart");
        fpAdresses.removeValidation("rend");
        fpAdresses.removeValidation("rdesc");
        fpAdresses.removeValidation("naddr");
        fpAdresses.removeValidation("nmask");
        fpAdresses.removeValidation("ndesc");
        
        if (this.value === 'address'){
          $(buttonException).hide();
          $(buttonSeparator).hide();
          /** @type {element} */ var ipAdressElement = $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
          $( ipAdressElement ).attr("value", ipAdress);
          fpAdresses.setTitle(i18n.labelDiscoScopesAddressHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesAddressDesc);
          fpAdresses.appendFieldRow( {  label    : i18n.labelDiscoScopesAddressAddress
                                      , required : true
                                      , element  :  ipAdressElement
                                      , hint     : i18n.labelSpecifyIpAddressFieldDescription } );
          e.data.fpExclusion.hide();
        } else if (this.value === 'range'){
          $(buttonException).show();
          $(buttonSeparator).show();
          fpAdresses.setTitle(i18n.labelDiscoScopesRangeHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesRangeDesc);
          /** @type {element} */ var ipAdressElement = $("<input size='50' maxlength='250' name='rstart' id='rstart' value='' aria-required='true' class='ibm-text' aria-describedby='descrstart' type='text'>").get(0) ;
          $( ipAdressElement ).attr("value", ipAdress);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeStart
                                                  , required : true
                                                  , element  : ipAdressElement
                                                  , hint     : i18n.labelDiscoScopesRangeStartDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeEnd
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='rend' id='rend' value='' aria-required='true' class='ibm-text' aria-describedby='descrend' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesRangeEndDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  : $("<input size='50' maxlength='250' name='rdesc' id='rdesc' value='' class='ibm-text' aria-describedby='descrdesc' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
          e.data.fpExclusion.show();
          
        } else if (this.value === 'network'){
          $(buttonException).show();
          $(buttonSeparator).show();
          fpAdresses.setTitle(i18n.labelDiscoScopesNetworkHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesNetworkDesc);
          /** @type {element} */ var ipAdressElement = $("<input size='50' maxlength='250' name='naddr' id='naddr' value='' aria-required='true' class='ibm-text' aria-describedby='descnaddr' type='text'>").get(0)
          $( ipAdressElement ).attr("value", ipAdress);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkAddress
                                                  , required : true
                                                  , element  : ipAdressElement 
                                                  , hint     : i18n.labelDiscoScopesNetworkAddressDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkMask
                                                  , required : true
                                                  , element  : $("<input size='50' maxlength='250' name='nmask' id='nmask' value='' aria-required='true' class='ibm-text' aria-describedby='descnmask' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesNetworkMaskDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  : $("<input size='50' maxlength='250' name='ndesc' id='ndesc' value='' class='ibm-text' aria-describedby='descndesc' type='text'>").get(0) 
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
          e.data.fpExclusion.show();
        }
      };
      //Do not show exclusion panel by default
      this.fpExclusion.hide();
      fpDiscoveryOptions.putHiddenInput("scopeset",pid);
      fpDiscoveryOptions.putHiddenInput("sepid",sepid);
      fpDiscoveryOptions.linkTo(fpAdresses);
      fpAdresses.linkTo(this.fpExclusion);
      
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {element}                   */   var selectIP        = $("<select name='option' id='option' size='1' ></select>").get(0);
      /** @type {element}                   */   var option          = document.createElement( 'option' );
      $( option ).attr('value', 'address');
      $( option ).text(i18n.labelDiscoScopesSelectAddress);
      $( selectIP ).append(option);
      
      option   = document.createElement('option');
      $( option ).attr('value', 'range');
      $( option ).text(i18n.labelDiscoScopesSelectRange);
      $( selectIP ).append(option);
      
      option   = document.createElement('option');
      $( option ).attr('value', 'network');
      $( option ).text(i18n.labelDiscoScopesSelectNetwork);
      
      $(selectIP).append(option);
      
      fpDiscoveryOptions.setDescription(i18n.labelDiscoScopesSelectDescription);
      fpDiscoveryOptions.appendFieldRow( { label    : i18n.labelDiscoScopesComboBoxLabel
                                  , required : true
                                  , element  : selectIP 
                                  } );
      $( selectIP ).on('change'   ,{ fpExclusion : this.fpExclusion }, changeHandler );
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : $.proxy(function( event )
              {
              fpDiscoveryOptions.putHiddenInput("noOfRows",this.noOfExclusionRows);
              fpDiscoveryOptions.submit();
              $( fpDiscoveryOptions ).on( 'executionSucceeded', function( event )
              {
                (new ShowCommand( {
                  action : 'scopesets/show'
                    , clearSystemMessages : false 
                    , parameters : { pid    : sepid
                      } 
                })).execute();
              } );
              // Show errors
              $( fpDiscoveryOptions ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        fpDiscoveryOptions.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                      delete event.response.payload.errors;
                    }
                  } );
              
              
              },this)
            }
          )
        );
      getScopeElementCmd.setAction("scopes/getScopeElement/" + pid);
      getScopeElementCmd.execute();
      $(getScopeElementCmd).on("executionSucceeded", $.proxy(function(event){
        /** @type {Object} */ var scopeElement  = event.response.payload.scopeElement;
        if (scopeElement.type === 0){
          $( selectIP ).val("address");
          ipAdress = scopeElement.ipAddress;
          /** @type {element}*/ var inputAddress  = $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text' >").get(0);
          $( inputAddress ).attr("value", scopeElement.ipAddress);
          fpAdresses.setTitle(i18n.labelDiscoScopesAddressHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesAddressDesc);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesAddressAddress
                                                  , required : true
                                                  , element  :  inputAddress
                                                  , hint     : i18n.labelSpecifyIpAddressFieldDescription } );
          $( buttonException ).hide();
          $( buttonSeparator ).hide();
          this.fpExclusion.hide();
        } else if (scopeElement.type === 1){
          $( selectIP ).val("range");
          ipAdress = scopeElement.start;
          $( buttonException ).show();
          $( buttonSeparator ).show();
          /** @type {element}*/ var inputRstart =  $("<input size='50' maxlength='250' name='rstart' id='rstart' value='' aria-required='true' class='ibm-text' aria-describedby='descrstart' type='text' >").get(0);
          $(inputRstart).attr("value", scopeElement.start);
          /** @type {element}*/ var inputRend   = $("<input size='50' maxlength='250' name='rend' id='rend' value='' aria-required='true' class='ibm-text' aria-describedby='descrend' type='text' >").get(0);
          $(inputRend).attr("value", scopeElement.end);
          /** @type {element}*/ var inputRdesc  = $("<input size='50' maxlength='250' name='rdesc' id='rdesc' value='' class='ibm-text' aria-describedby='descrdesc' type='text' >").get(0);
          $(inputRdesc).attr("value", scopeElement.name);
          fpAdresses.setTitle(i18n.labelDiscoScopesRangeHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesRangeDesc);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeStart
                                                  , required : true
                                                  , element  : inputRstart
                                                  , hint     : i18n.labelDiscoScopesRangeStartDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesRangeEnd
                                                  , required : true
                                                  , element  :  inputRend
                                                  , hint     : i18n.labelDiscoScopesRangeEndDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  :  inputRdesc
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
            if( !TV.typeIsUndefined( typeof scopeElement.exclusions ) ){
              this.createExclusionElementsRows(scopeElement.exclusions,pid,sepid);  
            }
          this.fpExclusion.show();
        } else if (scopeElement.type === 2){
          $( selectIP).val("network");
          ipAdress = scopeElement.ipAddress;
          $( buttonException ).show();
          $( buttonSeparator ).show();
          fpAdresses.setTitle(i18n.labelDiscoScopesNetworkHeading);
          fpAdresses.setDescription(i18n.labelDiscoScopesNetworkDesc);
          /** @type {element}*/ var inputNAddr = $("<input size='50' maxlength='250' name='naddr' id='naddr' value='' aria-required='true' class='ibm-text' aria-describedby='descnaddr' type='text' >").get(0);
          $( inputNAddr ).attr("value",scopeElement.ipAddress);
          /** @type {element}*/ var inputNMask = $("<input size='50' maxlength='250' name='nmask' id='nmask' value='' aria-required='true' class='ibm-text' aria-describedby='descnmask' type='text'>").get(0)
          $( inputNMask ).attr("value",scopeElement.subnetMask);
          /** @type {element}*/ var inputNDesc = $("<input size='50' maxlength='250' name='ndesc' id='ndesc' value='' class='ibm-text' aria-describedby='descndesc' type='text' >").get(0);
          $( inputNDesc ).attr("value",scopeElement.name);
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkAddress
                                                  , required : true
                                                  , element  :  inputNAddr
                                                  , hint     : i18n.labelDiscoScopesNetworkAddressDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesNetworkMask
                                                  , required : true
                                                  , element  :  inputNMask
                                                  , hint     : i18n.labelDiscoScopesNetworkMaskDesc } );
          fpAdresses.appendFieldRow( { label    : i18n.labelDiscoScopesDescription
                                                  , element  :  inputNDesc
                                                  , hint     : i18n.labelDiscoScopesDescriptionDesc } );
          if ( !TV.typeIsUndefined( typeof scopeElement.exclusions ) ) {
            this.createExclusionElementsRows(scopeElement.exclusions, pid,sepid);  
          }
          this.fpExclusion.show();
        }
      },this));
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
      buttonException = ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : true
            , label   : i18n.buttonAddExclusion
            , onclick : $.proxy(function( event )
              {
                $.proxy(this.addExclusionRow(),this);
              },this)
            }
          )
      buttonSeparator=ButtonFactory.buildButtonSeparator ;
      $(buttonException).hide();
      $(buttonSeparator).hide();
      $( buttonContainer ).append( buttonException );
      $( buttonContainer ).append(buttonSeparator)
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
                (new ShowCommand( {
                  action : 'scopesets/index'
                    , clearSystemMessages : false} 
                 )).execute();
              }
            }
          )
        );
      $(fpDiscoveryOptions.form).attr("aria-label", i18n.labelDiscoScopesSelectHeading);
      $(fpAdresses.form).attr("aria-label", i18n.labelDiscoScopesAddressHeading);
      $(this.fpExclusion.form).attr("aria-label", i18n.labelDiscoScopeExclusionHeading);
      contentArea.append( fpDiscoveryOptions.getDomNode() );
      contentArea.append( fpAdresses.getDomNode() );
      contentArea.append( this.fpExclusion.getDomNode() );
      contentArea.append( buttonContainer );
      //special string to go back -2 in the history
      history.pushState("backAction", null, null);
      this.show( contentArea.getDomNode() );
    };
    tsa.cmd.handlers.ScopesHandler.prototype.ipRule = function( value ){
      /** @type {object} */  var regEx     = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
      /** @type {string} */  var matchedIp = "";
      /** @type {boolean} */ var valid     = true;
      if( value != "" ) {
        matchedIp = value.match(regEx);
        if(matchedIp != null) {
          /** @type {Array[string]} */ var split = matchedIp[0].split('.');
          for (/** @type {integer} */ var i = split.length - 1; i >= 0; i--) {
            /** @type {string} */ var s = split[i];
            if(s.length == 0 || isNaN(s) || s < 0 || s >255)
               valid = false;
         };
        
        } else {
          valid = false;
       }
      }
      return ( valid ? null : "Invalid IP" );
    };
    tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionHost = function( cell, noOfRows, exclusionForm , exclusion, defaultIP){
      
      exclusionForm.addValidationRule('excAddr['+noOfRows+'][ip]', tsa.cmd.handlers.ScopesHandler.prototype.ipRule);
      /** @type {Element} */ var input = $("<input type='text' size='15' id='excAddr' placeholder='" + tsa.i18n.labelDiscoScopesExclusionIpAdress + "' data-index='' class='ibm-text ipscope'>").get(0);
      if( exclusion !=null){
        $( input ).attr( 'value', exclusion.ipAddress);
      }else if(defaultIP!=null){
        $( input ).attr( 'value', defaultIP);
      }
      $( input ).attr( 'name', 'excAddr['+noOfRows+'][ip]' );
      $(cell).append(input);
      input   = $("<input type='hidden' name='dummy'>").get(0);
      $(cell).append(input);
      input   = $("<input  size='30' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' data-index='' class='ibm-text' type='text'>").get(0);
      $( input ).attr('name', 'excAddrDesc[' + noOfRows + '][ip]');
      if(exclusion!=null) { 
        $( input ).attr( 'value', exclusion.name );
        }
      $(cell).append(input);
    };
    tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionRange = function( cell, noOfRows, exclusionForm , exclusion, defaultIP){
      
      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopesHandler.prototype.ipRule;
      
      exclusionForm.addValidationRule('excRstart['+noOfRows+'][range]', ipRule);
      
      input = $("<input size='15'  placeholder='" + tsa.i18n.labelDiscoScopesExclusionStarIp + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.start );
      }else if (defaultIP!=null){
        $( input ).attr( 'value', defaultIP );
      }
      $( input ).attr('name', 'excRstart['+noOfRows+'][range]');
      
      $(cell).append(input);
      
      input   = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionEndIp + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      
      $( input ).attr( 'name', 'excRend['+noOfRows+'][range]' );
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.end );
      }
      exclusionForm.addValidationRule('excRend['+noOfRows+'][range]', ipRule);
      $(cell).append(input);
      
      input   = $("<input size='25' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' class='ibm-text' type='text'>  ").get(0);
      $( input ).attr( 'name', 'excRdesc['+noOfRows+'][range]' );
      if( exclusion !=null ){
        $( input ).attr( 'value', exclusion.name );
      }
      $(cell).append(input);
    };
    tsa.cmd.handlers.ScopesHandler.prototype.buildExclusionSubnet = function(  cell, noOfRows, exclusionForm, exclusion, defaultIP){
      /** type{function}*/
      var ipRule = tsa.cmd.handlers.ScopesHandler.prototype.ipRule;
      input = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionSubnet + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      $( input ).attr( 'name', 'excNaddr['+noOfRows+'][subnet]' );
      
      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.ipAddress );
      }else if(defaultIP!=null){
        $( input ).attr( 'value', defaultIP );
      }
      exclusionForm.addValidationRule('excNaddr['+noOfRows+'][subnet]', ipRule);
      $(cell).append(input);
     
      input   = $("<input size='15' placeholder='" + tsa.i18n.labelDiscoScopesExclusionSubnetMask + "' aria-required='true' data-index='' class='ibm-text  ipscope' type='text'>").get(0);
      $( input ).attr( 'name', 'excNmask['+noOfRows+'][subnet]' );
      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.subnetMask );
      }
      exclusionForm.addValidationRule('excNmask['+noOfRows+'][subnet]', ipRule);
      $(cell).append(input);
      
      input   = $("<input size='25' placeholder='" + tsa.i18n.labelDiscoScopesExclusionDescription + "' class='ibm-text' type='text'> ").get(0);
      $( input ).attr( 'name', 'excDesc['+noOfRows+'][subnet]' );
      if(exclusion!=null){
        $( input ).attr( 'value', exclusion.name );
        }
      $(cell).append(input);
    };
    return tsa.cmd.handlers.ScopesHandler;
  }
  
  
);