/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/IconFactory'

  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , TablePanel
          , ButtonFactory
          , FormPanel
          , ShowCommand
          , Command
          , IconFactory)
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopesetsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.ScopesetsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopesetsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopesetsHandler.prototype ).constructor = tsa.cmd.handlers.ScopesetsHandler;

    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ScopesetsHandler.prototype._index = function()
    {
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelDiscoScopesHeading
                                                                                , helpDoc       : 'sshelp_discovery_scope_sets_index_panel.html'
                                                                                , showBackToTop : true } );
      /** @type {tsa.cmd.Command} */             var getScopesetsCmd         = new Command({ action:'scopesets/getScopeSets'
                                                                                              ,clearSystemMessages:false });
      /** @type {tsa.widgets.TablePanel} */      var tablePanel              = new TablePanel( {  title        : i18n.labelDiscoScopesHeading
                                                                                                  , emptyMsg : i18n.labelDiscoScopesNoSets
                                                                                                  , columns      : [ { id : 'name'   , label : i18n.labelDiscoScopesName , width : '80%' , sortable : false}
                                                                                                                   , { id : 'action' , label : i18n.labelActionsColumn   , width : '20%' , sortable : false } ] } );
      /** @type {tsa.widgets.FormPanel}     */   var formPanelNTP               = new FormPanel( { title  : i18n.labelClockNtpHeading} );
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer            = ButtonFactory.buildButtonContainer();
      contentArea.appendMessage( { text : i18n.labelDiscoScopesCaption } );

      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonDiscoScopesAddSet
            , onclick : function( event )
              {
              (new ShowCommand( {
                   action : 'scopesets/new'
                 , clearSystemMessages : false } )
               ).execute();
              }
            }
          )
        );

      $(getScopesetsCmd).on('executionSucceeded', function(event){
        /** @type {number} */ var scopesets = event.response.payload.scopeSets;
        /** @type {Array.<(string|Element)>} */ var result          = null;

        if( !TV.typeIsUndefined( typeof scopesets ) )
        {
          $.each( scopesets, function( icaIdx, set ){
              /** @type {Element} */ var aName     = document.createElement("a");
              var clickDisplayHandler = function ( event ) {
                (new ShowCommand( {
                    action : event.data.URL
                  , clearSystemMessages : false
                  , parameters          : { pid : event.data.pid } } )
                 ).execute();
              }
              /**@type function() */
              var clickTriggerHandler = function ( event ) {
                tsa.clearSysMessages();
                var proceed = true;
                if(!TV.typeIsUndefined( typeof event.data.confirm )){
                  proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?")
                }
                if(proceed){
                  var actionCmd = new Command( { action : event.data.URL
                                                      , clearSystemMessages : false  } );
                  actionCmd.execute();
                  $( actionCmd ).on( 'executionSucceeded', function( event )
                       {
                        (new ShowCommand( {
                           action : 'scopesets/index'
                             , clearSystemMessages : false}
                          )).execute();
                        } );
                }
              };
              $( aName ).attr( 'href', '#' );
              $( aName ).on( 'click', { pid : set.pid , URL : 'scopesets/show', pid : set.pid } , clickDisplayHandler);

              /** @type {Element} */ var dActions   = document.createElement("div");
              /** @type {Element} */ var editIcon   = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit   + " '" + set.name + "'"} );
              /** @type {Element} */ var runIcon    = IconFactory.buildActionIcon( { type : 'runb'     , caption : i18n.labelDiscoStart   + " '" + set.name + "' "+ i18n.labelDiscoDiscovery } );
              /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + " '" + set.name + "'"} );
              /** @type {Element} */ var statusIcon = null;
              if (set.enabled){
                statusIcon = IconFactory.buildActionIcon( { type : 'active'   , caption :  "'" + set.name + "' " + i18n.labelCollectionEnabled} );
              } else {
                statusIcon = IconFactory.buildActionIcon( { type : 'offline'   , caption : "'" + set.name + "' " + i18n.labelCollectionDisabled} );
              }


              $( editIcon   ).on( 'click', { URL : 'scopesets/edit', pid : set.pid  } , clickDisplayHandler );
              $( statusIcon ).on( 'click', { URL : 'scopesets/ajaxToggle/'   + set.pid, name: set.name   } , clickTriggerHandler );
              $( runIcon    ).on( 'click', { URL : 'scopesets/ajaxDiscover/' + set.pid, name: set.name } , clickTriggerHandler );
              $( deleteIcon ).on( 'click', { URL : 'scopesets/ajaxDelete/'   + set.pid, name: set.name , confirm : true} , clickTriggerHandler );

              $( aName    ).text( set.name );

              /** @type {Element} */ var aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( editIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( deleteIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( statusIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);

              aRef    = document.createElement("a");
              $( aRef ).attr( 'href', '#' );
              $( aRef ).append( runIcon );
              $( dActions ).append( aRef );
              $( dActions ).append(tsa.NBSP);
              result = [ aName , dActions ];

              tablePanel.appendRow( result );
          });
        }
       });
      getScopesetsCmd.execute();
      contentArea.append( tablePanel.getDomNode() );
      contentArea.append(buttonContainer);
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetsHandler.prototype._new = function(){
      tsa.clearSysMessages();
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelScopeSetHeading
                                                                                , helpDoc       : 'sshelp_discovery_scope_set_new_panel.html'
                                                                                , showBackToTop : true } );
      /** @type {tsa.widgets.FormPanel} */       var formPanel               = new FormPanel( { title  : i18n.labelDescribeScopeSetHeading } );
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer         = ButtonFactory.buildButtonContainer();
      formPanel.setDescription( i18n.labelScopeSetEnterDesc);
      formPanel.appendFieldRow( { label    : i18n.labelScopeSetNameField
                                  , required : true
                                  , element  : $( '<input size="50" maxlength="250" name="name" id="name" value="" aria-required="true" class="ibm-text" type="text">' ).get(0)
                                  } );
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonSave
            , onclick : function( event )
              {
              formPanel.setActionAttribute("scopesets/ajaxCreate/");
              formPanel.submit();
              $( formPanel ).on( 'executionSucceeded', function( event )
              {
                (new ShowCommand( {
                  action : 'scopes/new'
                    , clearSystemMessages : false
                    , parameters : { pid : event.response.payload.pid} } )
                ).execute();

              } );
              $( formPanel ).on( 'executionFailed', function( event )
                  {
                    if( TV.typeIsObject( typeof event.response.payload.errors ) )
                    {
                      for( /** @type {string} */ var fieldName in event.response.payload.errors )
                      {
                        formPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                      }
                    }
                  } );
              }
            }
          )
        );

      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'cancel'
            , grayout : true
            , label   : i18n.buttonCancel
            , onclick : function( event )
              {
              (new ShowCommand( {
                action : 'scopesets/index'
                  , clearSystemMessages : false}
               )).execute();
              }
            }
          )
        );
      contentArea.append( formPanel.getDomNode() );
      contentArea.append( buttonContainer );
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetsHandler.prototype._show = function(){
      // Redirect to edit
      this._edit();
    };
    //-------------------------------------------------------------------------
    tsa.cmd.handlers.ScopesetsHandler.prototype._edit = function(){
      /** @type {string}                  */     var pid                     = this.getParameter('pid');
      /** @type {string}                  */     var name                    = null;
      /** @type {Object}                  */     var i18n                    = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */     var contentArea             = new ContentArea ( { title         : i18n.labelScopeSetHeading
                                                                                , helpDoc       : 'sshelp_discovery_scope_set_show_edit_panel.html'
                                                                                , showBackToTop : true } );
      /** @type {tsa.cmd.Command} */             var getScopesetCmd          = new Command({ action:'scopesets/getScopeSet'
                                                                                            ,clearSystemMessages:false
                                                                                            ,  parameters          : { name : pid } });
      /** @type {tsa.cmd.Command} */             var getScopesetsCmd         = new Command({ action:'scopesets/getScopeSets'
                                                                                            ,clearSystemMessages:false });
      /** @type {tsa.widgets.TablePanel} */      var generalTablePanel       = new TablePanel( {  title        : i18n.labelScopeSetGeneralHeading
                                                                                                , columns      : [ { id : 'name'   , width : '80' }
                                                                                                                 , { id : 'value' } ] } );
      /** @type {tsa.widgets.TablePanel} */      var ipCountTablePanel       = new TablePanel( {  title        : i18n.labelScopeSetIpCountHeading
                                                                                                , description  : i18n.labelScopeSetIpCountDecription
                                                                                                , columns      : [ { id : 'name'   , width : '80' }
                                                                                                                 , { id : 'value'                 } ] } );
      /** @type {tsa.widgets.TablePanel} */      var scopesTablePanel        = new TablePanel( {  title       : i18n.SummaryForScopesTable
                                                                                                , emptyMsg     : i18n.labelScopeSetNoElements
                                                                                                , columns      : [  { id : 'type'     , label : i18n.labelScopeSetType  , width : '20%'}
                                                                                                                  , { id : 'value'    , label : i18n.labelScopeSetValue , width : '30%'}
                                                                                                                  , { id : 'desc'     , label : i18n.labelScopeSetDesc  , width : '40%'}
                                                                                                                  , { id : 'actions'  , label : i18n.labelActionsColumn , width : '15%'}
                                                                                                                  ] });

      /** @type {String} */                      var scopeIPCount                = null;
      /** @type {String} */                      var scopesetRecommendedIPCount  = null;
      /** @type {tsa.widgets.ButtonFactory} */   var buttonContainer             = ButtonFactory.buildButtonContainer();
      /** @type {element} */                     var buttonMove                  = ButtonFactory.buildButton({  type    : 'arrow'
                                                                                                          , grayout : true
                                                                                                          , label   : i18n.buttonScopeSetMoveElements
                                                                                                          , onclick : function( event )
                                                                                                            {
                                                                                                              (new ShowCommand( {
                                                                                                                action : 'scopes/move'
                                                                                                              , clearSystemMessages : false
                                                                                                              , parameters :{ pid : pid , name : name }} )
                                                                                                              ).execute();
                                                                                                            }
                                                                                                          });

      $(getScopesetsCmd).on('executionSucceeded', function(event){
        // hide button move if there is only one or less scopeset
        if(!TV.typeIsUndefined( typeof event.response.payload.scopeSets ) && event.response.payload.scopeSets.length < 2){
          $(buttonMove).hide();
        }
      });
      getScopesetCmd.execute();
      $(getScopesetCmd).on('executionSucceeded', function(event){
        //
        // Populate General pane
        //
        getScopesetsCmd.execute();
        generalTablePanel.appendRow( [i18n.labelScopeSetName,event.response.payload.scopeSet.displayName] );
        name = event.response.payload.scopeSet.displayName;

        //
        // Populate IP count pane
        //
        // Appending messages to the content area...
        scopeIPCount = event.response.payload.scopeSet.scopeSetCount;
        scopesetRecommendedIPCount = event.response.payload.scopeSet.scopeSetRecommendedIPCount;
        ipCountTablePanel.appendRow( [i18n.labelScopeSetCount, scopeIPCount] );

        //
        // Populate Scopes pane
        //
        if ( !TV.typeIsUndefined( typeof event.response.payload.scopeSet.scopeElements ) )
        {
          $.each( event.response.payload.scopeSet.scopeElements, function( idx, element ){
            /** @type {string} */ var type = null;
             if(element.type==0){
               type = i18n.labelScopeSetAddress;
             } else if(element.type==1) {
               type = i18n.labelScopeSetRange;
             } else if(element.type==2) {
               type = i18n.labelScopeSetNetwork;
             } else{
               type = "unkown";
             }
             /**@type function() */
             var clickTriggerHandler = function ( event ) {
               tsa.clearSysMessages();
               var proceed = true;
               if(!TV.typeIsUndefined( typeof event.data.confirm )){
                 proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?")
               }
               if(proceed){
                 var actionCmd = new Command( { action : event.data.URL
                                                     , clearSystemMessages : false });
                 actionCmd.execute();
                 $( actionCmd ).on( 'executionSucceeded', function( event )
                      {
                       (new ShowCommand( {
                          action : 'scopesets/edit'
                          , clearSystemMessages : false
                          , parameters : { pid : pid }}
                         )).execute();
                       } );
               }
             };
             /** @type {Element} */ var dActions   = document.createElement("div");
             /** @type {Element} */ var editIcon   = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit   + " '" + element.externalForm + "'"} );
             /** @type {Element} */ var runIcon    = IconFactory.buildActionIcon( { type : 'runb'     , caption : i18n.labelDiscoStart   + " '" + element.externalForm + "' " + i18n.labelDiscoDiscovery } );
             /** @type {Element} */ var deleteIcon = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete + " '" + element.externalForm + "'"} );
             $( editIcon   ).on( 'click',  function(){
               (new ShowCommand( {
                 action : 'scopes/edit'
                   , clearSystemMessages : false
                   , parameters : {  pid    : element.pid
                                   ,  sepid : pid }
                                 })).execute();
             } );
             $( runIcon    ).on( 'click', { URL : "scopes/ajaxDiscover/" + element.pid } , clickTriggerHandler);
             $( deleteIcon ).on( 'click', { URL : "scopes/ajaxDelete/" + element.pid , name : element.externalForm  , confirm : true } , clickTriggerHandler);
             /** @type {Element} */ var aRef    = document.createElement("a");
             $( aRef ).attr( 'href', '#' );
             $( aRef ).append( editIcon );
             $( dActions ).append( aRef );
             $( dActions ).append(tsa.NBSP);

             aRef    = document.createElement("a");
             $( aRef ).attr( 'href', '#' );
             $( aRef ).append( deleteIcon );
             $( dActions ).append( aRef );
             $( dActions ).append(tsa.NBSP);

             aRef    = document.createElement("a");
             $( aRef ).attr( 'href', '#' );
             $( aRef ).append( runIcon );
             $( dActions ).append( aRef );
             $( dActions ).append(tsa.NBSP);

             scopesTablePanel.appendRow( [type, element.externalForm, element.displayName, dActions ] );
            });


          }else{
            //hide if there is no scopes to move
            $(buttonMove).hide();
          }
          contentArea.append(generalTablePanel.getDomNode());
          if (scopeIPCount > 0) {
            contentArea.appendMessage( { level : 'INFO'  , border : false , text : tsa.bind(i18n.labelScopeSetIPCountInfo, [scopesetRecommendedIPCount]) } );
          }
          contentArea.append(ipCountTablePanel.getDomNode());
          contentArea.append(scopesTablePanel.getDomNode());
          contentArea.append(buttonContainer);
      });
      $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonScopeSetAddElement
            , onclick : function( event )
              {
              (new ShowCommand( {
                action : 'scopes/new'
              , clearSystemMessages : false
              , parameters :{ pid : pid }} )
              ).execute();
              }
            }
          )
        );

      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

      $( buttonContainer ).append(buttonMove);

      contentArea.append(generalTablePanel.getDomNode());
      contentArea.append(ipCountTablePanel.getDomNode());
      contentArea.append(scopesTablePanel.getDomNode());
      contentArea.append(buttonContainer);
      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.ScopesetsHandler;
  }

);