/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 1
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery',
    'tsa/TypeVerificator',
    'tsa/cmd/handlers/AbstractHandler',
    'tsa/widgets/ContentArea',
    'tsa/widgets/TablePanel',
    'tsa/widgets/ButtonFactory',
    'tsa/widgets/FormPanel',
    'tsa/cmd/ShowCommand',
    'tsa/cmd/Command',
    'tsa/widgets/IconFactory'
  ],

  function( $,
            TV,
            AbstractHandler,
            ContentArea,
            TablePanel,
            ButtonFactory,
            FormPanel,
            ShowCommand,
            Command,
            IconFactory)
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopesetsvmwareHandler' );

    //-------------------------------------------------------------------------

    /**
     *  @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters.
     */

    tsa.cmd.handlers.ScopesetsvmwareHandler = function( parameters ) {
        if ( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) {
            parameters = /** @type {?} */ ( {} );
        }

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype = new AbstractHandler( {} );
    ( tsa.cmd.handlers.ScopesetsvmwareHandler.prototype ).constructor = tsa.cmd.handlers.ScopesetsvmwareHandler;

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** MAIN SCREEN - Provide list of dynamic scope sets                       ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype._index = function() {
        var i18n = tsa.i18n;

        var contentArea = new ContentArea ( { title         : i18n.labelDiscoVmwareScopesHeading,
                                              helpDoc       : 'sshelp_discovery_vmware_scope_sets_index_panel.html',
                                              showBackToTop : true } );
        var tablePanel = new TablePanel( { title    : i18n.labelDiscoVmwareScopesHeading,
                                           emptyMsg : i18n.labelDiscoVmwareScopesNoSets,
                                           columns  : [ { id : 'name'   , label : i18n.labelNameColumn    , width : '80%', sortable : false  },
                                                        { id : 'action' , label : i18n.labelActionsColumn , width : '20%', sortable : false } ] } );

        var buttonContainer = ButtonFactory.buildButtonContainer();
        contentArea.appendMessage( { text : i18n.labelDiscoVmwareScopesCaption } );

        /*
         * Build the Add button to add a VMware Dynamic Scope Set.
         */
        var addButton = ButtonFactory.buildButton( { type    : 'add',
                                                     grayout : true,
                                                     label   : i18n.buttonDiscoVmwareScopesAddSet } );

        $(addButton).on( 'click', function( ) {
            (new ShowCommand( { action              : 'scopesetsvmware/new',
                                clearSystemMessages : true } )
            ).execute();
        } );

        $(buttonContainer).append(addButton);

        /*
         * Run command to get all VMware Dynamic Scope Sets
         */
        var getScopesetsCmd = new Command( { action              : 'scopesets/getScopeSets',
                                             parameters          : { 'filter' : 'Vmware' },
                                             clearSystemMessages : false });

        getScopesetsCmd.execute();
        $(getScopesetsCmd).on('executionSucceeded', function(event) {
            var scopesets = event.response.payload.scopeSets;
            var result    = null;

            if (!TV.typeIsUndefined(typeof scopesets)) {
                /*
                 * Find just the VMware scope sets.  Eliminate the hidden Linux and Windows scope sets when creating
                 * the list of VMware Dynamic Scope Sets.
                 */
                var vmwareScopesets = [];
                $.each( scopesets, function( icaIdx, set ) {
                    if (!set.name.endsWith("_LINUX") && !set.name.endsWith("_WINDOWS")) {
                        vmwareScopesets.push(set);
                    }
                } );

                /*
                 * Create a row in the table for each VMware Dynamic Scope Set.
                 */
                $.each( vmwareScopesets, function( icaIdx, set ) {
                    var divName = document.createElement("div");
                    var aName   = document.createElement("a");

                    var clickDisplayHandler = function ( event ) {
                        (new ShowCommand( { action              : event.data.URL,
                                            clearSystemMessages : false,
                                            parameters          : { pid : event.data.pid } } )
                        ).execute();
                    };

                    /**@type function() */
                    var clickTriggerHandler = function(event) {
                        tsa.clearSysMessages();
                        var proceed = true;
                        if (!TV.typeIsUndefined(typeof event.data.confirm)) {
                            proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?");
                        }
                        if (proceed) {
                            var actionCmd = new Command( { action              : event.data.URL,
                                                           clearSystemMessages : false  } );
                            actionCmd.execute();
                            $( actionCmd ).on( 'executionSucceeded', function() {
                                (new ShowCommand( {action : 'scopesetsvmware/index',
                                                   clearSystemMessages : false}
                                )).execute();
                            } );
                        }
                    };

                    $( aName ).attr( 'href', '#' );
                    $( aName ).on( 'click', { pid : set.pid,
                                              URL : 'scopesetsvmware/show',
                                              pid : set.pid } , clickDisplayHandler);

                    /*
                     * Create the action icons and add to row for this dynamic scope set.
                     */
                    var dActions   = document.createElement("div");
                    var editIcon   = IconFactory.buildActionIcon( { type    : 'edit',
                                                                    caption : i18n.labelButtonEdit + " '" + set.name + "'"} );
                    var runHoverText = i18n.labelDiscoVmwareStartDiscovery.replace("{0}", set.name);
                    var runIcon    = IconFactory.buildActionIcon( { type    : 'runb',
                                                                    caption :  runHoverText } );
                    var deleteIcon = IconFactory.buildActionIcon( { type    : 'delete',
                                                                    caption : i18n.labelButtonDelete + " '" + set.name + "'"} );
                    var statusIcon = null;

                    if (set.enabled) {
                        statusIcon = IconFactory.buildActionIcon( { type    : 'active',
                                                                    caption : "'" + set.name + "' " + i18n.labelCollectionEnabled} );
                    } else {
                        statusIcon = IconFactory.buildActionIcon( { type    : 'offline',
                                                                    caption : "'" + set.name + "' " + i18n.labelCollectionDisabled} );
                    }

                    $( editIcon   ).on( 'click', { URL : 'scopesetsvmware/edit',
                                                   pid : set.pid  } , clickDisplayHandler );
                    $( statusIcon ).on( 'click', { URL : 'scopesets/ajaxToggle/'   + set.pid,
                                                   name: set.name   } , clickTriggerHandler );
                    $( runIcon    ).on( 'click', { URL : 'scopesetsvmware/ajaxDiscoverScopeSet/' + set.pid } , clickTriggerHandler );
                    $( deleteIcon ).on( 'click', { URL     : 'scopesetsvmware/ajaxDelete/'   + set.pid,
                                                   name    : set.name,
                                                   confirm : true} , clickTriggerHandler );

                    $( divName ).append( tsa.NBSP);
                    $( divName ).append(aName);
                    $( aName   ).append( set.name);


                    var aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( editIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( deleteIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( statusIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( runIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    result = [ divName , dActions ];

                    tablePanel.appendRow( result );
                } );

            }
        } );

        contentArea.append( tablePanel.getDomNode() );
        contentArea.append(buttonContainer);
        this.show( contentArea.getDomNode() );
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** NEW SCOPE SET SCREEN                                                   ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype._new = function() {
        var i18n = tsa.i18n;

        var contentArea = new ContentArea ( { title         : i18n.labelVmwareScopeSetHeading,
                                              helpDoc       : 'sshelp_discovery_vmware_scope_set_new_panel.html',
                                              showBackToTop : true } );

        // Check with TSA server if internal SSH key has been defined or not.  This impacts what options are available for
        // the Linux credentials "Authentication Type" field.
        //
        // To keep from having the code be indented too much, the bulk of the processing is done in the createNewScreen() function
        // immediately below.
        var getApplianceSshKeyCommand = new Command( { action              : 'credentials/ajaxCheckInternalSshKey',
                                                       clearSystemMessages : false } );

        $( getApplianceSshKeyCommand ).on( 'executionSucceeded', $.proxy( function( ) {
            this.createNewScreen(contentArea, true);
        }, this ) );

        $( getApplianceSshKeyCommand ).on( 'executionFailed', $.proxy( function( ) {
            this.createNewScreen(contentArea, false);
        }, this ) );

        getApplianceSshKeyCommand.execute();

        this.show( contentArea.getDomNode() );
    };


    //
    // Main function the creates the content for the new scope set screen.
    //
    // @param contentArea Main panel area where the new scope set panes are added to.
    // @param isPkiDefined If true, indicates that the TSA appliance SSH key has been configured; otherwise false.
    //                     NOTE: By default, TSA is shipped with no TSA appliance SSH key.  The TSA administrator
    //                           has to create one in order for it to exist.
    //
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.createNewScreen = function(contentArea, isPkiDefined) {
        var i18n = tsa.i18n;

        var buttonContainer         = ButtonFactory.buildButtonContainer();
        var scopeSetPanel           = new FormPanel( { title : i18n.labelDescribeScopeSetHeading } );
        var scopePanel              = new FormPanel( { title : i18n.labelDiscoVmwareScopesAddressHeading } );
        var vmwarePanel             = new FormPanel( { title       : i18n.labelDiscoVmwareCredsEnterAccessHeading,
                                                       description : i18n.labelAccessInformationPaneDescription });
        var virtualMachineTypePanel = new FormPanel( { title       : i18n.labelDiscoVmwareCredsEnterVirtualMachineSectionHeading,
                                                       description : i18n.labelDiscoVmwareCredsEnterVirtualMachineSectionDesc });
        var vmwareLinuxPanel        = new FormPanel( { title       : i18n.labelDiscoVmwareCredsEnterAccessLinuxHeading,
                                                       description : i18n.labelAccessInformationPaneDescription });
        var testFormLinux           = new FormPanel( { title       : i18n.labelDiscoVmwareCredsLinuxTestHeading,
                                                       description : i18n.labelDiscoVmwareCredsTestDesc } );
        var vmwareWindowsPanel      = new FormPanel( { title       : i18n.labelDiscoVmwareCredsEnterAccessWindowsHeading,
                                                       description : i18n.labelAccessInformationPaneDescription });
        var testFormWindows         = new FormPanel( { title       : i18n.labelDiscoVmwareCredsWindowsTestHeading,
                                                       description : i18n.labelDiscoVmwareCredsTestDesc } );

        // Link all FormPanels
        scopeSetPanel.linkTo(scopePanel);
        scopePanel.linkTo(vmwarePanel);
        vmwarePanel.linkTo(virtualMachineTypePanel);
        virtualMachineTypePanel.linkTo(vmwareLinuxPanel);
        vmwareLinuxPanel.linkTo(testFormLinux);
        testFormLinux.linkTo(vmwareWindowsPanel);
        vmwareWindowsPanel.linkTo(testFormWindows);

        // Create Scope Set fields
        scopeSetPanel.setDescription( i18n.labelVmwareScopeSetEnterDesc);
        scopeSetPanel.appendFieldRow( { label    : i18n.labelScopeSetNameField,
                                        required : true,
                                        element  : $( '<input size="50" maxlength="250" name="name" id="name" value="" aria-required="true" class="ibm-text" type="text">' ).get(0)
                                      } );

        scopePanel.appendFieldRow( { label    : i18n.labelIpAddressField,
                                     required : true,
                                     element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0),
                                     hint     : i18n.labelDiscoVmwareScopesAddressAddressDesc } );

        var scopeSetGroup = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);

        // Define check box for Linux virtual machines.
        //
        // Create function that hides or displays the panes associated with Linux virtual machines as the
        // check box is selected / deselected.
        scopesetInput = $("<input name='virtualMachines' class='checkBoxClass' type='checkbox' checked>").get(0);
        $(scopesetInput).on('click', function() {
            console.log(this.value);

            if (this.checked) {
                vmwareLinuxPanel.show();
                testFormLinux   .show();
            } else {
                vmwareLinuxPanel.hide();
                testFormLinux   .hide();
            }
        });
        $(scopesetInput).attr("id", "checkBoxLinux");
        $(scopesetInput).attr("value", "Linux");
        scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
        $(scopeLabel).text("Linux");
        $(scopeLabel).attr("for", "Linux");
        $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));

        // Define check box for Windows virtual machines.
        //
        // Create function that hides or displays the panes associated with Windows virtual machines as the
        // check box is selected / deselected.
        scopesetInput = $("<input name='virtualMachines' class='checkBoxClass' type='checkbox' checked>").get(0);
        $(scopesetInput).on('click',function() {
            console.log(this.value);

            if (this.checked) {
                vmwareWindowsPanel.show();
                testFormWindows   .show();
            } else {
                vmwareWindowsPanel.hide();
                testFormWindows   .hide();
            }
        });
        $(scopesetInput).attr("id", "checkBoxWindows");
        $(scopesetInput).attr("value", "Windows");
        scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
        $(scopeLabel).text("Windows");
        $(scopeLabel).attr("for", "Windows");
        $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));

        virtualMachineTypePanel.appendFieldRow( { label    : i18n.labelDiscoVmwareCredsEnterVirtualMachineSectionTypes,
                                                  required : false,
                                                  aRequire : false,
                                                  element  : scopeSetGroup } );

        // Create VMware Scope Set Credential fields
        var vmwareHostCredName  = $( '<input type="text"     id="HostAuth_name"             name="HostAuth_name"             size="50" maxlength="250" />' ).get(0);
        var vmwareHostUserName  = $( '<input type="text"     id="HostAuth_username"         name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        var vmwareHostPassword1 = $( '<input type="password" id="HostAuth_password"         name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        var vmwareHostPassword2 = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        /*
         * EVENT HANDLER - For Linux credentials only, notified when the "Authentication type" pull-down value is changed.
         */
        var onHostAuthAuthTypeChanged = function( event ) {
            // Elements
            var panel          = event.data.vmwarePanel;
            var uploadKey      = event.data.vmwareHostUploadKey;
            var uploadKeyType  = event.data.vmwareHostUploadKeyType;
            var authType       = event.data.vmwareHostAuthType;
            var hostPassword1  = event.data.vmwareHostPassword1;
            var hostPassword2  = event.data.vmwareHostPassword2;
            var row            = event.data.rowPanel;
            var uploadKeyField = uploadKey ;
            var keyTypeMessage = uploadKeyType;

            // String
            var authSshKey     = event.data.hostAuthSshKey;

            // Boolean conditions
            var pkiGlobal          = ( $( authType ).val() === 'authType_pki'        );
            var pkiScoped          = ( $( authType ).val() === 'authType_pki_scoped' );
            var isPKI              = (pkiGlobal || pkiScoped);
            var hideSSHKeyRow      = !isPKI;
            var hideSSHKeyUpload   = ( hideSSHKeyRow || pkiGlobal );
            var hideKeyTypeMessage = pkiScoped ;

            $( row            )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
            $( uploadKeyField )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
            $( keyTypeMessage )[ hideKeyTypeMessage ? 'hide' : 'show' ]();

            if (hideSSHKeyUpload){
                panel.removeValidation( authSshKey );
            } else {
                panel.addValidationRule( authSshKey, 'required' );
            }

            // remove required field for passphrase
            if (pkiScoped) {
                panel.removeValidation( $(hostPassword1).attr('id') );
                panel.removeValidation( $(hostPassword2).attr('id') );
            } else {
                panel.addValidationRule( $(hostPassword1).attr('id'), 'required' );
                panel.addValidationRule( $(hostPassword2).attr('id'), 'required' );
            }

            // Password or Passphrase? use the right label...
            var labelValue = isPKI ? i18n.labelPassphraseField : i18n.labelPasswordField ;
            var labelNode  = $( hostPassword1 ).closest( 'tr' ).find( 'label' );
            labelNode.empty();

            if (!pkiScoped) {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }
            labelValue = isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField ;
            labelNode = $( hostPassword2 ).closest( 'tr' ).find( 'label' );
            labelNode.empty();
            if (!pkiScoped) {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            } else {
                labelNode.append(labelValue);
            }

            if (isPKI) {
                $( keyTypeMessage ).text( pkiGlobal ? i18n.labelDiscoVmwareCredsInternalSSHKey : i18n.labelDiscoVmwareCredsExternalSSHKey );
            }
        };

        vmwarePanel.appendFieldRow( { label : i18n.labelCredentialName       , required : true , element : vmwareHostCredName  } );
        vmwarePanel.appendFieldRow( { label : i18n.labelUserNameField        , required : true , element : vmwareHostUserName  } );
        vmwarePanel.appendFieldRow( { label : i18n.labelPasswordField        , required : true , element : vmwareHostPassword1 } );
        vmwarePanel.appendFieldRow( { label : i18n.labelConfirmPasswordField , required : true , element : vmwareHostPassword2 } );

        // Default values to trigger change handler
        var VmwareAuthValues  = new Object();
        VmwareAuthValues.data = new Object();
        VmwareAuthValues.data.vmwarePanel         = vmwarePanel;
        VmwareAuthValues.data.vmwareHostPassword1 = vmwareHostPassword1;
        VmwareAuthValues.data.vmwareHostPassword2 = vmwareHostPassword2;

        // Create test credential button for VMware and define the action taken when the button is clicked.
        var testVmwareButton = this.createTestButton(scopeSetPanel, 'scopesetsvmware/ajaxTestVmwareCredential');

        var row = vmwarePanel.layoutTableBody.insertRow(-1);
        var td = document.createElement("td");
        var pButton = document.createElement("p");
        $(pButton).append(testVmwareButton);
        $(td).attr("colspan", 2);
        $(td).append(pButton);
        $(row).append(td);

        /*
         * Create VMware Linux Scope Set Credential fields
         */

        // Determine if TSA SSH Server Key has been defined.  This affects which options are available in authentication type pull-down widget.
        var linuxVmwareHostHostAuthType = null;
        if (isPkiDefined) {
            linuxVmwareHostHostAuthType = $( '<select name="HostAuthLinux_authtype" id="HostAuthLinux_authtype" >' +
                                                 '<option value="authType_default" selected="selected"' + '>' + this.i18nAuthType( 'authType_default'    ) + '</option>' +
                                                 '<option value="authType_password"'                    + '>' + this.i18nAuthType( 'authType_password'   ) + '</option>' +
                                                 '<option value="authType_pki_scoped"'                  + '>' + this.i18nAuthType( 'authType_pki_scoped' ) + '</option>' +
                                                 '<option value="authType_pki"'                         + '>' + this.i18nAuthType( 'authType_pki'        ) + '</option>' +
                                             '</select>' ).get(0);
        } else {
            linuxVmwareHostHostAuthType = $( '<select name="HostAuthLinux_authtype" id="HostAuthLinux_authtype" >' +
                                                 '<option value="authType_default" selected="selected"' + '>' + this.i18nAuthType( 'authType_default'    ) + '</option>' +
                                                 '<option value="authType_password"'                    + '>' + this.i18nAuthType( 'authType_password'   ) + '</option>' +
                                                 '<option value="authType_pki_scoped"'                  + '>' + this.i18nAuthType( 'authType_pki_scoped' ) + '</option>' +
                                             '</select>' ).get(0);
        }

        // Elements
        var linuxVmwareHostCredName      = $( '<input type="text"     id="HostAuthLinux_name"             name="HostAuthLinux_name"             size="50" maxlength="250" />' ).get(0);
        var linuxVmwareHostUserName      = $( '<input type="text"     id="HostAuthLinux_username"         name="HostAuthLinux_username"         size="50" maxlength="250" />' ).get(0);
        var linuxVmwareHostPassword1     = $( '<input type="password" id="HostAuthLinux_password"         name="HostAuthLinux_password"         size="50" maxlength="250" />' ).get(0);
        var linuxVmwareHostPassword2     = $( '<input type="password" id="HostAuthLinux_password_confirm" name="HostAuthLinux_password_confirm" size="50" maxlength="250" />' ).get(0);
        var linuxVmwareHostUploadKey     = $( '<input type="file" id="HostAuthLinux_sshKey" name="HostAuthLinux_sshKey" />' ).get(0);
        var linuxVmwareHostUploadKeyType = document.createElement( 'p' );

        var linuxVmwareHostUploadKeyDiv  = document.createElement( 'div' );

        $( linuxVmwareHostUploadKeyType ).text( i18n.labelDiscoVmwareCredsInternalSSHKey );

        $( linuxVmwareHostUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( linuxVmwareHostUploadKeyDiv ).append( linuxVmwareHostUploadKeyType );
        $( linuxVmwareHostUploadKeyDiv ).append( linuxVmwareHostUploadKey     );

        vmwareLinuxPanel.appendFieldRow( { label: i18n.labelCredentialName                , required : true  , element : linuxVmwareHostCredName     } );
        vmwareLinuxPanel.appendFieldRow( { label: i18n.labelUserNameField                 , required : true  , element : linuxVmwareHostUserName     } );
        vmwareLinuxPanel.appendFieldRow( { label: i18n.labelPasswordField                 , required : true  , element : linuxVmwareHostPassword1    } );
        vmwareLinuxPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField          , required : true  , element : linuxVmwareHostPassword2    } );
        vmwareLinuxPanel.appendFieldRow( { label: i18n.labelCredentialsAuthenticationType , required : false , element : linuxVmwareHostHostAuthType } );
        var rowLinuxPanel = vmwareLinuxPanel.appendFieldRow( { label: i18n.labelDiscoVmwareCredsSSHKey   , required: true  , element: linuxVmwareHostUploadKeyDiv } );
        vmwareLinuxPanel.removeValidation( "HostAuthLinux_sshKey" );
        $( linuxVmwareHostHostAuthType ).on( 'change',
            { vmwarePanel              : vmwareLinuxPanel,
              vmwareHostUploadKey      : linuxVmwareHostUploadKey,
              vmwareHostUploadKeyType  : linuxVmwareHostUploadKeyType,
              vmwareHostAuthType       : linuxVmwareHostHostAuthType,
              vmwareHostPassword1      : linuxVmwareHostPassword1,
              vmwareHostPassword2      : linuxVmwareHostPassword2,
              vmwareHostUploadKeyDiv   : linuxVmwareHostUploadKeyDiv,
              rowPanel                 : rowLinuxPanel,
              hostAuthSshKey           : "HostAuthLinux_sshKey"
            },
            onHostAuthAuthTypeChanged );

        // Values to enable default configuration
        var linuxAuthValues  = new Object();
        linuxAuthValues.data = new Object();
        linuxAuthValues.data.vmwarePanel             = vmwareLinuxPanel;
        linuxAuthValues.data.vmwareHostUploadKey     = linuxVmwareHostUploadKey;
        linuxAuthValues.data.vmwareHostUploadKeyType = linuxVmwareHostUploadKeyType;
        linuxAuthValues.data.vmwareHostAuthType      = linuxVmwareHostHostAuthType;
        linuxAuthValues.data.vmwareHostPassword1     = linuxVmwareHostPassword1;
        linuxAuthValues.data.vmwareHostPassword2     = linuxVmwareHostPassword2;
        linuxAuthValues.data.vmwareHostUploadKeyDiv  = linuxVmwareHostUploadKeyDiv;
        linuxAuthValues.data.rowPanel                = rowLinuxPanel;

        // Create test credential button for Linux and define the action taken when the button is clicked.
        var testLinuxButton = this.createTestButton(scopeSetPanel, 'scopesetsvmware/ajaxTestLinuxCredential');

        // Test form for Linux
        var testhostFieldLinux = $("<input size='50' maxlength='250' name='aaddrlinux' id='aaddrlinux' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
        testFormLinux.appendFieldRow( { label    : i18n.labelIpAddressField,
                                        required : false,
                                        element  : testhostFieldLinux } );
        testFormLinux.appendHR();
        testFormLinux.appendJointRow( testLinuxButton );


        /*
         * Create Vmware Windows Scope Set Credential fields
         */
        /** @type {Element} */ var windowsVmwareHostName          = $( '<input type="text"     id="HostAuthWindows_name"  name="HostAuthWindows_name"  size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var windowsVmwareHostUserName      = $( '<input type="text"     id="HostAuthWindows_username"  name="HostAuthWindows_username"  size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var windowsVmwareHostPassword1     = $( '<input type="password" id="HostAuthWindows_password" name="HostAuthWindows_password"  size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var windowsVmwareHostPassword2     = $( '<input type="password" id="HostAuthWindows_password_confirm" name="HostAuthWindows_password_confirm" size="50" maxlength="250" />' ).get(0);

        vmwareWindowsPanel.appendFieldRow( { label: i18n.labelCredentialName       , required : true , element : windowsVmwareHostName      } );
        vmwareWindowsPanel.appendFieldRow( { label: i18n.labelUserNameField        , required : true , element : windowsVmwareHostUserName  } );
        vmwareWindowsPanel.appendFieldRow( { label: i18n.labelPasswordField        , required : true , element : windowsVmwareHostPassword1 } );
        vmwareWindowsPanel.appendFieldRow( { label: i18n.labelConfirmPasswordField , required : true , element : windowsVmwareHostPassword2 } );

        // Create test credential button for Windows and define the action taken when the button is clicked.
        var testWindowsButton = this.createTestButton(scopeSetPanel, 'scopesetsvmware/ajaxTestWindowsCredential');

        // Test form for Windows
        var testhostFieldWindows = $("<input size='50' maxlength='250' name='aaddrwindows' id='aaddrwindows' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0);
        testFormWindows.appendFieldRow( { label    : i18n.labelIpAddressField,
                                          required : false,
                                          element  : testhostFieldWindows } );
        testFormWindows.appendHR();
        testFormWindows.appendJointRow( testWindowsButton );

        /*
         * Create save button and define action when button is clicked.
         */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      grayout : false,
                                                      label   : i18n.buttonSave } );
        $( buttonContainer ).append(saveButton);

        $( saveButton ).on( 'click', function( ) {
            scopeSetPanel.setActionAttribute("scopesetsvmware/ajaxCreate/");
            scopeSetPanel.submit();

            $( scopeSetPanel ).on( 'executionSucceeded', function( event ) {
                (new ShowCommand( { action              : 'scopesetsvmware/index',
                                    clearSystemMessages : false,
                                    parameters          : { pid : event.response.payload.pid} } )
                ).execute();
            } );

            $( scopeSetPanel ).on( 'executionFailed', function( event ) {
                if ( TV.typeIsObject( typeof event.response.payload.errors ) ) {
                    for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                        scopeSetPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );


        // call onHostAuthAuthTypeChanged to trigger default values.
        onHostAuthAuthTypeChanged( linuxAuthValues );

        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );

        /*
         * Create cancel button and define action when button is clicked.
         */
        var cancelButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                        grayout : true,
                                                        label   : i18n.buttonCancel } );
        $( buttonContainer ).append(cancelButton);

        $( cancelButton ).on( 'click', function( ) {
            (new ShowCommand( { action              : 'scopesetsvmware/index',
                                clearSystemMessages : false }
            )).execute();
        } );

        /*
         * Add all the panes to the panel.
         */
        contentArea.append( scopeSetPanel          .getDomNode() );
        contentArea.append( scopePanel             .getDomNode() );
        contentArea.append( vmwarePanel            .getDomNode() );
        contentArea.append( virtualMachineTypePanel.getDomNode() );
        contentArea.append( vmwareLinuxPanel       .getDomNode() );
        contentArea.append( vmwareWindowsPanel     .getDomNode() );

        contentArea.append( buttonContainer );
    };


    /*
     * Set-up test credential button based on the type of credential being tested (VMware, Linux, Windows).
     *
     * @param firstForm Form that will contain the test credential button.
     * @param actionAttribute Action taken when test credential button is clicked.
     *
     * @return Test credential button.
     */
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.createTestButton = function(scopeSetPanel, actionAttribute) {
        var i18n = tsa.i18n;
        var testButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonTestCredential } );

        $(testButton).on('click', function( ) {
            tsa.clearSysMessages();

            var action = scopeSetPanel.getActionAttribute();  // Save current value and restore after panel is submitted.

            scopeSetPanel.setActionAttribute(actionAttribute);
            scopeSetPanel.submit();
            scopeSetPanel.setActionAttribute(action);

            $( scopeSetPanel ).on( 'executionFailed', function( event ) {
                if ( TV.typeIsObject( typeof event.response.payload.errors ) )  {
                    for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                        scopeSetPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );

        return testButton;
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** SHOW SCOPE SET SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype._show = function() {
        // Redirect to edit
        this._edit();
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT SCOPE SET SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype._edit = function() {
        var i18n = tsa.i18n;

        var pid            = this.getParameter('pid');
        // "name" is filled in by getScopesetCmd and used later by getCredCommand.  Therefore, edit has to ensure
        // getScopesetCmd completes before getCredCommand is started.
        var name           = null;
        var contentArea    = new ContentArea ( { title   : i18n.labelVmwareScopeSetHeading,
                                                 helpDoc : 'sshelp_discovery_vmware_scope_set_show_edit_panel.html' } );
        var getScopesetCmd = new Command( { action              : 'scopesets/getScopeSet',
                                            clearSystemMessages : false,
                                            parameters          : { name : pid } } );

        /*-----------------*/
        /* GENERAL PANE    */
        /*-----------------*/
        var tablePanelGeneral = new TablePanel( { title   : i18n.labelGeneralPaneHeading,
                                                  columns : [ { id : 'name'   , width : '80'}, { id : 'value' } ] } );

        /*--------------------*/
        /* VMWARE SCOPES PANE */
        /*--------------------*/
        /** NOTE: emptyMsg should never occur for this panel. */
        var tablePanelVmwareScopes = new TablePanel( { title    : i18n.labelVmwareScopesHeading,
                                                       emptyMsg : i18n.labelVmwareScopeSetNoElements,
                                                       columns  : [  { id : 'type'    , label : i18n.labelTypeColumn    , width : '20%'},
                                                                     { id : 'value'   , label : i18n.labelValueColumn   , width : '30%'},
                                                                     { id : 'actions' , label : i18n.labelActionsColumn , width : '15%'}
                                                                  ] });

        var buttonContainerVmwareScopes = ButtonFactory.buildButtonContainer();

        // FUTURE - create the label
        var buttonNewScope = null;

        /*---------------------------*/
        /* CREDENTIALS PANE - VMWARE */
        /*---------------------------*/
        /** NOTE: emptyMsg should never occur for this panel. */
        var tablePanelVmwareCred = new TablePanel( { title    : i18n.labelDiscoVmwareCredsVmwareCredentialsHeading,
                                                     emptyMsg : i18n.labelDiscoVmwareCredsNoCreds,
                                                     columns  : [  { id : 'name'     , label : i18n.labelCredentialName , width : '20%' , sortable : false },
                                                                   { id : 'userName' , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                   { id : 'actions'  , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        var buttonContainerVmwareCredentials = ButtonFactory.buildButtonContainer();


        /*--------------------------*/
        /* CREDENTIALS PANE - Linux */
        /*--------------------------*/
        var tablePanelLinuxCred = new TablePanel( { title    : i18n.labelDiscoVmwareCredsLinuxCredentialsHeading,
                                                    emptyMsg : i18n.labelDiscoVmwareCredsNoCreds,
                                                    columns  : [  { id : 'name'     , label : i18n.labelCredentialName , width : '20%' , sortable : false },
                                                                  { id : 'type'     , label : i18n.labelDiscoCredsType , width : '20%' , sortable : false },
                                                                  { id : 'userName' , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                  { id : 'actions'  , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        var buttonContainerLinuxCredentials = ButtonFactory.buildButtonContainer();

        /*----------------------------*/
        /* CREDENTIALS PANE - WINDOWS */
        /*----------------------------*/
        var tablePanelWindowsCred = new TablePanel( { title    : i18n.labelDiscoVmwareCredsWindowsCredentialsHeading,
                                                      emptyMsg : i18n.labelDiscoVmwareCredsNoCreds,
                                                      columns  : [  { id : 'name'     , label : i18n.labelCredentialName , width : '20%' , sortable : false },
                                                                    { id : 'userName' , label : i18n.labelUserNameColumn , width : '15%' , sortable : false },
                                                                    { id : 'actions'  , label : i18n.labelActionsColumn  , width : '15%' , sortable : false } ] } );

        var buttonContainerWindowsCredentials = ButtonFactory.buildButtonContainer();


        var infoParagraph = document.createElement("div");

        getScopesetCmd.execute();
        $(getScopesetCmd).on('executionSucceeded', $.proxy( function(event) {
            name = event.response.payload.scopeSet.displayName;
            var displayName = name.replace("_VmwareCrt_","");

            buttonNewScope = ButtonFactory.buildButton({ type    : 'add',
                                                         grayout : true,
                                                         label   : i18n.buttonDiscoVmwareScopeAdd,
                                                         onclick : function( ) { (new ShowCommand( { action              : 'scopesvmware/new',
                                                                                                     clearSystemMessages : true,
                                                                                                     parameters          : { pid          : pid,
                                                                                                                             scopeSetName : displayName }} )
                                                                                 ).execute();
                                                       } });

            // Create Credentials Pane new button.  This is deferred so that the VMware dynamic scope set name can be determined.

            $( buttonContainerVmwareCredentials ) .append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerVmwareCredentials ) .append(this.createAddCredentialsButton(pid, i18n.buttonDiscoVmwareCredsAddVmware,  i18n.labelDiscoVmwareCredsVmwareType,  displayName));
            $( buttonContainerLinuxCredentials )  .append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerLinuxCredentials )  .append(this.createAddCredentialsButton(pid, i18n.buttonDiscoVmwareCredsAddLinux,   i18n.labelDiscoVmwareCredsLinuxType,   displayName));
            $( buttonContainerWindowsCredentials ).append( ButtonFactory.buildButtonSeparator );
            $( buttonContainerWindowsCredentials ).append(this.createAddCredentialsButton(pid, i18n.buttonDiscoVmwareCredsAddWindows, i18n.labelDiscoVmwareCredsWindowsType, displayName));

            // Add information to General panel
            tablePanelGeneral.appendRow( [i18n.labelNameField, displayName] );


            if ( !TV.typeIsUndefined( typeof event.response.payload.scopeSet.scopeElements ) )  {

                var scopeElementsLength = event.response.payload.scopeSet.scopeElements.length;

                // Add row for each VMware vCenter or ESXi scope
                $.each( event.response.payload.scopeSet.scopeElements, function( idx, element ) {
                    var type = null;
                    if (element.type === 0) {
                        type = i18n.labelScopeSetAddress;
                    }

                    /**@type function() */
                    var clickTriggerHandler = function ( event ) {
                        tsa.clearSysMessages();
                        var proceed = true;
                        if (!TV.typeIsUndefined( typeof event.data.confirm )) {
                            proceed = confirm(i18n.labelDeleteConfirmation + " '" + event.data.name + "' ?");
                        }
                        if (proceed) {
                            var actionCmd = new Command( { action : event.data.URL, clearSystemMessages : false });
                            actionCmd.execute();
                            $( actionCmd ).on( 'executionSucceeded', function( ) {
                                (new ShowCommand( {action : 'scopesetsvmware/edit',
                                                   clearSystemMessages : false,
                                                   parameters : { pid : pid }}
                                )).execute();
                            } );
                        }
                    };

                    var dActions   = document.createElement("div");
                    var editIcon   = IconFactory.buildActionIcon( { type    : 'edit',
                                                                    caption : i18n.labelButtonEdit   + " '" + element.externalForm + "'"} );
                    var runHoverText = i18n.labelDiscoVmwareStartDiscovery.replace("{0}", element.externalForm);
                    var runIcon    = IconFactory.buildActionIcon( { type    : 'runb',
                                                                    caption :  runHoverText } );
                    var deleteIcon = IconFactory.buildActionIcon( { type    : 'delete',
                                                                    caption : i18n.labelButtonDelete + " '" + element.externalForm + "'"} );
                    $( editIcon   ).on( 'click',  function() {
                        (new ShowCommand( {action              : 'scopesvmware/edit',
                                           clearSystemMessages : true,
                                           parameters          : {  pid          : element.pid,
                                                                    sepid        : pid,
                                                                    scopeSetName : displayName }
                                          })).execute();
                    } );

                    $( runIcon    ).on( 'click', { URL : "scopesetsvmware/ajaxDiscoverScopeElement/" + element.pid } , clickTriggerHandler);
                    $( deleteIcon ).on( 'click', { URL     : "scopes/ajaxDelete/" + element.pid,
                                                   name    : element.externalForm,
                                                   confirm : true } , clickTriggerHandler);
                    var aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( editIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    if (scopeElementsLength > 1) {
                        aRef = document.createElement("a");
                        $( aRef ).attr( 'href', '#' );
                        $( aRef ).append( deleteIcon );
                        $( dActions ).append( aRef );
                        $( dActions ).append(tsa.NBSP);
                    }

                    aRef = document.createElement("a");
                    $( aRef ).attr( 'href', '#' );
                    $( aRef ).append( runIcon );
                    $( dActions ).append( aRef );
                    $( dActions ).append(tsa.NBSP);

                    tablePanelVmwareScopes.appendRow( [type, element.externalForm, dActions ] );
                });

                // Add button for Add New VMware vCenter or ESXi
                $(buttonContainerVmwareScopes).append(ButtonFactory.buildButtonSeparator);
                $(buttonContainerVmwareScopes).append(buttonNewScope);

                var rowButtonScopes    = tablePanelVmwareScopes.tableBody.insertRow(-1);
                var tdButtonScopes     = document.createElement("td");
                var tdbuttonFillScopes = document.createElement("td");
                $(tdButtonScopes).append( buttonContainerVmwareScopes );
                $(tdButtonScopes).attr("colspan", 2);
                $(tdbuttonFillScopes).attr("colspan", 4);
                $(rowButtonScopes).append(tdButtonScopes, tdbuttonFillScopes);


                var selectIPVmware = document.createElement( 'select' );
                // Timezone selector properties...
                selectIPVmware.name = "timezone";
                selectIPVmware.id   = "timezone";

                var opt = document.createElement( 'option' );
                opt.value = "Unassigned";

                $( opt ).text( "Unassigned" );
                $( opt ).attr('selected' , 'selected');
                $( selectIPVmware ).append( opt );

                $.each( event.response.payload.scopeSet.scopeElements, function( idx, element ){
                    opt = document.createElement( 'option' );
                    opt.value = element.pid;
                    $( opt ).text( element.externalForm );
                    $( selectIPVmware ).append( opt );

                });

            } else {
                // hide if there is no scopes to move
                $(buttonNewCredentials).hide();
            }

            var vmwareName = name.replace("_VmwareCrt_","");
            var templateInfo = function(value) {
                var pTemplate  = document.createElement("p");
                var brTemplate = $("<br class='break'>").get(0);
                $(pTemplate).append(value, brTemplate);
                return pTemplate;
            };

            var getPartitionScopeSet = new Command( { action              :  'scopesets/getScopeSets',
                                                      parameters          : { 'filter' : 'Vmware'},
                                                      clearSystemMessages : false } );
            getPartitionScopeSet.execute();
            $( getPartitionScopeSet ).on( 'executionSucceeded', function( event ) {
                var scopesets = event.response.payload.scopeSets;

                if( !TV.typeIsUndefined( typeof scopesets ) ) {
                    var missingPartitionCreation = true;
                    $.each( scopesets, function( icaIdx, set ) {
                        if (set.name.startsWith(vmwareName) && set.name.match(/(_LINUX|_WINDOWS)$/) && set.enabled){
                            $(infoParagraph).append(templateInfo(set.name));
                            missingPartitionCreation = false;
                        }
                    });

                    if (missingPartitionCreation) {
                        $(infoParagraph).append(templateInfo(i18n.labelVmwareNonePartition));
                    }
                }

            });

            // Create credential panels
            // Start logic to retrieve and display credentials
            var getCredCommand = new Command( { action              : 'credentials/getCredentials',
                                                parameters          : { 'filter' : 'EVERYTHING'},
                                                clearSystemMessages : false } );
            getCredCommand.execute();

            $( getCredCommand ).on( 'executionSucceeded', $.proxy( function( event ) {
                // Here we use the new function
                var credentials = event.response.payload.credentials;

                $.proxy(this.retrieveVmwareCredentials (tablePanelVmwareCred,  buttonContainerVmwareCredentials,  credentials, pid, name), this);
                $.proxy(this.retrieveLinuxCredentials  (tablePanelLinuxCred,   buttonContainerLinuxCredentials,   credentials, pid, name), this);
                $.proxy(this.retrieveWindowsCredentials(tablePanelWindowsCred, buttonContainerWindowsCredentials, credentials, pid, name), this);
            }, this ) );

        }, this ) );


        contentArea.append(tablePanelGeneral     .getDomNode());
        contentArea.append(tablePanelVmwareScopes.getDomNode());
        contentArea.append(tablePanelVmwareCred  .getDomNode());
        contentArea.append(tablePanelLinuxCred   .getDomNode());
        contentArea.append(tablePanelWindowsCred .getDomNode());
        this.show(contentArea.getDomNode());
    };


    /*
     * Set-up add credential button on show / edit dynamic scope set panel.
     *
     * @param pid ID of the dynamic scope set.
     * @param buttonLabel Label to be associated with the button
     * @param buttonType Type of add button (VMware, Linux, or Windows)
     * @param displayName Displayable version of the dynamic scope set name
     *
     * @return Constructed button.
     */
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.createAddCredentialsButton = function( pid, buttonLabel, buttonType, displayName ) {
        var addButton = ButtonFactory.buildButton( { type    : 'add',
                                                     grayout : true,
                                                     label   : buttonLabel } );

        $(addButton).on('click', function () {
             (new ShowCommand( { action              : 'credentialsvmware/new',
                                 clearSystemMessages : true,
                                 parameters          : { pid          : pid,
                                                         type         : buttonType,
                                                         scopeSetName : displayName }} )
             ).execute();
        } );

        return addButton;
    };


    /**************************************************************************
     * RETRIEVE CREDENTIAL FUNCTIONS FOR EDIT SCOPE SET                       *
     *                                                                        *
     * There is a separate function for each credential type:                 *
     *     - VMware vCenter / ESXi       retrieveVmwareCredentials            *
     *     - Linux                       retrieveLinuxCredentials             *
     *     - Windows                     retrieveWindowsCredentials           *
     **************************************************************************/


    //
    // Retrieve VMware credentials
    //
    // @param tablePanelCred Panel used for VMware credentials.
    // @param buttonContainer Container for buttons associated with VMware credentials.
    // @param credentials List of all credentials.
    // @param pid PID for this VMware Dynamic Scope Set.
    // @param scopesetName Name of the VMware Dynamic Scope Set.
    //
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.retrieveVmwareCredentials = function(tablePanelCred,
                                                                                           buttonContainer,
                                                                                           credentials,
                                                                                           pid,
                                                                                           scopesetName) {
        var i18n = tsa.i18n;

        var tablePanel      = null;
        var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };

        var displayScopeSetName = scopesetName.replace("_VmwareCrt_", "");

        if ( TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 ) ) {

            var columnData = [];
            var prunedCred = [];

            // Prune credential not assigned to current vmware scopeset
            $.each( credentials, function( credIndex, cred ) {
                if (cred.name.endsWith("_VmwareCrt_") && (cred.scopeSetName === scopesetName)) {
                    prunedCred.push(cred);
                }
            } );

            credentials = prunedCred;

            columnData.push( { id : 'name'     , label : i18n.labelDiscoCredsName     , width : '20%' , sortable : false },
                             { id : 'userName' , label : i18n.labelDiscoCredsUserName , width : '15%' , sortable : false },
                             { id : 'actions'  , label : i18n.labelActionsColumn      , width : '15%' , sortable : false } );

            tablePanelProps.columns = columnData;

        } else {
            tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }

        tablePanel = new TablePanel( tablePanelProps );

        var credentialsLength = credentials.length;

        $.each( credentials, function( credIndex, cred ) {

            // If scopeset is null or does not have the tag in the name then continue
            if (!cred.name.endsWith("_VmwareCrt_")) {
                return;
            }

            var vmwareCredentialSuffix = "_SCOPESET_" + displayScopeSetName + "_VMWARE_VmwareCrt_";

            cred.name = cred.name.substring(0, cred.name.indexOf(vmwareCredentialSuffix));

            /** @type {Array.<Element>} */ var elements = [];
            // Elements
            var nameSpan     = document.createElement( 'span' );
            var nameAnchor   = document.createElement( 'a' );
            var actionsSpan  = document.createElement( 'span' );
            var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit     + ' "' + cred.name + '"' } );
            var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete   + ' "' + cred.name + '"' } );
            var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsDown } );
            var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsUp   } );

            // Boolean conditions for determining whether up or down arrows for changing credential order is needed.
            var isFirst = ( credIndex === 0 );
            var isLast  = ( credIndex === ( credentials.length - 1 ) );

            // Column 1 - Name:
            nameAnchor.href = '#';
            $( nameAnchor ).addClass( 'ibm-feature-link' );
            $( nameAnchor ).css( { 'font-weight' : 'bold' } );
            $( nameAnchor ).text( cred.name );
            $( nameAnchor ).on( 'click', function( event ) {
                ( new ShowCommand( { action     : 'credentialsvmware/show/' + cred.pid,
                                     parameters : { pid : pid }
                                   } ) ).execute();

                try {
                    event.stopPropagation();
                } catch ( error ) { /* Ignore */ }

                return false;
            } );

            $( nameSpan ).append( nameAnchor );
            $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
            elements.push( nameSpan );

            // Column 2 - User name:
            elements.push( ( cred.userName !== null ) ? cred.userName : '' );

            // Column 3 - Actions:
            $( actionsSpan ).append( editIcon   );
            $( editIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.editCredentialClicked( cred.pid, pid ); } );
            $( editIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );

            if (credentialsLength > 1) {
                /*
                 * Do not allow deletion if there is only one VMware credential.
                 */
                $( actionsSpan ).append( deleteIcon );
                $( deleteIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );
                $( deleteIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.deleteCredentialClicked( cred.name, cred.pid , pid); } );
            }

            if ( !isLast  ) {
                $( actionsSpan ).append( moveDownIcon );
                $( moveDownIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, 1 );  } );
            }

            if ( !isFirst ) {
                $( actionsSpan ).append( moveUpIcon );
                $( moveUpIcon   ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, -1 ); } );
            }

            elements.push( actionsSpan );

            // Append the row...
            tablePanelCred.appendRow( elements );

        } );

        var rowButton    = tablePanelCred.tableBody.insertRow(-1);
        var tdButton     = document.createElement("td");
        var tdbuttonFill = document.createElement("td");
        $(tdButton).append( buttonContainer );
        $(tdButton).attr("colspan", 2);
        $(tdbuttonFill).attr("colspan", 4);
        $(rowButton).append(tdButton,tdbuttonFill);
    };


    //
    // Retrieve Linux credentials
    //
    // @param tablePanelCred Panel used for Linux credentials.
    // @param buttonContainer Container for buttons associated with Linux credentials.
    // @param credentials List of all credentials.
    // @param pid PID for this VMware Dynamic Scope Set.
    // @param scopesetName Name of the VMware Dynamic Scope Set.
    //
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.retrieveLinuxCredentials = function(tablePanelCred,
                                                                                          buttonContainer,
                                                                                          credentials,
                                                                                          pid,
                                                                                          scopesetName) {
        var i18n = tsa.i18n;

        var tablePanel      = null;
        var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };

        var displayScopeSetName = scopesetName.replace("_VmwareCrt_", "");

        var scopeSetNameLinux   = scopesetName.replace("_VmwareCrt_","_LINUX_VmwareCrt_");

        if ( TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 ) ) {
            var columnData       = [];
            var prunedCred = [];

            // Prune credential not assigned to current vmware scopeset
            $.each( credentials, function( credIndex, cred ) {
                if (cred.name.endsWith("_VmwareCrt_") && (cred.scopeSetName === scopeSetNameLinux)) {
                    prunedCred.push(cred);
                }
            } );

            credentials = prunedCred;

            columnData.push( { id : 'name'     , label : i18n.labelDiscoCredsName     , width : '20%' , sortable : false },
                             { id : 'type'     , label : i18n.labelDiscoCredsType     , width : '20%' , sortable : false },
                             { id : 'userName' , label : i18n.labelDiscoCredsUserName , width : '15%' , sortable : false },
                             { id : 'actions'  , label : i18n.labelActionsColumn      , width : '15%' , sortable : false } );

            tablePanelProps.columns = columnData;

        } else {
            tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }

        tablePanel = new TablePanel( tablePanelProps );

        $.each( credentials, $.proxy( function( credIndex, cred ) {

            // If scopeset is null or does not have the tag in the name then continue
            if (!cred.name.endsWith("_VmwareCrt_")) {
                return;
            }

            var linuxCredentialSuffix = "_SCOPESET_" + displayScopeSetName + "_LINUX_VmwareCrt_";

            cred.name = cred.name.substring(0, cred.name.indexOf(linuxCredentialSuffix));

            var elements     = [];
            var nameSpan     = document.createElement( 'span' );
            var nameAnchor   = document.createElement( 'a' );
            var actionsSpan  = document.createElement( 'span' );
            var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit     + ' "' + cred.name + '"' } );
            var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete   + ' "' + cred.name + '"' } );
            var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsDown } );
            var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsUp   } );

            var isFirst = ( credIndex === 0 );
            var isLast  = ( credIndex === ( credentials.length - 1 ) );

            // Column 1 - Name:
            nameAnchor.href = '#';
            $( nameAnchor ).addClass( 'ibm-feature-link' );
            $( nameAnchor ).css( { 'font-weight' : 'bold' } );
            $( nameAnchor ).text( cred.name );
            $( nameAnchor ).on( 'click', function( event ) {
                ( new ShowCommand( { action     : 'credentialsvmware/show/' + cred.pid,
                                     parameters : { pid : pid }
                                   } ) ).execute();

                try {
                    event.stopPropagation();
                } catch ( error ) { /* Ignore */ }

                return false;
            } );

            $( nameSpan ).append( nameAnchor );
            $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
            elements.push( nameSpan );

            // Column 2 - authentication type
            //
            // Only applies to Linux credentials.  VMware (which uses a web service) and Windows (which uses SMB or remote management)
            // only support password authentication.
            elements.push( this.i18nAuthType(cred));

            // Column 3 - User name:
            elements.push( ( cred.userName !== null ) ? cred.userName : '' );

            // Column 4 - Actions:
            $( actionsSpan ).append( editIcon   );
            $( editIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.editCredentialClicked( cred.pid, pid ); } );
            $( editIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );

            $( actionsSpan ).append( deleteIcon );
            $( deleteIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );
            $( deleteIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.deleteCredentialClicked( cred.name, cred.pid , pid); } );

            if (!isLast) {
                $( actionsSpan ).append( moveDownIcon );
                $( moveDownIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, 1 ); } );
            }

            if (!isFirst) {
                $( actionsSpan ).append( moveUpIcon );
                $( moveUpIcon   ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, -1 );  } );
            }

            elements.push( actionsSpan );

            // Append the row...
            tablePanelCred.appendRow( elements );

        }, this ) );

        var rowButton    = tablePanelCred.tableBody.insertRow(-1);
        var tdButton     = document.createElement("td");
        var tdbuttonFill = document.createElement("td");
        $(tdButton).append( buttonContainer );
        $(tdButton).attr("colspan", 2);
        $(tdbuttonFill).attr("colspan", 4);
        $(rowButton).append(tdButton,tdbuttonFill);
    };


    //
    // Retrieve Windows credentials
    //
    // @param tablePanelCred Panel used for Windows credentials.
    // @param buttonContainer Container for buttons associated with Windows credentials.
    // @param credentials List of all credentials.
    // @param pid PID for this VMware Dynamic Scope Set.
    // @param scopesetName Name of the VMware Dynamic Scope Set.
    //
    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.retrieveWindowsCredentials = function( tablePanelCred , buttonContainer, credentials, pid, scopesetName) {

        var i18n            = tsa.i18n;

        var tablePanel      = null;
        var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };

        var displayScopeSetName = scopesetName.replace("_VmwareCrt_", "");

        var scopeSetNameWindows = scopesetName.replace("_VmwareCrt_","_WINDOWS_VmwareCrt_");

        if ( TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 ) ) {
            var columnData = [];
            var prunedCred = [];

             // Prune credential not assigned to current vmware scopeset
            $.each( credentials, function( credIndex, cred ) {
                if (cred.name.endsWith("_VmwareCrt_") && (cred.scopeSetName === scopeSetNameWindows )) {
                    prunedCred.push(cred);
                }
            } );

            credentials = prunedCred;

            columnData.push( { id : 'name'     , label : i18n.labelDiscoCredsName     , width : '20%' , sortable : false },
                             { id : 'userName' , label : i18n.labelDiscoCredsUserName , width : '15%' , sortable : false },
                             { id : 'actions'  , label : i18n.labelActionsColumn      , width : '15%' , sortable : false } );

            tablePanelProps.columns = columnData;

        } else {
            tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
        }

        tablePanel = new TablePanel( tablePanelProps );

        $.each( credentials, function( credIndex, cred ) {

            // if scopeset is null or does not have the tag in the name then continue
            if (!cred.name.endsWith("_VmwareCrt_") ){
                return;
            }

            var windowsCredentialSuffix = "_SCOPESET_" + displayScopeSetName + "_WINDOWS_VmwareCrt_";

            cred.name = cred.name.substring(0, cred.name.indexOf(windowsCredentialSuffix));

            var elements     = [];
            var nameSpan     = document.createElement( 'span' );
            var nameAnchor   = document.createElement( 'a' );
            var actionsSpan  = document.createElement( 'span' );
            var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : i18n.labelButtonEdit     + ' "' + cred.name + '"' } );
            var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : i18n.labelButtonDelete   + ' "' + cred.name + '"' } );
            var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsDown } );
            var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : i18n.labelDiscoCredsMove + ' "' + cred.name + '"' + i18n.labelDiscoCredsUp   } );

            var isFirst = ( credIndex === 0 );
            var isLast  = ( credIndex === ( credentials.length - 1 ) );

            // Column 1 - Name:
            nameAnchor.href = '#';
            $( nameAnchor ).addClass( 'ibm-feature-link' );
            $( nameAnchor ).css( { 'font-weight' : 'bold' } );
            $( nameAnchor ).text( cred.name );
            $( nameAnchor ).on( 'click', function( event ) {
                ( new ShowCommand( { action     : 'credentialsvmware/show/' + cred.pid,
                                     parameters : { pid : pid }
                                   } ) ).execute();

                try {
                    event.stopPropagation();
                } catch ( error ) { /* Ignore */ }

                return false;
            } );

            $( nameSpan ).append( nameAnchor );
            $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
            elements.push( nameSpan );

            // Column 2 - User name:
            elements.push( ( cred.userName !== null ) ? cred.userName : '' );

            // Column 3 - Actions:
            $( actionsSpan ).append( editIcon   );
            $( editIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.editCredentialClicked( cred.pid, pid ); } );
            $( editIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );

            $( actionsSpan ).append( deleteIcon );
            $( deleteIcon ).css( { 'cursor' : 'pointer', 'margin-right' : '0.5em' } );
            $( deleteIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.deleteCredentialClicked( cred.name, cred.pid, pid ); } );

            if (!isLast) {
                $( actionsSpan ).append( moveDownIcon );
                $( moveDownIcon ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, 1 ); } );
            }

            if (!isFirst) {
                $( actionsSpan ).append( moveUpIcon );
                $( moveUpIcon   ).on( 'click', function( ) { tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked( cred.pid, pid, -1 ); } );
            }

            elements.push( actionsSpan );

            // Append the row...
            tablePanelCred.appendRow( elements );

        } );

        var rowButton    = tablePanelCred.tableBody.insertRow(-1);
        var tdButton     = document.createElement("td");
        var tdbuttonFill = document.createElement("td");
        $(tdButton).append( buttonContainer );
        $(tdButton).attr("colspan", 2);
        $(tdbuttonFill).attr("colspan", 4);
        $(rowButton).append(tdButton,tdbuttonFill);
    };


    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== EVENT HANDLERS                                                         ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/

    /**************************************************************************
     * DELETE CREDENTIAL EVENT HANDLER                                        *
     *                                                                        *
     * @param credentialName Name of the credential.                          *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this VMware Dynamic Scope Set.                   *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.deleteCredentialClicked = function( credentialName, credentialPID , pid) {
        if ( TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 ) ) {
            if ( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + credentialName + '"?' ) ) {
                var deleteCmd = new Command( { action     : 'credentialsvmware/deleteCredential',
                                               parameters : { 'pid' : credentialPID } } );

                $( deleteCmd ).on( 'executionSucceeded', function( event ) {
                    if ( !TV.typeIsUndefined( typeof event.response.sysMessages[0] ) ) {
                        // Clear default message
                        tsa.clearSysMessages();
                        tsa.printSysMessage( { level   : 'info',
                                               message : event.response.sysMessages[0].message } );
                    }

                    ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                         parameters          : { pid : pid },
                                         clearSystemMessages : false } ) ).execute();
                } );

                deleteCmd.execute();
            }

        } else {
            logger.warn( 'Invalid credential PID. Deletion request did not proceed.' );
        }
    };


    //-------------------------------------------------------------------------

    /**************************************************************************
     * EDIT CREDENTIAL EVENT HANDLER                                          *
     *                                                                        *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this VMware Dynamic Scope Set.                   *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.editCredentialClicked = function( credentialPID, pid ) {
        ( new ShowCommand( { action     : 'credentialsvmware/edit/' + credentialPID,
                             parameters : { entityId : credentialPID , pid : pid } } ) ).execute();
    };


    //-------------------------------------------------------------------------

    /**************************************************************************
     * MOVE CREDENTIAL EVENT HANDLER                                          *
     *                                                                        *
     * @param credentialPID The credential PID.                               *
     * @param pid The PID of this VMware Dynamic Scope Set.                   *
     * @param offset Negative to move credential up, positive to move down.   *
     **************************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.moveCredentialClicked = function( credentialPID, pid, offset ) {
        if( TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 ) ) {
            if( TV.typeIsNumber( typeof offset ) && !isNaN( offset ) && ( offset !== 0 ) ) {
                /** @type {tsa.cmd.Command} */ var moveCmd = new Command( { action     : 'scopesetsvmware/ajaxMoveCredential',
                                                                            parameters : { 'pid'    : credentialPID,
                                                                            'offset'   : offset } } );

                $( moveCmd ).on( 'executionSucceeded', function( ) { ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                                                                          clearSystemMessages : false,
                                                                                          parameters          : { pid : pid } } ) ).execute();
                                                                   } );

                moveCmd.execute();

            } else {
                logger.error( 'Invalid offset. Movement request did not proceed.' );
            }

        } else {
            logger.error( 'Invalid credential PID. Movement request did not proceed.' );
        }
    };


    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== GENERAL FUNCTIONS                                                      ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/


    /**************************************************************
     * CONVERT AUTHENTICATION TYPE TO DISPLAYABLE VALUE           *
     *                                                            *
     * @param authtype Authentication type string.                *
     * @return Displayable value for authentication type.         *
     **************************************************************/

    tsa.cmd.handlers.ScopesetsvmwareHandler.prototype.i18nAuthType = function ( credential ) {
        /** @type {string} */ var i18nLabel = '';
        /** @type {string} */ var authType  = '';
        
        if ( TV.typeIsString (typeof credential) )
        {
          authType = credential;
        }
        else if( !TV.typeIsUndefined( typeof credential.properties.authtype ) )
        {
         
          /** @type {boolean} */ var isPKI             = ( credential.properties.authtype.indexOf( 'authType_pki' ) === 0 );
          /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                           ( credential.scopeSetName.length > 0 ) );
          /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
          authType = credential.properties.authtype;
          /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );
          if( isPKI )
          {
            authType =   scopeSetHasSSHKey  ? 'authType_pki_scoped' : 'authType_pki' ;
          }
        }
        else 
        {
          authType = credential.authClassLabel;
        }

        switch( authType ) {
            case 'authType_default'    : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypeDefault   ; break;
            case 'authType_password'   : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
            case 'authType_pki'        : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePki       ; break;
            case 'authType_pki_scoped' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
            default                    : i18nLabel = authtype;
        }

        return i18nLabel;
    };



    //  -------------------------------------------------------------------------
    return tsa.cmd.handlers.ScopesetsvmwareHandler;

    }
);
