/*
 * IBM Technical Support Appliance Version 2 Release 4 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/VerificationToolkit'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , VT
          , Command
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ScopesettingsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ScopesettingsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ScopesettingsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopesettingsHandler.prototype ).constructor = tsa.cmd.handlers.ScopesettingsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * The maximum recommended number of scope entries.
     * @private
     * @type {number}
     */
    tsa.cmd.handlers.ScopesettingsHandler.MAX_RECOMMENDED_ENTRIES = 400;
    
    /**
     * The content area widget instance.
     * @private
     * @type {tsa.widgets.ContentArea}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.contentArea = null;
    
    /**
     * Instructions message.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.instructionsMsg = null;
    
    /**
     * The form panel widget instance.
     * @private
     * @type {tsa.widgets.FormPanel}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.formPanel = null;
    
    /**
     * The HTML Input for the scope name.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.nameInput = null;
    
    /**
     * The HTML Input for the file containing scope entries.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.fileInput = null;
    
    /**
     * The map of file entries and their attributes.
     * @private
     * @type {Object.<string,{valid:boolean
     *                       ,count:number}>}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.fileEntries = null;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype._index = function()
    {
      /** @type {Object} */ var i18n = tsa.i18n;
      
      this.contentArea = new ContentArea ( { title       : i18n.labelImportScopeHeading
                                           , helpDoc     : 'sshelp_scopeset_settings_panel.html' } );
      this.formPanel   = new FormPanel   ( { title       : i18n.labelImportScopeSetHeading
                                           , description : i18n.labelImportScopeSetDesc } );
      
      this.nameInput = $( '<input type="text" name="name" id="name"    size="50" maxlength="250" required/>' ).get( 0 );
      this.fileInput = $( '<input type="file" name="fileUpload" id="fileUpload" required/>' ).get(0);
      
      // Clear previous message
      $( this.fileInput ).on( 'change', function(){tsa.clearSysMessages();});
      
      this.formPanel.appendFieldRow( { label    : i18n.labelImportScopeSetNameHeading
                                     , required : true
                                     , element  : this.nameInput } );
      this.formPanel.appendFieldRow( { label    : i18n.labelImportScopeSet
                                     , required : true
                                      , element  : this.fileInput 
                                      , hint : i18n.labelImportScopeSetNameDesc} );
      
      this.formPanel.appendButton( { type    : 'arrow'
                                   , label   : i18n.buttonImportScopeSet
                                   , onclick : $.proxy( this.onImportButtonClicked, this ) } );
      
      this.instructionsMsg = this.contentArea.appendMessage( { text : i18n.labelImportScopeSetCaption
                                                                    + '\n\n'
                                                                    + i18n.labelImportScopeSetLabelCaption } );
      this.contentArea.append( this.formPanel.getDomNode() );
      
      this.show( this.contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Load file entries.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.loadFileEntries = function()
    {
      this.fileEntries = {};
      
      if( !TV.typeIsUndefined( typeof FileReader           ) &&
          !TV.typeIsUndefined( typeof this.fileInput.files ) )
      {
        if( this.fileInput.files.length > 0 )
        {
          /** @type {FileReader} */ var fileReader = new FileReader();
          
          fileReader.onload = $.proxy( function( progressEvent )
          {
            /** @type {Array.<string>} */ var lines         = fileReader.result.split( '\n' );
            /** @type {number}         */ var blankLines    = 0;
            /** @type {number}         */ var validCount    = 0;
            /** @type {number}         */ var invalidCount  = 0;
            /** @type {number}         */ var validDupCount = 0;
            
            $.each( lines, $.proxy( function( lineIndex, line )
            {
              if( ( line = $.trim( line ) ).length > 0 )
              {
                if( !TV.typeIsObject( typeof this.fileEntries[ line ] ) )
                {
                  /** @type {boolean} */ var isValid = VT.isValidIPv4( line );
                  
                  this.fileEntries[ line ] = { valid : isValid
                                             , count : 1 };
                  validCount   += ( isValid ? 1 : 0 );
                  invalidCount += ( isValid ? 0 : 1 );
                }
                else
                {
                  this.fileEntries[ line ].count++;
                  validDupCount += ( this.fileEntries[ line ].valid ? 1 : 0 );
                  invalidCount  += ( this.fileEntries[ line ].valid ? 0 : 1 );
                }
              }
              else
              {
                blankLines++;
              }
            }
            , this ) );
            
            logger.info( 'File entries loaded. Results:'
                       + '\n    ' + 'Total lines'              + ': ' + lines.length  + '.'
                       + '\n    ' + 'Blank lines'              + ': ' + blankLines    + '.'
                       + '\n    ' + 'Valid entries'            + ': ' + validCount    + '.'
                       + '\n    ' + 'Duplicated valid entries' + ': ' + validDupCount + '.'
                       + '\n    ' + 'Invalid entries'          + ': ' + invalidCount  + '.' );
            
            // Confirm:
            this.showConfirmationDialog();
          }
          , this );
          
          // Reading using default encoding: utf-8
          fileReader.readAsText( this.fileInput.files[0] );
        }
        else
        {
          logger.warn( 'File entries can\'t be loaded: file not selected.' );
        }
      }
      else
      {
        logger.error( 'Cannot read file: unsupported browser.' );
      }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Button: click event handler.
     * @param {Event} event Event
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.onImportButtonClicked = function( event )
    {
      if( this.formPanel.validate() )
      {
        this.loadFileEntries();
      }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show confirmation dialog after locally loading/analyzing the entries.
     * If problems were detected within the entry file, they will be displayed here.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.showConfirmationDialog = function()
    {
      if( this.fileEntries !== null )
      {
        /** @type {Array.<string>} */ var valid       = [];
        /** @type {Array.<string>} */ var invalid     = [];
        /** @type {Array.<string>} */ var validDup    = [];
        /** @type {boolean}        */ var tooMany     = false;
        /** @type {boolean}        */ var noneValid   = false;
        
        for( /** @type {string} */ var key in this.fileEntries )
        {
          /** @type {Object} */ var entry = this.fileEntries[ key ];
          
          if( !entry.valid )
          {
            invalid.push( key );
          }
          else
          {
            valid.push( key );
            
            if( entry.count > 1 )
            {
              validDup.push( key );
            }
          }
        }
        
        if( noneValid = ( valid.length === 0 ) )
        {
          alert( tsa.i18n.labelImportScopeSetProblemNoneValid );
        }
        else if( ( tooMany = ( valid.length > tsa.cmd.handlers.ScopesettingsHandler.MAX_RECOMMENDED_ENTRIES ) ) ||
                 ( invalid  .length > 0 ) ||
                 ( validDup .length > 0 ) )
        {
          /** @type {Element} */ var problemsMsgNode = document.createElement( 'h3' );
          /** @type {Element} */ var tooManyMsgNode  = null;
          /** @type {Element} */ var invalidMsgNode  = null;
          /** @type {Element} */ var validDupMsgNode = null;
          /** @type {Element} */ var validMsgNode    = null;
          /** @type {Element} */ var confirmMsgNode  = document.createElement( 'h3' );
          /** @type {string}  */ var tooManyMsgText  = tsa.i18n.labelImportScopeSetStatus;
          /** @type {string}  */ var invalidMsgText  = tsa.i18n.labelImportScopeSetProblemInvalid;
          /** @type {string}  */ var validDupMsgText = tsa.i18n.labelImportScopeSetProblemValidDup;
          /** @type {string}  */ var validMsgText    = tsa.i18n.labelImportScopeSetValidCount;
          /** @type {Element} */ var buttonContainer = ButtonFactory.buildButtonContainer();
          /** @type {string}  */ var bullet          = String.fromCharCode( 0x2022 );
          
          $( this.instructionsMsg ).hide();
          $( this.formPanel.getDomNode() ).hide();
          
          $.each( invalid  , function( i, entry ) { invalidMsgText  += '\n' + bullet + ' ' + entry; } );
          $.each( validDup , function( i, entry ) { validDupMsgText += '\n' + bullet + ' ' + entry; } );
          
          validMsgText += ' ' + ( valid.length );
          
          $( problemsMsgNode ).text( tsa.i18n.labelImportScopeSetProblemsFound );
          if (tooMany)
          {
            $( confirmMsgNode  ).text( tsa.i18n.labelImportScopeSetConfirmLimit );
          } 
          else
          {
            $( confirmMsgNode  ).text( tsa.i18n.labelImportScopeSetConfirm );
          }
          this.contentArea.append( problemsMsgNode );
          
          if( tooMany              ) tooManyMsgNode  = this.contentArea.appendMessage( { level : 'WARN', border : false, text : tooManyMsgText  } );
          if( invalid  .length > 0 ) invalidMsgNode  = this.contentArea.appendMessage( { level : 'WARN', border : false, text : invalidMsgText  } );
          if( validDup .length > 0 ) validDupMsgNode = this.contentArea.appendMessage( { level : 'WARN', border : false, text : validDupMsgText } );
          
          var closeDialog = $.proxy( function( event )
          {
            if( tooManyMsgNode  !== null ) $( tooManyMsgNode  ).remove();
            if( invalidMsgNode  !== null ) $( invalidMsgNode  ).remove();
            if( validDupMsgNode !== null ) $( validDupMsgNode ).remove();
            
            $( problemsMsgNode ).remove();
            $( confirmMsgNode  ).remove();
            $( validMsgNode    ).remove();
            $( buttonContainer ).remove();
            
            $( this.instructionsMsg ).show();
            $( this.formPanel.getDomNode() ).show();
            $( this.fileInput ).val( '' );
          }
          , this );
          /** @type {Element} */ var onAccept = null; 
          /** @type {Element} */ var onCancel = null;
          if(tooMany)
          {
            onAccept = $.proxy( function( event )
            {
              closeDialog();
            }
            , this );
          }
          else
          {
            onCancel = $.proxy( function( event )
            {
              closeDialog();
            }, this );
          
            onAccept = $.proxy( function( event )
            {
              closeDialog();
              this.importScopeSet( $( this.nameInput ).val(), valid );
            }, this );
          }
          if (!tooMany) 
          {
            $( buttonContainer ).append( ButtonFactory.buildButton( { type : 'cancel' , label : tsa.i18n.buttonCancel , onclick : onCancel } ) );
            $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
          }
          $( buttonContainer ).append( ButtonFactory.buildButton( { type : 'arrow'  , label : tsa.i18n.buttonAccept , onclick : onAccept } ) );
          
          this.contentArea.append( confirmMsgNode );
          validMsgNode = this.contentArea.appendMessage( { level : 'INFO', border: false, text : validMsgText } );
          this.contentArea.append( buttonContainer );
        }
        else
        {
          // A simple dialog is enough...
          if( window.confirm( tsa.i18n.labelImportScopeSetConfirm ) )
          {
            this.importScopeSet( $( this.nameInput ).val(), valid );
          }
        }
      }
      else
      {
        logger.error( 'The entries have not been loaded.' );
      }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Scope Set: perform the server request.
     * @private
     * @param {string}         name    The scope name.
     * @param {Array.<string>} entries The entries, previously validated IPv4 addresses.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.importScopeSet = function( name, entries )
    {
      /** @type {tsa.cmd.Command} */ var cmd = new Command( { action     : 'scopesettings/ajaxImportScopeSet'
                                                            , parameters : { 'name'    : name
                                                                           , 'entries' : entries.join( ' ' ) }
                                                            } );
      $( cmd ).on( 'executionSucceeded', $.proxy( function( event )
      {
        $( this.nameInput ).val( '' );
      }
      , this ) );
      
      $( cmd ).on( 'executionFailed', $.proxy( function( event )
      {
          if( TV.typeIsObject( typeof event.response.payload.errors ) )
          {
            for( /** @type {string} */ var fieldName in event.response.payload.errors )
            {
                this.formPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
            }
          }
      }
      , this ) );
      
      cmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ScopesettingsHandler;
  }
);