/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery',
    'tsa/TypeVerificator',
    'tsa/cmd/handlers/AbstractHandler',
    'tsa/widgets/ContentArea',
    'tsa/widgets/TablePanel',
    'tsa/widgets/ButtonFactory',
    'tsa/widgets/FormPanel',
    'tsa/cmd/ShowCommand',
    'tsa/cmd/Command',
    'tsa/widgets/IconFactory'
  ],

  function( $,
            TV,
            AbstractHandler,
            ContentArea,
            TablePanel,
            ButtonFactory,
            FormPanel,
            ShowCommand,
            Command,
            IconFactory)
{
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ScopesvmwareHandler' );

    //-------------------------------------------------------------------------

    /**
     * @constructor
     * @extends tsa.cmd.AbstractHandler
     * Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     * @param {Object} parameters The parameters.
     */
    tsa.cmd.handlers.ScopesvmwareHandler =  function( parameters ) {
        if ( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) {
            parameters = /** @type {?} */ ( {} );
        }

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ScopesvmwareHandler.prototype = new AbstractHandler( {} );
    ( tsa.cmd.handlers.ScopesvmwareHandler.prototype ).constructor = tsa.cmd.handlers.ScopesvmwareHandler;


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** ADD IP ADDRESS TO VMWARE DYNAMIC SCOPE SET SCREEN                      ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesvmwareHandler.prototype._new = function() {

        var i18n            = tsa.i18n;
        var contentArea     = new ContentArea ( { title         : i18n.labelDiscoVmwareScopesHeading,
                                                  helpDoc       : 'sshelp_discovery_vmware_scopes_new_panel.html',
                                                  showBackToTop : true } );
        var generalForm     = new FormPanel( { title: i18n.labelGeneralPaneHeading } );
        var fpAddresses     = new FormPanel( { title       : i18n.labelDiscoScopesAddressHeading,
                                               action      : 'scopes/ajaxCreateDynamicScope',
                                               description : i18n.labelDiscoVmwareScopesAddressDesc } );
        var pid             = this.getParameter('pid');
        var scopeSetName    = this.getParameter('scopeSetName');
        var buttonSeparator = null;

        var nameSpan = document.createElement( 'span' );
        $( nameSpan ).css( { 'font-weight' : 'bold' } );
        $( nameSpan ).text( scopeSetName );

        generalForm.appendFieldRow( { label: i18n.labelScopeSetField, element: nameSpan } );

        fpAddresses.appendFieldRow( { label    : i18n.labelDiscoScopesAddressAddress,
                                      required : true,
                                      element  : $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text'>").get(0),
                                      hint     : i18n.labelSpecifyIpAddressFieldDescription } );
        var buttonContainer = ButtonFactory.buildButtonContainer();
        var selectIP        = $("<select name='option' id='option' size='1'></select>").get(0);
        var option          = document.createElement( 'option' );


        $( option ).attr('value'    , 'address' );
        $( option ).attr('selected' , 'selected');
        $( option ).text(i18n.labelDiscoScopesSelectAddress );
        $( selectIP ).append(option);

        option   = document.createElement( 'option' );
        $( option ).attr( 'value', 'range' );
        $( option ).text( i18n.labelDiscoScopesSelectRange );
        $( selectIP ).append(option);

        option   = document.createElement( 'option' );
        $( option ).attr( 'value', 'network' );
        $( option ).text( i18n.labelDiscoScopesSelectNetwork );
        $(selectIP).append(option);

        /*
         * Create "save" button.
         */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      grayout : false,
                                                      label   : i18n.buttonSave } );
        $(saveButton).on('click', $.proxy(function( ) {
            fpAddresses.putHiddenInput("pid", pid);
            fpAddresses.submit();

            $( fpAddresses ).on( 'executionSucceeded', function( ) {
                (new ShowCommand( { action              : 'scopesetsvmware/show',
                                    clearSystemMessages : false,
                                    parameters          : { pid : pid }
                } ) ).execute();
            } );

            // Show errors
            $( fpAddresses ).on( 'executionFailed', function( event ) {
                if ( TV.typeIsObject( typeof event.response.payload.errors ) ) {
                    for ( var fieldName in event.response.payload.errors ) {
                        fpAddresses.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                    }
                    delete event.response.payload.errors;
                }
            } );
        } ) );

        /*
         * Create "cancel" button.
         */
        var cancelButton = ButtonFactory.buildButton( { type    : 'cancel',
                                                        grayout : true,
                                                        label   : i18n.buttonCancel } );
        $(cancelButton).on('click', function () {
            (new ShowCommand( { action              : 'scopesetsvmware/show',
                                clearSystemMessages : true,
                                parameters          : { pid : pid }
            } ) ).execute();
        } );

        /*
         * Append buttons to panel.
         */
        buttonSeparator = ButtonFactory.buildButtonSeparator ;
        $( buttonSeparator ).hide();

        $( buttonContainer ).append( saveButton );
        $( buttonContainer ).append( buttonSeparator );
        $( buttonContainer ).append( cancelButton );

        $(fpAddresses.form).attr("aria-label", i18n.labelDiscoScopesAddressHeading);
        contentArea.append( generalForm.getDomNode() );
        contentArea.append( fpAddresses.getDomNode() );
        contentArea.append( buttonContainer );
        this.show( contentArea.getDomNode() );
    };


    //--------------------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT EXISTING IP ADDRESS IN VMWARE DYNAMIC SCOPE SET SCREEN            ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    tsa.cmd.handlers.ScopesvmwareHandler.prototype._edit = function() {
        var i18n                    = tsa.i18n;
        var contentArea             = new ContentArea ( { title   : i18n.labelDiscoVmwareScopesHeading,
                                                          helpDoc : 'sshelp_discovery_vmware_scopes_edit_panel.html' } );
        var generalForm             = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );
        var fpAddresses             = new FormPanel( { title       : i18n.labelDiscoScopesAddressHeading,
                                                       action      : 'scopes/ajaxEditScopeElement',
                                                       description : i18n.labelDiscoVmwareScopesAddressDesc } );
        var getScopeElementCmd      = new Command( { clearSystemMessages : false } );
        var pid                     = this.getParameter('pid');
        var sepid                   = this.getParameter('sepid');
        var scopeSetName            = this.getParameter('scopeSetName');
        var buttonSeparator         = null;
        var ipAddress               = null;

        var nameSpan = document.createElement( 'span' );
        $( nameSpan ).css( { 'font-weight' : 'bold' } );
        $( nameSpan ).text( scopeSetName );

        generalForm.appendFieldRow( { label: i18n.labelScopeSetField, element: nameSpan } );

        fpAddresses.putHiddenInput("scopeset",pid);
        fpAddresses.putHiddenInput("sepid",sepid);
        fpAddresses.putHiddenInput("option","address");

        var buttonContainer = ButtonFactory.buildButtonContainer();
        var selectIP        = $("<select name='option' id='option' size='1' ></select>").get(0);
        var option          = document.createElement( 'option' );
        $( option ).attr('value', 'address');
        $( option ).text(i18n.labelDiscoScopesSelectAddress);
        $( selectIP ).append(option);

        option = document.createElement('option');
        $( option ).attr('value', 'range');
        $( option ).text(i18n.labelDiscoScopesSelectRange);
        $( selectIP ).append(option);

        option = document.createElement('option');
        $( option ).attr('value', 'network');
        $( option ).text(i18n.labelDiscoScopesSelectNetwork);

        $(selectIP).append(option);

        /*
         * Create "save" button
         */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      grayout : false,
                                                      label   : i18n.buttonSave } );

        $(saveButton).on('click', $.proxy(function(  ) {
             fpAddresses.submit();
             $( fpAddresses ).on( 'executionSucceeded', function(  ) {
                 ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                      clearSystemMessages : false,
                                      parameters          : { pid : sepid }
                 } ) ).execute();
             } );

            // Show errors
            $( fpAddresses ).on( 'executionFailed', function( event ) {
                if( TV.typeIsObject( typeof event.response.payload.errors ) ) {
                    for( var fieldName in event.response.payload.errors ) {
                        fpAddresses.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                    }
                    delete event.response.payload.errors;
                }
            } );
        } , this ) );

        /*
         * Get information about the scope element (ie, IP address) to edit.
         */
        getScopeElementCmd.setAction("scopes/getScopeElement/" + pid);
        getScopeElementCmd.execute();

        $(getScopeElementCmd).on("executionSucceeded", $.proxy(function(event) {
            var scopeElement = event.response.payload.scopeElement;
            if (scopeElement.type === 0) {
                $( selectIP ).val("address");
                ipAddress = scopeElement.ipAddress;
                var inputAddress = $("<input size='50' maxlength='250' name='aaddr' id='aaddr' value='' aria-required='true' class='ibm-text' aria-describedby='descaaddr' type='text' >").get(0);
                $( inputAddress ).attr("value", scopeElement.ipAddress);
                fpAddresses.setTitle(i18n.labelDiscoScopesAddressHeading);
                fpAddresses.setDescription(i18n.labelDiscoVmwareScopesAddressDesc);
                fpAddresses.appendFieldRow( { label    : i18n.labelDiscoScopesAddressAddress,
                                              required : true,
                                              element  : inputAddress,
                                              hint     : i18n.labelSpecifyIpAddressFieldDescription } );
            }
        } , this ) );


        /*
         * Create "cancel" button
         */
        var cancelButton = ButtonFactory.buildButton( { type    : 'cancel',
                                                        grayout : true,
                                                        label   : i18n.buttonCancel } );

        $(cancelButton).on('click', function () {
            ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                 clearSystemMessages : false,
                                 parameters          : { pid : sepid } } ) ).execute();
        } );

        /*
         * Add content to the panel.
         */
        $( buttonContainer ).append(saveButton);
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator );
        buttonSeparator = ButtonFactory.buildButtonSeparator ;

        $(buttonSeparator).hide();

        $( buttonContainer ).append(buttonSeparator);
        $( buttonContainer ).append(cancelButton);

        $(fpAddresses.form).attr("aria-label", i18n.labelDiscoScopesAddressHeading);
        contentArea.append( generalForm.getDomNode() );
        contentArea.append( fpAddresses.getDomNode() );
        contentArea.append( buttonContainer );
        this.show( contentArea.getDomNode() );
    };



    tsa.cmd.handlers.ScopesvmwareHandler.prototype.ipRule = function( value ) {
        var regEx     = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
        var matchedIp = "";
        var valid     = true;
        if ( value !== "" ) {
            matchedIp = value.match(regEx);
            if (matchedIp !== null) {
                var split = matchedIp[0].split('.');
                for (var i = split.length - 1; i >= 0; i--) {
                    var s = split[i];
                    if (s.length === 0 || isNaN(s) || s < 0 || s > 255) {
                        valid = false;
                    }
                }

            } else {
                valid = false;
            }
        }
        return ( valid ? null : "Invalid IP" );
    };

    return tsa.cmd.handlers.ScopesvmwareHandler;

}

);
