/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ContentArea
          , TablePanel
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'SensorsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.SensorsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.SensorsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SensorsHandler.prototype ).constructor = tsa.cmd.handlers.SensorsHandler;

    //-------------------------------------------------------------------------
//    /**
//     * Show main screen.
//     */
    tsa.cmd.handlers.SensorsHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                      = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea               = new ContentArea ( { title : i18n.labelSensorsHeading
                                                                                                 , helpDoc :'sshelp_sensors_panel.html'
                                                                                                 , showBackToTop : true} );
      /** @type {Element}                   */ var downloadSensorsButton     = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer           = ButtonFactory.buildButtonContainer();

      contentArea.appendMessage( { text : i18n.labelSensorsCaption } );

      /** @type {tsa.cmd.Command} */ var getSensorsListCommand = new Command ( { action             : 'sensors/getSensorsList'
                                                                               , clearSystemMessages: false   });
      getSensorsListCommand.execute();
      $( getSensorsListCommand ).on( 'executionSucceeded', function( e ){

        var columnHeaders = new Array();
        columnHeaders.push({id:"name" , label:i18n.labelSensorsName , width: '40%'});
        $.each(e.response.payload.profiles, function(index, profile){
            columnHeaders.push({id:profile.name , label:profile.name , width: '12%'});
        });

        /** @type {tsa.widgets.TablePanel} */ var tablePanelSensorsProfiles  = new TablePanel( { title    : i18n.labelSensorsProfilesHeading
                                                                                               , columns  : columnHeaders
                                                                                               , emptyMsg : i18n.labelSensorsNoSensors
                                                                                               });

        contentArea.append(tablePanelSensorsProfiles.getDomNode());

        $.each(e.response.payload.sensors, function(index, sensor){
          var sensorArray = new Array();
          sensorArray.push(sensor.name);
          $.each(sensor.enabled, function(index, val){
            val? sensorArray.push("X"):sensorArray.push("");
          });
          tablePanelSensorsProfiles.appendRow(sensorArray);
        });

        // Appending button to content area...
        downloadSensorsButton = ButtonFactory.buildButton( { type : 'download' , grayout : false , label : i18n.buttonSensorsDownload ,
                                 onclick : function( event ){window.location = '/appliance/sensors/download/sensors.csv';} })

        $( buttonContainer ).append( downloadSensorsButton );
        contentArea.append( buttonContainer );

      });

      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.SensorsHandler;
  }
);