/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/Command'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , ShowCommand
          , Command
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'SetupwizardHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.SetupwizardHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.SetupwizardHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SetupwizardHandler.prototype ).constructor = tsa.cmd.handlers.SetupwizardHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SetupwizardHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n         = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea  = new ContentArea ( { title   : i18n.labelSetupwizardHeading
                                                                                    , helpDoc : 'sshelp_setupwizard_panel.html' } );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer        = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdCancel          = new ShowCommand( { action     : 'main/index' } );
      /** @type {tsa.cmd.ShowCommand}       */ var showCmdStart           = new ShowCommand( { action     : 'connectivity/index' , clearSystemMessages : false , 
    	                                                                                           parameters : { "setupWizardEnabled" : true } } );
      $("#ibm-navigation").show();
      $("#WizardList").remove();
      
      
      //Check if the wizard process is active with the setupWizardEnabled parameter = true.
      if (this.getParameter("setupWizardEnabled") == "true") {
    	      /** hide the left menu **/
    	      //hide left menu
          $("#ibm-primary-links").hide();
          $("#ibm-primary-links").css("opacity", "0.0");
      }
      
      contentArea.appendMessage( { text : i18n.labelSetupwizardCaption } );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonlabelStartSetupwizard
          , onclick : function( event )
            {
        	      //hide the cancel button for wait screen
              $("#ajax-wait-btn-container").hide();
              //hide left menu
        	      $("#ibm-primary-links").hide();
              $("#ibm-primary-links").css("opacity", "0.0");
        	      showCmdStart.execute();
            }
          }
        )
      );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      
      $( buttonContainer ).append(
        ButtonFactory.buildButton(
          { type    : 'cancel'
          , grayout : true
          , label   : i18n.buttonlabelExitSetupwizardPage
          , onclick : function( event )
            {
        	     $("#ajax-wait-btn-container").show();
        	     $("#ajax-wait-msg"          ).text(i18n.labelPleaseWait);
        	     $("#ibm-primary-links").show();
              showCmdCancel.execute();
            }
          }
        )
      );
      
      contentArea.append( buttonContainer );
      
      this.show( contentArea.getDomNode() );
    };
    
    /**
     * Show completed details Setup Wizard
     */
    tsa.cmd.handlers.SetupwizardHandler.prototype._view = function()
    {

        /** @type {Object}                    */           var i18n                   = tsa.i18n;
        /** @type {tsa.widgets.ContentArea}   */           var contentArea            = new ContentArea ( { title   : i18n.labelSetupwizardCompletedHeading
                                                                                                          , helpDoc : 'sshelp_setupwizardcompleted_panel.html' } );
        /** @type {tsa.widgets.ButtonFactory} */           var buttonContainer        = ButtonFactory.buildButtonContainer();
        /** @type {tsa.cmd.ShowCommand}       */           var showCmdCancel          = new ShowCommand( { action     : 'main/index' } );
        
        /** @type {Object}                    */           var varSetupWizardEnabledReboot = this.getParameter("setupWizardEnabled");
        
        /** hide the left menu **/
        $("#ibm-navigation").show();
        $("#WizardList").remove();
        //hide left menu
        $("#ibm-primary-links").hide();
        $("#ibm-primary-links").css("opacity", "0.0");
        
        //Hyper Link to IBM TSA site
        var redirectLink = document.createElement("a");
        redirectLink.href = "https://www.ibm.com/services/us/resources/it-asset-discovery-demo/#!/install-tsa";
        redirectLink.target = '_blank';
        redirectLink.innerHTML = i18n.labelSetupwizardCompletedCaptionLink;
        
        var text = document.createTextNode(i18n.labelSetupwizardCompletedCaptionInf);
        
        var redirectContent = document.createElement("p");
        redirectContent.appendChild(text);
        redirectContent.appendChild(redirectLink);
        
        contentArea.appendMessage( { text : i18n.labelSetupwizardCompletedCaption } );
        contentArea.append(redirectContent)
        
       
        
        
        $( buttonContainer ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : true
            , label   : i18n.buttonlabelContinueSetupwizardPage
            , onclick : $.proxy( function( event )
              {
                this.exitWizard();
                
                if (!TV.typeIsUndefined( typeof varSetupWizardEnabledReboot ) && null != varSetupWizardEnabledReboot && varSetupWizardEnabledReboot == 'false') {

                    if (confirm('System restart is required to update your clock settings. Restart system now?')) {
                        tsa.cmd.handlers.SetupwizardHandler.rebootSystem('auto');
                    } else {
                        tsa.cmd.handlers.SetupwizardHandler.rebootSystem('user');
                    }

                } else  {
                    showCmdCancel.execute();
                }
              } , this )
            }
          )
        );
        
        contentArea.append( buttonContainer );
        
        this.show( contentArea.getDomNode() );
      
     };
    
    //-------------------------------------------------------------------------
    /**
     * Instance getter.
     * @return {tsa.cmd.handlers.SetupwizardHandler}
     */
    tsa.cmd.handlers.SetupwizardHandler.getInstance = function()
    {
      return tsa.cmd.handlers.SetupwizardHandler.instance;
    };
    
    tsa.cmd.handlers.SetupwizardHandler.prototype.generateList = function(productList, numberLine) {
        
        var ul = document.createElement('ul');
        ul.setAttribute('id', 'WizardList'  );
        
        productList.forEach(renderProductList);
        function renderProductList(element, index, arr) {
          var li = document.createElement('li');
          if (index < numberLine) {
              li.setAttribute('style', 'padding: 0px 5px 0px; height:25px; width: 100%; display: flex; justify-content: left; text-align:left; align-items: center; color:gray');
              var groupIcon = document.createElement('img');
              groupIcon.alt = "";
              groupIcon.src = tsa.attributes.imagesPath + '/' + 'ok16.gif';
              groupIcon.setAttribute('style', 'margin: 0px 3px');
              $(li).append(groupIcon);
          } else if (index == numberLine) {
              li.setAttribute('style', 'padding: 0px 5px 0px; height:25px; width: 100%; display: flex; justify-content: left; text-align:left; align-items: center; background-color: White;');
              var groupIcon = document.createElement('img');
              groupIcon.alt = "";
              groupIcon.src = tsa.attributes.imagesPath + '/' + 'active16.gif';
              groupIcon.setAttribute('style', 'margin: 0px 3px');
              $(li).append(groupIcon);
          } else {
              li.setAttribute('style', 'padding: 0px 5px 0px; height:25px; width: 100%; display: flex; justify-content: left; text-align:left; align-items: center; color:gray');
              var groupIcon = document.createElement('img');
              groupIcon.alt = "";
              groupIcon.src = tsa.attributes.imagesPath + '/' + 'offline16.gif';
              groupIcon.setAttribute('style', 'margin: 0px 3px');
              $(li).append(groupIcon);
          }
          ul.appendChild(li);
          li.innerHTML = li.innerHTML + element;
        }
        $("#ibm-navigation").append(ul);
        
    };
    
    /** hide the left menu * */
    tsa.cmd.handlers.SetupwizardHandler.prototype.hideLeftMenu = function() {
        $("#ibm-primary-links").hide();
        $("#WizardList"       ).remove();
        $("#flash"            ).css("visibility", "hidden");
        $("#flash"            ).css("height", "0px");
    };
    
    /** hide the alert panel * */
    tsa.cmd.handlers.SetupwizardHandler.prototype.hideAlertPanel = function() {
        $("#flash").css("visibility", "hidden");
        $("#flash").css("height"    , "0px");
    };
    
    /** hide the exit * */
    tsa.cmd.handlers.SetupwizardHandler.prototype.exitWizard = function() {
        $("#ibm-primary-links"      ).show();
        $("#ibm-primary-links"      ).css("opacity", "1");
        $("#WizardList"             ).remove();
        $("#flash"                  ).css("display", "block");
        $("#flash"                  ).css("visibility", "visible");
        $("#flash"                  ).css("height", "auto");
        $("#flash"                  ).show();
        $("#ajax-wait-btn-container").show();
        $("#ajax-wait-msg"          ).text(tsa.i18n.labelPleaseWait);
    };
    
    /** show the message checking updates ....*/
    tsa.cmd.handlers.SetupwizardHandler.prototype.checkingUpdates = function() {
        $("#ajax-wait-msg"          ).text(tsa.i18n.labelCheckingForUpdates);
    }
    
    /** remove wizard list and left menu * */
    tsa.cmd.handlers.SetupwizardHandler.prototype.removeWizardLetfMenu = function() {
        $("#WizardList").remove();
        /** hide left menu* */
        $("#ibm-primary-links").hide();
    };
    
    /** remove wizard list and show left menu * */
    tsa.cmd.handlers.SetupwizardHandler.prototype.removeWizardShowLetfMenu = function() {
        $("#WizardList").remove();
        $("#ibm-primary-links").show();
    };
    
    
    /**
     * Request for restarting system.
     * @param {string} restart Restart mode. Use either:
     *                         'auto' : Restart will be initiated by the system.
     *                         'user' : Restart should be initiated by the user.
     */
    tsa.cmd.handlers.SetupwizardHandler.rebootSystem = function( restart )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;
       
       cmd = new Command( { action              : 'clock/ajaxReboot'
                          , clearSystemMessages : false
                          , parameters          : { "reboot" : restart } 
                          } );
       cmd.execute();
       
       $(cmd).on( 'executionSucceeded', function( event ){
              $("#WizardList").remove();
              $("#ibm-primary-links").show();
                          ( new ShowCommand( { action : 'main/index' , clearSystemMessages : true  } ) ).execute();
                  });
    };
    
    return tsa.cmd.handlers.SetupwizardHandler;
  }
);