/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'ShutdownHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.ShutdownHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.ShutdownHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ShutdownHandler.prototype ).constructor = tsa.cmd.handlers.ShutdownHandler;

    //-------------------------------------------------------------------------

    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ShutdownHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n              = tsa.i18n;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ContentArea}   */ var contentArea       = null;
      /** @type {tsa.widgets.FormPanel}     */ var formPanelSuspend  = null;
      /** @type {tsa.widgets.FormPanel}     */ var formPanelResume   = null;
      /** @type {tsa.widgets.FormPanel}     */ var formPanelRestart  = null;
      /** @type {tsa.widgets.FormPanel}     */ var formPanelPoweroff = null;
      /** @type {Element}                   */ var suspendButton     = null;
      /** @type {Element}                   */ var resumeButton      = null;
      /** @type {Element}                   */ var restartButton     = null;
      /** @type {Element}                   */ var poweroffButton    = null;



       // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelShutdownHeading
                                      , helpDoc : 'sshelp_shutdown_panel.html' } );
      contentArea.appendMessage( { text : i18n.labelShutdownCaption } );

      //SUSPEND FORM
      formPanelSuspend = new FormPanel( { title  : i18n.labelShutdownSuspendHeading } );
      formPanelSuspend.setDescription( i18n.labelShutdownSuspendDesc );
      suspendButton = ButtonFactory.buildButton ({
            type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonShutdownSuspend
          , onclick : function( event ){
            var suspendCmd = new Command({ action : 'shutdown/ajaxSuspend', clearSystemMessages : false,  blockUI : false });
            if ( confirm(i18n.labelShutdownSuspendConfirm) ) {
              tsa.clearSysMessages();
              suspendCmd.execute();
              }
            }
      });

      var suspendDiv = document.createElement( 'div' );
      $( suspendDiv ).append( suspendButton );
      formPanelSuspend.appendJointRow( suspendDiv );

      //RESUME FORM
      formPanelResume = new FormPanel( { title  : i18n.labelShutdownResumeHeading });
      formPanelResume.setDescription( i18n.labelShutdownResumeDesc );
      resumeButton = ButtonFactory.buildButton ({
        type    : 'arrow'
      , grayout : false
      , label   : i18n.buttonShutdownResume
      , onclick : function( event ){
          var resumeCmd = new Command({ action : 'shutdown/ajaxResume', clearSystemMessages : false,  blockUI : false });
          if ( confirm(i18n.labelShutdownResumeConfirm) ) {
             tsa.clearSysMessages();
             resumeCmd.execute();
             }
          }
      });

      var resumeDiv = document.createElement( 'div' );
      $( resumeDiv ).append( resumeButton );
      formPanelResume.appendJointRow( resumeButton );


      //RESTART FORM
      formPanelRestart = new FormPanel( { title  : i18n.labelShutdownRestartHeading } );
      formPanelRestart.setDescription( i18n.labelShutdownRestartDesc );
      restartButton = ButtonFactory.buildButton ({
          type    : 'arrow'
        , grayout : false
        , label   : i18n.buttonShutdownRestart
        , onclick : function( event ){
          var restartCmd = new Command({ action : 'shutdown/ajaxRestart', clearSystemMessages : false,  blockUI : false });
          if ( confirm(i18n.labelShutdownRestartConfirm) ) {
            restartCmd.execute();
            }

          $( restartCmd ).on( 'executionSucceeded', function( event )
              {
              // a job or update is running, verify restart
               if(event.response.payload.displayConfirmation)
               {
                 if ( confirm(event.response.payload.message) ) {
                 tsa.cmd.handlers.ShutdownHandler.restartPoweroff('Restart');
                 }
               }
               ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
              });
          }
      });

      var restartDiv = document.createElement( 'div' );
      $( restartDiv ).append( restartButton );
      formPanelRestart.appendJointRow( restartButton );


      //POWEROFF FORM
      formPanelPoweroff = new FormPanel( { title  : i18n.labelShutdownPwrOffHeading } );
      formPanelPoweroff.setDescription( i18n.labelShutdownPwrOffDesc );
      poweroffButton = ButtonFactory.buildButton ({
          type    : 'arrow'
        , grayout : false
        , label   : i18n.buttonShutdownPwrOff
        , onclick : function( event ){
                  var poweroffCmd = new Command({ action : 'shutdown/ajaxPoweroff', clearSystemMessages : false,  blockUI : false });
                  if ( confirm(i18n.labelShutdownPwrOffConfirm) ) {
                    poweroffCmd.execute();
                    }
                  $( poweroffCmd ).on( 'executionSucceeded', function( event ) {
                    //a job or update is running, verify poweroff
                     if(event.response.payload.displayConfirmation)
                     {
                       if ( confirm(event.response.payload.message) ) {
                      	 tsa.cmd.handlers.ShutdownHandler.restartPoweroff('Poweroff');
                       }
                     }
                     ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
                });
        }
      });

      var poweroffDiv = document.createElement( 'div' );
      $( poweroffDiv ).append( poweroffButton );
      formPanelPoweroff.appendJointRow( poweroffButton );

      //Link forms
      formPanelSuspend.linkTo( formPanelResume );
      formPanelResume.linkTo ( formPanelRestart );
      formPanelRestart.linkTo( formPanelPoweroff );

      //Append forms
      contentArea.append( formPanelSuspend.getDomNode() );
      this.show( contentArea.getDomNode() );

    };

    //-------------------------------------------------------------------------


    /**
     * Request to restart or power off system.
     * @param {string} type action to be done. Use either:
     *                         'Poweroff' : Power off the system.
     *                         'Restart'  : Restart the system.
     */
    tsa.cmd.handlers.ShutdownHandler.restartPoweroff = function( type )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;

       cmd = new Command( { action              : 'shutdown/ajax'+type
                          , clearSystemMessages : false
                          , parameters          : { "confirmed" : type}
                          } );
       cmd.execute();
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.ShutdownHandler;
  }
);