/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand' 
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , ButtonFactory
          , FormPanel
          , Command
          , ShowCommand
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'SupportHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.SupportHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.SupportHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SupportHandler.prototype ).constructor = tsa.cmd.handlers.SupportHandler;

    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SupportHandler.prototype._index = function()
    {
      /** @type {Object}                  	*/ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea = new ContentArea ( { title : i18n.labelSupportHeading
    	  																		 , helpDoc  : 'sshelp_logging_and_trace_panel.html' } );
      /** @type {tsa.cmd.Command}           */ var getTraceCmd 		  = new Command( { action : 'support/getTraceInfo', clearSystemMessages : false } );
  	  /** @type {Element}                   */ var appErrorRadio      = document.createElement( 'input' );
      /** @type {Element}                   */ var appWarnRadio		  = document.createElement( 'input' );
      /** @type {Element}                   */ var appInfoRadio       = document.createElement( 'input' );
      /** @type {Element}                   */ var appDebugRadio      = document.createElement( 'input' );
      /** @type {Element}                   */ var appTraceRadio      = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrFatalRadio  = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrErrorRadio  = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrWarnRadio	  = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrInfoRadio   = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrDebugRadio  = document.createElement( 'input' );
      /** @type {Element}                   */ var discMgrTraceRadio  = document.createElement( 'input' );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer 	  = ButtonFactory.buildButtonContainer();
  	  /** @type {tsa.widgets.FormPanel}   	*/ var formPanelAppliance = null;
  	  /** @type {tsa.widgets.FormPanel}   	*/ var formPanelDiscMgr   = null;

      contentArea.appendMessage( { text : i18n.labelSupportCaption } );

      //APPLIANCE TRACE LEVEL
      formPanelAppliance = new FormPanel( { title  : i18n.labelSupportLevelHeading
    	  								  , action : 'support/ajaxUpdateTraceInfo' } );
      formPanelAppliance.setDescription( i18n.labelSupportLevelDesc );

      //Radio buttons
      appErrorRadio.type   = 'radio';
      appErrorRadio.name   = 'level';
      appErrorRadio.value  = 'error';

      appWarnRadio.type    = 'radio';
      appWarnRadio.name    = 'level';
      appWarnRadio.value   = 'warn';

      appInfoRadio.type    = 'radio';
      appInfoRadio.name    = 'level';
      appInfoRadio.value   = 'info';

      appDebugRadio.type   = 'radio';
      appDebugRadio.name   = 'level';
      appDebugRadio.value  = 'debug';

      appTraceRadio.type   = 'radio';
      appTraceRadio.name   = 'level';
      appTraceRadio.value  = 'trace';

     /** @type {Element} */ var appSpan       = document.createElement( 'span' );
     /** @type {Element} */ var appErrorLabel = document.createElement( 'label' );
     /** @type {Element} */ var appWarnLabel  = document.createElement( 'label' );
     /** @type {Element} */ var appInfoLabel  = document.createElement( 'label' );
     /** @type {Element} */ var appDebugLabel = document.createElement( 'label' );
     /** @type {Element} */ var appTraceLabel = document.createElement( 'label' );

     $( appErrorLabel ).append( appErrorRadio , document.createTextNode( i18n.labelSupportLevelError ) );
     $( appWarnLabel  ).append( appWarnRadio  , document.createTextNode( i18n.labelSupportLevelWarn  ) );
     $( appInfoLabel  ).append( appInfoRadio  , document.createTextNode( i18n.labelSupportLevelInfo  ) );
     $( appDebugLabel ).append( appDebugRadio , document.createTextNode( i18n.labelSupportLevelDebug ) );
     $( appTraceLabel ).append( appTraceRadio , document.createTextNode( i18n.labelSupportLevelTrace ) );
     $( appSpan    	  ).append( appErrorLabel , document.createElement( 'br' ) , appWarnLabel , document.createElement( 'br' ) ,
    		 					appInfoLabel  , document.createElement( 'br' ) , appDebugLabel, document.createElement( 'br' ) ,
    		 					appTraceLabel );
     $( appSpan ).addClass( 'ibm-input-group' );
     $( appSpan ).attr( 'role', 'radiogroup'  );
     $( appSpan ).attr( 'aria-label', 'level' );

     formPanelAppliance.appendJointRow( appSpan );


     //DISCOVERY MANAGER TRACE LEVEL
     formPanelDiscMgr = new FormPanel( { title  : i18n.labelDiscMgrSupportLevelHeading } );
     formPanelDiscMgr.setDescription( i18n.labelDiscMgrSupportLevelDesc );

     //All modules checkbox
     var allDiv = document.createElement( 'div' );
     var allCheckbox   = document.createElement( 'input' );
 	 allCheckbox.type  = 'checkbox';
 	 allCheckbox.name  = 'selectAll';
 	 allCheckbox.value = 'ALL_SELECTED';
 	 $( allCheckbox  ).on( 'change', function( event ) { tsa.changesCanBeLost( true ); } );

 	 /** type{element}**/ var labelCheckInput = $("<label style='font-weight:normal'></label>").get(0);
 	 $( labelCheckInput ).append( allCheckbox );
 	 $( labelCheckInput ).append( i18n.labelSelectAllProperties );
 	 $( allDiv ).append( labelCheckInput );
 	 
 	 formPanelDiscMgr.appendJointRow( allDiv );

 	 //Radio buttons
     discMgrFatalRadio.type   = 'radio';
     discMgrFatalRadio.name   = 'discMgrLevel';
     discMgrFatalRadio.value  = 'FATAL';

     discMgrErrorRadio.type   = 'radio';
     discMgrErrorRadio.name   = 'discMgrLevel';
     discMgrErrorRadio.value  = 'ERROR';

     discMgrWarnRadio.type    = 'radio';
     discMgrWarnRadio.name    = 'discMgrLevel';
     discMgrWarnRadio.value   = 'WARN';

     discMgrInfoRadio.type    = 'radio';
     discMgrInfoRadio.name    = 'discMgrLevel';
     discMgrInfoRadio.value   = 'INFO';

     discMgrDebugRadio.type   = 'radio';
     discMgrDebugRadio.name   = 'discMgrLevel';
     discMgrDebugRadio.value  = 'DEBUG';

     discMgrTraceRadio.type   = 'radio';
     discMgrTraceRadio.name   = 'discMgrLevel';
     discMgrTraceRadio.value  = 'TRACE';

    /** @type {Element} */ var discMgrSpan       = document.createElement( 'span'  );
    /** @type {Element} */ var discMgrFatalLabel = document.createElement( 'label' );
    /** @type {Element} */ var discMgrErrorLabel = document.createElement( 'label' );
    /** @type {Element} */ var discMgrWarnLabel  = document.createElement( 'label' );
    /** @type {Element} */ var discMgrInfoLabel  = document.createElement( 'label' );
    /** @type {Element} */ var discMgrDebugLabel = document.createElement( 'label' );
    /** @type {Element} */ var discMgrTraceLabel = document.createElement( 'label' );

    $( discMgrFatalLabel ).append( discMgrFatalRadio , document.createTextNode( i18n.labelSupportLevelFatal ) );
    $( discMgrErrorLabel ).append( discMgrErrorRadio , document.createTextNode( i18n.labelSupportLevelError ) );
    $( discMgrWarnLabel  ).append( discMgrWarnRadio  , document.createTextNode( i18n.labelSupportLevelWarn  ) );
    $( discMgrInfoLabel  ).append( discMgrInfoRadio  , document.createTextNode( i18n.labelSupportLevelInfo  ) );
    $( discMgrDebugLabel ).append( discMgrDebugRadio , document.createTextNode( i18n.labelSupportLevelDebug ) );
    $( discMgrTraceLabel ).append( discMgrTraceRadio , document.createTextNode( i18n.labelSupportLevelTrace ) );
    $( discMgrSpan  	 ).append( discMgrFatalLabel , document.createElement( 'br' ) , discMgrErrorLabel, document.createElement( 'br' ),
    							   discMgrWarnLabel  , document.createElement( 'br' ) , discMgrInfoLabel , document.createElement( 'br' ),
    							   discMgrDebugLabel , document.createElement( 'br' ) , discMgrTraceLabel );
    $( discMgrSpan       ).addClass( 'ibm-input-group' );
    $( discMgrSpan       ).attr( 'role', 'radiogroup'  );
    $( discMgrSpan		 ).attr( 'aria-label', 'discMgrLevel' );

    formPanelDiscMgr.appendJointRow( discMgrSpan );

    //link forms
    formPanelAppliance.linkTo( formPanelDiscMgr );

    $( getTraceCmd ).on( 'executionSucceeded', function( event ) {
    	var lvl = event.response.payload.level;

    	if(lvl === 'error'){
    		 appErrorRadio.checked = true;
    	}else if(lvl === 'warn'){
    		 appWarnRadio.checked = true;
    	}else if(lvl === 'info'){
    		 appInfoRadio.checked = true;
    	}else if(lvl === 'debug'){
    		 appDebugRadio.checked = true;
    	}else {
    		 appTraceRadio.checked = true;
    	}

    	var discmgrlvl = event.response.payload.discMgrLevel;

    	if(discmgrlvl === 'FATAL'){
    		discMgrFatalRadio.checked = true;
    	}else if(discmgrlvl === 'ERROR'){
    		discMgrErrorRadio.checked = true;
    	}else if(discmgrlvl === 'WARN'){
    		discMgrWarnRadio.checked = true;
    	}else if(discmgrlvl === 'INFO'){
    		discMgrInfoRadio.checked = true;
    	}else if(discmgrlvl === 'DEBUG'){
    		discMgrDebugRadio.checked = true;
    	}else {
    		discMgrTraceRadio.checked = true;
    	}
   });

	//Save button
    var saveButton = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonSave
		, onclick : function( event ){
			formPanelAppliance.submit();
		}
	});

	$( formPanelAppliance ).on( 'executionSucceeded', function( event ) {
		( new ShowCommand( { action : 'main/index' , clearSystemMessages : false })).execute();
	});

  	//Cancel button
	var cancelButton = ButtonFactory.buildButton({ type : 'cancel', grayout : false , label : i18n.buttonCancel
		, onclick :  function( event ) {
			( new ShowCommand( { action : 'main/index' })).execute();
		  }
	});

	$( buttonContainer ).append( saveButton );
	$( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
	$( buttonContainer ).append( cancelButton );

	//Append forms and buttons
	contentArea.append( formPanelAppliance.getDomNode() );
	contentArea.append( buttonContainer );

	getTraceCmd.execute();

    this.show( contentArea.getDomNode() );

    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.SupportHandler;
  }
);