/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/ButtonFactory'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , Command
          , ContentArea
          , FormPanel
          , ButtonFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'SysfilesHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.SysfilesHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.SysfilesHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SysfilesHandler.prototype ).constructor = tsa.cmd.handlers.SysfilesHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SysfilesHandler.prototype._index = function()
    {
      /** @type {Object}                 	*/ var i18n        		= tsa.i18n;
      /** @type {tsa.widgets.ContentArea} 	*/ var contentArea 		= null;
      /** @type {tsa.widgets.TablePanel}    */ var tabsPanel		= null;
      /** @type {Element}                   */ var saveButton		= null;
      /** @type {Element}                   */ var undoButton		= null;
      /** @type {Element}                   */ var textArea			= null;
      /** @type {Element}                   */ var fileSelector  	= null; 
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer	= ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.FormPanel}     */ var textFormPanel	= new FormPanel( { action : 'sysfiles/ajaxUpdate', border : false} );
      /** @type {tsa.cmd.Command}           */ var getFilesCmd		= new Command( { action  : 'sysfiles/getSysFiles' } );
      
      // Initialize the  area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelSystemFilesHeading
                                      , helpDoc : 'sshelp_sysfiles_panel.html' } );
      
      contentArea.appendMessage( { text : i18n.labelSystemFilesCaption } );
      
      var cautionText = tsa.NBSP + "<span style='font-weight: bold'>" + i18n.labelCaution + ":</span><br/>" + i18n.labelSystemFilesDisclaimer;
      contentArea.appendMessage( { level : 'WARN'  , border : true , text : cautionText , textIsHtml : true } );
      
      var infoText = tsa.NBSP + "<span style='font-weight: bold'>" + i18n.labelNote + ": </span>" + i18n.labelSystemFilesRestartMessage;
      contentArea.appendMessage( { level : 'INFO'  , border : true , text : infoText , textIsHtml : true } );
      
      contentArea.appendMessage( {text : i18n.labelSystemFilesInstructions } );
      
      var div1 = document.createElement( 'div' );
      $( div1 ).addClass( 'ibm-container' );
    
	  var div2 = document.createElement( 'div' );
	  $( div2 ).addClass( 'ibm-container-body' );
	  
	  var div3 = document.createElement( 'div' );
	  $( div3 ).addClass( 'ibm-alternate-rule' );
	  
	  var hr = document.createElement( 'hr' );
	  $( div3 ).append( hr );
	  
	  var firstSelection = true;
	  
	  //get system files
      $( getFilesCmd ).on( 'executionSucceeded', function( event ){
    	  var textMap 		= {};
    	  var sysFilesList	= event.response.payload.sysFiles;
    	  var path			= null;
    	  
    	  var div1 =  document.createElement( 'div' );
    	  $( div1 ).addClass( 'ibm-container' );
    	  
    	  var div2 =  document.createElement( 'div' );
    	  $( div2 ).addClass( 'ibm-container-body' );
		  
    	  var divHr =  document.createElement( 'div' );
    	  $( divHr ).addClass( 'ibm-alternate-rule' );
    	  
    	  var hr = document.createElement( 'hr' );
    	  $( divHr ).append( hr );
    	  $( div2 ).append( divHr );   	      	      	  
    	  
    	  
    	  var p = document.createElement( 'p' );
    	  fileSelector  = document.createElement( 'select' );
    	  $( fileSelector ).attr( { 'id'  : 'fileSelector' } );
    	  $( fileSelector ).css( { 'font-size'  : 'small' } );
    	  fileSelector.name = 'path';
    	     	  
    	  var option = $('<option value="-">'+i18n.labelSystemFilesSelect+'</option>').get(0);
    	  $(fileSelector).append( option );
    	  
    	  $.each( sysFilesList, function( fileIndex, file )
    	  {
    		 path = file.path;
    		 textMap[ path ] = file.text; //set map of file text
    		 option = $('<option value="'+path+'">'+path+'</option>').get(0);
    		 $(fileSelector).append( option );  
    	  });
    	  
    	  $( fileSelector ).on( 'change', function( event ){
    		  $( textArea ).val( textMap[ $(fileSelector).val() ] );
    		  if(firstSelection){
    			  $("#fileSelector option[value='-']").remove();
    			  firstSelection = false;
    		  }
    	  } );
	  
    	  $( p ).append( fileSelector );
    	  $( div2 ).append( p );
    	  
    	  var divHr2 =  document.createElement( 'div' );
    	  $( divHr2 ).addClass( 'ibm-rule' );
    	  
    	  var hr2 = document.createElement( 'hr' );
    	  $( divHr2 ).append( hr2 );
    	  $( div2 ).append( divHr2 ); 
    	  
    	  var div4 =  document.createElement( 'div' );
    	  $( div4 ).css( { 'position' : 'relative' } );
    	  
  	   	  //FILE CONTENT AREA
    	  textArea = document.createElement( 'textarea' );
    	  $( textArea ).val ( "" );
    	  $( textArea ).attr( 'id'  , 'text' );
    	  $( textArea ).attr( 'name', 'text' );
    	  $( textArea ).css( { 'height'		  : '28em' 
    		  				 , 'width' 		  : '100%' 
    		  				 , 'border-width' : '0px' 
    		  				 , 'font-family'  : 'monospace'
    		  				 , 'white-space'  : 'pre'
    		  				 , 'word-wrap'    : 'normal'
    		  				 , 'overflow'     : 'scroll' 
    		  				 , 'font-size'    : 'medium' } );
    	 
    	  $( div4 ).append( textArea );
    	  $( div2 ).append( div4 );
    	  $( div1 ).append( div2 );
    	  
    	  textFormPanel.appendJointRow( div1 );
    	  
    	  //BUTTONS
          saveButton  = ButtonFactory.buildButton( { type : 'arrow' , label : i18n.buttonSave } );
          undoButton  = ButtonFactory.buildButton( { type : 'undo'  , label : i18n.buttonUndo , grayout : true} );
          
          $( saveButton ).on( 'click', function( event ){
        	  if( $(fileSelector).val() == "-" || $(textArea).val() == "" || $(textArea).val() == null){
        		  alert(i18n.labelSystemFilesLoadBeforeSaving);
        		  
        	  } else{
        		  textFormPanel.submit();
        		  textMap [ $(fileSelector).val() ] =  $(textArea).val();
        	  }
           } );
          
          $( undoButton ).on( 'click', function( event ){
        	  $( textArea ).val( textMap[ $(fileSelector).val() ] );
           } );
                    
          $( buttonContainer ).append( saveButton );
          $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
          $( buttonContainer ).append( undoButton );
          textFormPanel.appendJointRow( buttonContainer );
          
          contentArea.append( textFormPanel.getDomNode() );
          
      });
	  
      //Get file info
      getFilesCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.SysfilesHandler;
  }
);