/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------



define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/FormPanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , IconFactory
          , TablePanel
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'TimeoutsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     * 
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.TimeoutsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.TimeoutsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.TimeoutsHandler.prototype ).constructor = tsa.cmd.handlers.TimeoutsHandler;

    //-------------------------------------------------------------------------
    
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.TimeoutsHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                      = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea               = new ContentArea ( { title         : i18n.labelTimeoutsHeading
                                                                                                 , helpDoc       : 'sshelp_sensor_timeouts_panel.html' 
                                                                                                 , showBackToTop : true} );
      /** @type {Element}                   */ var resetDefaultsSensorsButton = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer            = ButtonFactory.buildButtonContainer();

      /** @type {tsa.cmd.Command} */ var getTimeoutsListCommand = new Command ( { action            : 'timeouts/getTimeoutsList'
                                                                               , clearSystemMessages: false   });
      contentArea.appendMessage( { text : i18n.labelTimeoutsCaption } );
      
      // Run function that fills in the table entries
      getTimeoutsListCommand.execute();
      $( getTimeoutsListCommand ).on( 'executionSucceeded', function( e ){

        var columnHeaders = new Array();
        columnHeaders.push({id:"name"           , label:i18n.labelTimeoutsNameColumnHeader           , width: '70%'});
        columnHeaders.push({id:"defaultTimeout" , label:i18n.labelTimeoutsDefaultTimeoutColumnHeader , width: '10%'});
        columnHeaders.push({id:"currentTimeout" , label:i18n.labelTimeoutsCurrentTimeoutColumnHeader , width: '10%'});
        columnHeaders.push({id:"actions"        , label:i18n.labelActionsColumn                      , width: '10%'});

        /** @type {tsa.widgets.TablePanel} */ var tablePanelTimeouts  = new TablePanel( { title    : i18n.labelTimeoutsHeading
                                                                                        , columns  : columnHeaders
                                                                                        , emptyMsg : i18n.labelTimeoutsNoSensors
                                                                                        });
        
        contentArea.append(tablePanelTimeouts.getDomNode());
        
        $.each(e.response.payload.timeouts, function(index, timeout){
            /** @type {Array.<Element>} */ var elements     = [];
            /** @type {Element}         */ var actionsSpan  = document.createElement( 'span' );
            /** @type {Element}         */ var editIcon = IconFactory.buildActionIcon( { type : 'edit' , caption : i18n.labelButtonEdit + ' "' + timeout.name + '"' } );

            // When edit icon is pushed, values currently displayed are sent as parameters to _edit function.  
            // Given how infrequently this will be done, not worth re-querying the time-out values which typically
            // should not change.
            $( editIcon ).on( 'click', function( event ) { tsa.cmd.handlers.TimeoutsHandler.editTimeoutClicked( timeout.name
                                                                                                              , timeout.defaultTimeout
                                                                                                              , timeout.currentTimeout); } );           
            elements.push( timeout.name );
            elements.push( timeout.defaultTimeout );
            elements.push( timeout.currentTimeout );
            
            $( actionsSpan ).append( editIcon );
            elements.push( actionsSpan );
            
            // Alignment of values in row:
            //    Name    - left-justified (default)
            //    Default - right-justified
            //    Current - right-justified
            //    Actions - centered
            var row = tablePanelTimeouts.appendRow( elements );
            $(row).find('td').eq(1).each (function() {
                $(this).css({ 'text-align' : 'right'})
            }); 
            $(row).find('td').eq(2).each (function() {
                $(this).css({ 'text-align' : 'right'})
            }); 
            $(row).find('td').eq(3).each (function() {
                $(this).css({ 'text-align' : 'center'})
            }); 
        });

        // Appending reset all sensors to default values button to content area...        
        resetTimeoutsButton =  ButtonFactory.buildButton( { type    : 'arrow' ,
                                                            grayout : false ,
                                                            label   : i18n.buttonTimeoutsReset , 
                                                            onclick : function( event )
                                                            {
                                                               if( window.confirm( tsa.i18n.labelResetToDefaultsConfirm ) )
                                                               {
                                                                  logger.info( 'Sensor time-outs will be reset to defaults...' );
                                                                  tsa.cmd.handlers.TimeoutsHandler.resetDefaults();
                                                               }
                                                             }
                                                           }
                                                         );
        $( buttonContainer ).append( resetTimeoutsButton );
        contentArea.append( buttonContainer );
        
      });

      this.show( contentArea.getDomNode() );
    };
  
  
  //-------------------------------------------------------------------------
  
  /**
   * Edit sensor time-out - event handler.  This function is called when the edit icon is
   * clicked next to a specific sensor.
   * @param {string} name           The name of the sensor.
   * @param {string} defaultTimeout The default time-out for this sensor.
   * @param {string} currentTimeout The current time-out for this sensor.
   */
  tsa.cmd.handlers.TimeoutsHandler.editTimeoutClicked = function( name, defaultTimeout, currentTimeout )
  {
    ( new ShowCommand( { action : 'timeouts/edit'
                       , parameters : { name : name 
                                      , defaultTimeout : defaultTimeout 
                                      , currentTimeout : currentTimeout }} ) ).execute();
  };
  
  //-------------------------------------------------------------------------
  
  /**
   * Show the sensor time-out edit screen.
   */
  tsa.cmd.handlers.TimeoutsHandler.prototype._edit = function( )
  {
    /** @type {Object}                  */ var i18n            = tsa.i18n;
    
    // Retrieves the parameters passed to this function
    /** @type {String}                  */ var name            = this.getParameter( 'name' );
    /** @type {String}                  */ var defaultTimeout  = this.getParameter( 'defaultTimeout' );
    /** @type {String}                  */ var currentTimeout  = this.getParameter( 'currentTimeout' );
    
    /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelTimeoutsEditHeading
                                                                                  , helpDoc       : 'sshelp_edit_sensor_timeout_panel.html' } );
    /** @type {tsa.widgets.FormPanel}   */ var timeoutForm     = this.createTimeoutChangeForm( name, defaultTimeout, currentTimeout );  
    /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();


    // Buttons; Save & Cancel...
    $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                            , label   : i18n.buttonSave
                                                            , onclick : function( event ) 
                                                            { timeoutForm.setActionAttribute("timeouts/putTimeout");
                                                              timeoutForm.submit(); } } ) );  
    $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
    $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                            , label   : i18n.buttonCancel
                                                            , onclick : function( event ) 
                                                            { ( new ShowCommand( { action : 'timeouts/index' } ) ).execute(); } } ) );
  
    // Appending content...
    contentArea.append( timeoutForm.getDomNode() );
    contentArea.append( buttonContainer );
    
    // Content area must be shown...
    this.show( contentArea.getDomNode() );
    
    
    // Action when save is successful
    $( timeoutForm ).on( 'executionSucceeded', function( event )
    {
        tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelTimeoutsUpdated, name ) } );
        
        ( new ShowCommand( { action              : 'timeouts/index'
                           , clearSystemMessages : false } ) ).execute();  
    } );

    // Action when save fails due to bad input.
    $( timeoutForm ).on( 'executionFailed', function( event )
    {
       if ( TV.typeIsObject( typeof event.response.payload.errors ) ) {
           for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                timeoutForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
           }
           delete event.response.payload.errors;
       }
    } );
  };

  //-------------------------------------------------------------------------

  /**
   * Build the creation/edition form.
   * @param {Object=} credential The credential data to initialize the form. Optional.
   * @return {tsa.widgets.FormPanel} The resulting form panel.
   * @private
   */
  tsa.cmd.handlers.TimeoutsHandler.prototype.createTimeoutChangeForm = function( )
  {
    /** @type {Object}                */ var i18n = tsa.i18n;
  
    // Retrieves the parameters passed to this function
    /** @type {String}                  */ var name            = this.getParameter( 'name' );
    /** @type {String}                  */ var defaultTimeout  = this.getParameter( 'defaultTimeout' );
    /** @type {String}                  */ var currentTimeout  = this.getParameter( 'currentTimeout' );

    /** @type {tsa.widgets.FormPanel} */ var timeoutInfoFormPanel   = null;
    /** @type {tsa.widgets.FormPanel} */ var timeoutChangeFormPanel = null;
      
    /** @type {Element} */ var nameSpan           = document.createElement( 'span' );
    /** @type {Element} */ var defaultTimeoutSpan = document.createElement( 'span' );
    
    /** @type {Element} */ var currentTimeoutField = $( '<input type="text" id="currentTimeout" name="currentTimeout" size="20" maxlength="20" />' ).get(0);

    // Initializing form panels...
    // There are two panels.  The first contains the values that cannot be changed (sensor name and default
    // time-out.  The second contains the current time-out which can be modified.
    timeoutInfoFormPanel = new FormPanel( { title       : i18n.labelTimeoutsEditInfoTableHeading } );
    
    timeoutChangeFormPanel = new FormPanel( { title       : i18n.labelTimeoutsEditChangeTableHeading 
                                            , description : i18n.labelTimeoutsEditChangeTableDescription } );
   
    $( nameSpan )          .css( { 'font-weight' : 'bold' } );
    $( nameSpan )          .text( name           );
    $( defaultTimeoutSpan ).text( defaultTimeout );

    timeoutInfoFormPanel.appendFieldRow( { label: i18n.labelTimeoutsName,           element: nameSpan } );
    timeoutInfoFormPanel.appendFieldRow( { label: i18n.labelTimeoutsDefaultTimeout, element: defaultTimeoutSpan } );
    
    timeoutChangeFormPanel.appendFieldRow( { label: i18n.labelTimeoutsEditChangeTableTimeout , required: true , element: currentTimeoutField } );
    /** @type {Element} */ var currentTimeoutFieldInitialized = ( currentTimeoutField );
    $( currentTimeoutFieldInitialized ).val( currentTimeout );
    
    timeoutInfoFormPanel.putHiddenInput( 'name'           , name );
    timeoutInfoFormPanel.putHiddenInput( 'defaultTimeout' , defaultTimeout )
    
    // Multiple form panels, but submitted as a single form...
    timeoutInfoFormPanel.linkTo( timeoutChangeFormPanel );
    
    // Return the form...
    return timeoutInfoFormPanel;
  };
  
  //-------------------------------------------------------------------------
  
  /**
   * Reset all sensor time-outs - event handler.  This function is called when the 
   * "Reset to defaults" button is clicked.
   */
  tsa.cmd.handlers.TimeoutsHandler.resetDefaults = function( )
  {
    /** @type {tsa.cmd.Command} */ var cmd = null;
    
    cmd = new Command( { action : 'timeouts/ajaxResetDefaults' } );
    cmd.execute();
    $( cmd ).on( 'executionSucceeded', function( event )
        {
          (new ShowCommand( {  
              action : 'timeouts/index'
            , clearSystemMessages : false } )  
          ).execute();
        } );
  };

  //-------------------------------------------------------------------------

  return tsa.cmd.handlers.TimeoutsHandler;
}

);