/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/handlers/SetupwizardHandler'
  ],

  function($
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel
          , SetupwizardHandler )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'TransmissionHandler' );
    var archiveFile = null;

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.TransmissionHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.TransmissionHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.TransmissionHandler.prototype ).constructor = tsa.cmd.handlers.TransmissionHandler;

    //-------------------------------------------------------------------------


    /**
     * Show main screen.
     */
    tsa.cmd.handlers.TransmissionHandler.prototype._index = function()
    {

    /** @type {Object}                    */ var i18n                     = tsa.i18n;
    /** @type {tsa.widgets.ContentArea}   */ var contentArea              = null;
    /** @type {tsa.widgets.TablePanel}    */ var scheduleTablePanel       = null;
    /** @type {tsa.widgets.TablePanel}    */ var historyTablePanel        = null;
    /** @type {Element}                   */ var editScheduleButton       = null;
    /** @type {Element}                   */ var runTransmissionButton    = null;
    /** @type {boolean}                   */ var downloadLast             = false;
    /** @type {boolean}                   */ var isTsaRegistered          = false;
    /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer          = ButtonFactory.buildButtonContainer();
    /** @type {tsa.cmd.Command}           */ var getJobInfoCmd            = new Command( { action              : 'transmission/getJobInfo'
                                                                                         , clearSystemMessages : false } );

    // Initialize the content area (its title and its help document)...
    contentArea = new ContentArea ( { title   : i18n.labelCollectionSchedHeading
                                    , helpDoc : 'sshelp_collection_schedule_panel.html' } );

    // Appending messages to the content area...
    contentArea.appendMessage( { text : i18n.labelCollectionSchedOverview } );

    // Building the schedule information table...
    scheduleTablePanel = new TablePanel( { title    : i18n.labelCollectionSchedScheduleHeading
                                         , emptyMsg : i18n.labelCollectionSchedStatusDisabled
                                         , columns  : [ { id : 'propertyName'  , width : '30%' }
                                                      , { id : 'propertyValue' , width : '70%' } ] } );

    // Building the history table...
    historyTablePanel = new TablePanel( { title    : i18n.labelCollectionSchedHistoryHeading
                                        , emptyMsg : i18n.labelCollectionSchedNoJobHistory
                                        , columns  : [ { id: 'status'   , label: i18n.labelCollectionSchedStatus   , width: '15%' }
                                                     , { id: 'name' , label: i18n.labelCollectionSchedInstance , width: '30%' }
                                                     , { id: 'state'    , label: i18n.labelCollectionSchedState    , width: '15%' }
                                                     , { id: 'comments' , label: i18n.labelCollectionSchedComments , width: '45%' } ] } );

    // Preparing the retrieval of the transmission job information (how to handle the response from server)...
    $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
    {
      /** @type {Object}  */ var jobInfo  = event.response.payload.jobInfo;
      /** @type {Object}  */ var schedule = jobInfo.schedule;
      /** @type {Object}  */ var history  = jobInfo.jobHistory;
      
      archiveFile     = event.response.payload.lastArchiveFile;
      downloadLast    = event.response.payload.downloadLast;
      isTsaRegistered = event.response.payload.isTsaRegistered;

      // Populating schedule table (if available; if not, the empty-message will be shown)...
      if( schedule !== null )
      {
        scheduleTablePanel.appendRow( [ i18n.labelCollectionSchedNextRun  , jobInfo.nextPerformanceTS  ] );
        scheduleTablePanel.appendRow( [ i18n.labelCollectionSchedSchedule , schedule.formatted         ] );
      }

      // Populating history table (if available; if not, the empty-message will be shown)...
      if( history !== null )
      {
        $.each( history, function( entryIndex, entry )
        {
          /** @type {Element} */ var commentList = $( '<ul style="list-style-type: disc; margin:0; padding-left: 20px;"></ul>' ).get(0);

          $.each( entry.comments, function( commentIndex, comment )
          {
            /** @type {Element} */ var commentElement = $( '<li style="padding-top: 0px;"></li>' ).get(0);

            $( commentElement ).text( comment );
            $( commentList ).append( commentElement );
          } );
          var statusCode = entry.statusCode;
          var stateText  = entry.stateText;
          if (entry.statusMessage == "Unknown") 
          {
            statusCode = -1;
            stateText = "Unknown";
          }
          if (entry.jobTypeName != "DataSnapshot")
          {
            historyTablePanel.appendRow( [ IconFactory.buildSeverityIcon( { severity : statusCode } )
                                         , entry.name
                                         , stateText
                                         , commentList ] );
          }
        } );
      }


    // Buttons...
    editScheduleButton       = ButtonFactory.buildButton( { type : 'arrow' , grayout : false , label : i18n.buttonCollectionSchedEditSchedule } );
    runTransmissionButton    = ButtonFactory.buildButton( { type : 'arrow' , grayout : true  , label : i18n.buttonCollectionSchedRunNow       } );

    $( editScheduleButton ).on( 'click', function( event )
    {
      ( new ShowCommand( { action : 'transmission/edit/com.ibm.servicestation.FullCollectionJob' } ) ).execute();
    } );

    $( runTransmissionButton ).on( 'click', function( event )
    {
      if ( isTsaRegistered )
      {
        ( new Command( { action     : 'transmission/ajaxRunTransmission'
                       , parameters : { 'tx' : 'false' } } ) ).execute();
      }
      else
      {
        if ( window.confirm(i18n.labelCollectionSchedIsTsaRegistered) ) 
	    {
	      ( new ShowCommand( { action              : 'registration/index'
	                         , clearSystemMessages : false } ) ).execute();
	    }
      }
    } );

    $( buttonContainer ).append( editScheduleButton );
    $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
    $( buttonContainer ).append( runTransmissionButton );
  });

    // Appending content...
    contentArea.append( scheduleTablePanel.getDomNode() );
    contentArea.append( historyTablePanel.getDomNode() );
    contentArea.append( buttonContainer );

    // Performing the job information retrieval...
    getJobInfoCmd.execute();

    // Content area must be shown...
    this.show( contentArea.getDomNode() );

    };


    //-------------------------------------------------------------------------
    /**
     * Show edit schedule screen.
     */
    tsa.cmd.handlers.TransmissionHandler.prototype._edit = function()
    {
      /** @type {Object}                    */ var i18n                = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea         = null;
      /** @type {tsa.widgets.FormPanel}     */ var enablementFormPanel = null;
      /** @type {tsa.widgets.FormPanel}     */ var scheduleFormPanel   = null;
      /** @type {Element}                   */ var enablementSelector  = document.createElement( 'select' );
      /** @type {Element}                   */ var hourSelector        = document.createElement( 'select' );
      /** @type {Element}                   */ var minuteSelector      = document.createElement( 'select' );
      /** @type {Element}                   */ var weekModeRadio       = document.createElement( 'input' );
      /** @type {Element}                   */ var monthModeRadio      = document.createElement( 'input' );
      /** @type {Array.<Element>}           */ var weekDayCheckboxes   = [];
      /** @type {Element}                   */ var weekDaysRow         = null;
      /** @type {Array.<Element>}           */ var monthDayCheckboxes  = [];
      /** @type {Element}                   */ var monthDaysRow        = null;
      /** @type {Element}                   */ var saveButton          = null;
      /** @type {Element}                   */ var cancelButton        = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer     = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}           */ var getJobInfoCmd       = new Command( { action              : 'transmission/getJobInfo'
                                                                                      , clearSystemMessages : false } ); 
      /** @type {Element}                   */ var varSetupWizardEnabled          = this.getParameter("setupWizardEnabled");
      $( enablementSelector ).attr("id", "enabled"   );
      $( hourSelector       ).attr("id", "hour"      ); 
      $( minuteSelector     ).attr("id", "minute"    );
      $( weekModeRadio      ).attr("id", "weekMode"  );
      $( monthModeRadio     ).attr("id", "monthMode" );
      // Initializing content area...
      if ( varSetupWizardEnabled || (varSetupWizardEnabled == false || varSetupWizardEnabled == 'false') ) {
          contentArea = new ContentArea( { title   : i18n.labelCollectionSchedHeading
              , helpDoc : 'sshelp_wizard_collection_schedule_panel.html' } );
      } else {
          contentArea = new ContentArea( { title   : i18n.labelCollectionSchedHeading
              , helpDoc : 'sshelp_collection_schedule_panel.html' } );
      }

      // Initializing form panels...
      enablementFormPanel = new FormPanel( { title       : i18n.labelCollectionSchedEnableHeading
                                           , description : i18n.labelCollectionSchedEnableCaption
                                           , action      : 'transmission/putJobSchedule' } );

      scheduleFormPanel = new FormPanel( { title       : i18n.labelCollectionSchedScheduleHeading
                                         , description : i18n.labelCollectionSchedScheduleCaption } );

      $( enablementFormPanel ).on( 'executionSucceeded', function( event )
       {
                    tsa.printSysMessage({
                        level   : 'info',
                        message : i18n.labelCollectionSchedUpdated
                    });
                    /**
                     * wizard process
                     */
                    if ((event.response.payload.setupWizardEnabled == 'true')
                             && (event.response.payload.setupWizardEnabledReboot == 'true')) {
                        /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                        /** hide the left menu * */
                        setupwizardHandler.hideLeftMenu();
                        setupwizardHandler.removeWizardLetfMenu();
                        setupwizardHandler.checkingUpdates();
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard('update/check', "true" );
                    } else if (event.response.payload.setupWizardEnabled == 'true') {
                        /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                        /** hide the left menu * */
                        setupwizardHandler.hideLeftMenu();
                        setupwizardHandler.removeWizardLetfMenu();
                        setupwizardHandler.checkingUpdates();
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard('update/check', "false" );
                    } else {
                        (new ShowCommand({
                            action              : 'transmission/edit/com.ibm.servicestation.FullCollectionJob',
                            clearSystemMessages : false
                        })).execute();
                    }
                } );

      // Multiple form panels, but submitted as a single form...
      enablementFormPanel.linkTo( scheduleFormPanel );

      // Enablement selector...
      enablementSelector.name = 'enabled';

      $.each( [ { value: 'true'  , label: i18n.labelCollectionSchedEnabled  }
              , { value: 'false' , label: i18n.labelCollectionSchedDisabled } ]
            , function( optionDataIndex, optionData )
              {
                /** @type {Element} */ var option = document.createElement( 'option' );

                $( option ).attr( 'value', optionData.value );
                $( option ).text( optionData.label );
                $( enablementSelector ).append( option );
              } );

      enablementFormPanel.appendFieldRow( { label    : i18n.labelCollectionSchedEnable
                                          , required : true
                                          , element  : enablementSelector } );

      /** @type {function(Event)} */
      var enablementChanged = function( event )
      {
        /** @type {boolean} */ var disabled = ( $( enablementSelector ).val() === 'false' );

        hourSelector   .disabled = disabled;
        minuteSelector .disabled = disabled;
        weekModeRadio  .disabled = disabled;
        monthModeRadio .disabled = disabled;

        $.each( weekDayCheckboxes.concat( monthDayCheckboxes ), function( checkboxIndex, checkbox )
        {
          checkbox.disabled = disabled;
        } );
      };

      $( enablementSelector ).on( 'change', enablementChanged );

      // Hour selector...
      hourSelector.name = 'hour';

      for( /** @type {number} */ var hour=0; hour<24; hour++)
      {
        $( hourSelector ).append( $( '<option value="' + hour + '">' + ( ( hour < 10 ) ? '0' : '' ) + hour + '</option>' ) );
      }

      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtHour
                                        , required : true
                                        , element  : hourSelector } );

      // Minute selector...
      minuteSelector.name = 'minute';

      for( /** @type {number} */ var minute=0; minute<60; minute+=5)
      {
        $( minuteSelector ).append( $( '<option value="' + minute + '">' + ( ( minute < 10 ) ? '0' : '' ) + minute + '</option>' ) );
      }

      scheduleFormPanel.appendFieldRow( { label    : i18n.scheduleAtMinute
                                        , required : true
                                        , element  : minuteSelector } );

      // Day mode radio buttons...
      weekModeRadio.type    = 'radio';
      weekModeRadio.name    = 'daymode';
      weekModeRadio.value   = 'week';
      weekModeRadio.checked = true;
      monthModeRadio.type   = 'radio';
      monthModeRadio.name   = 'daymode';
      monthModeRadio.value  = 'month';

      scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleDayMode
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span       = document.createElement( 'span' );
                         /** @type {Element} */ var weekLabel  = document.createElement( 'label' );
                         /** @type {Element} */ var monthLabel = document.createElement( 'label' );

                         $( weekLabel  ).append( weekModeRadio  , document.createTextNode( i18n.scheduleDayModeWeek  ) );
                         $( monthLabel ).append( monthModeRadio , document.createTextNode( i18n.scheduleDayModeMonth ) );
                         $( span       ).append( weekLabel, document.createElement( 'br' ), monthLabel );
                         $( span       ).addClass( 'ibm-input-group' );
                         $( span       ).attr( 'role', 'radiogroup' );
                         $( span       ).attr( 'aria-label', i18n.scheduleDayMode );

                         return span;
                       } )()
      } );

      // Days of week checkboxes...
      weekDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnDays
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var span = document.createElement( 'span' );

                         $( span ).addClass( 'ibm-input-group' );
                         $( span ).attr( 'role', 'group' );
                         $( span ).attr( 'aria-label', i18n.scheduleOnDays );

                         $.each( [ 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ], function( wdIndex, wd )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="weekday" value="' + ( wdIndex + 1 ) + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label>' + i18n[ 'schedule' + wd.charAt(0).toUpperCase() + wd.substring(1) ] + '</label>' ).get(0);

                           $( label ).prepend( checkbox );
                           weekDayCheckboxes.push( checkbox );
                           $( span ).append( label, document.createElement( 'br' ) );
                         } );

                         return span;
                       } )()
      } );

      // Days of month checkboxes...
      monthDaysRow = scheduleFormPanel.appendFieldRow( {
          label    : i18n.scheduleOnMonthDays
        , hint     : i18n.scheduleOnMonthDaysDesc
        , required : true
        , aRequire : false
        , element  : ( function()
                       {
                         /** @type {Element} */ var div = document.createElement( 'div' );

                         $( div ).addClass( 'ibm-input-group' );
                         $( div ).attr( 'role', 'group' );
                         $( div ).attr( 'aria-label', i18n.scheduleOnDays );

                         for( /** @type {number} */ var md=1; md<=31; md++ )
                         {
                           /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="monthday" value="' + md + '" />' ).get(0);
                           /** @type {Element} */ var label    = $( '<label style="display: inline-block; font-weight: normal; margin-right: 0.8em">'
                                                                  + ( ( md < 10 ) ? '0' : '' ) + md
                                                                  + '</label>' ).get(0);
                           $( label ).prepend( checkbox );
                           monthDayCheckboxes.push( checkbox );
                           $( div ).append( label );

                           if( ( md % 7 ) === 0 )
                           {
                             $( div ).append( document.createElement( 'br' ) );
                           }
                         };

                         return div;
                       } )()
      } );

      // Event handler: day mode changed...
      /** @type {function(Event)} */
      var dayModeChanged = function( event )
      {
        logger.info( 'Day mode changed: processing started...' );

        $( weekDaysRow  ).css( { 'display' : ( weekModeRadio  .checked ? '' : 'none' ) } );
        $( monthDaysRow ).css( { 'display' : ( monthModeRadio .checked ? '' : 'none' ) } );

        // Selecting a mode will enable all of the checkboxes of the other mode...
        if ( weekModeRadio  .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', true ); } );
        if ( monthModeRadio .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', true ); } );

        // If the user triggered the event (event is an object, and it is not null), clear the check-boxes of the mode he selected...
        if( TV.typeIsObject( typeof event ) && ( event !== null ) )
        {
          if( weekModeRadio  .checked ) $.each( weekDayCheckboxes  , function( index, element ) { $( element ).prop( 'checked', false ); } );
          if( monthModeRadio .checked ) $.each( monthDayCheckboxes , function( index, element ) { $( element ).prop( 'checked', false ); } );
        }

        logger.info( 'Day mode changed: processing finished.' );
      };

      $( weekModeRadio  ).add( monthModeRadio ).on( 'change', dayModeChanged );

      // Preparing the retrieval of the transmission job information (how to handle the response from server)...
      $( getJobInfoCmd ).on( 'executionSucceeded', function( event )
      {
        /** @type {Object} */ var jobInfo   = event.response.payload.jobInfo;
        /** @type {Object} */ var schedule  = jobInfo.schedule;
        /** @type {Object} */ var isEnabled = ( schedule !== null );

        // Populating schedule table (if available; if not, the empty-message will be shown)...
        if( isEnabled )
        {
          $( enablementSelector ).val( 'true' );
          $( hourSelector       ).val( schedule.hour   );
          $( minuteSelector     ).val( schedule.minute );

          weekModeRadio  .checked = ( schedule.dayOfWeek.length !== 7);
          monthModeRadio .checked = ( schedule.dayOfWeek.length === 7);

          $.each( weekDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfWeek.indexOf( checkboxIndex + 1 ) >= 0 );
          } );

          $.each( monthDayCheckboxes, function( checkboxIndex, checkbox )
          {
            checkbox.checked = ( schedule.dayOfMonth.indexOf( parseInt( $( checkbox ).val(), 10 ) ) >= 0 );
          } );
        }
        else
        {
          $( enablementSelector ).val( 'false' );
        }

        enablementChanged( null );
        dayModeChanged( null );
      } );

        /**
          * Check the setupWizardEnabled and setupWizardEnabled = false parameters to see if any is true and display the screen in wizard mode
          */
        if (varSetupWizardEnabled || (varSetupWizardEnabled == false || varSetupWizardEnabled == 'false')) {
            /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
            // hide alert panel
            setupwizardHandler.hideAlertPanel();
            setupwizardHandler.hideLeftMenu();

            // create the menu wizard
            productList = [ i18n.labelnameConnectivity, i18n.labelnameRegistration, i18n.labelnameClock, i18n.labelnameTransmition, i18n.labelnameIU ];
            setupwizardHandler.generateList( productList, 3 );

            // variable for indicate the wizard process
            enablementFormPanel.appendFieldRow({
                label    : '',
                required : false,
                element  : $('<input type="hidden" size="50" name="setupWizardEnabled" id="setupWizardEnabled" value="true" />').get(0)
            });

            if ((varSetupWizardEnabled == false || varSetupWizardEnabled == 'false') ) {
                // variable for indicate the wizard process need reboot
                enablementFormPanel.appendFieldRow({
                    label    : '',
                    required : false,
                    element  : $('<input type="hidden" size="50" name="setupWizardEnabledReboot" id="setupWizardEnabledReboot" value="true" />').get(0)
                });
            }

            /**
             * show the back, next and skip button
             */
            $(buttonContainer).append(ButtonFactory.buildButton({
                type    : 'back',
                grayout : true,
                label   : i18n.buttonlabelBackSetupwizardPage,
                onclick : function(event) {
                    setupwizardHandler.removeWizardLetfMenu();
                    if ((varSetupWizardEnabled == false || varSetupWizardEnabled == 'false')) {
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard('clock/index', "true" );
                    } else {
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard('clock/index', "false" );
                    }
                }
            }));
            $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
            $(buttonContainer).append(ButtonFactory.buildButton({
                type    : 'arrow',
                grayout : false,
                label   : i18n.buttonlabelSaveContinueSetupwizardPage,
                onclick : function(event) {
                    enablementFormPanel.submit();
                }
            }));
            $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
            $(buttonContainer).append(ButtonFactory.buildButton({
                type : 'arrow',
                grayout : true,
                label : i18n.buttonlabelSkipSetupwizardPage,
                onclick : function(event) {
                    /** hide left menu* */
                    setupwizardHandler.removeWizardLetfMenu();
                    setupwizardHandler.checkingUpdates();
                    if ((varSetupWizardEnabled == false || varSetupWizardEnabled == 'false')) {
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard ('update/check', "true" );
                    } else {
                        tsa.cmd.handlers.TransmissionHandler.redirectWizard ('update/check', "false" );
                    }

                }
            }));
            $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
            $(buttonContainer).append(ButtonFactory.buildButton({
                type : 'cancel',
                grayout : true,
                label : i18n.buttonlabelExitSetupwizardPage,
                onclick : function(event) {
                    // exit wizard
                    setupwizardHandler.exitWizard();
                    (new ShowCommand({
                        action : 'main/index'
                    })).execute();
                }
            }));
        } else { 
	      // Buttons...
	      saveButton   = ButtonFactory.buildButton( { type : 'arrow'  , label : i18n.buttonSave   } );
	      cancelButton = ButtonFactory.buildButton( { type : 'cancel' , label : i18n.buttonCancel } );
	
	      $( saveButton ).on( 'click', function( event )
	      {
	        enablementFormPanel.submit();
	      } );
	
	      $( cancelButton ).on( 'click', function( event )
	      {
	        ( new ShowCommand( { action : 'transmission/index' } ) ).execute();
	      } );
	
	      $( buttonContainer ).append( saveButton );
	      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
	      $( buttonContainer ).append( cancelButton );
      }
      // Appending content...
      contentArea.append( enablementFormPanel.getDomNode() );
      contentArea.append( scheduleFormPanel.getDomNode() );
      contentArea.append( buttonContainer );

      // Performing the job information retrieval...
      getJobInfoCmd.execute();

      // Content area must be shown...
      this.show( contentArea.getDomNode() );
    };
    
    /**
     * Show Commands for wizard
     * @param {string} action Action for the ShowCommand.
     * @param {string} reboot Flag to reboot when wizard finished
     * @param {string} parameters Parameters for the Show Command
     */
    tsa.cmd.handlers.TransmissionHandler.redirectWizard = function( action, reboot  )
    {                
                      var parameters = {};
                      if (reboot == "true") {
                        parameters.setupWizardEnabled = false;  
                       } else {
                        parameters.setupWizardEnabled = true;
                       }
                       (new ShowCommand({
                           action              : action,
                           clearSystemMessages : true,
                           parameters          : parameters
                       })).execute();
    };

    return tsa.cmd.handlers.TransmissionHandler;
  }


);
