/*
 * IBM Technical Support Appliance Version 2 Release 3 Modification 0
 *
 * (C) Copyright IBM Corp. 2018
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/widgets/TablePanel'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , TablePanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'UnknowndevicesHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.UnknowndevicesHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.UnknowndevicesHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.UnknowndevicesHandler.prototype ).constructor = tsa.cmd.handlers.UnknowndevicesHandler;
    
  //sidebar to show info of elements
    tsa.cmd.handlers.UnknowndevicesHandler.prototype.infoParagraph   = null;
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.UnknowndevicesHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea = new ContentArea ( { title : i18n.labelUnknownDevicesHeading 
                                                                               , helpDoc : 'sshelp_unknown_devices_panel.html'} );
      /** @type {tsa.widgets.TablePanel}  */ var tablePanel  = null;
      
      contentArea.appendMessage( { text : i18n.labelUnknownDevicesCaption } );
      
      /** @type {element} */ var disclaimerText = tsa.NBSP + "<strong>" + i18n.labelNote + ":</strong><p>" + i18n.labelUnknownDevicesDisclaimer+"</p>";
      contentArea.appendMessage( { level : 'INFO'  , text : disclaimerText , textIsHtml : true } );
      
      this.infoParagraph = document.createElement("div");
      
      /** @type {element}                  */    var paragraph              = document.createElement("p");
      
      $(paragraph).text(i18n.labelUnknownDevicesNoObjectSelected);
      
      $(this.infoParagraph).append(paragraph);
      
      contentArea.appendSidebarPanel( { title: i18n.labelUnknownDevicesObjectInfo, body: this.infoParagraph } );
      
      /** @type {function()}*/
      var entryHandler = function( entry )
      {
          
        /** @type {Array.<(string|Element)>} */ var result   = null;
        /** @type {Element}                  */ var aName    = document.createElement("a");
        /** @type {Element}                  */ var sName    = document.createElement("strong");
        
        $(aName).text(entry.displayName);
        $(aName).append(sName);
        $(aName).attr("href","#")
        
        result = [aName,entry.address,entry.lastModifiedTS];
        
        /** @type {function()}*/
        var displayInfoTrigger = $.proxy(function(){
            
            /** @type {tsa.cmd.Command}*/ var getInfoCmd = new Command( { action:'unknowndevices/getInfo/' + entry.pid
                                                                                     , clearSystemMessages : false } );
            getInfoCmd.execute();
            
            $( getInfoCmd ).on( 'executionSucceeded', $.proxy(function(e) {
                
                /** @type {Object} */  var object = e.response.payload.object;
                
                $(this.infoParagraph).empty();
               
                /** @type {function()}*/
                var templateInfo = function(name, value){
                    /** @type {Element}*/ var pTemplate  = document.createElement("p");
                    /** @type {Element}*/ var sTemplate  = document.createElement("strong");
                    /** @type {Element}*/ var brTemplate = $("<br class='break'>").get(0);
                    $(sTemplate).text(name+":");
                    $(pTemplate).append(sTemplate, brTemplate, value);
                    return pTemplate;
                }
                if(tsa.isDebugModeOn()){
                    $(this.infoParagraph).append(templateInfo(i18n.labelDiscoRunsPID,object.pid));
                }
                $.each( object.properties, $.proxy(function( idx, property )
                 {
                    $(this.infoParagraph).append(templateInfo(property.name,property.value));
                    
                    },this));
                
        
                
            },this) );
       
       },this);
        
       $(aName).on('click',displayInfoTrigger);
       
       return result;
      };
      
      tablePanel   = new TablePanel( { title        : i18n.labelAuthenticationStatusObjectsHeading
          , summary      : i18n.labelSummaryStatusDesc
          , emptyMsg     : i18n.labelSummaryNoStatus
          , ajaxAction   : 'unknowndevices/getList'
          , entryHandler : $.proxy(entryHandler,this)
          , columns      : [ { id : 'name'      , label : i18n.labelUnknownDevicesName         , width : '50%' , sortable : true }
                           , { id : 'address'   , label : i18n.labelUnknownDevicesAddress      , width : '25%' , sortable : true }
                           , { id : 'modified'  , label : i18n.labelUnknownDevicesLastModified , width : '25%' , sortable : true }] } );
       
      contentArea.append( tablePanel.getDomNode() );
      this.show( contentArea.getDomNode() );
    };
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.UnknowndevicesHandler;
  }
);
 