/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/FormPanel'
  , 'tsa/cmd/handlers/SetupwizardHandler'
  ],
  
  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , Command
          , ShowCommand
          , IconFactory
          , TablePanel
          , ButtonFactory
          , FormPanel
          , SetupwizardHandler
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'UpdateHandler' );
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.UpdateHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.UpdateHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.UpdateHandler.prototype ).constructor = tsa.cmd.handlers.UpdateHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.UpdateHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        	         = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea 		= null;
      /** @type {tsa.widgets.FormPanel}   */ var checkUpdateFormPanel 	= null;
      /** @type {tsa.widgets.FormPanel}   */ var docFormPanel  		= null;
      /** @type {Element}                 */ var checkUpdateButton    	= null;
      /** @type {Element}                 */ var docButton    		= null;
      
      // Initialize the content area (its title and its help document)...
      contentArea = new ContentArea ( { title   : i18n.labelUpdateHeading
          , helpDoc : 'sshelp_update_panel.html' } );
      
      // Append description the content area...
      contentArea.appendMessage( { text : i18n.labelUpdateCaption } );
      
      
      //UPDATE FORM
      checkUpdateFormPanel = new FormPanel( { title  : i18n.labelUpdateCheck4Heading } );
      checkUpdateFormPanel.setDescription( i18n.labelUpdateCheck4Desc );
      checkUpdateButton = ButtonFactory.buildButton ({
            type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonUpdateCheck4Update
          , onclick : function( event ){
        	  ( new ShowCommand( { action : 'update/check' } ) ).execute();
          }
      });

      var updateDiv = document.createElement( 'div' );
      $( updateDiv ).append( checkUpdateButton );
      checkUpdateFormPanel.appendJointRow( updateDiv );
      
      //DOCUMENTATION FORM
      docFormPanel = new FormPanel( { title  : i18n.labelUpdateGetDocHeading } );
      docFormPanel.setDescription( i18n.labelUpdateGetDocDesc );
      docButton = ButtonFactory.buildButton ({
            type    : 'arrow'
          , grayout : false
          , label   : i18n.buttonUpdateGetDoc
          , onclick : function( event ){
        	  ( new ShowCommand( { action : 'documentation/index' } ) ).execute();
            }
      });

      var docDiv = document.createElement( 'div' );
      $( docDiv ).append( docButton );
      docFormPanel.appendJointRow( docDiv );
      
      
      //Link forms
      checkUpdateFormPanel.linkTo( docFormPanel );
      
      //Append forms
      contentArea.append( checkUpdateFormPanel.getDomNode() );
      
      this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------
    /**
     * Show update schedule screen.
     */
    tsa.cmd.handlers.UpdateHandler.prototype._check = function()
    {
      /** @type {Object}                    */ var i18n             = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea      = null;
      /** @type {tsa.widgets.FormPanel}     */ var updateFormPanel  = null;
      /** @type {tsa.widgets.TablePanel}    */ var updateTablePanel = null; 
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer  = ButtonFactory.buildButtonContainer();
      /** @type {string}                    */ var fromLicense      = this.getParameter ( 'fromlicense' );
      /** @type {tsa.cmd.Command}           */ var checkUpdateCmd   = new Command( { action              : 'update/getUpdate'
                                                                                   , clearSystemMessages : false } );
      /** @type {Element}                   */ var setupWizardEnabledHelpPage       = this.getParameter("setupWizardEnabled");
    
      // Initialize the content area (its title and its help document)...
         if ( setupWizardEnabledHelpPage || (setupWizardEnabledHelpPage == false || setupWizardEnabledHelpPage == 'false') ) {
             contentArea = new ContentArea ( { title   : i18n.labelUpdateHeading
                 , helpDoc : 'sshelp_wizard_update_panel.html' } );
         } else {
             contentArea = new ContentArea ( { title   : i18n.labelUpdateCheckHeading
                 , helpDoc : 'sshelp_update_panel.html' } );
         }
      // Append description the content area...
      contentArea.appendMessage( { text : i18n.labelUpdateCheckDesc } );
      
      //check update status
      $( checkUpdateCmd ).on( 'executionSucceeded', function( event ){
    	  var updateInfo  = event.response.payload.updates;
          
    	  
    	  if(updateInfo == null){ //no update exists
    		  updateFormPanel = new FormPanel( { title : i18n.labelUpdateCheckAvailHeading } );
    		  
    		  var src  = tsa.attributes.imagesPath + '/' + tsa.i18n[ 'imageStatusInfo16' ];
    		  var icon = document.createElement( 'img' );
     		  var div  = document.createElement( 'div' );
    		  var span = document.createElement( 'span' );
    		  
              $( icon ).attr( 'alt', src );
              $( icon ).attr( 'src', src  );
              $( icon ).css( { 'vertical-align' : 'middle' } );
              $( span ).css( { 'white-space' : 'nowrap', 'font-size' : 'small', 'font-style' : 'italic' } );
              $( span ).append( icon );
     		  $( span ).append( document.createTextNode( tsa.NBSP + i18n.labelUpdateCheckNoUpdatesAvailable ) );
     		  $( div  ).append( span );
    		  
    		  updateFormPanel.appendJointRow( div );
    		  contentArea.append( updateFormPanel.getDomNode() );
    		  
    		  //wizard process
    		  var varSetupWizardEnabledReboot = event.response.payload.setupWizardEnabledReboot;
                         if (( !TV.typeIsUndefined( typeof event.response.payload.setupWizardEnabled ) && event.response.payload.setupWizardEnabled == 'true')
                                || ( !TV.typeIsUndefined( typeof varSetupWizardEnabledReboot ) && varSetupWizardEnabledReboot == 'true')) {
                             /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler = new SetupwizardHandler();
                            /** hide the left menu * */
                            setupwizardHandler.hideLeftMenu();
                            // create the menu wizard
                            productList = [ i18n.labelnameConnectivity, i18n.labelnameRegistration, i18n.labelnameClock, i18n.labelnameTransmition, i18n.labelnameIU ];

                            /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler = new SetupwizardHandler();
                            setupwizardHandler.generateList(productList, 4);

                            $(buttonContainer).append(ButtonFactory.buildButton({
                                type : 'back',
                                grayout : true,
                                label : i18n.buttonlabelBackSetupwizardPage,
                                onclick : function(event) {
                                    if (varSetupWizardEnabledReboot == "true") {
                                        
                                        tsa.cmd.handlers.UpdateHandler.redirectWizard ('transmission/edit/com.ibm.servicestation.FullCollectionJob', "true" );
                                    } else {
                                        tsa.cmd.handlers.UpdateHandler.redirectWizard ('transmission/edit/com.ibm.servicestation.FullCollectionJob', "false" );
                                    }
                                }
                            }));
                            $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
                            $(buttonContainer).append(ButtonFactory.buildButton({
                                type : 'arrow',
                                grayout : true,
                                label : i18n.buttonlabelFinishSetupwizardPage,
                                onclick : function(event) {
                                    // exit wizard
                                    setupwizardHandler.hideAlertPanel();
                                    setupwizardHandler.removeWizardLetfMenu();
                                    
                                    if (null != varSetupWizardEnabledReboot && varSetupWizardEnabledReboot == 'true') {
                                        tsa.cmd.handlers.UpdateHandler.redirectWizard ('setupwizard/view', "true" );
                                    } else {
                                        tsa.cmd.handlers.UpdateHandler.redirectWizard ('setupwizard/view', "false" );
                                    }
                                }
                            }));
                            contentArea.append(buttonContainer);
                        }
		  
    	  }else{ //an update exists
    		  
    	      var name 		    = i18n.labelUpdateGenericDescription;
    		  var releaseDate   = updateInfo.releaseDate;
    		  var version       = updateInfo.version;
    		  var readmeDetails = updateInfo.readmeDetails;
    		  var updateLicense = event.response.payload.licenseRequired;
    		  updateFormPanel = new FormPanel( { border : false } );
    		  
    		  contentArea.appendMessage( { level : 'INFO'  , border : true , text : i18n.labelUpdateCheckRestartsAfter } );
    		  updateTablePanel = new TablePanel( { title       : i18n.labelUpdateCheckAvailHeading
    			  								 , description : i18n.labelUpdateCheckUpdateAvailable
    			  								 , columns     : [ { id : 'property' , label : '' , width : '20%' , sortable : false } ] } );
    		 
    		  var nameSpan = document.createElement( 'span' );
    		  $( nameSpan ).css( { 'font-weight' : 'bold' } );
    		  $( nameSpan ).append( name );
    		  
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateDesc+":", nameSpan ] );
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateDate+":", releaseDate ] );
    		  updateTablePanel.appendRow( [ i18n.labelUpdateCheckUpdateVersion+":", version ] );
    		  contentArea.append( updateTablePanel.getDomNode() );
    		 
    		  
    		  //details overlay
    		  var div1   = document.createElement( 'div' );
    		  var div2   = document.createElement( 'div' );
    	      var p      = document.createElement( 'p'   );
    	      var aClose = $( '<a class="ibm-common-overlay-close" href="#close" style="text-decoration: none">'+i18n.buttonOverlayClose + '</a>').get(0);
    	      
    	      $( aClose ).on("click", function(e){ $( div1 ).hide(); });
    	         
    	      $( div1 ).addClass( 'ibm-common-overlay  ibm-overlay-alt-two' );
    	      $( div1 ).attr( 'id', 'overlay750' );
    	      $( div1 ).css( { 'font-color' : '#666666' } );
    	      $( div2 ).addClass( 'ibm-head');
    	      $( p    ).append( aClose );
    	      $( div2 ).append( p );
    	      $( div1 ).append( div2 );
    	          
    	      var div3  = document.createElement( 'div' );
    	      $( div3 ).addClass( 'ibm-body' );
    	      	
    	      var div4 = $('<div class="ibm-main ibm-code-sample" style="height:30em; background-color:#ffffff !important; border:0px; margin:0px; overflow:scroll" ></div>').get(0);
    	     
    	      var a2    = $( '<a class="ibm-access"><!-- Accessibility anchor --></a>' ).get(0);
    	      $( div4 ).append( a2 );
    	        
    	      var div5  = document.createElement( 'div' );
    	      $( div5 ).attr( 'id', 'updateDetailsTitle' );
    	        
    	      var h1  = document.createElement( 'h1' );
    	      $( h1 ).append(i18n.labelUpdateViewDetailsTitle)
    	      $( div5 ).append( h1 )
    	      $( div4 ).append( div5 );
    	        
    	      var div6  = document.createElement( 'div' );
    	      $( div6 ).addClass( 'ibm-container ibm-alternate' );
    	        
    	      var div7  = document.createElement( 'div' );
    	      $( div7 ).addClass( 'ibm-container-body' );
    	      $( div7 ).attr( 'id', 'updateDetailsList' );
    	      
    	      var pre = $('<pre>' + i18n.labelUpdateEnhancementDesc + '</pre>').get(0);
    	      $( div7 ).append( pre );
    	        
    	      var ul = document.createElement( 'ul' );
    	      
    	      //update details & history
    	      if( readmeDetails !== null )
    	      {
    	    	  var detail = null;
    	          $.each( readmeDetails, function( entryIndex, entry )
    	          {
    	        	  var p2 = document.createElement( 'p' );
    	        	  
    	        	  detail = $( '<li style="font-weight: bold; font-size:small"></li>' ).get(0);
    	        	  $( detail ).text( entry.heading );
    	        	  $( p2 ).append( detail );
    	        	  
    	        	  var p3 = document.createElement( 'p' );
    	        	  $( p3 ).attr( 'id', 'update-date' );
    	        	  $( p3 ).append( entry.date );
    	        	  $( p2 ).append( p3 );
    	        	  
    	        	  var ul2 = document.createElement( 'ul' );
    	        	  $.each( entry.enhancements, function( enhancementIndex, enhancement )
    	        	  {
    	        		  var enhancementText = $( '<li></li>' ).get(0);
    	        		  $( enhancementText ).css( { 'font-size' : '140%', 'font-color' : '#666666' } )
    	        		  $( enhancementText ).text( enhancement );
    	        		  $( ul2 ).append( enhancementText );
    	        		  $( p2 ).append( ul2 ); 
    	        	  });
    	        	  
    	        	 $( ul ).append( p2 );
    	        	  
    	          });
    	        
    	          $( div7 ).append( ul );
    	          $( div6 ).append( div7 );
    	          $( div4 ).append( div6 );
    	          $( div3 ).append( div4 );
    	          $( div1 ).append( div3 );
    	         
    	          var div8  = document.createElement( 'div' );
    	          $( div8 ).addClass( 'ibm-footer' );
    	          $( div1 ).append( div8 );
    	          
    	        }
    	       
    	      $( div1 ).hide();
    	      
    	      //append details overlay  
    	      updateFormPanel.appendJointRow( div1 );
    	        
    	      //View and Accept License button
    	      var viewLicenseCmd = new Command({ action : 'update/ajaxViewLicense', clearSystemMessages : false,  blockUI : false });
      		  var updateLicenseButton = ButtonFactory.buildButton({ 
      			    type    : 'arrow'  
      		      , label   : i18n.buttonViewLicenseNow 
      		      , onclick : function( event ){
        	  				 viewLicenseCmd.execute();
        	               }
      	         });

      	      $( viewLicenseCmd ).on( 'executionSucceeded', function( event ){
                   var updateLicenseVRM = event.response.payload.updateLicenseVRM;
                   ( new ShowCommand( { action : 'updatelicense/index' , clearSystemMessages : false 
                                  , parameters : { "updateLicenseVRM" : updateLicenseVRM } } ) ).execute();
              });

              //var wizard reboot	and normal process
              /** @type {Element}                 */var varSetupWizardEnabledReboot = event.response.payload.setupWizardEnabledReboot;
              /** @type {Element}                 */var setupWizardEnabled          = event.response.payload.setupWizardEnabled;

              //perform update button
              var updateCmd = new Command({ action : 'update/ajaxPerform', clearSystemMessages : false,  blockUI : false });
              var updateButton = ButtonFactory.buildButton({ 
                        type    : 'arrow'  
                      , label   : i18n.buttonUpdateCheckUpdateNow 
                      , onclick : function( event ){
                         if ((setupWizardEnabled == 'true') || (varSetupWizardEnabledReboot == 'true' ) ) {
                             // variable for indicate the wizard process
                             updateCmd.setParameter("setupWizardEnabled", "true");
                         }
                      updateCmd.execute();
                      }
              });

              $( updateCmd ).on( 'executionSucceeded', function( event ){
                    // exit wizard
                   /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                   /** hide the left menu * */
                   //setupwizardHandler.exitWizard();
                   
                   if (( !TV.typeIsUndefined( typeof setupWizardEnabled )          && setupWizardEnabled == 'true') || 
                       ( !TV.typeIsUndefined( typeof varSetupWizardEnabledReboot ) && varSetupWizardEnabledReboot == 'true' ) ) {
                       
                       /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                       setupwizardHandler.hideAlertPanel();
                       setupwizardHandler.removeWizardLetfMenu();
                       
                       tsa.cmd.handlers.UpdateHandler.redirectWizard ('setupwizard/view', "false" );
                   } else {
                          ( new ShowCommand( { action : 'main/index' , clearSystemMessages : false  } ) ).execute();
                   }
              });

              //view documentation button
              var viewButton = ButtonFactory.buildButton({ 
                      type    : 'arrow' 
                    , label   : i18n.buttonUpdateViewDetails 
                    , onclick : function( event ){
                                ibmCommon.Overlays.show('overlay750', this);
                    }
              });

              if ( updateLicense ){
                  updateTablePanel.setDescription( i18n.labelUpdateCheckLicenseAvailable );
                  $( updateLicenseButton ).show();
                  $( updateButton ).hide();
              }else{
                  updateTablePanel.setDescription( i18n.labelUpdateCheckUpdateAvailable );
                  $( updateButton ).show();
                  $( updateLicenseButton ).hide();
              }

              //wizard process add the back button
              if ((setupWizardEnabled == 'true') || (varSetupWizardEnabledReboot == 'true' ) ) {
                  /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                  setupwizardHandler.hideLeftMenu();
                  // add the back button
                  $( buttonContainer ).append(
                          ButtonFactory.buildButton(
                                  {     type    : 'back'
                                      , grayout : true
                                      , label   : i18n.buttonlabelBackSetupwizardPage
                                      , onclick : function( event ) {
                                          $("#WizardList").remove();
                                          
                                          /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                                          setupwizardHandler.hideAlertPanel();
                                          setupwizardHandler.removeWizardLetfMenu();
                                          
                                          if (varSetupWizardEnabledReboot == "true") {
                                              tsa.cmd.handlers.UpdateHandler.redirectWizard ('transmission/edit/com.ibm.servicestation.FullCollectionJob', "true" );
                                          } else {
                                              tsa.cmd.handlers.UpdateHandler.redirectWizard ('transmission/edit/com.ibm.servicestation.FullCollectionJob', "false" );
                                          }
                                      }
                                  }
                          )
                  );
                  $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
              }
              $( buttonContainer ).append( updateLicenseButton );
              $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
              $( buttonContainer ).append( updateButton );
              $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
              $( buttonContainer ).append( viewButton );

              //wizard process add skip button
              if (( !TV.typeIsUndefined( typeof setupWizardEnabled )          && setupWizardEnabled == 'true') 
               || ( !TV.typeIsUndefined( typeof varSetupWizardEnabledReboot ) && varSetupWizardEnabledReboot == 'true' ) ) {
                  /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                  /** hide the left menu **/
                  setupwizardHandler.removeWizardLetfMenu();

                  // create the menu wizard
                  productList = [ i18n.labelnameConnectivity, i18n.labelnameRegistration, i18n.labelnameClock, i18n.labelnameTransmition, i18n.labelnameIU ];
                  setupwizardHandler.generateList( productList, 4 );

                  $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
                  $( buttonContainer ).append(
                          ButtonFactory.buildButton(
                                  {     type    : 'arrow'
                                      , grayout : true
                                      , label   : i18n.buttonlabelSkipSetupwizardPage
                                      , onclick : function( event ) {
                                         
                                          /** @type {tsa.cmd.handlers.SetupwizardHandler} */ var setupwizardHandler          = new SetupwizardHandler();
                                          setupwizardHandler.hideAlertPanel();
                                          setupwizardHandler.removeWizardLetfMenu();
                                          
                                          if ( varSetupWizardEnabledReboot == 'true' ) {
                                              tsa.cmd.handlers.UpdateHandler.redirectWizard ('setupwizard/view', "true" );
                                          } else {
                                              tsa.cmd.handlers.UpdateHandler.redirectWizard ('setupwizard/view', "false" );
                                          }
                                      }
                                  }
                          )
                  );
              }
              updateFormPanel.appendJointRow( buttonContainer );
              contentArea.append( updateFormPanel.getDomNode() );

          }
      });
      
      checkUpdateCmd.setParameter("fromlicense", fromLicense);
      
      //wizard process
      /** @type {Element}                   */ var setupWizardEnabled = this.getParameter("setupWizardEnabled");
      if (setupWizardEnabled) {
          checkUpdateCmd.setParameter("setupWizardEnabled", "true");
      }
      //wizard process with reboot setupWizardEnabledReboot
      if ((setupWizardEnabled == false  || setupWizardEnabled == 'false' )) {
          checkUpdateCmd.setParameter("setupWizardEnabledReboot", "true");
      }
      
      // check for update
      checkUpdateCmd.execute();
      
      this.show( contentArea.getDomNode() );
    };
    
    
    //-------------------------------------------------------------------------
    
    /**
     * Request for restarting system.
     * @param {string} restart Restart mode. Use either:
     *                         'auto' : Restart will be initiated by the system.
     *                         'user' : Restart should be initiated by the user.
     */
    tsa.cmd.handlers.UpdateHandler.rebootSystem = function( restart )
    {
       /** @type {tsa.cmd.Command} */ var cmd = null;
       
       cmd = new Command( { action              : 'clock/ajaxReboot'
                          , clearSystemMessages : false
                          , parameters          : { "reboot" : restart } 
                          } );
       cmd.execute();
       
       $(cmd).on( 'executionSucceeded', function( event ){
              $("#WizardList").remove();
              $("#ibm-primary-links").show();
 			  ( new ShowCommand( { action : 'main/index' , clearSystemMessages : true  } ) ).execute();
 		  });
    };
    
    /**
     * Show Commands for wizard
     * @param {string} action Action for the ShowCommand.
     * @param {string} reboot Flag to reboot when wizard finished
     * @param {string} parameters Parameters for the Show Command
     */
    tsa.cmd.handlers.UpdateHandler.redirectWizard = function( action, reboot  )
    {                
                      var parameters = {};
                      if (reboot == "true") {
                        parameters.setupWizardEnabled = "false";  
                       } else {
                        parameters.setupWizardEnabled  = "true";
                       }
                       (new ShowCommand({
                           action              : action,
                           clearSystemMessages : true,
                           parameters          : parameters
                       })).execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.UpdateHandler;
  }
);