/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TabsPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/FormPanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ContentArea
          , TabsPanel
          , TablePanel
          , ButtonFactory
          , IconFactory
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'UseracctsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.UseracctsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.UseracctsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.UseracctsHandler.prototype ).constructor = tsa.cmd.handlers.UseracctsHandler;

    //sidebar to show info of elements
    tsa.cmd.handlers.UseracctsHandler.prototype.infoParagraphGeneral   = null;
    tsa.cmd.handlers.UseracctsHandler.prototype.infoParagraphMemberOf   = null;
    //-------------------------------------------------------------------------
   /**
     * Show main screen.
     */
    tsa.cmd.handlers.UseracctsHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                   = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea            = null;
      /** @type {Element}                   */ var addAccountButton       = null;
      /** @type {Element}                   */ var addGroupButton         = null;
      /** @type {Element}                   */ var bodyTabAccounts        = document.createElement( 'p' );
      /** @type {Element}                   */ var bodyTabGroups          = document.createElement( 'p' );
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer        = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.TabsPanel}     */ var tabsPanel              = null;
      
      
      contentArea = new ContentArea ( { title         : i18n.labelUsersAndGroupsHeading
                                      , showBackToTop : true
                                      , helpDoc : 'sshelp_users_panel.html' } );
      
        $( bodyTabAccounts ).text( i18n.labelUserAccountsCaption );
        $( bodyTabGroups ).text( i18n.labelUserGroupsCaption );
        tabsPanel = new TabsPanel( { tabs: [ { title  : i18n.labelUserAccountsTab , body : bodyTabAccounts
                                             , action : "useraccts/index" }
                                           , { title  : i18n.labelUserGroupsTab   , body : bodyTabGroups
                                             , action : "usergrps/index" }
                                           ] } );

        tablePanel  = new TablePanel  ( { title        : i18n.labelUserAccountsHeading
                                        , summary      : i18n.labelActivityLogTableSummary
                                        , emptyMsg     : i18n.labelUserNoAccounts
                                        , ajaxAction   : 'useraccts/getUserAccts'
                                        , entryHandler : $.proxy(this.infoEntryHandler,this)
                                        , columns      : [ { id       : 'userid'
                                                           , label    : i18n.labelUserAccountNameColumn
                                                           , width    : '20%'
                                                           , sortable : true }
                                                         , { id       : 'fullname'
                                                           , label    : i18n.labelUserAccountFullNameColumn
                                                           , width    : '20%'
                                                           , sortable : true }
                                                         , { id       : 'desc'
                                                           , label    : i18n.labelUserAccountDescColumn
                                                           , width    : '25%'
                                                           , sortable : true }
                                                         , { id       : 'passwordage'
                                                           , label    : i18n.labelUserAccountPasswordAgeColumn
                                                           , width    : '25%'
                                                           , sortable : true }
                                                         , { id       : 'actions'
                                                           , label    : i18n.labelActionsColumn
                                                           , width    : '10%'
                                                           , sortable : false } ] } );

      contentArea.append( tabsPanel.getDomNode() );
      contentArea.append( tablePanel.getDomNode() );
      
      $( buttonContainer ).append(
              ButtonFactory.buildButton(
                { type    : 'arrow'
                , grayout : false
                , label   : i18n.buttonUserAddAccount
                , onclick : function( event )
                            {
                                ( new ShowCommand( { action : 'useraccts/new' } )).execute();
                             }                  
                }
              )
          );
      
      this.infoParagraphGeneral  = document.createElement("div");
      this.infoParagraphMemberOf = document.createElement("div");
      /** @type {element} */    var paragraphGeneral  = document.createElement("p");
      /** @type {element} */    var paragraphMemberOf = document.createElement("p");
      
      $( paragraphGeneral  ).text( i18n.labelUserAccountNoneSelected );
      $( paragraphMemberOf ).text( i18n.labelUserAccountNoneSelected );
      
      $( this.infoParagraphGeneral  ).append( paragraphGeneral  );
      $( this.infoParagraphMemberOf ).append( paragraphMemberOf );      
      
      contentArea.append( buttonContainer );
      contentArea.appendSidebarAccordionPanel( [ { title: i18n.labelUserAccountGeneralHeading, body: this.infoParagraphGeneral } 
                                               , { title: i18n.labelUserAccountGroupsHeading,  body: this.infoParagraphMemberOf } ] );
      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------
  
    tsa.cmd.handlers.UseracctsHandler.prototype.infoEntryHandler = function( entry )
    {
        
       /** @type {Array.<(string|Element)>} */ var result       = null;
       /** @type {Element}                  */ var dUserId      = document.createElement("div");
       /** @type {Element}                  */ var dfullName    = document.createElement("div");
       /** @type {Element}                  */ var dDescription = document.createElement("div");
       /** @type {Element}                  */ var dPasswordAge = document.createElement("div");
       /** @type {Element}                  */ var dActions     = document.createElement("div");
       /** @type {Object}                   */ var i18n         = tsa.i18n;
       /** @type {Element}                  */ var actionsSpan  = document.createElement( 'span' );
              
       //event to handle click
       /** @type {function()}               */ 
       var clickEventHandler  = $.proxy(function(event){
         /** @type {string}          */ var actionURL    = 'useraccts/getInfo/' + entry.pid;
         /** @type {tsa.cmd.Command} */ var cmdInfo      = new Command( { action : actionURL
                                                                        , clearSystemMessages:false} );
         
           cmdInfo.execute();
           
           $(cmdInfo).on('executionSucceeded', $.proxy(function(event){
             
             $( this.infoParagraphGeneral  ).empty();
             $( this.infoParagraphMemberOf ).empty();
             
             /** @type {Object}                  */  var i18n             = tsa.i18n; 
             /** @type {Object}                   */ var object           = event.response.payload.object;
             /** @type {Element}                  */ var pUserName        = document.createElement("p");
             /** @type {Element}                  */ var pFullName        = document.createElement("p");
             /** @type {Element}                  */ var pDescription     = document.createElement("p");
             /** @type {Element}                  */ var pGroups          = document.createElement("p");
             /** @type {Element}                  */ var sUserName        = document.createElement("strong");
             /** @type {Element}                  */ var sFullName        = document.createElement("strong");
             /** @type {Element}                  */ var sDescription     = document.createElement("strong");
             /** @type {Element}                  */ var sGroups          = document.createElement("strong");             
                          
             $(sUserName   ).text(i18n.labelUserAccountName     + ':');
             $(sFullName   ).text(i18n.labelUserAccountFullName + ':');
             $(sDescription).text(i18n.labelUserAccountDesc     + ':');
             
             $(pUserName   ).append(sUserName    ,document.createElement( 'br' ),document.createTextNode( object.userName    != null  ? object.userName    : "" ));
             $(pFullName   ).append(sFullName    ,document.createElement( 'br' ),document.createTextNode( object.fullName    != null  ? object.fullName    : "" ));
             $(pDescription).append(sDescription ,document.createElement( 'br' ),document.createTextNode( object.description != null  ? object.description : "" ));
             $.each(object.groups, function(index, group){
               $(pGroups).append(sGroups , document.createElement( 'br' ), document.createTextNode( group.group ));
             });
             
             
             $(this.infoParagraphGeneral ).append( pUserName    );
             $(this.infoParagraphGeneral ).append( pFullName    );
             $(this.infoParagraphGeneral ).append( pDescription );
             $(this.infoParagraphMemberOf).append( pGroups      );
               
             }, this));
       },this);
       
       /** @type {Element} */ var editIcon    = IconFactory.buildActionIcon( { type : 'edit'   , caption : i18n.labelButtonEdit   + ' "' + entry.userid + '"' } );
       /** @type {Element} */ var deleteIcon  = IconFactory.buildActionIcon( { type : 'delete' , caption : i18n.labelButtonDelete + ' "' + entry.userid + '"' } );
       /** @type {Element} */ var aId        = document.createElement("a");
       /** @type {Element} */ var sId        = document.createElement("strong");
       
       $(sId).append(dUserId);
       $(aId).append(sId);
       $(aId).attr("href","#");
       
       $( sId      ).on( 'click', clickEventHandler );
       
       $( actionsSpan ).append( editIcon );
       
       if ( entry.userid !== "admin") 
       { 
         $( actionsSpan ).append( deleteIcon ); 
       } 
       
       /**@type {function()}*/
       var clickHandlerDelete = function( )
       {
         
           if( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + entry.userid + '"?' ) )
           {
             /** @type {tsa.cmd.Command} */ var deleteCmd = new Command( { action     : 'useraccts/deleteUserAcct/' + entry.pid } );
             
             $( deleteCmd ).on( 'executionSucceeded', function( event )
             { 
                 
                 var urlRedirect= 'useraccts/index';
                 if(TV.typeIsUndefined (event.response.payload.logout)){
                     urlRedirect= 'main/index';
                 }
               ( new ShowCommand( { action              : urlRedirect
                                  , clearSystemMessages : false } ) ).execute();
             } );
             
             deleteCmd.execute();
           }
         
           
         };
       $( editIcon     ).on( 'click', function( event ) { ( new ShowCommand( { action : 'useraccts/edit/' + entry.pid } ) ).execute(); } );
       $( deleteIcon   ).on( 'click', clickHandlerDelete );
       
       $( dUserId      ).text( entry.userid      );
       $( dfullName    ).text( entry.fullname    );
       $( dDescription ).text( entry.desc        );
       
       /**/ var passAge = parseInt(entry.passwordage);
       if(passAge == -1){
           $( dPasswordAge ).append(  tsa.i18n.labelUserAccountPasswordInResetState );
       } else if (passAge == -2) {
           $( dPasswordAge ).append(  tsa.i18n.labelUserAccountPasswordAgeNotAvailable ) ;
       } else {
           $( dPasswordAge ).append( entry.passwordage + " " + tsa.i18n.labelDays);
       }
       
       
       $( dActions     ).append( actionsSpan       );
       
       return [ aId , dfullName , dDescription , dPasswordAge , dActions];
    };
  //-------------------------------------------------------------------------

  tsa.cmd.handlers.UseracctsHandler.prototype._new = function()
  {
      /** @type {String}                  */ var userAccountPID  = this.getParameter( 'entityId' );
      /** @type {Object}                  */ var i18n            = tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelUserAccountHeading
                                                                                    , helpDoc : 'sshelp_user_account_panel.html' } );
      /** @type {Array.<Element>}       */ var userGroupCheckboxes = [];
      var actionFormURL = (userAccountPID!=null ? 'useraccts/putUserAcct':'useraccts/postUserAcct');
      /** @type {tsa.widgets.FormPanel} */ var generalForm         = new FormPanel( {  title       : i18n.labelUserAccountGeneralHeading
                                                                                     , action      : actionFormURL
                                                                                     , description : i18n.labelUserAccountGeneralDesc } );
      /** @type {tsa.widgets.FormPanel} */ var passwordForm        = new FormPanel( { title       : i18n.labelPasswordEnterHeading
                                                                                    , description : i18n.labelPasswordEnterDesc } );
      /** @type {tsa.widgets.FormPanel} */ var memberOfForm        = new FormPanel( { title       : i18n.labelUserAccountGroupsHeading
                                                                                    , description : i18n.labelUserAccountGroupsDesc } );
      /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
      /** @type {tsa.cmd.Command}         */ var getUserGrpsCmd  = new Command( { action              : 'useraccts/getUserGrps'
                                                                                , clearSystemMessages : false } );
      
      if(userAccountPID!=null){
          generalForm.putHiddenInput( 'pid', userAccountPID );
      }
      generalForm.linkTo(passwordForm);
      passwordForm.linkTo(memberOfForm);
      /** @type {Element} */ var nameField           = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var fullNameField       = $( '<input type="text" id="fullName" name="fullName" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var descField           = $( '<input type="text" id="descField" name="descField" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var newPasswordField    = $( '<input type="password" id="newPasswordField" name="newPasswordField" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var confPasswordField   = $( '<input type="password" id="newPasswordField_confirm" name="newPasswordField_confirm" size="50" maxlength="250" />' ).get(0);
      /** @type {Element} */ var disableAcctField    = $( '<span class="ibm-input-group" role="group" aria-label="Disable Account"><input type="checkbox" id="passwordOptions1" name="passwordOptions1" value="ACCOUNT_DISABLE" /><label for="passwordOptions1">'+i18n.labelUserAccountAccountDisabled+'</label></span>' ).get(0);
            
      //General form fields
      generalForm.appendFieldRow( {    label : i18n.labelUserAccountName
                                  , required : true  
                                  , element  : nameField 
                                  , hint     : i18n.labelUserAccountNameDesc } );
    
      generalForm.appendFieldRow( { label: i18n.labelUserAccountFullName
                                    , required: false
                                    , element: fullNameField
                                    , hint: i18n.labelUserAccountFullNameDesc } );
      
      generalForm.appendFieldRow( { label: i18n.labelUserAccountDesc
                                    , required: false
                                    , element: descField  
                                    , hint : i18n.labelUserAccountDescDesc } );
      
    //if is a new user then make it required
      var isCreate = (userAccountPID == null);
    //Password form fields
      passwordForm.appendFieldRow( { label: i18n.labelPasswordEnterPassw1 
                                     , required: isCreate
                                     , element: newPasswordField } );
      passwordForm.appendFieldRow( { label: i18n.labelPasswordEnterPassw2 
                                   , required: isCreate
                                   , element: confPasswordField } );
      passwordForm.appendFieldRow( { label: i18n.labelUserAccountPasswOptions 
                                     , required: false
                                     , element: disableAcctField  } );
        
      //Member of form fields
      /** @type {Element} */ var span = document.createElement( 'span' );
      
      $( span ).addClass( 'ibm-input-group' );
      $( span ).attr( 'role', 'group' );
      $( span ).attr( 'aria-label', i18n.labelUserAccountGroups );
      getUserGrpsCmd.execute();
      $( getUserGrpsCmd ).on( 'executionSucceeded', function( event ) {
              var userAcctGroup = event.response.payload.userAcctGroup;
              $.each(userAcctGroup, function(index, group) 
                      {
                         /** @type {Element} */ var checkbox = $( '<input type="checkbox" name="group" value="' + ( group.pid ) + '" />' ).get(0);
                         /** @type {Element} */ var label    = $( '<label>' + group.name + '</label>' ).get(0);
                         $( label ).prepend( checkbox );
                         $( span ).append( label, document.createElement( 'br' ) );
                      } );
              memberOfForm.appendFieldRow( {  label    : i18n.labelUserAccountGroups
                                            , required : true
                                            , aRequire : false
                                            , element  : span } );
           //If edit then retrieve all the info
           if (userAccountPID != null ) {
               /** @type {tsa.cmd.Command} */ var getUserAcctCmd  = new Command( {   action             : 'useraccts/getUserAcct/' + userAccountPID
                                                                                  , clearSystemMessages : false } );
               getUserAcctCmd.execute();
               $( getUserAcctCmd ).on( 'executionSucceeded', function( event ) {
                   for( var key in event.response.payload ) {
                       $( '[name="' + key + '"]' ).val( event.response.payload[ key ] );
                     }
                   $.each(event.response.payload.groups, function(index, group) 
                           {
                               $(span).find( '[value="' + group.pid + '"]' ).attr('checked',true);   
                           });
                   
                   
                     
                   if(event.response.payload.passwordOptions1){
                       $( '[name="passwordOptions1"]' ).attr('checked',true);
                   }
                   
                   if(userAccountPID==="00"){
                       
                       $( '[name="passwordOptions1"]' ).prop('disabled' , true );
                       $( '[name="group"]' ).prop('disabled' , true );
                       generalForm.putHiddenInput("group","10");
                   }
               });
           }
      });
      
     
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                              , label   : i18n.buttonSave
                                                              , onclick : function( event ) {
                                                                    generalForm.submit(); 
                                                                    } } ) );
      
      $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
      $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                , label   : i18n.buttonCancel
                , onclick : function( event ) { 
                    ( new ShowCommand( { action : 'useraccts/index' } )).execute(); 
                    }} ) );
      $( generalForm ).on( 'executionSucceeded', function( event )
        {
              ( new ShowCommand( { action : 'useraccts/index' } ) ).execute();
        } );
      
      $( generalForm ).on( 'executionFailed', function( event )
              {
                if ( TV.typeIsObject( typeof event.response.payload.errors ) ) 
                {
                  for ( /** @type {string} */ var fieldName in event.response.payload.errors ) 
                  {
                      generalForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                  }
                    delete event.response.payload.errors;
                }
              } );
      
      
      
      contentArea.append( generalForm.getDomNode() );
      contentArea.append( passwordForm.getDomNode() );
      contentArea.append( memberOfForm.getDomNode() );      
      contentArea.append( buttonContainer );  
      // here the contentArea
      this.show( contentArea.getDomNode() );
  };
  //-------------------------------------------------------------------------  
  tsa.cmd.handlers.UseracctsHandler.prototype._edit = function()
  {
      this._new();
  };
  //-------------------------------------------------------------------------

    return tsa.cmd.handlers.UseracctsHandler;
  }
);