/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/TabsPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/FormPanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ContentArea
          , TabsPanel
          , TablePanel
          , ButtonFactory
          , IconFactory
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'UsergrpsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.UsergrpsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.UsergrpsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.UsergrpsHandler.prototype ).constructor = tsa.cmd.handlers.UsergrpsHandler;
    //sidebar to show info of elements
    tsa.cmd.handlers.UsergrpsHandler.prototype.infoParagraph   = null;
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.UsergrpsHandler.prototype._index = function()
    {
      /** @type {Object}                    */ var i18n                  = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */ var contentArea           = null;
      /** @type {Element}                   */ var addAccountButton      = null;
      /** @type {Element}                   */ var addGroupButton        = null;
      /** @type {tsa.widgets.ButtonFactory} */ var buttonContainer       =  ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.TabsPanel}     */ var tabsPanel             = null;
      /** @type {Element}                   */ var editIcon              = IconFactory.buildActionIcon( { type : 'edit' , caption :  'Edit'} );
      /** @type {Element}                   */ var deleteIcon            = IconFactory.buildActionIcon( { type : 'delete' , caption :  'Delete'} );
      /** @type {Element}                   */ var bodyTabAccounts       = document.createElement( 'p' );
      /** @type {Element}                   */ var bodyTabGroups         = document.createElement( 'p' );

      contentArea = new ContentArea ( { title : i18n.labelUsersAndGroupsHeading
                                      , showBackToTop : true
                                      , helpDoc : 'sshelp_users_panel.html'} );
      
      $( bodyTabAccounts ).text( i18n.labelUserAccountsCaption );
      $( bodyTabGroups   ).text( i18n.labelUserGroupsCaption   );
      
      tabsPanel = new TabsPanel( { tabs: [ { title  : i18n.labelUserAccountsTab , body : bodyTabAccounts
                                           , action : "useraccts/index"}
                                         , { title  : i18n.labelUserGroupsTab   , body : bodyTabGroups
                                           , action : "usergrps/index"} ] } );

      tablePanel  = new TablePanel  ( { title        : i18n.labelUserGroupsHeading
                                      , summary      : i18n.labelActivityLogTableSummary
                                      , emptyMsg     : i18n.labelUserNoGroups
                                      , ajaxAction   : 'usergrps/getUserGrps'
                                      , entryHandler : $.proxy(this.infoEntryHandler,this)
                                      , columns      : [ { id       : 'name'
                                                         , label    : i18n.labelUserGroupNameColumn
                                                         , width    : '40%'
                                                         , sortable : true }
                                                       , { id       : 'desc'
                                                         , label    : i18n.labelUserGroupDescColumn
                                                         , width    : '50%'
                                                         , sortable : true }
                                                       , { id       : 'actions'
                                                         , label    : i18n.labelActionsColumn
                                                         , width    : '10%'
                                                         , sortable : false } ] } );

      tabsPanel.goToTab(1);
      
      
      this.infoParagraphGeneral           = document.createElement("div");
      this.infoParagraphScopeRestrictions = document.createElement("div");
      this.infoParagraphMembers           = document.createElement("div");
      
      /** @type {element} */ var paragraphGeneral           = document.createElement("p");
      /** @type {element} */ var paragraphScopeRestrictions = document.createElement("p");
      /** @type {element} */ var paragraphMembers           = document.createElement("p");
      
      $( paragraphGeneral           ).text( i18n.labelUserGroupNoneSelected );
      $( paragraphScopeRestrictions ).text( i18n.labelUserGroupNoneSelected );
      $( paragraphMembers           ).text( i18n.labelUserGroupNoneSelected );
      
      $( this.infoParagraphGeneral           ).append( paragraphGeneral           );
      $( this.infoParagraphScopeRestrictions ).append( paragraphScopeRestrictions );
      $( this.infoParagraphMembers           ).append( paragraphMembers           );
      
      $( buttonContainer ).append(
              ButtonFactory.buildButton(
                { type    : 'arrow'
                , grayout : false
                , label   : i18n.buttonUserAddGroup
                , onclick : function( event )
                            {
                                ( new ShowCommand( { action : 'usergrps/new' } )).execute();
                             }                  
                }
              )
          );
      
      contentArea.append( tabsPanel .getDomNode()  );
      contentArea.append( tablePanel.getDomNode() );
      contentArea.append( buttonContainer         );
      
      contentArea.appendSidebarAccordionPanel( [ { title: i18n.labelUserGroupGeneralHeading, body: this.infoParagraphGeneral } 
                                               , { title: i18n.labelUserGroupScopeSetsHeading, body: this.infoParagraphScopeRestrictions } 
                                               , { title: i18n.labelUserGroupMembersHeading, body: this.infoParagraphMembers } ] );
      
      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------
    
    tsa.cmd.handlers.UsergrpsHandler.prototype.infoEntryHandler = function( entry )
    {
        
       /** @type {Array.<(string|Element)>} */ var result             = null;
       /** @type {Element}                  */ var dName              = document.createElement("div");
       /** @type {Element}                  */ var dDescription       = document.createElement("div");
       /** @type {Element}                  */ var dActions           = document.createElement("div");       
       /** @type {Object}                   */ var i18n               = tsa.i18n;
       /** @type {Element}                  */ var actionsSpan        = document.createElement( 'span' );
       
       /**@type {function()}*/
       var clickHandlerDelete = function( )
       {
           //check if exist some member on this group
           /** @type {string}          */ var actionURLDelete    = 'usergrps/getInfo/' + entry.pid;
           /** @type {tsa.cmd.Command} */ var cmdInfoDelete      = new Command( { action : actionURLDelete
                                                                                , clearSystemMessages:false} );
           /** @type {boolean}         */ var existMember         = false;
           cmdInfoDelete.execute();
           $(cmdInfoDelete).on('executionSucceeded', $.proxy(function(event){
               /** @type {Object}          */ var object       = event.response.payload.object;
               if(!TV.typeIsUndefined (object.scopesets))
               {
                 $.each(object.members, function(index, member){
                     existMember = true;
                 });
               }
                     if (existMember) {
                         tsa.printSysMessage( new tsa.cmd.SystemMessage( { level   : 'info'
                             , message : i18n.labelGroupNotDelete } ) );
                     } else {
                         tsa.cmd.handlers.UsergrpsHandler.confirmDeleteGroup(entry.name, entry.pid);
                     }
           }, this));
           $( cmdInfoDelete ).on( 'executionFailed', function( event ){
               tsa.cmd.handlers.UsergrpsHandler.confirmDeleteGroup(entry.name, entry.pid);
           } );
         };
         
       /** @type {function()}               */ 
       var clickEventHandler  = $.proxy(function(event){
         /** @type {string}          */ var actionURL    = 'usergrps/getInfo/' + entry.pid;
         /** @type {tsa.cmd.Command} */ var cmdInfo      = new Command( { action : actionURL
                                                                        , clearSystemMessages:false} );
           cmdInfo.execute();
           $(cmdInfo).on('executionSucceeded', $.proxy(function(event){
             
             $( this.infoParagraphGeneral           ).empty();
             $( this.infoParagraphScopeRestrictions ).empty();
             $( this.infoParagraphMembers           ).empty();
             
             /** @type {Object}                  */  var i18n         = tsa.i18n; 
             /** @type {Object}                   */ var object       = event.response.payload.object;
             //General
             /** @type {Element}                  */ var pGroupName   = document.createElement("p");
             /** @type {Element}                  */ var pAuthLevel   = document.createElement("p");
             //
             //Scope restrictions
             /** @type {Element}                  */ var pScopesets   = document.createElement("p");
             //
             //Members
             /** @type {Element}                  */ var pMembers     = document.createElement("p");
             //
             /** @type {Element}                  */ var sGroupName   = document.createElement("strong");
             /** @type {Element}                  */ var sAuthLevel   = document.createElement("strong");
             /** @type {Element}                  */ var sScopesets   = document.createElement("strong");
             /** @type {Element}                  */ var sMembers     = document.createElement("strong");
                          
             $(sGroupName    ).text(i18n.labelUserGroupName           + ':');
             $(sAuthLevel    ).text(i18n.labelUserGroupAuthorityLevel + ':');
             $(pGroupName    ).append(sGroupName    , document.createElement( 'br' ),document.createTextNode( object.name        ));
             $(pAuthLevel    ).append(sAuthLevel    , document.createElement( 'br' ),document.createTextNode( object.authLevel   ));
             
             if(!TV.typeIsUndefined (object.scopesets))
             {
               $.each(object.scopesets, function(index, scopeset){
                   $(pScopesets).append(sScopesets , document.createElement( 'br' ),document.createTextNode( scopeset.scopeset ));
                 });
             }
             
             if(!TV.typeIsUndefined (object.scopesets))
             {
               $.each(object.members, function(index, member){
                 $(pMembers).append(sMembers , document.createElement( 'br' ),document.createTextNode( member.member ));
               });
             }
             
             $(this.infoParagraphGeneral           ).append( pGroupName    );
             $(this.infoParagraphGeneral           ).append( pAuthLevel    );
             $(this.infoParagraphScopeRestrictions ).append( pScopesets    );
             $(this.infoParagraphMembers           ).append( pMembers      );
             }, this));
       },this);
       
       /** @type {Element} */ var editIcon    = IconFactory.buildActionIcon( { type : 'edit'   , caption : i18n.labelButtonEdit   + ' "' + entry.userid + '"' } );
       /** @type {Element} */ var deleteIcon  = IconFactory.buildActionIcon( { type : 'delete' , caption : i18n.labelButtonDelete + ' "' + entry.userid + '"' } );
       /** @type {Element} */ var aId         = document.createElement("a");
       /** @type {Element} */ var sId         = document.createElement("strong");
       
       $(sId).append(dName);
       $(aId).append(sId);
       $(aId).attr("href","#");
       
       if( entry.name !== "Administrator") 
       {
           $( actionsSpan ).append( editIcon );
           $( actionsSpan ).append( deleteIcon ); 
       }
       
       $( dName        ).on( 'click', clickEventHandler );
       $( deleteIcon   ).on( 'click', clickHandlerDelete);
       $( editIcon     ).on( 'click', function( event ) { 
                                                           ( new ShowCommand( { action : 'usergrps/edit/' + entry.pid } ) ).execute(); 
                                                         } );       
       $( dName        ).text( entry.name );
       $( dDescription ).text( entry.desc );
       $( dActions     ).append( actionsSpan);
       
       return [ aId , dDescription , dActions];
    };
  //-------------------------------------------------------------------------

  tsa.cmd.handlers.UsergrpsHandler.prototype._new = function()
  {  
    /** @type {String}                  */ var userGroupPID = this.getParameter( 'entityId' );
    /** @type {String}                  */ var actionURL    = (userGroupPID!=null ? 'usergrps/putUserGrps':'usergrps/postUserGrps');
    /** @type {Object}                  */ var i18n         = tsa.i18n;
    /** @type {tsa.widgets.ContentArea} */ var contentArea  = new ContentArea ( { title : i18n.labelUserGroupsHeading
                                                                               , showBackToTop : true 
                                                                               , helpDoc : 'sshelp_user_group_panel.html' } );
    /** @type {Object}                  */ var i18n         = tsa.i18n;
    /** @type {tsa.widgets.FormPanel}   */ var userGrpForm  = new FormPanel( {  title       : i18n.labelUserAccountGeneralHeading
                                                                              , action      : actionURL
                                                                              , description : i18n.labelUserAccountGeneralDesc } );
    if(userGroupPID!=null)
    {
        userGrpForm.putHiddenInput( 'pid', userGroupPID );
    }
    /** @type {tsa.widgets.FormPanel}   */ var authorityForm   = new FormPanel( {  title       : i18n.labelUserGroupAuthorityHeading
                                                                                  , description : i18n.labelUserGroupAuthorityDesc } );
    /** @type {tsa.widgets.FormPanel}   */ var scopesetsForm   = new FormPanel( {  title       : i18n.labelUserGroupScopeSetHeading
                                                                                , description : i18n.labelUserGroupScopeSetDesc } );
    /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
    /** @type {tsa.cmd.Command}         */ var getScopesetsCmd = new Command( { action              : 'scopesets/getScopeSets'
                                                                               ,parameters          : { 'filter' : 'ALL' }
                                                                              , clearSystemMessages : false } );
    userGrpForm.linkTo(authorityForm);
    authorityForm.linkTo(scopesetsForm);
    
    //fill General Form
    /** @type {Element} */ var inputName           = $( '<input size="50" maxlength="250" name="name" id="name" value="" type="text">' ).get(0);
    /** @type {Element} */ var inputDesc           = $( '<input size="50" maxlength="250" name="desc" id="desc" value="" type="text">' ).get(0);
    
    userGrpForm.appendFieldRow( {    label : i18n.labelUserGroupName
                                , required : true  
                                , element  : inputName 
                                , hint     : i18n.labelUserGroupNameDesc } );
  
    userGrpForm.appendFieldRow( { label: i18n.labelUserGroupDesc
                                  , required: false
                                  , element: inputDesc
                                  , hint: i18n.labelUserGroupDescDesc } );
    
    //fill the Authority level form
    /** @type {element} */ var selectAuth  = $('<select name="authority" id="authority" size="1"></select>').get(0);
    
    var changeHandler = function(e) {
        /** @type {boolean} */ var disabled = ( $( selectAuth ).val() === 'DISCOVERY');
        if(disabled)
        {
            $('input[type=checkbox]').removeAttr('disabled');
        }else{
            $('input[type=checkbox]').attr('disabled','true');
        }
    };
    $(selectAuth).on("change", changeHandler);
    
    /** @type {element} */ var option = document.createElement( 'option' );
    $( option ).attr('value'    , 'ADMINISTRATOR' );
    $( option ).attr('selected' , 'selected');
    $( option ).text(i18n.labelAuthorityAdministrator );
    $( selectAuth ).append(option);
    
    option   = document.createElement( 'option' );
    $( option ).attr( 'value', 'DISCOVERY' );
    $( option ).text( i18n.labelAuthorityDiscovery );
    $( selectAuth ).append(option);
    
    option   = document.createElement( 'option' );
    $( option ).attr( 'value', 'VISITOR' );
    $( option ).text( i18n.labelAuthorityVisitor );
    $(selectAuth).append(option);
    
    authorityForm.appendFieldRow( {  label     : i18n.labelUserGroupAuthority
                                    , required : true
                                    , element  : selectAuth
                                    , hint     : i18n.labelUserGroupDescDesc } );
    
    var scopeSetGroup = $("<span class='ibm-input-group' role='group' aria-label='Scopes'></span>").get(0);
    $( getScopesetsCmd ).on( 'executionSucceeded', $.proxy( function( event ) {
        
        if( !TV.typeIsUndefined( typeof event.response.payload.scopeSets ) )
        { 
          $.each(event.response.payload.scopeSets, function(idxScopeset, element)
          {
            var scopesetId    = "scopeset" + idxScopeset;
            var scopesetInput = $("<input name='scopesets' class='checkBoxClass' type='checkbox'>").get(0);
            $(scopesetInput).attr("id", scopesetId);
            $(scopesetInput).attr("value", element.pid);
            var scopeLabel = $("<label  class='checkBoxLabelClass' ></label>").get(0);
            $(scopeLabel).text(element.displayName);
            $(scopeLabel).attr("for", scopesetId);
            $(scopeSetGroup).append(scopesetInput, scopeLabel, document.createElement("br"));
          });
         
          scopesetsForm.appendFieldRow( {  label    : i18n.labelUserGroupScopeSetName
                                                      , required : true
                                                      , aRequire : false
                                                      , element  : scopeSetGroup } );
          //Disable scopesets checkbox if Discovery level Auth is selected.
          $( selectAuth).trigger( "change" );
          
          if (userGroupPID!=null) 
          {
            var getScopeSetAssigned = new Command( { action              : 'usergrps/getUserGrp/'+ userGroupPID
                                                   , clearSystemMessages : false } );
            getScopeSetAssigned.execute();
            $( getScopeSetAssigned ).on( 'executionSucceeded', function( event )
            {
               for( var key in event.response.payload.object ) {
                  $( '[name="' + key + '"]' ).val( event.response.payload.object[ key ] );
                }
                $.each(event.response.payload.scopesets, function(index, scopeset) 
                        {
                            $("input:checkbox[value="+scopeset.pid+"]").attr("checked", true);
                        });
            }); 
          }
        }
    }, this ) );
    
    // Buttons...
    $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                            , label   : i18n.buttonSave
                                                            , onclick : function( event ) { 
                                                                userGrpForm.submit(); 
                                                                } } ) );
    $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
    $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                            , label   : i18n.buttonCancel
                                                            , onclick : function( event ) { 
                                                                ( new ShowCommand( { action : 'usergrps/index' } )).execute(); 
                                                                } } ) );
    
    getScopesetsCmd.execute();      
    
    $( userGrpForm ).on( 'executionFailed', function( event )
    {
      if ( TV.typeIsObject( typeof event.response.payload.errors ) ) 
      {
        for ( /** @type {string} */ var fieldName in event.response.payload.errors ) 
        {
            userGrpForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
        }
          delete event.response.payload.errors;
      }
    } );
    
    $( userGrpForm ).on( 'executionSucceeded', function( event )
            {
                  ( new ShowCommand( { action : 'usergrps/index' } ) ).execute();
            } );
    
    contentArea.append( userGrpForm.getDomNode() );
    contentArea.append( authorityForm.getDomNode() );
    contentArea.append( scopesetsForm.getDomNode() );
    contentArea.append( buttonContainer );
    
    
    this.show( contentArea.getDomNode() );
  };
  // -------------------------------------------------------------------------
  /**
  * Edit user account - event handler.
  * 
  * @param {string}
  *            The user account PID.
  */
  tsa.cmd.handlers.UsergrpsHandler.editUserGrpClicked = function(userGrpPID) 
  {
    (new ShowCommand({ action : 'usergrps/edit/' + userGrpPID })).execute();
  };
  //-------------------------------------------------------------------------
  tsa.cmd.handlers.UsergrpsHandler.prototype._edit = function()
  {
   this._new();
	  
  };
  //-------------------------------------------------------------------------
  
  
  /**
   *Confirm deleting a group
   */
  tsa.cmd.handlers.UsergrpsHandler.confirmDeleteGroup = function(name, pid)
  {                
      if( window.confirm( tsa.i18n.labelDeleteConfirmation + ' "' + name + '"?' ) )
      {
            /** @type {tsa.cmd.Command} */ var deleteCmd = new Command( { action     : 'usergrps/deleteUserGrps/' + pid } );
            $( deleteCmd ).on( 'executionSucceeded', function( event )
            { 
                
                var urlRedirect= 'usergrps/index';
                
              ( new ShowCommand( { action              : urlRedirect
                                 , clearSystemMessages : false } ) ).execute();
            } );
           deleteCmd.execute();
      }
  };
  
  
  //-------------------------------------------------------------------------

    return tsa.cmd.handlers.UsergrpsHandler;
  }
);