/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2019
 */

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV
          )
  {
    //---------------------------------------------------------------------------
    
    /**
     * This module is not a constructor, but a set of data and functions.
     */
    tsa.data.TimeZones = {};
    
    //---------------------------------------------------------------------------
    
    /**
     * The data, as an array, sorted by GMT offset.
     * @private
     * @type {Array.<Object>}
     */
    tsa.data.TimeZones._data =
    [    { id : 'GB'                   , name : { label : 'labelTimeZoneGB' } }
       , { id : 'CET'                  , name : { label : 'labelTimeZoneCET' } }
       , { id : 'EET'                  , name : { label : 'labelTimeZoneEET' } }
       , { id : 'Asia/Amman'           , name : { label : 'labelTimeZoneAsiaAmman' } }
       , { id : 'Asia/Jerusalem'       , name : { label : 'labelTimeZoneAsiaJerusalem' } }
       , { id : 'Africa/Nairobi'       , name : { label : 'labelTimeZoneAfricaNairobi' } }
       , { id : 'Iran'                 , name : { label : 'labelTimeZoneIran' } }
       , { id : 'Asia/Baku'            , name : { label : 'labelTimeZoneAsiaBaku' } }
       , { id : 'Asia/Kabul'           , name : { label : 'labelTimeZoneAsiaKabul' } }
       , { id : 'Etc/GMT-5'            , name : { label : 'labelTimeZoneEtcGMT5' } }
       , { id : 'Asia/Calcutta'        , name : { label : 'labelTimeZoneAsiaCalcutta' } }
       , { id : 'Asia/Kathmandu'       , name : { label : 'labelTimeZoneAsiaKathmandu' } }
       , { id : 'Etc/GMT-6'            , name : { label : 'labelTimeZoneEtcGMT6' } }
       , { id : 'Asia/Rangoon'         , name : { label : 'labelTimeZoneAsiaRangoon' } }
       , { id : 'Asia/Jakarta'         , name : { label : 'labelTimeZoneAsiaJakarta' } }
       , { id : 'Asia/Taipei'          , name : { label : 'labelTimeZoneAsiaTaipei' } }
       , { id : 'Australia/Perth'      , name : { label : 'labelTimeZoneAustraliaPerth' } }
       , { id : 'Japan'                , name : { label : 'labelTimeZoneJapan' } }
       , { id : 'Australia/Adelaide'   , name : { label : 'labelTimeZoneAustraliaAdelaide' } }
       , { id : 'Australia/Sydney'     , name : { label : 'labelTimeZoneAustraliaSydney' } }
       , { id : 'Etc/GMT-11'           , name : { label : 'labelTimeZoneEtcGMT11' } }
       , { id : 'Pacific/Auckland'     , name : { label : 'labelTimeZonePacificAuckland' } }
       , { id : 'Pacific/Fiji'         , name : { label : 'labelTimeZonePacificFiji' } }
       , { id : 'Pacific/Chatham'      , name : { label : 'labelTimeZonePacificChatham' } }
       , { id : 'Pacific/Tongatapu'    , name : { label : 'labelTimeZonePacificTongatapu' } }
       , { id : 'Etc/GMT+12'           , name : { label : 'labelTimeZoneEtcGMT12' } }
       , { id : 'US/Samoa'             , name : { label : 'labelTimeZoneUSSamoa' } }
       , { id : 'HST'                  , name : { label : 'labelTimeZoneHST' } }
       , { id : 'US/Alaska'            , name : { label : 'labelTimeZoneUSAlaska' } }
       , { id : 'PST8PDT'              , name : { label : 'labelTimeZonePST8PDT' } }
       , { id : 'MST7MDT'              , name : { label : 'labelTimeZoneMST7MDT' } }
       , { id : 'Mexico/BajaSur'       , name : { label : 'labelTimeZoneMexicoBajaSur' } }
       , { id : 'CST6CDT'              , name : { label : 'labelTimeZoneCST6CDT' } }
       , { id : 'Mexico/General'       , name : { label : 'labelTimeZoneMexicoGeneral' } }
       , { id : 'EST5EDT'              , name : { label : 'labelTimeZoneEST5EDT' } }
       , { id : 'America/Havana'       , name : { label : 'labelTimeZoneAmericaHavana' } }
       , { id : 'Canada/Atlantic'      , name : { label : 'labelTimeZoneCanadaAtlantic' } }
       , { id : 'Chile/Continental'    , name : { label : 'labelTimeZoneChileContinental' } }
       , { id : 'Brazil/West'          , name : { label : 'labelTimeZoneBrazilWest' } }
       , { id : 'America/Asuncion'     , name : { label : 'labelTimeZoneAmericaAsuncion' } }
       , { id : 'Canada/Newfoundland'  , name : { label : 'labelTimeZoneCanadaNewfoundland' } }
       , { id : 'America/Buenos_Aires' , name : { label : 'labelTimeZoneAmericaBuenos_Aires' } }
       , { id : 'Brazil/East'          , name : { label : 'labelTimeZoneBrazilEast' } }
       , { id : 'America/Montevideo'   , name : { label : 'labelTimeZoneAmericaMontevideo' } }
       , { id : 'Etc/GMT+2'            , name : { label : 'labelTimeZoneEtcGMT2' } }
       , { id : 'Atlantic/Azores'      , name : { label : 'labelTimeZoneAtlanticAzores' } }
    ];
    
    //---------------------------------------------------------------------------
    
    /**
     * The data, as a map (by ID).
     * @private
     * @type {Object.<string,Object>}
     */
    tsa.data.TimeZones._dataMap = {};
    
    $.each( tsa.data.TimeZones._data, function( index, timezone )
    {
      tsa.data.TimeZones._dataMap[ timezone.id ] = timezone;
    } );
    
    //---------------------------------------------------------------------------
    
    /**
     * Access method: get TimeZone by ID.
     * @param {string} id The TimeZone ID.
     * @return {Object}   The TimeZone data, or null if ID is not found.
     */
    tsa.data.TimeZones.getById = function( id )
    {
      /** @type {Object} */var timezone = null;
      
      if( TV.typeIsString( typeof id ) )
      {
        if( TV.typeIsObject( typeof tsa.data.TimeZones._dataMap[ id ] ) )
        {
          timezone = tsa.data.TimeZones._dataMap[ id ];
        }
      }
      
      return timezone;
    };
    
    //---------------------------------------------------------------------------
    
    /**
     * Access method: get TimeZones sorted by GMT offset.
     * @return {Array.<Object>} The TimeZones data.
     */
    tsa.data.TimeZones.getTimeZones = function()
    {
      return tsa.data.TimeZones._data.slice();
    };
    
    //---------------------------------------------------------------------------
    
    return tsa.data.TimeZones;
  }
);