/*
 * IBM Technical Support Appliance Version 2 Release 0 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator' ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ButtonFactory' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  This class must not be instantiated. */
    
    tsa.widgets.ButtonFactory = function( parameters )
    {
      throw 'This class must not be instantiated.';
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build button.
     * Parameters:
     *   type    : 'cart'     , 'arrow'    , 'cancel'   , 'customize' ,
     *             'download' , 'view'     , 'add'      , 'back'      ,
     *             'phone'    , 'chat'     , 'email'    , 'refresh'   ,
     *             'remove'   , 'undo'     , 'video'    , 'pda'
     *   grayout : Grayed-out  version of the button. Not all buttons support
     *             both versions. Optional; default: false.
     *   label   : The label of the button.
     *   name    : The name attribute. Optional. Default: none.
     *   onclick : Click event handler.
     *   
     * @param {{ type    : string
     *         , grayout : boolean
     *         , label   : string
     *         , onclick : function(Event)}} parameters The parameters.
     * @return {Element}
     */
    tsa.widgets.ButtonFactory.buildButton = function( parameters )
    {
      /** @type {Element} */ var button = null;
      
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        /** @type {string}   */ var type    = ( TV.typeIsString   ( typeof parameters.type    ) ? parameters.type    : 'arrow' );
        /** @type {boolean}  */ var grayout = ( TV.typeIsBoolean  ( typeof parameters.grayout ) ? parameters.grayout : false   );
        /** @type {string}   */ var label   = ( TV.typeIsString   ( typeof parameters.label   ) ? parameters.label   : '?'     );
        /** @type {string}   */ var name    = ( TV.typeIsString   ( typeof parameters.name    ) ? parameters.name    : ''      );
        /** @type {Function} */ var onclick = ( TV.typeIsFunction ( typeof parameters.onclick ) ? parameters.onclick : null    );
        /** @type {number}   */ var baseFS  = parseInt( $( 'body' ).css( 'font-size' ), 10 );
        
        button = document.createElement( 'input' );
        
        $( button ).attr( 'type', 'button' );
        $( button ).attr( 'value', label );
        $( button ).addClass( 'ibm-btn-' + type + '-' + ( grayout ? 'sec' : 'pri' ) );
        
        if( onclick !== null )
        {
          $( button ).on( 'click', onclick );
        }
        
        $( button ).css( { 'margin'    : '0px'
                         , 'font-size' : ( isNaN( baseFS ) ? 12 : Math.round( baseFS * 0.75 ) ) + 'px' } );
        
        // Set a name?
        if( name.length > 0 )
        {
          $( button ).attr( 'name', name );
        }
      }
      else
      {
        logger.error( 'Invalid button data.' );
      }
      
      return button;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build button separator.
     * @return {Element}
     */
    tsa.widgets.ButtonFactory.buildButtonSeparator = function()
    {
      /** @type {Element} */ var separator = document.createElement( 'span' );
      
      $( separator ).addClass( 'ibm-sep' );
      $( separator ).text( tsa.NBSP );
      
      return separator;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build button container.
     * @return {Element}
     */
    tsa.widgets.ButtonFactory.buildButtonContainer = function()
    {
      /** @type {Element} */ var container = document.createElement( 'p' );
      
      $( container ).addClass( 'ibm-buttons-row' );
      
      return container;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.widgets.ButtonFactory;
  }
);