/*
 * IBM Technical Support Appliance Version 2 Release 2 Modification 0
 *
 * (C) Copyright IBM Corp. 2017
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator' ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'IconFactory' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  This class must not be instantiated. */
    
    tsa.widgets.IconFactory = function( parameters )
    {
      throw 'This class must not be instantiated.';
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build action icon.
     * Parameters:
     *   type    : 'delete'
     *             'download'
     *             'edit'
     *             'movedown'
     *             'moveup'
     *             'cancel'
     *   caption : Optional. Caption to be displayed when mouse is over. Also used as "alt" text.
     * @param {{type:string}} parameters The parameters.
     * @return {Element}
     */
    tsa.widgets.IconFactory.buildActionIcon = function( parameters )
    {
      /** @type {Element} */ var span = null;
      
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        /** @type {string}  */ var type = ''; // unknown
        /** @type {Element} */ var icon = document.createElement( 'img' );
        /** @type {string}  */ var src  = '';
        
        switch( type = ( parameters.type + '' ) )
        {
          case 'active'   : src = 'active16.gif'  ; break;
          case 'collapse' : src = 'collapse16.png' ; break;
          case 'expand'   : src = 'expand16.png'  ; break;
          case 'expandb'  : src = 'expandb16.png' ; break;
          case 'offline'  : src = 'offline16.gif' ; break;
          case 'runb'     : src = 'run16b.png'    ; break;
          case 'info'     : src = 'info16.gif'    ; break;
          case 'delete'   : src = 'delete16.gif'  ; break;
          case 'download' : src = 'dn16.gif'      ; break;
          case 'edit'     : src = 'edit16.gif'    ; break;
          case 'movedown' : src = 'downb16.gif'   ; break;
          case 'moveup'   : src = 'upb16.png'     ; break;
          case 'cancel'   : src = 'cancel16.gif'  ; break;
          case 'stop'     : src = 'stop16.gif' ; break;
          case 'refresh'  : src = 'refresh16.gif' ; break;
          case 'bundle'   : src = 'bundle16.gif'  ; break;
          case 'bundlelib': src = 'bundle_library16.gif'  ; break;
          case 'bundlefrg': src = 'bundle_fragment16.gif'  ; break;
          default         : src = 'warn16.gif'
        }
        
        src  = tsa.attributes.imagesPath + '/' + src;
        span = document.createElement( 'span' );
        
        $( icon ).attr( 'src', src  );
        $( icon ).css( { 'vertical-align' : 'middle' } );
        $( span ).css( { 'white-space' : 'nowrap' } );
        $( span ).append( icon );
        
        if( TV.typeIsString( typeof parameters.caption ) )
        {
          $( icon ).attr( 'alt'   , parameters.caption );
          $( icon ).attr( 'title' , parameters.caption );
        }
      }
      else
      {
        logger.error( 'Invalid icon data.' );
      }
      
      return span;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build severity icon.
     * Parameters:
     *   severity : 0 or 'OK'    => Ok.
     *              1 or 'INFO'  => Information.
     *              2 or 'WARN'  => Warning.
     *              4 or 'ERROR' => Error.
     *              8 or 'CANC'  => Canceled.
     *              Any other    => Unknown.
     *   withText : Use true for adding text, false for the icon only. Optional. Default. false.
     *   
     * @param {{ severity : (string|number)
     *         , withText : boolean }} parameters The parameters.
     * @return {Element}
     * 
     */
    tsa.widgets.IconFactory.buildSeverityIcon = function( parameters )
    {
      /** @type {Element} */ var span = null;
      
      if( TV.typeIsObject( typeof parameters ) && ( parameters !== null ) )
      {
        /** @type {string}  */ var sev      = ''; // unknown
        /** @type {boolean} */ var withText = TV.typeIsBoolean( typeof parameters.withText ) && parameters.withText;
        /** @type {Element} */ var icon     = document.createElement( 'img' );
        /** @type {string}  */ var src      = '';
        /** @type {string}  */ var text     = '';
        
        if( TV.typeIsNumber( typeof parameters.severity ) )
        {
          switch( parameters.severity )
          {
            case 0: sev = 'OK'    ; break;
            case 1: sev = 'INFO'  ; break;
            case 2: sev = 'WARN'  ; break;
            case 4: sev = 'ERROR' ; break;
            case 8: sev = 'CANC'  ; break;
          }
        }
        else if( TV.typeIsString( typeof parameters.severity ) )
        {
          sev = parameters.severity.toUpperCase();
        }
        
        switch( sev )
        {
          case 'OK'    : src = 'Ok'      ; text = 'Ok'        ; break;
          case 'INFO'  : src = 'Info'    ; text = 'Info'      ; break;
          case 'WARN'  : src = 'Warn'    ; text = 'Warning'   ; break;
          case 'ERROR' : src = 'Error'   ; text = 'Error'     ; break;
          case 'CANC'  : src = 'Warn'    ; text = 'Cancelled' ; break;
          default      : src = 'Unknown' ; text = 'Unknown'   ;
        }
        
        src  = tsa.attributes.imagesPath + '/' + tsa.i18n[ 'imageStatus' + src + '16' ];
        text = tsa.i18n[ 'label' + text ];
        
        span = document.createElement( 'span' );
        
        $( icon ).attr( 'alt', text );
        $( icon ).attr( 'src', src  );
        $( icon ).css( { 'vertical-align' : 'middle' } );
        $( span ).css( { 'white-space' : 'nowrap' } );
        $( span ).append( icon );
        
        if( withText )
        {
          $( span ).append( document.createTextNode( tsa.NBSP + text ) );
        }
      }
      else
      {
        logger.error( 'Invalid icon data.' );
      }
      
      return span;
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.widgets.IconFactory;
  }
);