/*
 * IBM Technical Support Appliance Version 2 Release 6 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2020
 */

define(
  
  [ 'jquery'
  , 'tsa/TypeVerificator'
  ],
  
  function( $
          , TV )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'LeftNav' );
    
    //-------------------------------------------------------------------------
    /**
     * @constructor
     * @param {Object} parameters (none so far).
     */
    tsa.widgets.LeftNav = function( parameters )
    {
      /** @type {Array.<tsa.widgets.LeftNav.MenuItem>} */ var primaryMenuItems = [];
      
      tsa.widgets.LeftNav.instance = this;
      
      // All flags are initially set:
      this.flags = { whole : true    // Show the whole menu.
                   , debug : true    // Show the debug entries.
                   , dev   : true    // Show the development entries.
                   };
      
      this.domNode = $( '<div id="ibm-navigation" class="ibm-parent" aria-labelledby="ibm-access-navigation" role="navigation"></div>' ).get(0);
      $( this.domNode ).append( '<h2 id="ibm-access-navigation" class="ibm-access" style="display: none;">' + tsa.i18n.labelNavMinifiedCaption + '</h2>' );
      $( this.domNode ).append( this.minifiedElement = $( '<div>' + tsa.i18n.labelNavMinifiedCaption + ' ' + String.fromCharCode( 0x25bc ) + '</div>' ).get(0) );
      $( this.domNode ).append( this.rootMenuItemContainer = $( '<ul id="ibm-primary-links"></ul>' ).get(0) );
      $( this.domNode ).append( this.rootMenuSecondaryItemContainer = $( '<div id="ibm-secondary-navigation" aria-label="Navigation Secondary Menu" ></div>' ).get(0) );
      $(this.rootMenuSecondaryItemContainer).append('<h2>' + tsa.i18n.labelNavRelatedLinks + '</h2>');
      $(this.rootMenuSecondaryItemContainer).append($.parseHTML('<ul id="ibm-related-links"><li><a id="aNetworkID" href="#"></a></li></ul>'));
      $(this.rootMenuSecondaryItemContainer).hide();
      
      $( this.domNode         ).css( { 'border-radius'       : '0px 0px 5px 0px' } );
      $( this.minifiedElement ).css( { 'border-radius'       : '0px 0px 5px 0px'
                                     , 'background-color'    : '#c8d7e3'
                                     , 'color'               : '#293d6b'
                                     , 'border-top'          : '1px solid #e6edf2'
                                     , 'border-bottom'       : '1px solid #98b1c4'
                                     , 'border-right'        : '1px solid #98b1c4'
                                     , 'padding'             : '6px 10px'
                                     , 'width'               : '160px'
                                     , 'height'              : '14px'
                                     , 'font-family'         : 'arial,sans-serif'
                                     , 'font-size'           : '0.7em'
                                     , 'font-weight'         : 'bold' } );
      
      $.each( tsa.widgets.LeftNav.dataArray, $.proxy( function( dataIndex, data )
      {
        /** @type {tsa.widgets.LeftNav.MenuItem} */ var menuItem  = tsa.widgets.LeftNav.buildMenuItem( data );
        /** @type {Array.<Element>}              */ var listItems = menuItem.asHtml();
        
        primaryMenuItems.push( menuItem );
        
        $.each( listItems, $.proxy( function( listItemIndex, listItem )
        {
          $( this.rootMenuItemContainer ).append( listItem );
        }
        , this ) );
      }
      , this ) );
      
      this.rootMenuItem = new tsa.widgets.LeftNav.MenuItem( { id       : ''
                                                            , parent   : null
                                                            , children : primaryMenuItems
                                                            , label    : '' } );
      
      logger.info('LeftNav instance created.');
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * @constructor
     * Inner class: MenuItem
     * @param {{ id       : string
     *         , parent   : tsa.widgets.LeftNav.MenuItem
     *         , children : Array.<tsa.widgets.LeftNav.MenuItem>
     *         , label    : string
     *         , handler  : function(Event)}} parameters
     */
    tsa.widgets.LeftNav.MenuItem = function( parameters )
    {
      if( !TV.typeIsUndefined( typeof parameters.id       ) ) this.id       = parameters.id       ;
      if( !TV.typeIsUndefined( typeof parameters.label    ) ) this.label    = parameters.label    ;
      if( !TV.typeIsUndefined( typeof parameters.parent   ) ) this.parent   = parameters.parent   ;
      if( !TV.typeIsUndefined( typeof parameters.children ) ) this.children = parameters.children ;
      if( !TV.typeIsUndefined( typeof parameters.handler  ) ) this.handler  = parameters.handler  ;
    };
    
    /** @type {string}                               */ tsa.widgets.LeftNav.MenuItem.prototype.id                 = '';
    /** @type {string}                               */ tsa.widgets.LeftNav.MenuItem.prototype.label              = '';
    /** @type {tsa.widgets.LeftNav.MenuItem}         */ tsa.widgets.LeftNav.MenuItem.prototype.parent             = null;
    /** @type {Array.<tsa.widgets.LeftNav.MenuItem>} */ tsa.widgets.LeftNav.MenuItem.prototype.children           = null;
    /** @type {function(Event)}                      */ tsa.widgets.LeftNav.MenuItem.prototype.handler            = null;
    /** @type {Element}                              */ tsa.widgets.LeftNav.MenuItem.prototype.domNode            = null;
    /** @type {Element}                              */ tsa.widgets.LeftNav.MenuItem.prototype.domNodeForChildren = null;
    
    /** @return {string}                               */ tsa.widgets.LeftNav.MenuItem.prototype.getId                 = function() { return this.id                 ; };
    /** @return {string}                               */ tsa.widgets.LeftNav.MenuItem.prototype.getLabel              = function() { return this.label              ; };
    /** @return {tsa.widgets.LeftNav.MenuItem}         */ tsa.widgets.LeftNav.MenuItem.prototype.getParent             = function() { return this.parent             ; };
    /** @return {Array.<tsa.widgets.LeftNav.MenuItem>} */ tsa.widgets.LeftNav.MenuItem.prototype.getChildren           = function() { return this.children           ; };
    /** @return {function(Event)}                      */ tsa.widgets.LeftNav.MenuItem.prototype.getHandler            = function() { return this.handler            ; };
    /** @return {Element}                              */ tsa.widgets.LeftNav.MenuItem.prototype.getDomNode            = function() { return this.domNode            ; };
    /** @return {Element}                              */ tsa.widgets.LeftNav.MenuItem.prototype.getDomNodeForChildren = function() { return this.domNodeForChildren ; };
    
    /** @param {string}                               p */ tsa.widgets.LeftNav.MenuItem.prototype.setId       = function( p ) { this.id       = p; };
    /** @param {string}                               p */ tsa.widgets.LeftNav.MenuItem.prototype.setLabel    = function( p ) { this.label    = p; };
    /** @param {tsa.widgets.LeftNav.MenuItem}         p */ tsa.widgets.LeftNav.MenuItem.prototype.setParent   = function( p ) { this.parent   = p; };
    /** @param {Array.<tsa.widgets.LeftNav.MenuItem>} p */ tsa.widgets.LeftNav.MenuItem.prototype.setChildren = function( p ) { this.children = p; };
    /** @param {function(Event)}                      p */ tsa.widgets.LeftNav.MenuItem.prototype.setHandler  = function( p ) { this.handler  = p; };
    
    /**
     * Does this item have children?
     * @return {boolean}
     */
    tsa.widgets.LeftNav.MenuItem.prototype.hasChildren = function()
    {
      return ( ( this.getChildren() !== null ) && ( this.getChildren().length > 0 ) );
    };
    
    /**
     * Does this item have parent?
     * @return {boolean}
     */
    tsa.widgets.LeftNav.MenuItem.prototype.hasParent = function()
    {
      return ( this.getParent() !== null );
    };
    
    /**
     * Show.
     */
    tsa.widgets.LeftNav.MenuItem.prototype.show = function()
    {
      $( this.getDomNode()            ).show();
      $( this.getDomNodeForChildren() ).hide();
    };
    
    /**
     * Hide.
     */
    tsa.widgets.LeftNav.MenuItem.prototype.hide = function()
    {
      $( this.getDomNode()            ).hide();
      $( this.getDomNodeForChildren() ).hide();
    };
    
    /**
     * Find a descendant.
     * @param 
     */
    tsa.widgets.LeftNav.MenuItem.prototype.find = function( descendantId )
    {
      /** @type {tsa.widgets.LeftNav.MenuItem} */ var descendant = null;
      
      if( ( this.hasChildren() ) && TV.typeIsString( typeof descendantId ) )
      {
        $.each( this.children, function( childIndex, child )
        {
          if( child.getId() === descendantId )
          {
            descendant = child;
          }
          else if( child.hasChildren() )
          {
            descendant = child.find( descendantId );
          }
          
          return ( descendant === null );
        } );
      }
      
      return descendant;
    };
    
    /**
     * Build the HTML List Item Element(s) representing this MenuItem.
     * @return {Array.<Element>}
     */
    tsa.widgets.LeftNav.MenuItem.prototype.asHtml = function()
    {
      /** @type {Array.<Element>} */ var items = [];
      /** @type {Element}         */ var item  = document.createElement( 'li' );
      /** @type {Element}         */ var link  = document.createElement( 'a' );
      
      this.domNode = item;
      
      $( link ).attr( 'href', 'javascript:void(0)' );
      $( link ).text( tsa.i18n[ this.getLabel() ] );
      // Create id to handle redirection.
      if(this.getId()==="main"){
        $( item ).attr( 'id', "menu" + this.getId() );
      }
      $( item ).append( link );
      items.push( item );
      
      if( this.hasChildren() )
      {
        /** @type {Element} */ var itemForChildren = document.createElement( 'li' );
        /** @type {Element} */ var ulForChildren   = document.createElement( 'ul' );
        
        this.domNodeForChildren = itemForChildren;
        
        $( itemForChildren ).addClass( 'ibm-submenu' );
        $( itemForChildren ).css( { 'list-style' : 'none'
                                  , 'display'    : 'none' /* to show, use 'inline' */ } );
        
        $.each( this.getChildren(), function( childIndex, child )
        {
          /** @type {Array.<Element>} */ var childItems = child.asHtml();
          
          $.each( childItems, function( childItemIndex, childItem )
          {
            $( ulForChildren ).append( childItem );
          } );
        } );
        
        $( itemForChildren ).append( ulForChildren );
        items.push( itemForChildren );
      }
      
      $( link ).on( 'click', $.proxy( function( event )
      {
        if( !tsa.widgets.LeftNav.getInstance().isBlocked() )
        {
          if( tsa.confirmChangesCanBeDiscarded() )
          {
            /** @type {boolean} */ var isCollapsed = ( this.hasChildren() && ( $( this.domNodeForChildren ).css( 'display' ) === 'none' ) );
            
            // Remove the active flag from all other links:
            $( this.domNode ).closest( 'div' ).find( 'a' ).removeClass( 'ibm-is-active' );
            
            // Add the active flag to this link:
            $( this.domNode ).find( 'a' ).addClass( 'ibm-is-active' );
            
            // First level menu?
            if( !this.hasParent() )
            {
              // Close all other sub-menus:
              $( this.domNode ).closest( 'div' ).find( '.ibm-submenu' ).css( { 'display' : 'none' } );
            
              // Has children?
              if( this.hasChildren() )
              {
                // Show (or hide) this sub-menu:
                $( this.domNodeForChildren ).css( { 'display' : isCollapsed ? 'inline' : 'none' } );
              }
            }
            
            require( [ 'tsa/cmd/ShowCommand' ], $.proxy( function( ShowCommand )
            {
              /** @type {string} */ var command = 'index';
              /** @type {string} */ var handler = this.getId();
              
              ( new ShowCommand( { action : ( handler + '/' + command ) } ) ).execute();
            }
            , this ) );
          }
        }
        
        return false;
      }
      , this ) );
      
      return items;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * The instance.
     * @private
     * @type {tsa.widgets.LeftNav}
     */
    tsa.widgets.LeftNav.instance = null;
    
    /** Maps MenuItem ID to flags.
     *  @private
     *  @type {Object.<string,string>}
     */
    tsa.widgets.LeftNav.showMap = { 'debug'             : 'debug'
                                  , 'source'            : 'debug'
                                  , 'notifications'     : 'debug'
                                  , 'entityidsettings'  : 'debug'
                                  , 'discoverysettings' : 'debug'
                                  , 'timeouts'          : 'debug'
                                  , 'dev'               : 'dev' };
    
    /**
     * The array of menu data.
     * @const
     * @private
     * @type {Array.<Object>}
     */
    tsa.widgets.LeftNav.dataArray =
      [ { id : 'main'             , label    : 'labelNavSummary'          }
      , { id : 'logs'             , label    : 'labelNavActivityLog'      }
      , { id : 'inventorysummary' , label    : 'labelNavInventorySummary'
                                  , children : [ { id : 'inventorysettings'    , label : 'labelNavInventorySettings'    } ] }
      , { id : 'scopesetsmenu'    , label    : 'labelNavScopeSetCaption'
                                  , children : [   { id : 'scopesets'       , label : 'labelGeneralScopeSet'}
                                	              ,{ id : 'scopesettings'   , label : 'labelImportNavScopeset'}
                                                  ,{ id : 'scopesetshmc'    , label : 'labelNavScopesetHMC'  }
                                                  ,{ id : 'scopesetsvmware' , label : 'labelNavScopesetVmware'  }  ] }
      , { id : 'credentials'      , label    : 'labelNavDiscoCreds'       }
      , { id : 'discovery'        , label    : 'labelNavDiscoSched'       }
      , { id : 'discoruns'        , label    : 'labelNavDiscoHistory'     }
      , { id : 'discosettings'    , label    : 'labelNavDiscoSettings'
                                  , children : [ { id : 'discoverysettings'    , label : 'labelNavDiscoverySettings'    } // Only show on debug flag set
                                               , { id : 'timeouts'             , label : 'labelNavTimeouts'    }          // Only show on debug flag set
                                               , { id : 'connsettings'         , label : 'labelNavConnSettings'         } ] }
      , { id : 'transmission'     , label    : 'labelNavCollectionSched'  }
      , { id : 'admin'            , label    : 'labelNavAdminCaption'
                                  , children : [ { id : 'registration'         , label : 'labelNavRegistration'         }
                                               , { id : 'licenseview'          , label : 'labelNavLicenseView' 			}
                                               , { id : 'clock'                , label : 'labelNavClock'                }
                                               , { id : 'network'              , label : 'labelNavNetwork'              }
                                               , { id : 'connectivity'         , label : 'labelNavIbmConn'              }
                                               , { id : 'useraccts'            , label : 'labelNavUsers'                }
                                               , { id : 'password'             , label : 'labelNavPassword'             }
                                               , { id : 'security'             , label : 'labelNavSecurity'             }
                                               , { id : 'certificates'         , label : 'labelNavCertificates'         }
                                               , { id : 'backup'               , label : 'labelNavBackup'               }
                                               , { id : 'update'               , label : 'labelNavUpdate'               }
                                               , { id : 'support'              , label : 'labelNavSupport'              }
                                               , { id : 'schedrestart'         , label : 'labelNavScheduledRestart'     }
                                               , { id : 'datasnapshot'         , label : 'labelNavDataSnapshot'         }
                                               , { id : 'shutdown'             , label : 'labelNavShutdown'             } ] }
      , { id : 'tools'            , label    : 'labelNavToolsCaption'
                                  , children : [ { id : 'nettools'             , label : 'labelNavNettools'             }
                                               , { id : 'unknowndevices'       , label : 'labelNavUnknownDevices'       }
                                               , { id : 'authenticationstatus' , label : 'labelNavAuthenticationStatus' }
                                               , { id : 'dbtools'              , label : 'labelNavDBtools'              }
                                               , { id : 'setupwizard'          , label : 'labelNavSetupwizard'          }
                                               , { id : 'entityidsettings'     , label : 'labelNavEntityIdSettings'     } ] } // Only show on debug flag set
      , { id : 'documentation'    , label    : 'labelNavDocumentation' }
      , { id : 'debug'            , label    : 'labelNavDebugCaption'          // Only show on debug flag set
                                  , children : [ { id : 'sensors'              , label : 'labelNavSensors'     }
                                               , { id : 'jvm'                  , label : 'labelNavJvm'         }
                                               , { id : 'osgi'                 , label : 'labelNavOsgi'        }
                                               , { id : 'console'              , label : 'labelNavConsole'     }
                                               , { id : 'firewall'             , label : 'labelNavFirewall'    }
                                               , { id : 'properties'           , label : 'labelNavProperties'  }
                                               , { id : 'sysfiles'             , label : 'labelNavSystemFiles' } ] }
      , { id : 'dev'              , label    : 'labelNavDev'                   // Only show on dev flag set
                                  , children : [ { id : 'devWidgets'           , label : 'labelNavDevWidgets' }
                                               , { id : 'devUITests'           , label : 'labelNavDevUITests' }
                                               , { id : 'devUpdate'            , label : 'labelNavDevUpdate'  } ] }
      ];
    
    /**
     * The HTML DOM Node.
     * @private
     * @type {Element}
     */
    tsa.widgets.LeftNav.prototype.domNode = null;
    
    /**
     * Blocked flag (if set it causes the instance to ignore user events).
     * @private
     * @type {boolean}
     */
    tsa.widgets.LeftNav.prototype.blocked = false;
    
    /**
     * Minified flag.
     * @private
     * @type {boolean}
     */
    tsa.widgets.LeftNav.prototype.minified = false;
    
    /**
     * Control flags (for enabling/disabling menu sections).
     * @private
     * @type {Object.<string,boolean>}
     */
    tsa.widgets.LeftNav.prototype.flags = null;
    
    /**
     * MenuItem root.
     * @private
     * @type {tsa.widgets.LeftNav.MenuItem}
     */
    tsa.widgets.LeftNav.prototype.rootMenuItem = null;
    
    /**
     * MenuItem HTML Container.
     * @private
     * @type {Element}
     */
    tsa.widgets.LeftNav.prototype.rootMenuItemContainer = null;
    /**
     * MenuItem HTML Secondary links Container.
     * @private
     * @type {Element}
     */
    tsa.widgets.LeftNav.prototype.rootMenuSecondaryItemContainer = null;
    
    /**
     * HTML Element to be displayed when this menu is minified.
     * @private
     * @type {Element}
     */
    tsa.widgets.LeftNav.prototype.minifiedElement = null;
    
    
    //-------------------------------------------------------------------------
    
    /**
     * Instance getter.
     * @return {tsa.widgets.LeftNav}
     */
    tsa.widgets.LeftNav.getInstance = function()
    {
      return tsa.widgets.LeftNav.instance;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Block the instance (causing it to ignore user events).
     */
    tsa.widgets.LeftNav.prototype.block = function()
    {
      this.blocked = true;
      
      $( this.domNode ).css( { 'opacity' : '0.7' } );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Unblock the instance (causing it to normally handle user events again).
     */
    tsa.widgets.LeftNav.prototype.unblock = function()
    {
      $( this.domNode ).css( { 'opacity' : '1.0' } );
      
      this.blocked = false;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Is the instance blocked (ignoring user events)?.
     * @return {boolean}
     */
    tsa.widgets.LeftNav.prototype.isBlocked = function()
    {
      return this.blocked;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Minify (yielding room for main content).
     * @param {boolean=} flag Optional. Default: true. If false, it will be shown the standard way.
     */
    tsa.widgets.LeftNav.prototype.minify = function( flag )
    {
      flag = !TV.typeIsBoolean( typeof flag ) || flag;
      this.minified = flag;
      
      $( this.domNode ).css( { 'left'       : ( flag ? '-10px' : '' )
                             , 'top'        : ( flag ? '-30px' : '' )
                             , 'padding'    : ( flag ?   '0px' : '' )
                             , 'box-shadow' : ( flag ? '0px 0px 10px 0px rgba(0,0,0,0.3)' : '' ) } );
      
      $( this.domNode ).off( 'mouseover' );
      $( this.domNode ).off( 'mouseout'  );
      
      if( flag )
      {
        $( this.minifiedElement       ).show();
        $( this.rootMenuItemContainer ).hide();
        $( this.domNode ).on( 'mouseover' , $.proxy( function( event ) { $( this.rootMenuItemContainer ).show(); $( this.minifiedElement ).hide(); }, this ) );
        $( this.domNode ).on( 'mouseout'  , $.proxy( function( event ) { $( this.rootMenuItemContainer ).hide(); $( this.minifiedElement ).show(); }, this ) );
      }
      else
      {
        $( this.minifiedElement       ).hide();
        $( this.rootMenuItemContainer ).show();
      }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Is the instance minified (yielding room for main content)?.
     * @return {boolean}
     */
    tsa.widgets.LeftNav.prototype.isMinified = function()
    {
      return this.minified;
    };
    
    //-------------------------------------------------------------------------
    /**
     * Set/clear the flags. Update the menu according to the changes.
     * @param {{debug : boolean
     *         ,dev   : boolean
     *         ,whole : boolean}} flagMap
     */
    tsa.widgets.LeftNav.prototype.setFlags = function( flagMap )
    {
      /** @type {Object.<string,boolean>} */ var changed = {};
      
      for( /** @type {string} */ var flagname in this.flags )
      {
        if( TV.typeIsBoolean( typeof flagMap[ flagname ] ) &&
            ( flagMap[ flagname ] !== this.flags[ flagname ] ) )
        {
          this.flags[ flagname ] = flagMap[ flagname ];
          changed[ flagname ]    = flagMap[ flagname ];
        }
      }
      
      if( this.domNode !== null )
      {
        if( TV.typeIsBoolean( typeof changed.whole ) )
        {
          $( this.domNode )[ changed.whole ? 'show' : 'hide' ]();
        }
      }
      
      if( this.rootMenuItem !== null )
      {
        for( var menuItemId in tsa.widgets.LeftNav.showMap )
        {
          /** @type {tsa.widgets.LeftNav.MenuItem} */ var mi       = this.rootMenuItem.find( menuItemId );
          /** @type {string}                       */ var flagname = tsa.widgets.LeftNav.showMap[ menuItemId ];
          
          if( ( mi !== null ) && TV.typeIsString  ( typeof flagname            )
                              && TV.typeIsBoolean ( typeof changed[ flagname ] ) )
          {
            /** @type {boolean} */ var flagval = changed[ flagname ];
            
            mi[ flagval ? 'show' : 'hide' ]();
          }
        }
      }
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build the MenuItem instance based on the given data (plain object).
     * @param {Object} data The data (see: tsa.widgets.LeftNav.dataArray).
     * @return {tsa.widgets.LeftNav.MenuItem} The resulting MenuItem instance.
     */
    tsa.widgets.LeftNav.buildMenuItem = function( data )
    {
      /** @type {tsa.widgets.LeftNav.MenuItem} */ var menuItem = null;
      
      if( TV.typeIsObject( typeof data ) && ( data !== null ) )
      {
        menuItem = new tsa.widgets.LeftNav.MenuItem( { id     : data.id
                                                     , label  : data.label } );
        
        if( TV.typeIsObject( typeof data.children ) && TV.isInstanceOf( data.children, Array ) )
        {
          /** @type {Array.<tsa.widgets.LeftNav.MenuItem>} */ var children = [];
          
          $.each( data.children, function( index, childData )
          {
            /** @type {tsa.widgets.LeftNav.MenuItem} */ var childMenuItem = null;
            
            children.push( childMenuItem = tsa.widgets.LeftNav.buildMenuItem( childData ) );
            
            childMenuItem.setParent( menuItem );
          } );
          
          menuItem.setChildren( children );
        }
      }
      else
      {
        logger.error( 'Couldn\'t build MenuItem: invalid data structure.' );
      }
      
      return menuItem;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Get the DOM Node.
     * @return {Element}
     */
    tsa.widgets.LeftNav.prototype.getDomNode = function()
    {
      return this.domNode;
    };
    /**
     * Focus main menu 
     */
    tsa.widgets.LeftNav.prototype.focusMainMenu = function(  )
    {
      var menuId    = '#menumain';
      var isActive = $( menuId ).find('a').hasClass( "ibm-is-active" );
      if( ! isActive){
        currentMenu= $(this.getDomNode()).find(".ibm-is-active");
        $( currentMenu ).removeClass( 'ibm-is-active' );
        $( "li.ibm-submenu" ).css( "display", "none" );
        $(menuId).find( 'a' ).addClass( 'ibm-is-active' );
      }
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.widgets.LeftNav;
  }
);