
/*******************************************************************************
* Purpose : To produce a window name that is unique to the host. The window
* name must be unique in case the user has multiple concurrent
* sessions open with different hosts. Otherwise all sessions would
* try to share the new window and require re-authentication.
*
* Input : name - generic name for the window
*
* Returns : name with host name (with any periods removed) prepended
*******************************************************************************/
function getUniqueWindowName(name)  {
  var hostName = "localhost";
  var winName = "";
	
  for (i = 0; i < hostName.length; i++) {
  
    if ((hostName.charAt(i) == '_') ||
     ((hostName.charAt(i) >= '0') && (hostName.charAt(i) <= '9')) ||
	 ((hostName.charAt(i) >= 'a') && (hostName.charAt(i) <= 'z')) ||
	 ((hostName.charAt(i) >= 'A') && (hostName.charAt(i) <= 'Z')))
       winName = winName + hostName.charAt(i);
    }
  return (winName + name);
}

//******************************************************************************
//* Purpose: Open separate window to display help.
//*
//* Input: url - path to requested help
//*
//* Returns: none
//******************************************************************************
function openHelp(url) {

  var newurl = "";
  var end;
  
  if ((window.Help) && (!Help.closed)) {
     Help.close();
  } 
     
  // Need to convert any #'s in the url to %23 because
  // if we don't, the GET request will just request
  // the portion up to the first # (thinking the rest
  // is the anchor --- only a problem on IE)
  // find out if there's an anchor as part of the url.
  // If so, we shouldn't convert that # to %23
  
  end = url.search(/.html#/);
  
  // if no anchor, convert all #'s
  if (end < 0) {
    end = url.length;
  }
  
  // convert #'s to %23
  for (var loop = 0; loop < end ; loop++ ) {
  
    var c = url.charAt(loop);
    
    if (c == '#')
       newurl += "%23";
    else
       newurl += c;
   }
   
   // preserve the anchor
   for (var loop = end; loop < url.length ; loop++ ) {
   
     newurl += url.charAt(loop);
   }
   
   Help = window.open(newurl, getUniqueWindowName("Help"),'width=600,height=400,resizable=yes,scrollbars=yes,menubar=no,status=no' );
   Help.location = newurl;
// Help.location.reload(true); // to make sure it's refreshed
}

//******************************************************************************
//* Purpose: Open separate window to display a config summary.
//*
//* Input: url - path to requested help
//*
//* Returns: none
//******************************************************************************
function openConfig(url) {

  var newurl = "";
  var end;
  
  if ((window.Config) && (!Config.closed)) {
     Config.close();
  } 
     
  // Need to convert any #'s in the url to %23 because
  // if we don't, the GET request will just request
  // the portion up to the first # (thinking the rest
  // is the anchor --- only a problem on IE)
  // find out if there's an anchor as part of the url.
  // If so, we shouldn't convert that # to %23
  
  end = url.search(/.html#/);
  
  // if no anchor, convert all #'s
  if (end < 0) {
    end = url.length;
  }
  
  // convert #'s to %23
  for (var loop = 0; loop < end ; loop++ ) {
  
    var c = url.charAt(loop);
    
    if (c == '#')
       newurl += "%23";
    else
       newurl += c;
   }
   
   // preserve the anchor
   for (var loop = end; loop < url.length ; loop++ ) {
   
     newurl += url.charAt(loop);
   }
      
   Config = window.open(newurl, getUniqueWindowName("Config"),'width=500,height=700,resizable=yes,scrollbars=yes,menubar=no,status=no' );
   Config.location = newurl;
// Config.location.reload(true); // to make sure it's refreshed
}

//******************************************************************************
//* Purpose: confirmAndGo
//******************************************************************************
function confirmAndGo( msg, url )
{
	if (confirm(msg)) {
	
		 window.location = url;
	}
}

//******************************************************************************
//* Purpose: Allows chaining of window events. Especially useful for enabling
//*          pages that are assembled via disparate means to install function
//*          into window.onload, etc. handlers.
//*
//* Input: fn1 - existing handler to install into
//*        fn2 - new handler
//*
//* Returns: new handler
//******************************************************************************  
function chain(fn1, fn2) {
  
   return function() {
    
        if (fn1) {
          fn1();
        }
            
        if (fn2) {
          fn2();
        }
    }
}




